/** @license
 *  (C) Copyright ECMWF http://www.ecmwf.int
 *
 *  This software is licensed under the terms of the Apache Licence Version 2.0
 *  which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 *  In applying this licence, ECMWF does not waive the privileges and immunities
 *  granted to it by virtue of its status as an intergovernmental organisation nor
 *  does it submit to any jurisdiction.
 *
 */

define(['config', 'jquery', 'ajax_queue', 'bus', 'jquery-ui/widget', './ajax_indicator.css'], function(ECMWF, $, Ajaxqueue, Bus) {

$.widget('ui.ajaxIndicator', {

	options: {
		messageBus: Bus()
	},

	_init: function() {

		var self = this, o = this.options;

		this.element.find('img').orelse(function() {
			self.element.div('.activity-image');
		});

		Ajaxqueue.start(function () {
			self.element.fadeIn(100);
		});

		Ajaxqueue.stop(function () {
			self.element.fadeOut(200, function(){ self.element.hide(); });
		});

		this.element.click(function() {

			var timer;

			var div = $('<div>', {'class': 'ajax-queue'});
			div.dialog({
				title: 'AJAX Queue',
				resizable: true,
				beforeclose: function() {
					console.log('clearInterval');
					clearInterval(timer);
				}
			});

			var outstanding = { };
			var list = [];
			var trs  = [];

			var outstanding_tr = $('<tr>').appendTo($('<table>').appendTo(div));
			var table = $('<table>').appendTo(div);

			//var names = ["_seqno", "url", "queue", "name", "_canceled", "_status", "image_load", "exception", "req"];
			var names = ['_seqno', 'url', 'queue', 'name', '_canceled', '_status', 'image_load', 'exception'];

			var tr = $('<tr>').appendTo(table).addClass('header');
			$.each(names, function(_, name) {
				var s = name.replace('_', ' ');
				$('<td>').appendTo(tr).text(s);
			});

			function refresh() {
				var info = Ajaxqueue.info();

				$.each(info.outstanding,function(a,n) {
					if(!outstanding[a]) {
						$('<td>').text(a + ':').addClass('header').appendTo(outstanding_tr);
						outstanding[a] = $('<td>').appendTo(outstanding_tr);
					}
					outstanding[a].text(n);
				});

				function entry(i, p, active) {

					if(!list[i]) {
						var x = list[i] = {};
						var tr = trs[i] = $('<tr>').appendTo(table);
						$.each(names, function(_,name) {
							x[name] = $('<td>').appendTo(tr).addClass(name);
						});
					}

					x = list[i];
					trs[i].toggleClass('active', active);

					$.each(names, function(_,name) {
						var v = p[name];
						if(typeof(v) === 'boolean') {
							if(v) { v = name; }
							else { v = ''; }
						}
						if(v === undefined || v === null) {
							v = '-';
						}
						x[name].text(v);
					});
				}

				var n = info.active.length;
				$.each(info.active,function(i,p) { entry(i, p, true); });
				$.each(info.queue, function(i,p) { entry(i + n , p, false); });
				n = info.active.length + info.queue.length;
				$.each(trs, function(i,p) { trs[i].toggle(i < n); });
			}

			refresh();
			div.dialog('open');
			timer = setInterval(refresh, 1000);
		});
	},

	destroy: function(){
		console.log('destroy');
	}
});

});

