/** @license
 *  ECMWF Ajax-loader
 *  (C) Copyright ECMWF http://www.ecmwf.int
 *
 *  This software is licensed under the terms of the Apache Licence Version 2.0
 *  which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 *  In applying this licence, ECMWF does not waive the privileges and immunities
 *  granted to it by virtue of its status as an intergovernmental organisation nor
 *  does it submit to any jurisdiction.
 *
 */
/*
 *  This plugin creates or uses a set of dom elements to display loading status.
 *  There is a delay of 250ms before the loader appears.
 *  Call elem.loader('stop') to cancel.
 *
 *  This version of the Loader does not do css background switching due to a
 *  performance issue with MSIE8 doing a network request each time.
 *
 *  -we now change the visibility of both the outer and inner elements. The inner
 *   is needed for status switching, and the outer for applying composite styling.
 */
define(['config', 'jquery', 'element', './loader.css'], function(ECMWF, $){

	var icons = {aborted: 'ui-icon-alert'};

	$.fn.loader = function (status, msg) {

		return this.each(function () {
			var elem = $(this);

			if (status === 'destroy') {
				elem.find('> .loader').remove();
				return this;
			}

			var loader = elem.find('.loader').orelse(function () {
				return elem.div('.loader');
			});

			loader.stop(true, true).delay(250).queue(function (next) {
				// Hide previous state
				loader.find('div').stop().hide().parent().hide();

				if (status !== 'complete' && status !== 'canceled') {
					var current = loader.find('.' + status).orelse(function () {
						return loader.div({
							className: status + (icons[status] ? ' ' + icons[status] : ''),
							css: {position: 'absolute'}
						})
						.each(function(){ $(this).hide().parent().hide(); });
					});

					if (msg) {
						current.find('.loader-msg').orelse(function () {
							return current.div('.loader-msg');
						}).stop().text(msg).show();
					}
					else {
						current.find('.loader-msg').hide();
					}

					current.stop().show().parent().show();
				}

				next && next();
			});
		});
	};

	ECMWF.ajax_loader = function(item) {
		return function(status) {
			item.loader(status);
		};
	};
});
