/** @license
 *  (C) Copyright ECMWF http://www.ecmwf.int
 *
 *  This software is licensed under the terms of the Apache Licence Version 2.0
 *  which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 *  In applying this licence, ECMWF does not waive the privileges and immunities
 *  granted to it by virtue of its status as an intergovernmental organisation nor
 *  does it submit to any jurisdiction.
 *
 */
define(['config', 'jquery', 'bus', 'settings', 'browser', 'jquery-ui/slider', 'view-dialogue', './animation_controls.css'], function(ECMWF, $, Bus, Settings, Browser) {

$.widget('ui.animationControls', $.ui.viewDialogue2, {

	options: {
		name: 'controls',
		title: 'Controls',
		messageBus: Bus()
	},

	_init: function() {
		var self = this, o = this.options;

		self.enable_range_selectors = true;

		$.ui.viewDialogue2.prototype._init.call(this);

		var pref;
		Settings.each_pref(function(item){
			if(item.name === 'animation_speed') pref = item;
		});

		self.item.attr({id: 'controls'}).h4('Animation speed');

		var slider = self.item.div().slider({
			min: pref.range[0],
			max: pref.range[1],
			value: - pref.value + pref.range[0] + pref.range[1],
			step: 0.1,
			stop: function(event){
				var val = - parseFloat(slider.slider('values', 0)) + pref.range[0] + pref.range[1];
				$.setUserPreference('animation_speed', val);
				o.bus.send('animation-speed-set', val);
			}
		});

		// create an additional slider background to workaround issues with the handle extending beyond the background at endstops.
		self.item.div('.ui-widget ui-widget-content ui-corner-all dummy-slider');

		self.item.h4('.border').text('Animation range');

		self.selectors = $.each([{
				label: 'Start',
				help: 'Animation start time',
				time: null,
				get_current_time: function(){
					return self.selectors[0].time
						? self.selectors[0].time
						: (self.base_times ? self.base_times.values[0] : null);
				},
				get_current_date: function(){
					var time = self.selectors[0].get_current_time();
					return time ? time.valid_time : null;
				}
			}, {
				label: 'End',
				help: 'Animation end time',
				time: null,
				get_current_idx: function(){
					return self.base_times ? (self.base_times.values.length - 1) : -1;
				},
				get_current_time: function(){
					return self.selectors[1].time ?
						self.selectors[1].time
						: (self.base_times ? self.base_times.values[self.selectors[1].get_current_idx()] : null);
				},
				get_current_date: function(){
					var time = self.selectors[1].get_current_time();
					return time ? time.valid_time : null;
				}
			}],
			function(i, selector){
				self.enable_range_selectors &&
				(selector.el = self.item.div('<div class="label" style="">' + selector.label + '</div>'))
					.dateSelect({
						mode: 'enhanced',
						select: function(date){
							selector.time = lookup_date(date);
							console.log('animationControls: ' + selector.label + ' select!', [selector.time.valid_time.json], '--->', [date.json]);
							o.messageBus.send('animation-set', {range: {start: self.selectors[0].get_current_time(), end: self.selectors[1].get_current_time()}});
							selector.tooltip.data('uiModeTooltip') && selector.tooltip.modeTooltip('close');
						},
						on_open: function(){
							selector.tooltip.data('uiModeTooltip') && selector.tooltip.modeTooltip('disable');
						},
						on_close: function(){
							selector.tooltip.data('uiModeTooltip') && selector.tooltip.modeTooltip('enable');
						}
					});

				selector.tooltip = selector.el
					.find('.ui-selectmenu-button')
					.modeTooltip({
						content: function(){ return selector.help; }
					});
		});

		self.item.p('<i>The animation range can also be set by SHIFT-clicking on the main Time Slider. The position can be changed by dragging the orange markers.</i>');

		var increment_values = [1, 24, 12, 6, 3];
		var use_radio_for_increment = true;
		if(use_radio_for_increment){
			var vertical = false;
			self.item.h5('.border').text('Animation Increment');
			var radio = self.item.div('.increment')
				.radio({
					group: 'animation-increment',
					buttons: increment_values,
					labels: $.map(increment_values, function(i){ return i === 1 ? 'All' : i + ' hours'; })
				});
			self.item.find(':radio').click(function(){
				setTimeout(function(){ //allow default handler to run first.
					var interval = radio.find(':checked').val();
					$.inArray(increment_values, interval) && (self.animation_client.setInterval(+interval), self.animation_client._interval !== (+interval) && console.warn('interval!!!'));
				}, 10);
			});
			vertical && self.item.find('label').addClass('vertical').after('<br>');
			self.item.find('label:first').css({textAlign: 'left', paddingLeft: 4});
		}else{
			var ul = $('<ul>');
			var increments = $.each(['24 hours', '12 hours', '6 hours', '3 hours'], function(i, item){ ul.li(item); });
			var inc = self.item.div('<div class="label">Increment</div>')
				.selectorList({
					mode: 'ui-dialogue',
					select: function(_interval){
						inc.selectorList('setItem', _interval);
						var interval = parseInt(_interval, 10);
						interval && (self.animation_client.set_interval(interval));
					}
				})
				.selectorList('setItems', increments, increments[1], ul)
				.modeTooltip({
					content: function(){ return 'Animation increment';}
				});
		}

		self.animation_client = $.extend(new $.AnimatorClient(), {
			getFrames: function(){
				self.animation_client.updateFrameList();
				return [];
			},

			//return false unless time is at the correct interval
			//-the base_time is not checked, so the framelist will still be valid after a base_time change.
			filter: function(time){
				if(this._frames.length){
					for(var i in this._frames){
						var t = this._frames[i];
						if((t.valid_time.getTime() === time.valid_time.getTime())) return true;
					}
					return false;
				}
				return true;
			},

			updateFrameList: function(){
				this._frames = [];
				var time = self.selectors[0].get_current_time();
				var end = self.selectors[1].get_current_time();
				if(time){
					while((time.valid_time.getTime() - end.valid_time.getTime()) <= 0) {
						this._frames.push(time);
						time = time.add(this._interval);
					}
				}
			},

			setInterval: function(interval){
				console.log('setInterval: interval=', interval);
				self.animation_client._interval = interval;
				self.animation_client.updateFrameList();
				o.bus.send('animation-set', {filter: true});

				(interval === 1) ? self.animation_client.unregister() : self.animation_client.register();
			},

			register: function(){
				if(!self.animation_client._is_registered){
					$.animationAddFilter(self.animation_client);
					self.animation_client._is_registered = true;
				}
			},

			unregister: function(){
				if(self.animation_client._is_registered){
					$.animationRemoveFilter(self.animation_client);
					self.animation_client._is_registered = false;
				}
			},
			_is_registered: false,

			_interval: 1,
			_frames: []
		});

		o.messageBus
			.listen('chart-product-availability', function(event, msg){
				self.base_times = msg.valid_time;
				var dates = $.map(self.base_times.values, function(a) { return a.valid_time });
				self.enable_range_selectors &&
					$.each(self.selectors, function(){ this.el.dateSelect('setDates', dates, this.get_current_date()); });
				self.animation_client.updateFrameList();
			})
			.listen('chart-animation-selection-change', function(event, msg){
				var start_changed = (msg.start != self.selectors[0].time);
				var end_changed = (msg.end != self.selectors[1].time);

				self.selectors[0].time = msg.start;
				self.selectors[1].time = msg.end;

				self.enable_range_selectors &&
					$.each(self.selectors, function(){
						this.el.dateSelect('setDate', this.get_current_date());
					});

				if(start_changed || end_changed){
					self.animation_client.updateFrameList();
					o.bus.send('animation-set', {filter: true});
				}
			});

		if(Browser.msie && (Browser.version === '7.0' || Browser.version === '8.0')){ // prevent lightDialogue mouse interaction
			self.item.mousemove(function(event){

				var s = slider.data('uiSlider');
				if(s._mouseStarted){
					s._mouseDrag.apply(s, [event]);
					return false;
				}
			});
		}

		function lookup_date(time){
			if(!time) return;

			var dates = self.base_times.values;

			for(var i in dates){
				var item = dates[i].valid_time;
				if(item.getTime() === time.getTime()){
					return dates[i];
				}
			}
		}
	},

	destroy: function(){
		$.ui.viewDialogue.prototype.destroy.call(this);
	}
});

});

