/*
 *  Application
 *
 *  Provides a layer on top of Framework providing checks that resources have loaded.
 *  Normally, using $.framework.onReady() is preferable to Application.
 */
define(['config', 'jquery', 'ajax_queue', 'bus', 'utils', 'forecaster/browser_check'], function(ECMWF, $, Ajaxqueue, Bus, Utils) {

return function(on_application_loaded) {

	var self = this;

	Utils.same_origin() || (Ajaxqueue.options.host = location.protocol + '//' + ECMWF.options.backend_host);

	var errors = [];
	Bus().listen('log-error', function(_, e){
		errors.push(e);
	});

	function cannot_load_application(error) {
		// Assume jQuery not loaded
		var text = ['Failed to load application. Try reloading the page.'];
		if(error) {
			for(var n in error)
			{
				try{
					text.push((error.length > 1 ? n + ': ' : '') + error[n]);
				}
				catch(ignore) {}
			}
		}
		alert(text.join('\n'));
	}

	setTimeout(function() {
		function on_ready() {
			if(!$.fn.browserCheck()) return;
			if(errors.length){
				cannot_load_application(errors);
				return;
			}

			Bus().send('application-loaded', {});
			on_application_loaded(self);

			Bus().listen('fullscreen', function() {
				function launchFullScreen(element) {
					if(element.requestFullScreen) {
						element.requestFullScreen();
					} else if(element.mozRequestFullScreen) {
						element.mozRequestFullScreen();
					} else if(element.webkitRequestFullScreen) {
						element.webkitRequestFullScreen();
					}
				}

				launchFullScreen(document.documentElement);
			});
		}

		function a(){
			if(!ECMWF.options.loaded) {
				cannot_load_application(['Could not load options']);
			}

			$.framework.onReady(function() {
				if(__DEV__){
					on_ready();
				}else{
					try {
						on_ready();
					} catch(e) {
						cannot_load_application(e);
					}
				}

				setTimeout(function check_css_is_loaded() {
					// During transition to web2013 template, both the new and old styles are tested for.
					// Also note that not all pages currently have H1
					if(!(/^(?:rgb\(0, 0, 128\)|#000080)$/.test($('h1').css('color')) || /^(?:dotted)$/.test($('.text-link').css('border-bottom-style')) || $('h1').css('font-weight') + '' === '300'))
						cannot_load_application(['CSS not loaded']);
				}, 4000);

				$('#main > script').remove();
			});
		}

		if(__DEV__){
			a();
		}else{
			try {
				a();
			} catch(e) {
				cannot_load_application(e);
			}
		}
	}, 1);
}

});
