/*
 *  Mixin for the BasicChart widget to handle Epsgrams.
 */
define(['jquery', 'storage', 'chart/model/parameters', 'virtual-dom/h', 'virtual-dom/diff', 'virtual-dom/patch', 'virtual-dom/create-element', /*'form/handlers/location', */'utils', 'unit', 'time', 'chart-app/router', 'element'], function($, localStorage, Parameters, h, diff, patch, createElement, Utils, Units, Time, page) {

	var epsgram_history = [];

	function Epsgram() {
	}

	Epsgram.prototype = {
		epsgram_init: function() {
			console.log('epsgrams_init');
			var self = this;

			var is_wmo = self.instance.has_widget('wmo_locations');

			setTimeout(function() {
				switch(is_wmo){
					case true:
						require('form/handlers/country_location');
						break;
					case false:
						require('form/handlers/location');
						break;
				}
			}, 0);

			self._history.init.call(self, is_wmo);

			self.location_handler = function(obj){
				console.log('location.click');
				require(['dialogue2013'], function(Dialogue){
					new Dialogue({
						id: 'location-select',
						title: '<h2>Location</h2>'
					});
					var d = $('#location-select')
						.each(function(){
							if($('#location-select').hasClass('in')) $('#location-select').modal('hide');
						})
						.modal('show');

					var default_location = {lat: -50, lon: -9};
					var values = $.extend(default_location, Utils.map(self.instance.variables, function(v, i) {
						if(!!~$.inArray(i, ['lat', 'lon'])) return v.value;
					}));

					var content = d.find('.modal-body');
					content.each(function _blah(){
						content.empty();
						var footer = d.find('.modal-footer').empty();

						content.form({
							items: [
								{'type': is_wmo ? 'country_location' : 'location', name: 'location', title: 'Location', value: values}
							],
							ok: function(form_data){
								var location = form_data.location;
								if(is_wmo){
									console.log('basicChart.location.ok: location=', location);

									if(location.country){
										d.modal('hide');

										localStorage.ensgram_wmo_country = JSON.stringify(location.country);
										self.bus.send('wmo-country', location.country);
									}
								}
								else {
									console.log('basicChart.location.ok: lat=', location.lat, ' lon=', location.lon);
									if(location){
										d.modal('hide');
										self
											._set_parameter('lat', location.lat + '')
											._set_parameter('lon', location.lon + '')
											._set_parameter('altitude', location.altitude + '')
											._set_parameter('station_name', location.city || '');
									}
								}
							},
							cancel: function(form_data){
								d.modal('hide');
							}
						});
						content.find('.form-help, .form-item-title').remove();
						content.find('.form-buttons input#_ok').addClass('btn').appendTo(footer);
						content.find('.form-buttons input#_cancel').addClass('btn').appendTo(footer);
						content.find('.form-buttons').remove();
						content.find('.form-row').css({background: 'none'});

						//content.locationMap();
					});
				});
			}

			self.instance.on.image.subscribe(self, function() {
				/*
					$.fn.wwwChartParameterBar && // TODO
					self.parameter_bar.wwwChartParameterBar('setItems', $.extend(templates.forecast_base_time, {
						values: $.map(self.instance.data.availability.availability, function(a){ return a[0]; }),
						labels: $.map(self.instance.data.availability.availability, function(a){ return new Time(a).asText(); })
					}));
				*/

				self._history.update.call(self);
				self._metadata.update.call(self);

				self.instance.variables.location = self.instance.variables.location || new Parameters.dynamic({name: 'location', title: 'Location', type: 'location', values: []})

				// ... would need to pass this a real time value
				//self.instance.variables.base_time || (self.instance.variables.base_time = new Parameters.time(templates.forecast_base_time));
			});

			self.instance.events
				.bind('load-end', function(){
					self._stop_spinner();
					if(self.instance.error_msg){
						self.bus.send('log-error', {title: 'Plot failed', message: self.instance.error_msg});

					}else if(self.instance.data.is_fall_back) {
						self.bus.send('log-warning', {title: 'Fallback plot', message: 'The requested plot is not available. A fallback is displayed.'});
					}
				});
		},

		epsgram_destroy: function() {
		},

		_history: {
			block: $('.column-1'),
			init: function(is_wmo) {
				var self = this, history = self._history;

				history.storage = is_wmo ? 'ensgram_history_wmo' : 'ensgram_history';
				epsgram_history = JSON.parse(localStorage[history.storage] || '[]');
			},
			update: function() {
				var self = this, history = self._history, index;

				var json = this.instance.as_json().product;

				if((index = this.history_exists(json))){
					(epsgram_history.length >= 20) && epsgram_history.pop();

					(index > -1) && epsgram_history.splice(index, 1);
					delete json.values.location;
					epsgram_history.unshift({values: json.values});

					localStorage[history.storage] = JSON.stringify(epsgram_history);

					self.bus.send('location-changed');
				}
			}
		},

		_metadata: {
			update: function () {
				var self = this, meta = self._metadata;

				require(['help-panel', 'markup'], function() {
					if(!meta.block){
						meta.block = $('.chart-metadata')

						self.element.helpProvider({getContent: function(){
							return {content: meta.get_epsgram_help.call(self)};
						}});
					}

					$.when(self.element.helpProvider('getContent')).then(function(out){
						out || (out = {});
						out.content || meta.block.hide();
						out.content &&
							meta.block.show().find('.metadata-content')
								.markup('##Location information\n'  + out.content)
								.siblings('.title').find('> span').text(out.title || 'No Help')
								.trigger('loading', 'false');
					});
				});
			},

			get_epsgram_help: function() {
				var self = this;
				var list = [];

				if (!self.instance.data.metadata) {
					return 'Cannot get help information.';
				}
				var metadata = self.instance.data.metadata;

				if (!metadata.distance) {
					return 'Cannot display help information. Missing Distance data.';
				}

				var m = Units.get('m');
				var km = Units.get(metadata.distance.distance_unit || 'km');

				var gridpoint_type = (metadata.land_sea_mask < 0.5) ? 'sea' : 'land';

				metadata.title && list.push('### ' + metadata.title );
				list.push('Location requested: **' + $.formatGeoLocation(metadata.requested_location) + '**.  ');
				list.push('Nearest ENS grid point: **' + $.formatGeoLocation(metadata.location) + '**.');
				list.push('');

				if (metadata.height !== '' &&  metadata.height !== -9999 && metadata.db !== '10_days_wave') {
					console.log('metadata.height:<' , metadata.height, '>');
					list.push('Height of requested location (based on the GTOPO30 dataset): **' + m.asHTML(metadata.height) + '**, ');
					if (metadata.db != "plumes") {
						list.push('height of ENS orography at nearest grid point: **' + m.asHTML(metadata.eps_height) + '**, ');
						list.push('height of high resolution orography interpolated to ENS resolution at nearest grid point: **' + m.asHTML(metadata.deterministic_height) + '**.');
					}
					if (metadata.temperature_correction) {
						list.push('');
						if(metadata.temperature_adjustment) {
							list.push('Temperature parameter is adjusted to the requested location by **' + $.formatCelcius(metadata.temperature_adjustment.deterministic_adjustement, 1) + '**' + ' in high resolution forecast curve and ' + '**' + $.formatCelcius(metadata.temperature_adjustment.eps_adjustement, 1) + '** in ENS forecast curves.');
						}
						else
						{
							list.push('Temperature parameter is adjusted to the requested location');
						}
						list.push('');
					}
				}
				list.push('');
				list.push('');
				//TODO: Fix me later
				if (metadata.db === '10_days_wave') {
					list.push('The nearest ENS grid point is located at **' + km.asHTML(metadata.distance.distance) + ' ' + metadata.distance.direction + '** of the selected location.');
				}
				else {
					list.push('The nearest ENS grid point is a **' + gridpoint_type + ' point**, ');
					list.push('located at **' + km.asHTML(metadata.distance.distance) + ' ' + metadata.distance.direction + '** of the selected location.');
				}

				/* $.each(metadata, function(k,v) { list.push("* " + k + " = " + v); }); */

				return list.join('\n');
			},
			block: null
		},

		history_exists: function(data) {
			var data_to_add = data.values;
			var i = 0;
			for (var e in epsgram_history) {
				if(!epsgram_history[e].values){
					// incorrect format - ignore
				}else{
					var matched = true;

					for (var key in epsgram_history[e].values) {
						var axis = JSON.stringify(epsgram_history[e].values[key]);
						if (axis != JSON.stringify(data_to_add[key])){
							matched = false;
							break;
						}
					}

					if (matched)
						return i;
				}
				i++;
			}
			return -1;
		}
	}

	/*
	var templates = {
		forecast_base_time: {name: 'base_time', title: 'Forecast base time', type: 'retention', values: []}
	}
	*/

	$.widget('ui.parameterLocation', $.ui.parameterControl, {

		options: {
			title: 'Location ...'
		},

		_make_html: function() {
			var o = this.options;

			var chart = this.element.parents('.chart-column').find('.basic').basicChart('instance');

			(this.element
				.empty()
				.addClass('parameter-selector')
				.button({label: o.title}))
				.click(chart.location_handler);
		}
	});

	/*
	function VColumn(el, options) {
		this.element = el;
		this.sections = []

		this.dom_tree = h('div.vdom-root')
		// TODO do not modify the dom until we are ready
		this.element.append(this.dom_tree_root = createElement(this.dom_tree));

		this.render = $.throttle(300, this._render);
	}

	VColumn.prototype = {
		add: function(name, fn) {
			this.sections.push({name: name, render: fn})
			return this;
		},
		set_state: function(name, state) {
			var section = $.findWhere(this.sections, {name: name})
			section.state = state;
			this.render()
			return this;
		},
		_render: function() {
			var new_tree = h('div.vdom-root', $.map(this.sections, function(v){ return v.render(v.state); }))
			this.dom_tree_root = patch(this.dom_tree_root, diff(this.dom_tree, new_tree));
			this.dom_tree = new_tree;

			this._on_render && this._on_render();
		},
		on_render: function(fn) {
			this._on_render = fn;
			return this;
		}
	}
	*/

	return Epsgram;
});
