/*
 *  Base class for widget displaying list of chart parameter options.
 *
 *  Typically it will have a child widget such as chartParameterSelector
 */
define(['jquery', 'bus', 'toolbar'], function($, Bus){

	$.widget('ui.parameterControl', {
		options: {
			buttons: false, // forward, back, animate
			parameter: null,
			bus: Bus(),
			on_select: $.noop
		},

		_create: function() {
			var self = this, o = self.options;
			if(Array.isArray(o.parameter)) o.parameter = o.parameter[0];

			this.element.addClass(['parameter-control', this._dash_name()].join(' '));
			this._setItems();

			if(o.buttons){
				this.element
					.div('.float-left toolbar-container')
						.prependTo(this.element)
						.toolButton({
							tooltips: true,
							tools : [[
								['prev-step', function(){ console.log('prev'); }],
								['next-step', function(){ console.log('next'); }]
						]]});

				this.element
					.div('.float-right toolbar-container')
						.toolButton({tools: [['animation-toggle']], messageBus: o.messageBus, tooltips: true})
			}

			o.bus.listen(this, 'chart-content-update', function(_, msg) {
				// TODO we get PAST messages which are possibly for the old product - is it possible to weed these out?

				//FIXME: Warning message below is generated because there is more controls then 
				//       it should be in the new product. 
				//       It may cause other issues as there is a hanging object somewhere causing it.

				if(!msg.product.variables[o.parameter.name]) return console.warn('cannot update control for non-existant parameter: ', o.parameter.name, $.map(msg.product.variables, function(v){ return v.name; }));

				self._value = msg.product.variables[o.parameter.name].value;
				self._on_content_update(msg.changed);
			});

			o.parameter.subscribe(this, function(){
				self._on_parameter_change();
			});
		},

		_destroy: function() {
			var self = this, o = self.options;

			this.element.removeClass(this._dash_name());
			o.parameter.unsubscribe(this);
			o.bus.forget_me(this);
		},

		setItems: function(parameter) {
			var self = this, o = self.options;

			o.parameter = parameter;

			self._setItems();
		},

		_setItems: function() {
			var self = this, o = self.options;

			self._make_html();

			o.parameter.values && $.each(o.parameter.values, function(i, v) {
				self._make_item_html(i);
			});
		},

		_make_html: $.noop,
		_make_item_html: $.noop,
		_on_content_update: $.noop,
		_on_parameter_change: $.noop,

		_dash_name: function() {
			return this.widgetName.replace(/([a-z])([A-Z])/g, '$1-$2').toLowerCase();
		},

		bind: function(fn){
			var self = this;
			return function(){ return fn.apply(self, arguments); };
		}
	});
});
