/*
 *  A button with a drop down menu.
 *
 *  Unlike the previous version, this version uses the menuSelect widget.
 */
define(['jquery', 'bus', 'jquery-ui/selectmenu', 'perfect-scrollbar'], function($, Bus) {

	$.widget('ui.parameterSelectmenu', $.ui.selectmenu, {
		options: {
			classes: {
				'ui-selectmenu-menu': 'parameter-menu'
			},
		},

		_renderButtonItem: function(item) {
			var buttonItem = $('<span>');

			this._setText(buttonItem, this.options.label + ': ' + item.label);
			this._addClass(buttonItem, 'ui-selectmenu-text');

			return buttonItem;
		},

		_resizeButton: function() {
			var controls = this.element.parent().parent().parent().find('.parameter-control:visible')
			if(controls.length > 2){
				var width = controls.length === 3
					? 13
					: (controls.length === 4
						? 9
						: 8
					)
				controls.css({maxWidth: width + 'em'})
			}
		}
	});

	$.widget('ui.dateParameterMenu', $.ui.parameterSelectmenu, {
		_renderItem: function(ul, item) {
			var label = item.label.split(' ');

			return ul
				.li(item.disabled && '.ui-state-disabled')
				.html(
					$.map(['day', 'date', 'month', 'year'], function(v){ if(label.length) return '<span class="' + v + '">' + label.shift() + '</span>' }).join(' ') + ' ' +
					label.join(' ')
				);
		}
	});

	$.widget('ui.chartParameterMenu', {
		options: {
			name: 'missing',
			title: 'Untitled',
			value: null,
			values: [],
			labels: null,
			bus: Bus(),
			select: null,           // function to mark the correct menu item dom element as selected. Is a temporary workaround for base_time parameter being in unusual format.
			on_click: $.noop
		},

		_create: function(){
			var self = this, o = self.options;
			self.selected_value = o.value || null;

			if(o.value && !~$.inArray(o.value, o.values)) console.warn('chartParameterMenu: invalid value', o.value);

			var widget = o.name === 'base_time' ? 'dateParameterMenu' : 'parameterSelectmenu';

			o.labels || (o.labels = o.values);

			var menu = self.element.addClass('parameter-selector')
				.formSelect(self._make_items())
				// TODO setting value here is having no effect
				.val(o.value)
				[widget]({
					name: o.name,
					label: o.title,
					select: function(event, ui) {
						console.log('select.select: ui.item=', ui.item);
						o.on_click(event, ui, o.name);
						self.select(ui.item.element);
					},
					open: function(ev) {
						menu.menu
							.parent()
							.css({maxHeight: Math.max(40, $(window).height() - menu.menu.offset().top - 10)})
							.perfectScrollbar({suppressScrollX: true, wheelSpeed: ECMWF.wheelspeed || 10})
							.perfectScrollbar('update');
					}
				})
				[widget]('instance');

			self.element.find('.ui-selectmenu-button').addClass('fff');

			menu.menu.on('click', 'li', function(event) {
				var li = $(this);
				var i = li.index();
				var item = {index: i, value: self.element.find('select option:nth-child(' + (i + 1) + ')').attr('value'), label: li.text(), element: li};
				menu._select(item, event);
			});

			// TODO this version does use data-value
			o.bus.listen('chart-content-update', function(_, msg) {
				var instance = msg.product;
				if(self.selected_value !== instance.variables[o.name].value){
					self.selected_value = instance.variables[o.name].value;
					self.select($(this).find("[data-value='" + self.selected_value + "']"));
				}
			});

			self.menu = menu;

			self.select_by_value(o.value);
		},

		_destroy: function(){
			this.element.removeClass('parameter-selector');
		},

		setItems: function(parameter) {
			var o = this.options;

			o.values = parameter.values;
			o.labels = parameter.labels || [];
			this.menu_items = null;
			this.menu && this.menu.remove();
		},

		select_by_value: function(value) {
			var o = this.options, menu = this.menu;

			this.selected_value = value;

			var index = $.inArray(this.selected_value, o.values)
			menu.element.prop('selectedIndex', index)
			menu.refresh()
			this.select(menu.menu.find('li:eq(' + index + ')'))
		},

		select: function(el) {
			el.parent().find('span.ui-icon').remove();
			el.span('.ui-icon ui-icon-check');
		},

		_make_items: function() {
			var self = this, o = self.options;

			return this.menu_items = {
				options: $.map(o.values, function(a, i) {
					return [[a, o.labels[i] || a]];
				}),
				label: o.title
			}
		}
	});
});
