/*
 *  basic_chart/plugins/time_navigator
 *
 *  This chart mixin integrates the timeNavigator widget with the chart.
 *
 */
define(['jquery', 'bus', '../parameter_controls/factory'], function($, Bus) {

	return function(chart, o){
		this.name = 'time_navigator';
		var self = this, instance;

		if(typeof o.element === 'function'){
			o.element = o.element();
			self.el_was_added = true;
		}

		chart.bus.listen('product-swapped', function() {
			instance = chart.instance;

			chart.bus.listen_once('chart-product-availability', function (_, msg) {
				if(msg.valid_time.values.length > 1){
					self.element && self.element.show() || add_time_navigator();
				}else{
					self.element && self.element.hide();
				}
			});
		});

		function add_time_navigator() {

			require(['tool-button'], function() {
				require(['time-navigator'], function() {

					(self.element = (o.element || (self.el_was_added = true, chart.container.div('#time-navigator').insertAfter(chart.element))))
						.show()

						// The time_navigator is already a container, but this additional
						// instance adds the 'root' property so that it can react to changes.
						.container({
							root: chart.container.hasClass('containr') ? chart.container : self.element,
							layout: 'root'
						})

						.timeNavigator({
							messageBus : chart.bus,
							show_bt    : false,
							ready      : chart._loader.checks.tnav.resolve
						});

					// unfortunate that this has to be done explictly here. TODO make automatic
					chart.instance.variables.time.availability && chart.instance.availability(chart.instance.variables.time.value, function(avail){
						chart.instance.variables.time.format && (avail.format = chart.instance.variables.time.format);
						chart.bus.send('chart-product-availability', avail);
					});

					// FIXME this will only work on the product browser page
					Bus(chart.element).send('size-changed');

					chart
						._listen('slider-time-change', function(_, time) {
							console.log('basicChart.time_navigator: slider change: ', time && time.base_time && [time.base_time.format('mmdd:HH'), time.step, time.valid_time.format('mmdd:HH')]);
							Bus().send('add-page-state');
							instance.variables.time.set(time);
						})
						._listen('size-changed', function(_, size) {
							self.element.trigger('layout', size ? {width: size.width} : null);
						});
				});
			});
		}

		this.destroy = function() {
			this.element && this.element[this.el_was_added ? 'remove' : 'empty']();
		}
	}
});
