/*
 *  (C) Copyright ECMWF http://www.ecmwf.int
 *  This software is licensed under the terms of the Apache Licence Version 2.0
 *
 *  basic_chart/tools
 *
 *  This chart mixin adds a drop down menu for such actions as Print, Add-to-dashboard.
 *
 *  Was changed to use buttons instead of a drop down menu - for the menu version see revisions before 2016-12-10
 *
 */
define(['config', 'jquery', 'element', 'ajax_queue', 'toolbar', 'jquery-ui/button', './tools.css'], function(ECMWF, $, Dom, Ajaxqueue) {

	return function(chart, o){
		this.name = 'tools';
		var self = this;

		chart.bus.listen_once('product-swapped', function() {
			require(['user_info', 'tool-buttons/print'], function(User, PrintTool) {
				User.info.then(function(user) {
					var tool = new PrintTool.default();

					// temporary
					if((typeof o.parent === 'object') && o.parent.find('.option-print').length){
						return console.error('tools: options button already present'), (process.env.NODE_ENV !== 'prod' && console.trace());
					}

					if(typeof o.parent === 'string') o.parent = $(o.parent);

					$.each([
						{
							name: 'Print pdf',
							icon: 'ui-icon-print',
							select: function(e) {

								if(chart._print){
									chart._print(); // basic chart
								}else{
									if(chart.product.api > 1.0){
										chart.product.www_print(chart.product.title(chart.time, false, true), function(result){
											if(result && result.url){
												location.href = result.url;
											}
										});
									}else{
										chart.product.print({
											json: {
												product: chart.product.as_json(chart._viewport(), chart.time, chart.mail, 'pdf'),
												title: chart.product.title(chart.time, false, true)
											},
											complete: function(result) {
												if(result && result.result.url){
													location.href = result.result.url;
												}
											}
										});
									}
								}
								require(['notify'], function() {
									$.notify({id: 1, kind: 'ok', story: 'ok', message: 'Printing...'});
								});
							}
						},
						{
							name: 'Add to dashboard',
							icon: 'ui-icon-plus',
							classes: ['add-to-dashboard'],
							activatable: function() {
								//var is_interactive = !!chart.product.layerByID;
								//return !is_interactive && (ECMWF.options.site && ECMWF.options.site.dashboard);
								return ECMWF.options.site && ECMWF.options.site.dashboard && user.uid;
							},
							select: function(e) {
								console.log('add-to-dashboard click');
								(chart.options.instance || chart.product).add_to_dashboard({}, function(result) {
									console.log('Saved to dashboard');
									require(['notify'], function() {
										$.notify({id: 1, kind: 'ok', story: 'ok', message: 'Saved to <a href="' + ECMWF.options.site.dashboard + '">dashboard \'' + result.dashboard.title + '\'</a>'});
									});
								});
								require(['notify'], function() {
									$.notify({id: 1, kind: 'ok', story: 'ok', message: 'Saving...'});
								});
							}
						},
						/* TODO eccharts does not currently support opening named products
						{
							name: 'Open in eccharts',
							icon: 'ui-icon-extlink',
							classes: ['open-in-eccharts'],
							enabled: function() {
								return User.is_forecaster()
							},
							select: function(e) {
								console.log('open-in-eccharts click');
								var name = typeof chart.product.name === 'function' ? chart.product.name() : chart.product.name
								window.location.href = ECMWF.options.site.forecaster + '?widget=' + name;
								return false;
							}
						},
						*/
						{
							name: 'Permalink',
							icon: 'ui-icon-link',
							enabled: function() {
								return ECMWF.options.enable_sharing !== false && user.uid && !chart.product.is_eccharts();
							},
							/*
							activatable: function() {
								var is_interactive = !!chart.product.layerByID;
								return __DEV__ || !is_interactive;
							},
							*/
							select: function() {
								require(['dialogue2013'], function(Dialogue) {

									new Dialogue({
										id: 'chart-permalink',
										title: '<h2>Chart permanent link</h2>',
										html: $('<div>').div([
											Dom.p('The following URL can be used to download charts automatically'),
											Dom.textArea({id: 'download-link', rows: 5, cols: 60}).text('Loading...'),
											Dom.p().html("Add <i>'&format=pdf'</i> to the end of URL if you wish to download PDF version.")
										]).html()
									});

									$('#chart-permalink').modal('show');

									Ajaxqueue({
										url: '/plots/get-token/',
										dataType: 'jsonp',
										complete: function(result) {
											$('#download-link').text(result.token
												? chart.product.download_url(result.token, result.email)
												: 'Unable to obtain token, please make sure you are registered and logged in with your registered id to see the perma-link.')
										}
									});
								});
							}
						},
						{
							name: 'Download data',
							icon: 'ui-icon-arrowthickstop-1-s',
							enabled: function() {
								return __DEV__;
							},
							activatable: function() {
								chart.product.has_widget('data');
							},
							select: function() {
								console.log('TODO download data');
							}
						},
						/*
						},
						{
							name: 'Edit',
							icon: 'ui-icon-pencil',
							enabled: function() {
								return !!ECMWF.options.ajax.chart_content && User.is('ecmwf');
							},
							select: function() {
								window.open(ECMWF.options.ajax.chart_content + '/' + page._path.page + '?edit', 'chart-edit');
							}
						}
						{
							name: 'Speed up >',
							icon: 'ui-icon-arrow-1-n',
							select: Bus().sendfn('animation-speed-up')
						},
						{
							name: 'Slow down <',
							icon: 'ui-icon-arrow-1-s',
							select: Bus().sendfn('animation-speed-down')
						*/
						{
							name: 'Administer product',
							icon: 'ui-icon-gear',
							classes: ['open-in-admin'],
							enabled: function() {
								return ECMWF.options.site.chart_admin && User.is('ecmwf') && !chart.product.is_eccharts()
							},
							select: function(e) {
								var name = typeof chart.product.name === 'function' ? chart.product.name() : chart.product.name
								window.open(ECMWF.options.site.chart_admin + name, 'chart-admin');
								return false;
							}
						}
					], function(i, v){
						//var parent = typeof o.parent === 'string' ? $(o.parent) : o.parent;
						(!v.enabled || v.enabled()) &&
							o.parent.div('.option-' + v.name.toLowerCase().replace(/ /g, '-'))
								.button({label: v.name, icons: {primary: v.icon}, text: false, disabled: (v.activatable && !v.activatable())})
								.click(v.select);
					});

					/* -- stepping through parameter values is disabled for now
							-the purpose is related to animating arbitrary parameters.
					var retention_items = [];
					$.each(chart.instance.variables, function(i, param){
						$.merge(items, retention_items = [
							{
								name: 'Next ' + param.title,
								icon: 'ui-icon-arrowthick-1-e',
								activatable: function(){
									var p = chart.instance.variables[param.name];
									return $.inArray(p.value, p.values) !== p.values.length - 1;
								},
								select: function(e){
									chart.instance.variables[param.name].next();
									return false; // dont close menu
								}
							},
							{
								name: 'Prev ' + param.title,
								icon: 'ui-icon-arrowthick-1-w',
								activatable: function(){
									var p = chart.instance.variables[param.name];
									return !!$.inArray(p.value, p.values);
								},
								select: function(e){
									chart.instance.variables[param.name].previous();
									return false; // dont close menu
								}
							}
						]);
					});
					*/

					// TODO the contents of the actions menu are now dependent on the product type.
					// So if the product changes we need to update the menu
					// FOR NOW we do not change between product types on released pages so we are ok.
					(chart.bus || chart.options.messageBus).listen('product-swapped', function() {
						o.parent.find('.results-button, .back-button')[chart.product.has_widget('epsgram') ? 'hide' : 'show']();
					});
				});
			});
		});

		// TODO add product-swapped to the interactive chart?
		if(chart.product && chart.product.layerByID){
			chart.bus.send('product-swapped');
		}

		this.destroy = function() {
			o.parent.find('.ui-button').each(function() {
				var button = $(this)
				var classes = button.attr('class')
				if(classes.indexOf('option-') > -1) button.remove();
			})
		}
	}
});
