/*
 *  Chart Application base class
 */
import ECMWF from 'config'
import $ from 'jquery'
import Bus from 'bus'
import User from 'user_info'
import Utils from 'utils'
import products from 'vgrid/products'
import chart_handlers from 'product_browser/chart_handler'
import router from './router'
import Observable from 'observable'
import h from 'virtual-dom/h'
import virtualise from 'vdom-virtualize'
import RenderTree from 'render-tree'
import mixins from 'mixins'

var CH = chart_handlers;
var login_warning_done = false;

export default	$.extend({
		config: {
			page: {
				borderx: ECMWF.options.page.borderx || 52 // 4em from css
			},
			col1: {
				// note that setting size to zero at <700 is important because it causes the right column to be bigger causing the left column to wrap
				width: [[10000, 264], [1280, 220], [1024, 200], [700, 0]] // except for the last, these must match stylesheet
			}
		},

		col1: function(o) {
			var self = this;

			/*
			$('.top-bar, .footer-bar').appendTo(
				$('body').div('.scroll-wrapper')
					.perfectScrollbar({wheelSpeed: ECMWF.wheelspeed})
					.on('mouseenter', '.ps-container', function() {
						$(this).parents('.ps-container').find('.ps-scrollbar-y:first').css({opacity: 0.1});
					})
					.on('mouseleave', '.ps-container', function() {
						$(this).parents('.ps-container').find('.ps-scrollbar-y:first').css({opacity: ''});
					})
			);
			*/

			this.have_url = $.Deferred();
			this.on = {
				url: new Observable()
			}

			var classes = o.el.attr('class').split(' ')

			self.vdom = {
				l: new RenderTree({
					filters: [
					],
					sections: [
						{
							name: 'filters',
							title: 'Filters',
							render: function(){
								return !~$.inArray(self._package, ['web', 'tcyclone'])
									? $.map(self.vdom.l.filters, function(v){ return v.render() })
									: []
							}
						}
					],
					main: {
						render: function(){
							return h('div.' + classes.join('.'), $.map(
								self.vdom.l.sections,
								function(v){
									if(v.enabled !== false){
										var r = v.render();
										if(r && !Array.isArray(r)) r = [r]
										if(r.length){
											v.title && r.unshift(h('h2', [v.title]));
											return h('section.' + v.name + (v.hidden ? '.hidden' : ''), r)
										}
									}
								}
							))
						}
					},
					root: o.el[0],
					tree: virtualise(o.el[0]),
					ready: []
				})
			}

			this._package && products.set_package(this._package);

			products.subscribe(function() {
				if(!products._products.length && !login_warning_done){
					User.info.then(function(user) {
						user.uid || require(['dialogue2013'], function(Dialogue) {
							var url = (ECMWF.options.backend_host ? '//' + ECMWF.options.backend_host : '') + '/auth/login/?back=' + encodeURIComponent(location.href);

							new Dialogue({
								id: 'no-products-warning',
								title: '<h2>You are not logged in</h2>',
								html: '<p>Please note that many charts have restricted access.</p><p>If you are an ECMWF partner, please <a href="'+ url +'">log in</a> to get access to a larger range of products.</p><p>If the content you are trying to view is for licensed users only, such as a specific chart, you might still not be granted access once logged in'
							});
							$('#no-products-warning').modal('show');
						});
						login_warning_done = true;
					})
				}
			});

			products.on.items.subscribe(function() {
				self._package = products.query.package;
			})

			self.have_chart = $.Deferred(function(df) {
				Bus().listen('chart-added', function() {
					df.resolve();
				});
			})
		},

		col2: function(o) {
			this.widgets.col2.content = o.el.ensure('.panel-content');
		},

		col3: function(o) {
			var self = this;

			$('body').on('layout', function() { // triggered by ChartHandler and general container layout
				self._on_resize();
			})
		},

		scroll_to_col3: function() {
		},

		change_package: function(pkg) {
			var self = this;

			// 1- get new product list
			// 2- load the new products into the grid
			// 3- is new product selected? either display it, or go to the landing page

			products.set_package(pkg);
			products.set_list();
		},

		on_url_change: function(ctx, c) {
			var self = this, new_product, handler = chart_handlers.basic, prev = {};

			// note that we do not use chart_handler.state here because it cannot be guaranteed not to have already been updated unless the chart handler is called AFTER and VIA the url change.
			// this does NOT preserve the product on landing pages as originally intended.
			prev.name = router._path && router._path.page;
			prev.product = prev.name && products.product_by_name(prev.name);

			var is_landing_page = !ctx.params.product;

			// note difference betweeen product and page_product: product should be the currently selected product, even when on a landing page.
			var changed = $.extend({
				product: {
					widget: ctx.params && ((ctx.params.product && CH.state.list_product && ctx.params.product !== prev.name) || (CH.state.list_product && !ctx.params.product))
				},
				page_product: ctx.params.product !== (self.prev && self.prev.product),
				path: path_changed(prev.path, ctx.params.product_path),
				is_landing_page: is_landing_page !== self.is_landing_page
			}, c);

			if(changed.pkg){
				console.log('>>> on_url_change: package has changed!', this._package, ' --> ', [ctx.params['package']]);

				self.change_package(ctx.params['package']);
			}
else {

			if(changed.product.url){
				console.log('%con_url_change: product has changed!', 'background-color:#bdf', [prev.name], ' --> ', [ctx.params.product]);
				if(ctx.params.product){
					products.have.items.then(function() {
						if((new_product = products.product_by_name(ctx.params.product))){
							CH.state.params = ctx.query;
							self.on_product_select(new_product, ctx.query);

							products.selection.set(products.lookup_idx_by_name(ctx.params.product));
						}else{
							if(ctx.canonicalPath.replace(/\?.*/, '') !== router.original_path){
								console.log('assuming external url. exiting application...');
								location.href = ctx.canonicalPath;
								return;
							}
							console.log('product not found but path is unchanged. not exiting');

							require(['dialogue2013'], function(Dialogue) {
								new Dialogue({
									id: 'product-not-found',
									title: '<h2>Product not found</h2>',
									html: '<p>Could not find the product <i>' + ctx.params.product + '</i>.</p>' + (!!~$.inArray(ctx.params.product, ['obstat', 'swdfp', 'medium']) ? '<p><a href="' + ctx.pathname + '/">' + ECMWF.ucfirst(ctx.params.product) + ' package</a></p>' : '')
								});
								$('#product-not-found').modal('show').click(function() {
									$('#product-not-found').modal('hide');
								});
							});
						}
					});
				}else{
					self.scroll_to_col2();
				}
			}

			if(ctx.params.product && changed.product.url){
				self.scroll_to_col3();
			}

			if(changed.path){
				//path_filter.set([router.params.product_path]);
				self.update_category_links();
			}

			if(changed.path || changed.page_product || (!ctx.params.product && !self.is_landing_page) || (ctx.params.product && self.is_landing_page)){

				if(false){
					var url = [ctx.params.product_path.join('/')];
				}else{
					url = []; // package not in the content url
				}
				ctx.params.product && url.push(ctx.params.product);

				// page content
				(function() {
					console.log('on_url_change: content: using fallback: changed.page_product=', changed.page_product);
					if(is_landing_page){
						self.widgets.content.empty();
					}else{
						//if(changed.page_product){ // TODO PUTMEBACK changed.page_product is sometimes not set when it should be
						products.content(router.params.product, function(result) {
							self.have_chart.done(function() {
								self.widgets.content.html(result.text || result.description)
							});
						});
					}
				})();
			}
}

			if(changed.is_landing_page){
				this._on_resize();
			}

			self.is_landing_page = is_landing_page;

			if(handler.instance){
				handler.instance.set_parameters(ctx.query);

				handler.instance.have.availability.done(function() {
					handler.instance && $.extend(CH.state.params, Utils.map(handler.instance.variables, function(v){ return v.to_string(); }));
					/*__DEV__ &&*/ (!handler.instance) && console.warn('on_url_change: instance became unset!');
				});
			}

			self.have_url.resolve();
			self.on.url.emit(changed);

			function path_changed(old_path, new_path) {
				if(old_path && !new_path) return true;
				if(!old_path && new_path) return true;
				if(old_path.length !== new_path.length) return true;
				for(var i in new_path){
					if(old_path[i] !== new_path[i]) return true;
				}
				return false;
			}
		},

		on_product_select: function(product, params) {
			var self = this;

			if(!product.data || !product.data.package) return console.error('on_product_select: unexpected product format');

			// we don't get a loaded product here so cannot call product methods
			// - should we use 'product-swapped-and-loaded' instead?
			if(product.has_widget('epsgram')){
				products.remove_filter('facets');
			}
		},

		h1: function(text) {
			text &&
				$('h1')
					.text(text)
					.addClass('ready');
		},

		_on_resize: $.debounce(200, function(size) {
			// Increase the page width if the chart is wide.
			// Basic charts only (interactive charts do not change size).
			// This is a hack. There is no way this can work reliably in all situations.

			// It does not do any resizing, it only calls the layout fns

			// note that this needs to respond to resizing of both the window and the chart.

			var self = this;

			self._do_layout();

			if(!this.is_landing_page){
				$('#time-navigator').triggerHandler('container');
			}

			// Notify users for site-specific size handling
			// This is emitted after:
			//   1) basic chart size changes
			//   2) page changes between landing and product pages
			//   3) window is resized
			//
			// note that when going from landing page to chart page, layout may be done twice: initial layout, then again after chart arrives
			//
			this.widgets.main.triggerHandler('content-resize', {
				is_landing_page: this.is_landing_page,
				col2: this.col2,
				col3: this.col3,
				col1_width: this.bind(this._col1_width)
			});
		}),

		update_category_links: function() {
		},

		_col1_width: function() {
			var window_width = window.innerWidth; // media queries widths include the scrollbar

			var a = (ECMWF.options.col1 && ECMWF.options.col1.width) || this.config.col1.width
			var w = a[0]
			for(var i in a){
				if(a[i][0] < window_width) break;
				w = a[i]
			}
			return w[1];
		}

}, {bind: mixins.bind});
