define(['jquery', 'bus', 'virtual-dom/h', 'virtual-dom/diff', 'virtual-dom/patch', 'jquery-ui/resizable', 'hover-intent'], function($, Bus, h, diff, patch) {

$.widget('ui.panel', {
	options: {
		resizable: false,
		closeable: false,
		collapsible: true,
		fullscreenable: null, // click callback
		title: '',
		bus: Bus()
	},

	_create: function() {
		var self = this, o = self.options;

		self.content = this.element.ensure('.panel-content');
		self.top_right = self.element.ensure('.top-right').hide();

		this.element
			.addClass('panel')
			.bind('dblclick', function() {
				o.collapsible && self.collapse(function(){
					o.bus.send('size-changed');
				});
				return false;
			});

		o.closeable && self.top_right
			.div('.button button-close')
				.div('.ui-icon ui-icon-close fadeable')
				.click(function() {
					self.hide();
					return false;
				});

		o.fullscreenable && self.top_right
			.div('.button button-fullscreen')
				.div('.ui-icon ui-icon-arrow-4-diag fadeable')
				.click(function() {
					if(!self._fullscreen || !self._fullscreen.active){
						self._fullscreen = {
							active: true,
							restore: {
								parent: self.element.parent(),
								height: parseInt(self.element[0].style.height, 10),
								offset: self.element.offset()
							}
						}

						var header_height = 43;
						var size = {
							height: $(window).height() - header_height
						};

						$('#footer').hide();
						self.element.addClass('fullscreen').css($.extend({width: self.element.outerWidth()}, self.element.offset()))
							.appendTo('#main')
							.delayed(100, function() {
								self.element.css({height: '', width: 'auto', top: '', left: '', bottom: 10});
							});

					}else{
						self.element
							.css(self._fullscreen.restore.offset)
							.delayed(250, function() {
								self.element.removeClass('fullscreen').css({left: '', top: '', bottom: '', width: '', height: self._fullscreen.restore.height})
									.appendTo(self._fullscreen.restore.parent);
								$('#footer').show();
							});

						self._fullscreen.active = false;
						size = {height: self._fullscreen.restore.height};
					}

					o.fullscreenable(self._fullscreen.active, size);
				});

		o.resizable && self.element
			.resizable({
				handles: 's',
				stop: function(e, ui){
					if(ui.size.height > 10){
						self.content.height(ui.size.height - parseInt(self.element.css('paddingBottom'), 10) - parseInt(self.element.css('paddingTop'), 10));
						self.content.trigger('layout', {height: ui.size.height - 0 * parseInt(self.content.css('paddingBottom'), 10)});
					}
				}
			})
			.delayed(1000, function() {
				// This overrides the first incorrect container layout. TODO Prevent the initial setting from happening.
				self.element.resizable('option', 'stop').apply(self.element, [null, {size: {height: self.element.outerHeight()}}]);
			})

		self.element.hoverIntent({
			over: function() {
				self.top_right.find('*').length && self.top_right.show();
			},
			out: function(e) {
				self.top_right.hide();
			},
			timeout: 1000
		});
	},

	_init: function() {
	},

	collapse: function(done) {
		var self = this, o = self.options;

		self.collapsed = !self.collapsed;

		this.element.css({height: ''});
		this.content.slideToggle({duration: 200, complete: done || $.noop});

		if(o.title)
			self.collapsed
				? (self.title
					? self.title.show()
					: self.title = self.element.h2(o.title))
				: self.title.hide();

		deselect(); // double click causes selection which is not possible to prevent.
	},

	show: function() {
		this.element.css({height: 0, opacity: 0}).delayed(20, function(){ $(this).css({height: 300, paddingTop: '', paddingBottom: '', borderTop: '', borderBottom: '', opacity: 1}); });
		this.visible = true;
	},

	hide: function() {
		var self = this;
		this.visible && self.element.css({height: 0, paddingTop: 0, paddingBottom: 0, borderTopWidth: 0, borderBottomWidth: 0}).delayed(1000, function(){ self.element.css({opacity: 0}).delayed(500, function(){ self.element.css({display: 'none'}); })});
		this.visible = false;
	},

	set_title: function(title) {
		var self = this;
		this.title = this.element.find('h2').orelse(function(){ return self.element.h2(); }).text(this.options.title = title);
	},

	_destroy: function() {
		this.element.removeClass('panel');
	}
});

function deselect() {
	var selection = window.getSelection ? window.getSelection() : document.selection ? document.selection : null;
	if(!!selection) selection.empty ? selection.empty() : selection.removeAllRanges();
}

});
