/*
 *   There is more than one product collection manager available:
 *     - grid/products (this is the one currently in use)
 *     - basic_chart/model/basic_instance_list
 */

define(['config', 'jquery', 'ajax_queue', 'utils', 'bus', 'cache', './modernizr'], function(ECMWF, $, Ajaxqueue, Utils, Bus, Cache, Modernizr) {

ECMWF.options.ajax.backend.print_chart || (ECMWF.options.ajax.backend.print_chart = '/plots/get-map/'); // temporary while www is different.

		/*
		if(location.protocol === 'https'){
			if ('serviceWorker' in navigator) {
				navigator.serviceWorker.register('/service-worker.js', {scope: '/'}).then(function() {
					// Registration was successful. Now, check to see whether the service worker is controlling the page.
					if (navigator.serviceWorker.controller) {
						// If .controller is set, then this page is being actively controlled by the service worker.
						console.log('service worker ACTIVE');
					} else {
						// If .controller isn't set, then prompt the user to reload the page so that the service worker can take
						// control. Until that happens, the service worker's fetch handler won't be used.
						console.log('serviceworker registered, will handle requests after next reload');
					}
				}).catch(function(error) {
					// Something went wrong during registration. The service-worker.js file
					// might be unavailable or contain a syntax error.
					console.error('serviceworker registration: ', error);
				});
			} else {
				console.log('serviceworkers not supported')
			}
		}
		*/

$.extend(Ajaxqueue.options,
	{
		cache: Cache
	}
);

ECMWF.browser.msie9 && require(['./msie9']);

if(!Modernizr.history) require(['html5-history-api']);
/*
Modernizr.localstorage || r.push('storage');
ECMWF.browser.msie8 && r.push('msie_fixes');
*/
require(['grid/products', './router', 'mixins', 'page/query', 'facets', 'tooltip', 'tooltip/tipsy', 'throttle-debounce', 'messages'], function(products, router, mixins, qs){

	router.original_path = location.pathname; // for loop detection

	/*
	 * Ongoing improvements to url parsing
	 *  - the application can now be at the top level (eg /charts/medium/ is ok)
	 *  - TODO allow the app name to be absent (eg allow /medium/)
	 *
	 */
	tokens = router.app_tokens_by_path(location.pathname);

	if(!tokens.app) throw new Error('cannot parse url'); // TODO there is a default now so this will never throw

	router.set_base_path(tokens.base_path);
	router.app_path = tokens.app_path;

	// TODO move to application
	if(tokens.chart_package === 'tcyclone'){
		tokens.app = tokens.chart_package;
	}

	ECMWF.load_app(tokens.app, function(_page) {

		__DEV__ && Bus().send('event-start', ['page start']);

		ECMWF.wheelspeed = ECMWF.browser.mozilla ? 10 : 1;

		var main = $(ECMWF.options.dom.main || '#main');

		var col1 = main.find('.column-1');
		var col2 = main.find('.column-2');
		var col3 = main.find('.chart-column').orelse(function(){ return main.div('.column chart-column'); })

		var Page = function(def) {
			this.config = {col2: {}}
			$.extend(true, this, def);
			this.widgets = {
				main: main,
				col1: col1,
				col2: col2,
				col3: col3
			};
			this.query = {};
			this._package = tokens.chart_package;

			this.show = function() {
				// panels are not used by the main view - views that need them will have to inititialise them themselves
				//this.widgets.col2.panel($.extend({}, this.config.panel, this.config.col2.panel));
				//this.widgets.col3.panel(this.config.panel);

				this.col1.call(page, {el: col1});
				this.col2.call(page, {el: col2});
				this.col3.call(page, {el: col3});
			}

			this.get_layout_info = function() {
				// these values represent (mostly) the original, not dynamic, sizes.

				var margin = 10;

				return {
					mode: (parseInt(main.css('minWidth'), 10) === 1060 ? 'normal' : 'wide'),
					col1: {
						outer_width: col1.outerWidth() + margin
					},
					col2: {
						inner_width: col2.width(),
						outer_width: col2.outerWidth() + 2 * margin
					},
					margin: margin
				}
			}
			this.layout = this.get_layout_info();
		}

		var page = new Page(_page);
		page.show();

		Bus().listen('size-changed', page.bind(page._on_resize));

		$('body').container({layout: 'single'}).on('layout', function(_, size) {
			page.layout.mode = (parseInt(main.css('minWidth'), 10) === 1060 ? 'normal' : 'wide');
			// TODO this is a dupe? see chart_application._resize
			page._on_resize();
		});

		router
			.route('*', function parse(ctx, next) {
				ctx.query = qs.parse(ctx.querystring);
				next();
			})
			.route('*', function(ctx, next){
				// note that handling '*' interferes with proper 404 handling.

				// perhaps try '*/' and '*' to differentate between listing and product pages?

				var _tokens = router.app_tokens_by_path(tokens.base_path + ctx.params[0]);
				if(_tokens.app){
					var pkg = _tokens.chart_package;
					_tokens = _tokens.tokens;
					_tokens.shift();

					var is_product = ctx.params[0][ctx.params[0].length - 1] !== '/';

					ctx.params.product = (is_product && _tokens.pop()) || null;
					ctx.params.package = pkg;

					var changed = {
						pkg: pkg !== page._package,
						// TODO should perhaps use _new_path here as the semantics are clearer
						product: {url: (router._path.page || null) !== ctx.params.product},
						q: (function() {
							var a = qs.parse((router._path.q || '').replace('?', ''))
							var b = ctx.query
							var seen = {}
							var changed = $.map(
								$.merge(Object.keys(a), Object.keys(b)),
								function(v){ if(a[v] !== b[v] && !seen[v]){ seen[v] = true; return v; } }
							);
							console.log('ROUTE: qparam changed:', changed);
							return changed;
						})()
					}

					ctx.params.product_path = _tokens;
					router.params = ctx.params;

					//console.log('ROUTE: changed: ', router.path !== ctx.path, changed.product.url);
					($.isEmptyObject(router._path) || changed.product.url || changed.pkg || changed.q) && page.on_url_change(ctx, changed);
					false && set_breadcrumbs(ctx);

					// TODO is prev still needed? Is it something to do with product vs page_product?
					// In PageController, _new_path is the new path, and _path is the old.
					page.prev = {product: ctx.params.product};

					return;
				}

				//special case of redirecting to login page correctly
				//TODO should be a separate route.
				if (ctx.path.indexOf('/auth/login') > -1) {
					location.href = ctx.path;
					return;
				}

				console.log('route not found');
				location.href = ctx.canonicalPath;
			})
			.start();

		function set_breadcrumbs(ctx) {
			if(!ECMWF.chart_app || !page.config.enable_hierarchy) return;

			var path = ctx.params.product_path.slice(0);
			path.unshift('charts');
			var a = '';
			$('.breadcrumbs').html($.map(path, function(v){
				a += v + '/';
				var path = a.replace('charts/', '').replace(/\/$/, '');
				return '<a href="/' + ECMWF.chart_app.base_url.replace('charts', '') + a + '" data-path="' + path + '">' + v + '</a>';
			}).join(' &gt; '));

			var last = $('.breadcrumbs a:last').text();
			$('.breadcrumbs a:last').remove();
			$('.breadcrumbs').append(last);

			$('.breadcrumbs a').click(function() {
				path_filter.set([$(this).attr('data-path').split('/')]);
				return false;
			});
		}

		require(['shortcuts'], function() {
			var fullscreen = false, bus = Bus();

			$(document).shortcuts({
				shortcuts: [
					{ key: 70,                  signal: 'fullscreen'}, // 'f'
					{ key: $.ui.keyCode.LEFT,   signal: 'prev-step' },
					{ key: $.ui.keyCode.RIGHT,  signal: 'next-step' },
					{ key: $.ui.keyCode.COMMA,  signal: 'animation-speed-down' },
					{ key: $.ui.keyCode.PERIOD, signal: 'animation-speed-up' },

					{ key: 51, callback: function() { // '3'
						require(['chart/plugins/timers'], function(plugin) {
							var chart = $.fn.chart && $('.chart').chart('instance') || ($.fn.basicChart && $('.basic').basicChart('instance'));
							if(chart && chart.add_plugin('Timers', plugin)){
								require(['notify'], function() {
									$.notify({id: 1, kind: 'ok', story: 'ok', message: 'Calltree timers are now enabled.<br/>New requests will have timer information available.'});
								});
							}else{
								Bus().send('open-calltree');
							}
						});
					}},
					{ key: 61, callback: function() { // '='
						require(['chart/plugins/timers'], function(plugin) {
							var chart = $.fn.chart && $('.chart').chart('instance');
							if(chart && chart.add_plugin('Timers', plugin)){
								chart.options.messageBus.clear().listen_once('chart-content-update', function() {
									bus.send('show-timers');
								});
								chart._redraw();
							}else{
								Bus().send('show-timers');
							}
						});
					}},
					{ key: 83, callback: function(){ // 's'
					}}
			]});

			bus.listen('fullscreen', function() {
				if(fullscreen){
					$('header, #footer').show();
					$('.column-1, .main-column-wrapper').css({top: '', bottom: ''});
					fullscreen = false;
				}else{
					$('header, #footer').hide();
					$('.column-1, .main-column-wrapper').css({top: 0, bottom: 0});
					fullscreen = true;
				}

				page._on_resize();
			})
		})
	})
})

});
