define(['config', 'jquery', 'utils', 'ajax_queue', 'animator', 'log', 'ajax-loader'], function(ECMWF, $, Utils, Ajaxqueue) {

	return new function() {
		var animation_seqno = 0;
		var _Animator = this;

		/*
		 *  This object conforms to the AnimatorClient interface - see core/animator.js
		 */
		this.Animator = function ChartAnimator() {

			this._init = function(chart, animation_loader) {
				var self = this;

				animation_seqno++;
				self._chart = chart;

				self._started = false;
				self._paused = false;
				self._active = true; // what does it mean?
				self._first = true;

				self._calltimes = {
					plot: 0,
					network: 0
				};

				self._count = 0;
				self._complete = false;
				self._loader = false;

				self._fr = {};
				self._frames = [];
				self._current = null;

				self.img_pool = [];
				self.animation_loader = animation_loader;

				self._set_product();

				var chart_set_product = chart._set_product;
				chart._set_product = function() {
					chart_set_product.apply(chart, arguments);
					self._set_product();
				}

				/*
				 *   Expose the animator. Eg, applications can do: chart.chart('animator').stop(),
				 */
				self._chart.animator = function() {
					return self;
				}
			}

			this._set_product = function() {
				var self = this;

				self._destroy();
				self._tag = self._chart.name + "#" + (animation_seqno++);
			}

			this._show_loader = function(on) {
				var self = this;

				if (self.animation_loader) {
					if (on) {
						self.animation_loader.show();
					} else {
						self.animation_loader.hide();
					}
					return;
				}

				// Force hidden
				if (self._complete || self._paused || !self._active) {
					on = false;
				}

				if (self._loader != on) {
					self._chart.clip.loader(on ? 'progress' : 'complete');
					self._loader = on;
				}
			}

			this._fetchFrame = function(time) {

				var self = this;

				if (self._fr[time.key].submitted) return;

				//console.log("FETCH FRAME ", time.key);
				self._fr[time.key].submitted = true;

				function request(img, first) {

					function success(json) {
						if (json.is_fall_back) {
							console.warn('chartAnimator.success: plot failed, got fallback');
							self.pause();
							return error();
						}

						if (json.values && json.values.step && json.values.step != time.step) {
							console.warn('chartAnimator.success: step mismatch', json.values.step, ' != ', time.step);
							self.pause();
						}

						if (!self._active) {
							self._show_loader(false);
							return;
						}

						if(self._chart.product.validate_plot && !self._chart.product.validate_plot(json)){
							console.error('plot validation failed: ', self._chart.product.error_msg);
							self.pause();
							return error();
						}

						self._fr[time.key].ready = true;

						self._count = 0;
						self._complete = true;
						$.each(self._frames, function(_, time) {
							if (!self._fr[time.key].ready) {
								self._complete = false;
							} else {
								self._count++;
							}
						});

						self._notify('chart-frame-loaded', [time, json.result]);
						self._frame_ready(time);
						self._update_loader();

						if (self._complete) {
							self._complete = true;
							self._notify('chart-animation-load-complete', [self._frames]);
							self._load_complete();
							self._show_loader(false);
						}
					}

					function error() {
						// Try again
						// note the danger of runaway recursion
						if (self._active && !self._paused) {
							setTimeout(function() {
								self._active && request(img, first);
							}, 10000);
						}
					}

					function call_times(timers, tracker, browser) {
						var django = timers["django.total"];
						self._calltimes.plot += django;
						browser && (self._calltimes.network += browser.total - django);
					}

					var fallback_url = !!~$.inArray(self._chart.product.kind, ['chart', 'eccharts']) ? '/backend/get-chart/' : '/plots/get-map/';

					Ajaxqueue({
						url : self._chart.product.url || fallback_url, // TODO remove fallback
						data: JSON.stringify((self._chart.product.data && (self._chart.product.data.kind === 'instance' || self._chart.instance))
							?
								$.extend(true,
									{animate: true},
									self._chart.product.as_json(self._chart._viewport ? self._chart._viewport() : undefined, time)
								)
							: {
								product: self._chart.product.as_json(self._chart._viewport ? self._chart._viewport() : null, time),
								animate: first
									? {
										start: self._current,
										frames: self._frames
									} : null
							}
						),
						success: success,
						error: error,
						// TODO add chart ajax options
						cache: 12 * 60 * 60,
						priority: "normal",
						statistics: true,
						logErrors: "Load frame (" + time.key + ")",
						tag: self._tag,
						call_times: call_times,
						img: img
					});
				}

				self._fr[time.key].image = self._get_frame_img();
				request(self._fr[time.key].image, self._first);
				self._first = false;
			};

			this._frame_ready = function(time) {
				// To be overriden
			};

			this._update_loader = function() {
				// To be overriden
			};

			this._load_complete = function() {
				// To be overriden
			};

			this.frameReady = function(time) {

				var self = this;

				if (self._paused || !self._active) return false;

				if (self._fr[time.key].ready) {
					return true;
				}

				self._fetchFrame(time);

				return false;
			};

			this.showFrame = function(time) {
				var self = this;

				if (self._paused || !self._active){
					console.warn('chartAnimator.showFrame called in error', self._paused, self._active);
					return false;
				}

				if (self._fr[time.key].ready) {
					self._chart.showFrame(time, self._fr[time.key].image);
					return true;
				}

				return false;
			};

			this._notify = function(msg, data) {
				var self = this;
				self._chart.options.messageBus.send(msg, data);
			};

			this._frame_list_ready = function() {
				// Nothing to do by default
			};

			this._on_start = function(ready_to_start) {
				var self = this;

				self._show_loader(true);

				console.log("ChartAnimator START ", self._chart.name, self._chart.time);

				self._chart.product.availability(self._chart.time, function(avail) {

					var dates = avail.valid_time.values;

					self._dates = dates;
					self._total = dates.length;

					self._notify('chart-animation-start');
					ready_to_start();
				});
			};

			this.getFrames = function() {
				var self = this;
				return self._dates;
			};

			this.getPreferredFrame = function() {
				var self = this;
				return self._chart.time;
			};

			this.frameList = function(dates, current) {
				var self = this;
				self._frames = dates;
				self._current = current;
				self._total = dates.length;
				self._count = 0;

				var complete = true;
				$.each(dates, function(i, time) {
					self._fr[time.key] || (self._fr[time.key] = {});
					if (!self._fr[time.key].ready) {
						complete = false;
					} else {
						self._count++;
					}
				});

				if (!complete) {
					self._update_loader();
					// Give a change to sub-classes to take a decision
					self._frame_list_ready();
				}
				self._complete = complete;
				self._show_loader(!complete);
			};

			this._on_stop = function() {
				var self = this;
				self._active = false;

				// TODO who should be responsible for frame_prev ?
				self._chart.frame_prev && self._chart.img.attr('src', self._chart.frame_prev.attr('src'));
				self.clear();
			}

			this._on_pause = function() {
				var self = this;

				if (!self._paused) {
					self._chart.frame_prev && self._chart.img.attr('src', self._chart.frame_prev.attr('src'));

					self._notify('chart-animation-stop');

					// tempoarary fix for probe incorrect after animation pause
					self._chart.element.find('img.frame').hide();
					self._chart.img.show();

					self._show_loader(false);
					if (self.animation_loader) {
						self.animation_loader.pause();
					} else {
						$.logMessage('Animation paused', null, {group: 'animation_loader'});
					}
				}
			};

			this.resume = function() {
				var self = this;

				var paused = self._paused;

				ChartAnimator.prototype.resume.apply(this);

				if (paused) {
					self._notify('chart-animation-start');
					self._show_loader(!self._complete);
				}
			};

			this.clear = function() {
				var self = this;

				if(!$.isEmptyObject(self._fr)){
					self._chart.frame_prev = null;

					self._chart._show_frame(self._chart.img);
					self._destroy();
				}
			}

			this._destroy = function() {
				// this removes animation details but does not destroy the object itself.
				var self = this;

				ChartAnimator.prototype._destroy.apply(this);

				self._dates && $.each(self._dates, function() {
					this.valid_time.loaded = false;
				});

				Ajaxqueue.cancelByTag(self._tag); // Clear any outstanding requests

				$.each(self._fr, function(i, frame) {
					frame.image && self._release_frame_img(frame.image);
				});
				self._fr = {};
			}

			this.paused = function() {
				return this._paused;
			};

			this._loaded = function() {
				var self = this;
				return self._fr;
			}

			// A pool of images is used to prevent IE memory leaks
			this._get_frame_img = function() {
				var self = this;

				var img = (self.img_pool.length) ? self.img_pool.pop() : $("<img>");
				return img.hide().addClass('frame').appendTo(self._chart.clip).click(function(event) {
					return self._chart._on_click(event);
				});
			}

			this._release_frame_img = function(img) {
				var self = this;

				img.hide()
					.off()
					.detach()
					.attr({src: Utils.blank_gif});

				self.img_pool.push(img);
			}
		}
		this.Animator.prototype = new $.AnimatorClient();

		this.OnDemandAnimator = function OnDemandAnimator(chart, animation_loader) {

			this._init(chart, animation_loader);

			// Prevent pre-loading on the server side
			this._first = false;

			this._update_loader = function() {
				var self = this;
				self._chart.clip.loader('progress', self._count + '/' + self._total);
			};

			return this;
		}

		this.OnDemandAnimator.prototype = new this.Animator();

		this.PreloadAnimator = function PreloadAnimator(chart, animation_loader) {

			// Preload all frames
			this._frame_list_ready = function() {
				var self = this;

				var dates = self._frames,
					idx = self._current;

				self._started = new Date();

				// Preload
				function dates_by_play_order(dates) {
					var low = [];
					var hi = [];

					for (var i in dates) {
						if (dates[i].index(dates) < idx) low.push(dates[i]);
						else hi.push(dates[i]);
					}
					return hi.concat(low);
				}

				$.each(dates_by_play_order(dates), function(_, time) {
					self._fetchFrame(time);
				});
			};

			this._load_complete = function() {
				var self = this;
				var elapsed = (new Date()) - self._started;
				if (self.animation_loader) {
					self.animation_loader.complete(elapsed);
				} else {
					$.logMessage('Animation load complete (' + (Math.round(elapsed / 100) / 10) + 's)', null, {
						group: 'animation_loader'
					});
				}
			};

			this._update_loader = function(time) {
				var self = this;
				if (self.animation_loader) {
					self.animation_loader.progress(time, self._count, self._total);
				} else {
					$.logMessage('Animation loading...  ' + self._count + '/' + self._total, null, {
						group: 'animation_loader'
					});
				}
			};

			this.stop_listening = function(time) {
				// bus messages are bound as the chart, so if the chart is destroyed, these messages will be unbound there.
			}

			this._init(chart, animation_loader);

			return this;
		};

		this.PreloadAnimator.prototype = new this.Animator();

		/*
		 *  This object is the product of refactoring and serves no purpose other than lazy instantiation.
		 *  Its methods should be moved into ChartAnimator above.
		 */
		this.Controller = function(chart, preload, animation_loader) {

			this.chart = chart;

			var self = this;

			this.animation = function() {
				if (!this._animation) {
					this._animation = preload ? new _Animator.PreloadAnimator(chart, animation_loader) : new _Animator.OnDemandAnimator(chart, animation_loader);
				}
				return this._animation;
			}

			this._listen = function(msg, fn) {
				var self = this;
				self.chart.options.messageBus.listen(chart, msg, self._bind(fn));
				return self;
			}

			this._bind = function(fn) {
				var self = this;
				return function() {
					fn.apply(self, arguments);
				};
			}

			chart.bound ||
			self
				._listen('collect-animation-loaded-state', function(_, loaded) {
					loaded.chart = {
						dates: Utils.map(self.animation()._fr, function(frame){ return frame.ready; }),
						complete: self.animation()._complete
					};
				})
				._listen('tool-animation-toggle', function(){ return self.animation().toggle(); })
				._listen('tool-animation-start', function(){ self.animation().start(); })
				._listen('tool-animation-stop', function(){ self._animation && self._animation.stop(); })
				._listen('tool-animation-pause', function(){ self._animation && self._animation.pause(); });
			chart.bound = true;
		}
	}
});
