/*
 *  (C) Copyright ECWMF http://wwww.ecmwf.int
 *
 *  This software is licensed under the terms of the Apache Licence Version 2.0
 *  which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 *  In applying this licence, ECMWF does not waive the privileges and immunities
 *  granted to it by virtue of its status as an intergovernmental organisation nor
 *  does it submit to any jurisdiction.
 *
 */
define(['config', 'jquery', './parameters', 'ajax_queue', 'time'], function(ECMWF, $, Parameters, ajaxqueue, Time) {

	function BaseProduct() {

		this.data = {product: {}};

		this.set_parameter = function(parameter, value, done) {
			done = done || $.noop;
			var self = this;

			var param = typeof parameter === 'object' ? parameter : self.variables[parameter];
			console.assert(param, 'parameter=' + parameter);
			if(!param) return;

			if(!param.is_current(value)){
				self._loaded = false;
				param.set(value);
			} else {
				done();
			}
		}

		this.has_widget = function (widget) {
			return !!~$.inArray(widget, this.data.product.widgets);
		}

		this.is_eccharts = function() {
			return false
		}

		this.availability = function(time, ready) {
			var self = this;

			if (this._availability) {
				self._send_availability(time, ready);
				return;
			}

			ajaxqueue({
				url: '/backend/get-availability/',
				json: {
					product: this.as_json(null, null)
				},
				logErrors: 'Product availability',
				success: function(result) {
					self._update_number = ++BaseProduct.prototype._update_number;
					self._updated = result.updated;
					self._availability = $.map(result.availability, function(a) {
						return new Time(a);
					});
					self._send_availability(time, ready);
				}
			});
		}

		this._send_availability = function(time, ready) {
			if (!time) return ready();

			var self = this,
				times = [],
				cycles = [],
				old = false,
				a;

			$.each(self._availability, function(i, d) {
				if ((time.base_time.getTime() - d.base_time.getTime()) === 0) {
					times.push(d);
				}

				if (time.base_time.getTime() - d.base_time.getTime() < 0) {
					old = true;
				}

				if ((time.valid_time.getTime() - d.valid_time.getTime()) === 0) {
					cycles.push(d);
				}
			});

			a = {
				current: time,
				base_time: {},
				valid_time: {},
				availability: self._availability,
				update_number: self._update_number,
				updated: self._updated
			};

			a.valid_time.values = times.sort(function(a, b) {
				return a.valid_time.getTime() - b.valid_time.getTime();
			});
			a.valid_time.index = a.current.index(a.valid_time.values);

			a.base_time.values = cycles.sort(function(a, b) {
				return a.base_time.getTime() - b.base_time.getTime();
			});
			a.base_time.index = a.current.index(a.base_time.values);

			// True if there is no better base time for this valid time
			a.best_base_time = (a.base_time.index === a.base_time.values.length - 1);

			// This is an old cycle
			a.old_cycle = old;

			//console.log($.map(a.base_time.values,function(a) { return a.key; }));
			//console.log($.map(a.valid_time.values,function(a) { return a.key; }));
			ready(a);
		}

		this.print = function(request) {
			ajaxqueue($.extend(
				{
					url: ECMWF.options.ajax.backend.print_chart,
					priority: 'very-high',
					logErrors: 'Print'
				},
				request
			));
		}

		this.add_to_dashboard = function(options, done) {
			var self = this;

			ajaxqueue({
				priority: 'very-high',
				url: '/backend/save-product-to-dashboard/',
				json: {
					product: self.get_save_state(),
					options: $.extend({
							original_path: (self.has_widget('interactive') || self.has_widget('basic')) ? window.location.pathname : undefined,
						}, options)
				},
				logErrors: 'Save dashboard product',
				success: function(result) {
					done && done(result);
				}
			});
		}

		this.download_url = function(token, email) {
			// TODO: Get this from settings, or use ECMWF.fix_host.to_backend
			var full_host = location.protocol + '//' + location.hostname + (location.port ? ':' + location.port: '');
			var host = full_host.replace(/http:/, 'https:').replace(/www|eccharts/gi, 'apps').replace(/stage|eccharts-test/gi, 'apps-test');
			var name = typeof this.name === 'function' ? this.name() : this.name

			var url = host
				+ '/plots/product-download/' + (this.data.product.package || 'catalogue') + '/' + name + '/'
				+ '?' + $.map(this.variables, function(v) { return v.name + '=' + v.to_string(); }).join('&');

			return encodeURI(url + '&token=' + token + '&email=' + email);
		}
	}
	BaseProduct.prototype._update_number = 0;
	BaseProduct.prototype._req_defaults = {};

	return BaseProduct;
});
