/*
 *  WWW-style legends in a separate panel (ie not overlayed as in Eccharts)
 */
define(['jquery', 'utils', 'ajax_queue', 'chart'], function($, Utils, Ajaxqueue) {

	// legends do not work with jsonp
	if(Ajaxqueue.options.dataType === 'jsonp'){
		__DEV__ && console.warn('legends disabled');
		return;
	}

	var el = $('#chart-legends');

	$.chartRegisterPlugin('Legend', function(chart) {

		chart.options.messageBus.listen('chart-product-change', function(_, product) {
			product = product.product;

			el.no_collapse().empty().show();

			$.each(product.layers_by_id, function(i, layer) {
				if (layer.has_data){
					var legend_holder = $('<div>', {id: 'legend-' + layer.name}).prependTo(el);
					product.legend(layer, function(result){
						var img_holder = $('<div>')
						if (result.url) {
							var img = img_holder.img({src: result.url}).on('load', function() {
								chart.options.messageBus.send('chart-legend-loaded');
							});
						} else {
							img_holder.append('Legend failed'); // Put broken image here.
						}

						var tit = layer.title;
						var vars = tit.match(/(^|\s)\$(\w+)/g) || [];
						for (i in vars) {
							var v = vars[i].replace('$', '').trim();
							if (v in product.variables)
								tit = tit.replace('$' + v, product.variables[v].value);
						}

						legend_holder
							.append(img_holder)
							.span(tit)
					});
				}
			});
		});

		this.destroy = function() {
			el.empty().hide();
		}
	});

	$.fn.no_collapse = function() {
		return this.each(function() {
			var el = $(this);
			el.css({minHeight: el.height()}).delayed(2000, function(){ el.css({minHeight: ''}); });
		});
	}
});
