/*
 *  Show a marker flag on the chart at the current probe location.
 */
(function ($) {

	$.chartRegisterPlugin('marker', function(chart) {
		var self = chart, o = self.options;
		var marker, marker_position, show_marker = false;

		o.messageBus.listen_once('chart-content-update', function(_, msg) {
			marker = o.show_marker
				? self.clip.div('.marker').hide()
				: $();

			marker.click(function() {
				// Centre the marker
				var pt = self.product.pixel(marker_position.lat, marker_position.lon);
				var viewport = self._viewport();
				var dx = pt.x - viewport.width / 2;
				var dy = pt.y - viewport.height / 2;

				self.panMap(dx, dy);
			});

			if ($.fn.modeTooltip) {
				marker.modeTooltip({
						content: function() {
						var t = $.formatGeoLocation(marker_position);
						if (marker_position.title) {
							t = marker_position.title + ', ' + t;
						}
						return t;
					}
				});
			}

			self
				._listen('chart-content-update', function(_, msg) {
					plot_marker();
				})
				._listen('latlon-location-select', function(_, msg) {
					marker_position = msg;
					plot_marker();
				})
				._listen('visible-probe-windows', function(_, msg) {
					show_marker = (msg.count !== 0);
					plot_marker();
				})
				._listen('tool-show-marker-click', function(_, msg) {
					show_marker = !! msg;
					plot_marker();
				})
				._listen('tool-goto-marker-click', function() {
					o.messageBus.send('latlon-location-select', marker_position);
					o.messageBus.send('city-select', marker_position);
				});

			self._mouse.pan.start.push(function(){
				marker.hide();
			});

			self._mouse.wheel.push(function(event, delta) {
				marker.hide();
			});
		});

		function plot_marker() {
			if (o.show_marker && marker_position && show_marker) {
				var pt = self.product.pixel(marker_position.lat, marker_position.lon);
				if (pt) {
					marker.show().css({
						top: pt.y - 41,
						left: pt.x - 10
					});
				} else {
					marker.hide();
				}
			} else {
				marker.hide();
			}
		}

		this.destroy = function() {
			marker.remove();
		}
	});

} (jQuery));
