define(['jquery', 'magics'], function($, Magics) {

	$.chartRegisterPlugin('outline', function(chart) {
		var self = chart, o = self.options;
		var outline, valid_outline;

		o.messageBus.listen_once('chart-content-update', function(_, msg) {
			outline = o.use_outline
				? $('<img>', {'class': 'outline'})
					.hide()
					.appendTo(self.clip)
					.css({position: 'absolute', opacity: 0.4})
				: $();

			self._mouse.pan.start.push(function(){
				valid_outline && outline.show();
			});

			self._mouse.pan.drag.push(function(event, delta) {
				var start = self.drag.start;
				var end = {
					x: event.pageX,
					y: event.pageY
				};

				var top = self.drag.position.top + end.y - start.y;
				var left = self.drag.position.left + end.x - start.x;

				var view = this._viewport();

				// Outline may become ready as we drag
				valid_outline && outline.show();

				outline.css({
					top: top - view.height,
					left: left - view.width
				});
			});

			self._mouse.wheel.push(function(event, delta) {
				valid_outline && outline
					.show()
					.css({
						top: self.zoom.view.top - self.zoom.view.height,
						left: self.zoom.view.left - self.zoom.view.width,
						width: self.zoom.view.width * 3,
						height: self.zoom.view.height * 3
					});
			});

			self._mouse.pinch.push(function(scale) {
				self.valid_outline && outline
					.show()
					.css({
						top: self.zoom.view.top - self.zoom.view.height,
						left: self.zoom.view.left - self.zoom.view.width,
						width: self.zoom.view.width * 3,
						height: self.zoom.view.height * 3
					});
			});
		});

		self._listen('chart-content-update', function(_, msg) {
			if (o.use_outline) {
				valid_outline = true;
				outline.hide();
				var viewport = self.product.get_view(self.product.magics);

				self.product.get_outline({
					json: self.product.outline(viewport),
					name: self.name + '_outline',
					tag: self.name,
					cancelled_by: self.name,
					cache: 60 * 60 * 24,
					priority: 'low',
					img: outline,
					success: function(json) {
						var m = Magics(json.result.magics);
						outline.css({
							top: -viewport.height,
							left: -viewport.width,
							width: m.width,
							height: m.height
						});
						valid_outline = true;
					}
				});
			}
		});

		this.destroy = function() {
		}
	});

});
