import 'chart/utils'
import ECMWF from 'config'

(function ($) {

$.chartRegisterPlugin('chartPrint', function(chart) {

	chart._listen("tool-print-select", function () {
		var items = [
			{
				type: "text",
				name: "title",
				value: chart.product.title(chart.time, false, true),
				title: "Title"
			},
			{
				type: "choice",
				name: "size",
				value: $.getUserPreference("print_size") || "screen",
				values : [
					"screen",
					"a4_landscape",
					"a4_portrait",
					"a3_landscape",
					"a3_portrait"
				],
				title : "Size",
				labels : {
					screen : "Screen",
					a4_portrait  : "A4&nbsp;Portrait",
					a4_landscape : "A4&nbsp;Landscape",
					a3_portrait  : "A3&nbsp;Portrait",
					a3_landscape : "A3&nbsp;Landscape"
				}
			}
		];
		ECMWF.dialogue({
			title: "Print",
			items: items,
			width: 800,
			dialogue: {
				dialogClass: 'print-dialogue'
			},
			ok: function (choices) {
				$.logMessage('Printing...');

				console.log('choices=', choices, ' size=', choices.size);
				$.setUserPreference('print_size', choices.size);

				print(choices, {
					priority: "very-high",
					logErrors: "PDF",
					success: function (json) {
						if (json.result) {
							$.logMessage('Print - starting download ...', null, {timeout: 8000});
							location.href = json.result.url;
						}
						else $.logError("Print error");
					},
					error: function (json) {
						$.logError("Print error");
					}
				});
			}
		});
	});

	function print(options, args) {
		var product = chart.product;

		var prod    = product.as_json(chart._viewport(), chart.time, chart.mail, options.format || "pdf");
		var json    = $.extend(options, {product: prod});
		var request = $.extend({json: json, logErrors: "Print"}, args);

		product.print(request);
	}
});

} (jQuery));
