/*
 *   servicelib/metadata adds a hook into the ajax requests and calls the call_tree function
 *   which passes the debug data to the widget which waits for a command to display the dialogue.
 */
define(['jquery', '@ecmwf/timers', 'chart/model/base_product', 'servicelib/metadata'], function($, Timers, BaseProduct){

	$.chartRegisterPlugin('Timers', function(chart) {
		var plugin = this;
		var product = chart.product;
		$.extend(true, BaseProduct.prototype._req_defaults, {
			call_tree: function(tree, tracker) {
				Timers.call_tree(tree, tracker);
			},
			call_times: function(timers, tracker) {
				chart.times = make_timer_string(timers, tracker);
				Timers.timers(timers, tracker);
				chart.options.messageBus.send('timers-info', {timers: timers, tracker: tracker});
			}
		});

		function make_timer_string(timers, tracker) {
			var total = timers['browser.elapsed'];
			var django = timers['browser.django.elapsed'];
			var browser = total - django;

			function s(v) {
				var r = 10;
				return Math.round(v * r + 0.5) / r + 's';
			}

			return '(Network: ' + s(browser) + ' - Plot: ' + s(django)
				// + " - Total: "  + s(total)
				+ ')';
		}
	});
});
