/*
 *   The titleOverlay widget provides a semi transparent bar on the Chart widget
 *   as can be seen in the Eccharts Dashboard application.
 */
define(['jquery', 'chart'], function($) {

	$.chartRegisterPlugin('titleOverlay', function(chart) {
		var self = chart, o = self.options;

		//we have 3 versions of the title-overlay. titleOverlay3 leaks the least amount of memory in MSIE8
		var conserve_memory = true;
		if (conserve_memory) {
			self.title_overlay = o.show_title_overlay ? chart.element.div().titleOverlay3({
				chart: self,
				text: 'text'
			}) : $();
		} else {
			self.title_overlay = o.show_title_overlay ? chart.element.div().titleOverlay({
				chart: self,
				text: 'text'
			}) : $();
		}

		this.destroy = function() {
			if (o.show_title_overlay && self.title_overlay.data('titleOverlay3')) {
				self.title_overlay.titleOverlay3('destroy');
			}
			self.title_overlay.remove();
		}
	});

	false &&
	$.widget('ui.titleOverlay', {

		options: {
			chart: undefined,
			text: ''
		},

		_init: function (args) {
			var self = this, o = self.options;

			this.element.addClass('title-overlay text-overlay');

			self._apply_update = $.throttle(100, function () {
				self.element.html(o.text);
			});

			if (o.chart) {
				o.chart.options.messageBus
					//FIXME using wrong 'self'
					.listen(self, 'chart-content-update', function (_, msg) {
						// warning: o.chart.product.time may be incorrect (eg, in MSIE8) - must use msg.time.
						o.text = o.chart.product.title(msg.time, true);
						self.update();
					})
					.listen(self, 'chart-time-select', function (_, time) {
						o.text = o.chart.product.title(time, true);
						self.update();
					});

				this.element.click(function () {
					o.chart.options.messageBus.send('best-base-time');
					return false;
				});
			}
		},

		destroy: function (args) {
			this.element.removeClass('title-overlay text-overlay');
			$.Widget.prototype.destroy.call(this);
		},

		_apply: function (args) {},

		update: function (event) {
			var self = this;

			self._apply_update();
		}
	});

	// this overlay is designed to minimise memory leakage in MSIE8.
	$.fn.titleOverlay3 = function(args){

		var oo = $.extend({
			chart: undefined,
			text: ''
		}, args || {});

var self = this; //TODO

		return $(this).each(function() {
			var el = $(this);

			if(typeof args === 'string'){
				var d = el.data('titleOverlay3');
				switch(args){
					case 'destroy':
						el.removeClass('title-overlay .text-overlay');

						d.chart && d.chart.options.messageBus
							.forget('chart-content-update', d.content_update_handler)
							.forget('chart-time-select', d.time_select_handler)
							//.forget_me(self)

						//minimise the effect of the msie leak
						d.chart = null;
						d.content_update_handler = null;
						d.time_select_handler = null;
						el.removeData('titleOverlay3');

						el.off('click');

						//console.log('titleOverlay3.destroy: data=', el.data('titleOverlay3'));
						break;

					case 'update':
						d.text = arguments[2];
						d.update();
						break;

					default:
						break;
				}
			}else{

				d = {
					chart: oo.chart,
					content_update_handler: function(_, msg){
						// warning: o.chart.product.time may be incorrect (eg, in MSIE8) - must use msg.time.
						if(oo.chart.product){
							oo.text = oo.chart.product.title(msg.time, true);
							d.update();
						}
					},
					time_select_handler: function(_, time){
						if(oo.chart.product){
							oo.text = oo.chart.product.title(time, true);
							d.update();
						}
					},
					update: $.throttle(100, function () {
						el.html(oo.text);
					})
				};
				el.data('titleOverlay3', d);

				el.addClass('title-overlay text-overlay');

				if (oo.chart) {
					oo.chart.options.messageBus
						.listen(self, 'chart-content-update', d.content_update_handler) //TODO self not correct here
						.listen(self, 'chart-time-select', d.time_select_handler);

					el.click(function () {
						oo.chart.options.messageBus.send('best-base-time');
						return false;
					});
				}
			}
		});
	};
});
