
define(['config', 'jquery', 'ajax_queue', './model/base_product', 'basic_chart/mixins', './model/parameters', 'magics', './layer', 'time'], function(ECMWF, $, ajaxqueue, BaseProduct, mixins, Parameters, Magics, Layer, Time) {

window.ProductFactory = function(o) {
    return ((o.product && o.product.init) ? o.product : new Product()).init(o);
}

    var next_id = 0;

    function by_z_index(a, b) {
        return a.z_index - b.z_index;
    }

    function Product() {}
    Product.prototype = new BaseProduct();

    $.extend(Product.prototype, {

        // expected arguments are owner, product, projection, use_product_projection, values
        init: function(o) {

            // As of Jan 2018, get-map is used for _all_ plot requests except in Eccharts Forecaster
            // Feb 2018: no, get-map does not work for _some_ ecccharts charts, eg if product name is 'empty'
            this.api = (ECMWF.options.ajax.backend.get_chart === '/plots/get-map/') ? 2.0 : 1.0;

            this.magics = null;
            this._title = 'No name';
            this._availability = null;
            this._forced_values = o.values || {};
            this.variables = {};
            if (o.product === this || typeof o.product === 'string' || typeof o.product === 'undefined') {

                if (!o.product) {
                    o.product = 'empty';
                }

                this.state = {
                    product: o.product,
                    layers: [],
                    magics_viewport: null,
                    magics_state: null,
                    projection: o.projection,
                    use_product_projection: o.use_product_projection
                };

                this.name = o.product;
            } else {
                this.name = o.product.name;
                this.state = o.product.state || {
                    layers: []
                };
                this.state.magics_viewport = null;
                this._title = o.product.title;
            }
            this.owner = o.owner;
            this.layers_by_id = {};

            if(this.api > 1.0){
                this.variables.time = new Parameters.time();
            }

            return this;
        },

        set_owner: function(owner) {
            this.owner = owner;
        },

        get_chart: function(request) {
            var self = this;
            var success = request.success || $.noop;

				function _need_old_api(product) {

					if(product.product === 'empty') return true;
					return false;
				}

            if(self.api > 1.0){
                var new_request = $.extend(true, {}, request.json.product); // this variable only used for comparisons.

                delete new_request.time;
                delete new_request.layers;
                delete new_request.availability;
                delete new_request.magics_viewport;
                delete new_request.use_product_projection;
                delete new_request.viewport.top;
                delete new_request.viewport.left;

                if(JSON.stringify(new_request) === JSON.stringify(self.__prev_request)){
                    return console.log('product.get_chart: ignoring duplicate request');
                }
                self.__prev_request = new_request;
            }

            ajaxqueue($.extend(
                {
                    url:  _need_old_api(request.json.product) ? '/backend/get-chart/' : ECMWF.options.ajax.backend.get_chart,
                    error: $.noop
                },
                self._req_defaults,
                request,
                {
                    success: function(json) {
								if(__DEV__){
									var is_eccharts = self.api < 2 && json.result.layers && json.result.layers > 0;
									if(!self.validate([
										['has result', function(){ return $.isPlainObject(json.result); }],
										['has values', function(){ return !is_eccharts || json.values; }],
										['values are valid', function() {
											if(!is_eccharts || json.axis) {
												return !$.map(json.values, function(val, param) {
													var axis = $.findWhere(json.axis, {name: param});
													if(!~$.inArray(val, axis.values)) return val;
												}).length;
											}
											return true;
										}],
									])){
										return request.error(self.error_msg);
									}
								} else if (!json.result || !$.isPlainObject(json.result)) {
                            return request.error.apply(this, arguments);
                        }

                        success(json, self.___update(json.product, json.result.magics, json.availability, json.axis, json.values, json.widgets));
                    }
                }
            ));
        },

        get_outline: function(request) {
            ajaxqueue($.extend({
                url: '/backend/get-outline/'
            }, request));
        },

        destroy: function() {
            //makes no measurable difference in MSIE8 or MSIE7
            /*
                   this.magics = null;
                   this._availability = null;
                   this.name = null;
                   this._title = null;
                   this.owner = null;
                   this.layers_by_id = null;
                   */
        },

        layerVisibility: function(layer_id, visible) {
            var layer = this.layers_by_id[layer_id];
            if (layer) layer.visible = visible;
            this._availability = null;
        },

        layerOptions: function(layer_id, options) {
            var layer = this.layers_by_id[layer_id];
            layer.values = options;
            if (options.style) {
                layer.style = options.style;
            }
            this._availability = null;
        },

        layerDelete: function(layer_id) {
            if (this.state)
                this.state.product = 'empty';
            var layer = this.layers_by_id[layer_id];
            this.state.layers = $.grep(this.state.layers, function(a) {
                return a !== layer;
            });
            this._availability = null;
        },

        // Order contains list of layer ids
        layerOrder: function(order) {

            //var len = order.length
            var z = {};

            $.each(order, function(i, ord) {
                z[ord] = i;
            });

            $.each(this.state.layers, function(i, layer) {
                layer.z_index = z[layer.id];
            });

        },

        addLayers: function(names) {
            // TODO: Check if the layer is already there...
            //var len = this.state.layers.length;
            var self = this;

            if (this.state)
                this.state.product = 'empty';

            $.each(names, function(i, name) {
                self.state.layers.push({
                    name: name,
                    "z_index": 2000 - i,
                    visible: true,
                    inited: false
                });
            });
            this.state.layers.sort(by_z_index);
            this._availability = null;
        },

        ___update: function(result, magics, availability, axes, values, widgets) {
            var self = this, time;

            console.log("PRODUCT UPDATE", result, magics, availability)

            this.magics = Magics(magics);
            this.state.use_product_projection = false;
            this.data.product = {widgets: widgets};

            if (result.title) {
                this._title = result.title;
            }

            this.state.projection = result.projection;

            self.state.magics_viewport = self.magics.viewport();

            result.layers.sort(by_z_index);

            this.layers_by_id = {};

            $.each(result.layers, function(i, layer) {
                if (layer.time) {
                    time = new Time(layer.time);
                }

                if (!layer.id) {
                    layer.id = ++next_id;
                }

                self.layers_by_id[layer.id] = layer;

                if (!layer.values) {
                    layer.values = {};
                }
                if (!layer.styles) {
                    layer.styles = [];
                }
                if (!layer.variables) {
                    layer.variables = [];
                }

                delete layer.time;
                delete layer.values.base_time;
                delete layer.values.step;
                delete layer.values.valid_time;
            });

            if (availability) {
                self._updated = availability.updated;
                self._availability = $.map(availability.availability, function(a) {
                    return new Time(a);
                });
                self._update_number = ++BaseProduct.prototype._update_number;
            }

            this.state.layers = result.layers;

            // Store the variables as sent from the server.
            if(self.api > 1.0){
                if (axes) {
                    var variables = availability ? {time: self.variables.time} : {};
                    $.each(axes, function(i, v) {
                        if(values[v.name]) v.value = self.__prev_request.values[v.name] = values[v.name];

                        variables[v.name] = self.variables[v.name]
                            ? self.variables[v.name].set(v.value, v.values, v.labels)
                            : ((v.type === 'dynamic')
                                ? new Parameters.dynamic(v)
                                : new Parameters.string(v));
                    });
                    self.variables = variables;
                }

                if(availability){
                    self.__prev_request.values.time = [result.time.base_time.substr(0, 10), result.time.step, result.time.valid_time.substr(0, 10)];
                    self.variables.time.set(new Time(result.time), self._availability);
                }
            }

            return time;
        },

        get_save_state: function(full) {
            var state = $.cloneObject(this.state);

            if (!full) {

                $.each(state.layers, function(i, layer) {
                    if (!layer.values) {
                        layer.values = {};
                    }

                    // TODO: Check why this is needed (why check ?)

                    delete layer.time;
                    delete layer.values.base_time;
                    delete layer.values.step;
                    delete layer.values.valid_time;

                    // Just to save some bandwith
                    delete layer.variables;
                    delete layer.styles;
                    delete layer.id;
                    delete layer.inited;
                    delete layer.values.style;
                });
            }

            if(this.api > 1.0){
                state.values = state.values || {}
                for(var i in this.variables){
                    if(this.variables[i].value) state.values[this.variables[i].name] = this.variables[i].for_json();
                }
                delete state.values.time;
            }

				// temporary workaround for unexpected backend behaviour
				// (it is expected that the 'outer' product should be used when plotting,
				// not the 'inner' product prototype, but this is not the case)
				//
				// This allows us in many cases to use /backend/get-map/ for eccharts charts
				// (but not if product is built from the 'empty' product)
				if(typeof state.product === 'string' && this.name !== 'empty'){
					state.product = this.name
				}

            return {
                name: this.name,
                state: state,
                title: this._title
            };
        },

        clear_availability: function() {
            this._availability = null;
        },

        as_json: function(viewport, time, mail, format) {
            var self = this;

            var data = $.extend(true, {}, this.state, {
                viewport: this.get_view(viewport),
                format: format || ($.getUserPreference ? $.getUserPreference('image_format') : 'png')
            });

            // TODO: Come back here
            if (!self._availability) {
                data.availability = true;
            }

            self.state = self.state || {
                layers: []
            }
            data.layers && $.each(data.layers, function(i, layer) {

                layer.user_title = new Layer(layer).get_title();

                if (!self.state.layers[i].values) {
                    self.state.layers[i].values = {};
                }

                delete layer.styles; // save some bandwidth
                delete layer.units;
                delete layer.title;
                delete layer.variable_title;
                delete layer.variables;
                delete layer.has_probe;
                delete layer.has_data;
            });

            if (time && time.base_time) {
                data.time = time.toJSON();
            }

            // For now .... to support non-eccharts layers
            if (data.layers && data.layers.length === 1 && !data.layers[0].name) {
                delete data.layers;
            }

            data = $.cloneObject(data);

            if (!data.values) {
                data.values = {};
            }

            $.each(self._forced_values, function(k, v) {
                data.values[k] = v;
            });

            if(self.api > 1.0){
                for(var i in self.variables){
                    if(self.variables[i].value) data.values[self.variables[i].name] = self.variables[i].for_json();
                }
                time && (data.values.time = new Parameters.time(time).for_json());
                delete data.time;
                //delete data.projection;
            }

            return data;
        },

        serialise: function(viewport, time, mail, format) {

            var data = this.as_json(viewport, time, mail, format);
            return JSON.stringify(data);
        },

        get_view: function(view) {
            view = view || this.magics;

            if (!view)
                return {
                    top : 0,
                    left : 0
                };

            return {
                top : 0,
                left : 0,
                width : view.width,
                height : view.height
            }
        },

        layerList: function(filter) {
            filter || (filter = function() {
                return true;
            });
            return $.map(this.state.layers, function(l) {
                if (filter.call(l)) return l;
            });
        },

        full_map_projection_name: function() {
            return this.magics ? this.magics.full_map_projection_name() : "global";
        },

        outline: function(viewport) {
            if (this.magics) {
                return {
                    projection: this.magics.expand(viewport.width, viewport.height),
                    viewport: {
                        width: viewport.width * 3,
                        height: viewport.height * 3
                    }
                };
            }
        },

        position: function(x, y) {
            return this.magics ? this.magics.pc2ll(x, y) : null;
        },

        pixel: function(lat, lon) {
            return this.magics ? this.magics.ll2pc(lat, lon) : null;
        },

        product: function(name, use_product_projection) {
            var self = this;
            if (name) {
                this.name = name;
                this.state.product = name;
                this.state.use_product_projection = use_product_projection;
                this._availability = null;

                this.state.layers = [];
            } else {
                return this.name;
            }
        },

        projection: function(name) {
            if (name) {
                this.state.projection = name;
            }

            this.state.magics_viewport = null;
            this.magics = null;
            return this.state.projection;
        },

        resize: function(w, h) {
            if (this.magics) {
                var settings = this.magics.resize(w, h);
                this.state.projection = settings.projection;
            }
            return this.state.projection;
        },

        zoom: function(a, b, c, d) {
            var settings = this.magics.zoom(a, b, c, d);
            this.state.projection = settings.projection;
            return this.state.projection;
        },

        vertical_longitude: function(lon) {
            var settings =  this.magics.vertical_longitude(lon);
            this.state.projection = settings;
            return this.state.projection;
        },

        pan: function(dx, dy) {
            if (this.magics) {
                this.state.projection = this.magics.move(dx, dy);
                return this.state.projection;
            }
        },

        transform: function(matrix) {
            if (this.magics) {
                var settings = this.magics.tranform(matrix);
                this.state.projection = settings.projection;
                this.state.magics_viewport = settings.viewport;
                return this.state.projection;
            }
        },

        title: function(time, time_only, include_year) {
            if (time) {
                include_year = true;
                /*
                    var now  = new Date();
                    var diff = (now - time.base_time)/1000/60/60/24;
                    if(diff >= 365) {
                        include_year = true;
                    }
                    */

                if (time_only) {
                    return [
                        $.dateToTitle(time.base_time, include_year), 'VT',
                        $.dateToTitle(time.valid_time, include_year), 'Step', time.step
                    ].join(' ');
                } else {
                    return [
                        this._title, '-', $.dateToTitle(time.base_time, include_year), 'VT',
                        $.dateToTitle(time.valid_time, include_year), 'Step', time.step
                    ].join(' ');
                }
            } else {
                if (time_only) {
                    return '';
                } else {
                    return this._title;
                }
            }
        },

        size: function() {
            if (!this.magics) return {};

            return {
                width: this.magics.width,
                height: this.magics.height
            };
        },

        is_eccharts: function() {
				return this.state.product && (this.state.product === 'empty' || this.state.product.indexOf('uid_') === 0) // not sure how reliable this test is
        },

        /*
         *  Get legend of default size
         */
        legend: ajax_fn(function(layer, done) {
            return {
                url: '/backend/layer-legend/',
                priority: 'very-high',
                json: {layer: layer.id, product: $.extend({}, this.state, {viewport: null})}
            }
        }),

        layerByID: function(layer_id) {
            return this.layers_by_id[layer_id];
        },

        layer_icon: function(layer_id, img, time) {
            ajaxqueue({
                // TODO: pass the map name
                cancelled_by: this.owner,
                url: '/backend/layer-icon/',
                priority: 'low',
                json: {
                    product: this.as_json(null, time),
                    layer: layer_id
                },
                img: img
            });
            return this;
        },

        newAvailability: function(ready) {
            var self = this;

            if (!self._availability) {
                return;
            }

            ajaxqueue({
                url: '/backend/new-availability/',
                //cancelled_by : this.owner,
                json: {
                    product: self.as_json(null, null),
                    updated: self._updated
                },
                logErrors: 'New availability',
                success: function(result) {
                    if (result && result.availability) {
                        self._update_number = ++BaseProduct.prototype._update_number;
                        self._updated = result.updated;
                        self._availability = $.map(result.availability, function(a) {
                            return new Time(a);
                        });
                        ready(self._updated, self._availability);
                    }
                }
            });
        },

        // FIXME: what is this?
        /*
        compact_availability: function(ready) {

            ajaxqueue({
                url: ECMWF.options.ajax.backend.availability,
                json: {
                    product: this.as_json(null, null),
                    compact: true
                },
                success: ready
            });
        },
        */

        save_product: function(options, done) {
            ajaxqueue({
                priority: 'very-high',
                url: '/backend/save-product/',
                json: {
                    product: this.as_json(null, {}),
                    options: options
                },
                logErrors: 'Save product',
                success: function(result) {
                    console.log(result);
                    done && done(result);
                }
            });
        },

        save_projection: function(options, done) {
            ajaxqueue({
                priority: 'very-high',
                url: '/backend/save-projection/',
                json: {
                    product: this.as_json(null, {}),
                    options: options
                },
                logErrors: 'Save projection',
                success: function(result) {
                    console.warn(result);
                    done && done(result);
                }
            });
        },

        show_city: function(pos) {
            var d = 5,
                lat = parseFloat(pos.lat),
                lon = parseFloat(pos.lon);
            var north = lat + d,
                south = lat - d,
                west = lon - d,
                east = lon + d;

            console.log('SHOW CITY', lat, lon, north, south, west, east);

            if (north > 90) {
                north = 90;
            }
            if (south < -90) {
                south = -90;
            }

            this.state.projection = {
                lower_left_latitude: south,
                lower_left_longitude: west,
                projection: 'cylindrical',
                upper_right_latitude: north,
                upper_right_longitude: east
            };

            if (south > 45) {
                this.state.projection.projection = 'polar_stereographic';
                this.state.projection.map_hemisphere = 'north';
                this.state.projection.vertical_longitude = (east + west) / 2;
            }

            if (north < -45) {
                this.state.projection.projection = 'polar_stereographic';
                this.state.projection.map_hemisphere = 'south';
                this.state.projection.vertical_longitude = (east + west) / 2 + 180;
            }
        },

        // temporary method pending unification of backend interfaces for interactive charts
        www_print: function(title, done) {
            var self = this;

            // Make time compatible with eccharts time.
            var pr = self.as_json(null, null, null, 'pdf');
            if (pr.values) {
                if ($.inArray('time', pr.values)) {
                    var t = pr.values.time;
                    var ec_time = {'base_time': t[0]+'00', step: t[1], valid_time: t[2]+'00'};
                    pr.time = ec_time;
                    pr.values.time = ec_time;
                }
            }

            ajaxqueue({
                url: '/backend/print-chart/',
                priority: 'very-high',
                json: {
                    product: pr,
                    size: 'a4_landscape',
                    title: title
                },
                logErrors: 'Print',
                success: function(result){
                    console.log('print: success.', result, self.time);
                    result && result.result.url && done(result.result);
                }
            });
        }
    }, mixins);

function ajax_fn(fn) {
	return function(args, done) {
		ajaxqueue($.extend(
			{ complete: done || $.noop },
			fn.apply(this, arguments))
		)
		return this
	}
}

return Product;

});

