
define(['config', 'jquery', 'bus', 'browser'], function(ECMWF, $, Bus){
	require(['./testable', 'undo', 'matrix', 'magics', 'chart', './product', './layer', './latlon', 'container', './test.css'/*, 'chart_plugins/canvas_renderer'*/], function(ChartTestable) {

	$.framework.onReady(function TestApplication(){

		var pause_between_frames = 500;
		var initial_product = '2t_full'
		var products = ['2mT_wind30m', '2t_full', '700divergence_wind', '700vorticity_wind', 'efi_2t', 'efi_all', 'efi_tp', 'efi_wg', 'efi_ws', 'gusts_prob', 'mslp_uv200', 'mwp_percentile', 'mwp_prob'/*, 'rh_7huv'*/, 'rh_850uv', 'tp_tp_proba', 'tp_det'];
		var chart_options = {availability: false, product: initial_product, projection: 'europe', show_latlong: true, show_title_overlay: true, enable_undo: false, mousewheel: true};

		$.getUserPreference || ($.getUserPreference = $.noop);

		var bus = Bus();
		var el = $('#main-column, #mainColumn').orelse($('.node .content')).orelse($(ECMWF.options.dom.content_parent));

		var chart =
			el.div('#chart-container')
				.div({className: 'content', id: 'chart', css: {height: 200}})
					.chart(chart_options);

		el.clear();

		var product_select = el.formSelect({options: products})
			.val(initial_product)
			.change(function(_){
				var selected = $(this).find('option:selected');
				console.log(selected.text());
				bus.send('product-thumbnail-click', {name: selected.text()});
			});

		var status = el.div();

		bus.listen('chart-product-change', function(_, a){
			product_select.val(a.product.name);
		});

		new Test('test change projection', el, function start(){
				var ct = new ChartTestable(this);

				this.actions1 = {
					'0'  : ct.left,
					'10' : ct.up,
					'11' : ct.right,
					'21' : ct.up,
					'22' : ct.left,
					'32' : ct.up,
					'33' : ct.right,
					'43' : 'end'
				}

				var actions2 = {
					'0'   : ct.move_rand,
					'511' : 'end' //was originally 1024 but that takes too long to run.
				}
				var n_iterations = 512;

				var test = this;
				test._clear_timeout();

				var actions = new ActionList(actions2);

				actions.finished = function(){
					status.html('test finished.');
					Bus().forget('chart-content-update');
					test.start_button.show();

					el.div().a('clean up 1 (cache)'        ).click(function(){ destroy1(); });
					el.div().a('clean up 2 (chart outline)').click(function(){ destroy2(); });
					el.div().a('clean up 3 (chart imgs)'   ).click(function(){ destroy3(); });
					el.div().a('clean up 4 (chart destroy)').click(destroy4);
					el.div().a('clean up 5 (null chart)'   ).click(function(){ destroy5(); });

					setTimeout(function(){
						//destroy1();

						if(false && ECMWF.browser.msie) CollectGarbage(); //makes no difference.
					}, 4000)
				}

				//TODO testme
				bus.listen('chart-product-change', function(){
					test.start_button
						.show();

					bus.forget('chart-product-change');
				});

				actions.next();

				var t;
				bus.listen('chart-content-update', function(){
					if(!t){
						t = setTimeout(function(){
							t = null;
							if(actions.next())
								status.html(actions.n + ' / ' + n_iterations);
						}, pause_between_frames);
					}
				});

				function ActionList(actions){

					this.actions = actions;
					this.n = 0;

					this.get = function(n){
						var prev = actions[0];
						for(var i in this.actions){
							if(i > n) return prev;
							prev = this.actions[i];
						}
						return null;
					}

					this.next = function(){
						var _next = this.get(this.n++);
						if(test.running && $.isFunction(_next)) _next(this.n);
						else{
							console.log('stopping....');
							if(this.finished) this.finished();
							this.n = 0;
							return false;
						}
						return true;
					}
				}
			},
			function stop(){
				this.running = false;
			}
		);

		new Test('test animate', el, function _start(){
				//this.set_timeout(10000);
				this._clear_timeout();
				bus.send('tool-animation-toggle');
			},
			function _stop(){
				bus.send('tool-animation-toggle');
			}
		);

		new Test('test multiple animations', el, function start_(){
				var product_num = 1;
				var n_animations = 8;
				var frame = el.div();
				this._clear_timeout();
				var ct = new ChartTestable(this);
				ct.bus = bus;

				function next_animation(on_done){
					product_num++;
					setTimeout(function(){
						run_product_animation(function(){
							on_done();
						});
					}, 500);
				}

				function on_product_animation_done(){
					next_animation(function(){
						if(product_num < n_animations){
							on_product_animation_done();
						}
						else status.text('multiple animations test complete');
					});
				}

				run_product_animation(on_product_animation_done);

				bus.listen('chart-frame-loaded', function(_, msg){
					frame.text(msg.step);
				});

				function run_product_animation(finished){
					setTimeout(function(){
						status.text('changing product...');
						var product = products[product_num];
						if(product){
							ct.change_product(product, function(){
								status.text('product changed. waiting for availability...');

								setTimeout(function(){
									status.text('loading animation ' + product_num + ' ...');
									bus.send('tool-animation-toggle'); //start
									listen_once('chart-animation-load-complete', function(){
										console.warn('animation complete: ' + product_num);
										status.text('animation complete: ' + product_num);

										setTimeout(function(){
											listen_once('chart-animation-stop', function(){
												setTimeout(function(){
													frame.text('');
													finished();
												}, 1000);
											});
											bus.send('tool-animation-toggle'); //stop
										}, 2000);
									});
								}, 5000); //there is a bug in the chart re availability if we animate too soon after product load.
								          //TODO wait for 'chart-product-availability' message
							});
						}
					}, 1000);
				}
			},
			function stop_(){
				console.log('stop TODO');
			}
		);

		//temp!
		/*var clear_animation = */el.p().a('clear animation').attr('href', '#')
			.click(function(){
				chart.chart('clear_animation');
				return false;
			});

		// with this test the product serialisation is very small.
		new Test('test multiple product change', el, function start__(){
				var product_num = 0;
				var n_products = 16;
				var loop = 0;
				var n_loops = 20;
				var step_delay = 100;

				var test = this;
				test._clear_timeout();
				var ct = new ChartTestable(this);
				ct.bus = bus;

				function next_product(_on_next_product_done){
					status.text('changing product... ' + loop + '.' + product_num + ' (' + (product_num + loop * n_products) + '/' + (n_loops * n_products) +')');
					setTimeout(function(){
						ct.change_product(products[product_num], function(){
							if(test.running){
								status.text('product changed.');
								product_num++;
								_on_next_product_done();
							}
							else status.text('test cancelled.');
						});
					}, step_delay); // why does this have such a large impact on MSIE8 memory usage?
				}

				function on_next_product_done(){
					if(product_num < n_products) next_product(on_next_product_done);
					else {
						if(loop < n_loops){
							loop++
							product_num = 0;
							next_product(on_next_product_done);
						}else{
							status.text('product change done');
							el.div().a('clean up (chart destroy)').click(function(){ destroy4(); });
							stop();
						}
					}
				}

				next_product(on_next_product_done);
			},
			function stop__(){
				this.running = false;
			}
		);

		new Test('test create destroy', el, function start___(){
				/*
				use_outline: setting this to false saves 11MB (MSIE8 2011.02.14)
				*/
				var n_iterations = 25; // was 256;
				var test = this;
				test._clear_timeout();

				test.series(Test.make_list(create_destroy_chart, n_iterations));

				function create_destroy_chart(done){
					if(test.running){
						test.iter++;
						status.text('' + test.iter + ' of ' + n_iterations);

						destroy_chart();
						setTimeout(function(){
							create_chart();
							setTimeout(function(){
								done();
							}, 800);
						}, 50);
					}
				}

				function create_chart(){
					chart.chart($.extend({use_outline: false}, chart_options));
				}

				function destroy_chart(){
					chart.chart('destroy');

					if(!test.assert(chart.is(':empty'), 'node not empty')) return;
					return true;
				}
			},

			function stop___(){
			}
		);

		//TODO can now use bus.listen_once instead.
		function listen_once(name, fn){
			bus.forget(name);
			bus.listen(name, function(){
				bus.forget(name);
				fn();
			});
		}

		var test_magics = el.p().a('test magics').attr('href', '#').click(function(){
			test_magics.hide();
			status.text('testing magics...');

			var items = [];
			for(var i=0;i<10000;i++){
				var magics = {
					projection: {
						pcxmin: -8972360.0, 
						GPoffx: 0, 
						GPoffy: 0, 
						e: 0, 
						name: "polar_north", 
						top: 0, 
						pcymin: -6440280.0, 
						height: 200, 
						width: 720, 
						pcheight: 4674920.0, 
						pcwidth: 16829700.0, 
						coeff2: 11888300.0, 
						coeff3: 0, 
						lon0: 0, 
						coeff1: 1, 
						left: 0, 
						coeff7: 1, 
						coeff6: 1, 
						coeff5: 0, 
						coeff4: 0
					}
				} 
				items.push(magics_factory(magics));
			}
			status.text('magics test done. 10000 objects (~110MB) were created then destroyed. memory usage should be the same as before.');
			test_magics.show();
			return false;
		});

		//test multiple charts:
		var test_multiple = el.p().a('test multiple charts').attr('href', '#').click(function(){
			test_multiple.hide();
			chart.parent().css({'float': 'left', width: 380, height: 200});
			chart.css({width: 380, height: 200}).resize();

			//var chart2 =
				el.div({id: 'chart2-container', css: {'float': 'left'}}).insertAfter(chart.parent())//.clear()
					.div({'class': 'content', id: 'chart', css: {width: 380, height: 200}})
						.chart(chart_options);
			return false;
		});

		new Test('print', el, function start_print(){
				var test = this;
				this._clear_timeout();

				bus.listen('log-error', function(_, m){
					console.log('m=', m);
					status.text('error: ' + m.title);
					if(m.message.traceback){
						var s = 'xx' + m.message.traceback.join('<br>');
						el.p(s);
					}
					test.fail(m.title);
				});
				bus.send('tool-print-select');

				setTimeout(function(){ test.running && test.finished(); }, 3000);
			}
		);

		function retry(){
			false && setInterval(function(){
				$('.inform-dialogue .text-link:contains("retry")').click();
			}, 15000);
		}
		retry();

		function destroy1(){
			var size = $.cacheClear();
			el.div().p('cache cleared. size was: ' + (size/1000).toPrecision(2) + 'k');
		}

		function destroy2(){
			$('.chart-clip img.outline').remove();
			el.div().p('outline removed');
		}

		function destroy3(){
			$('.chart-clip img').remove();
			el.div().p('images removed');
		}

		function destroy4(){
			chart.chart('destroy');
			el.div().p('chart destroyed (including undo stack)');
		}

		function destroy5(){
			chart = null;
		}
	});

	});
});
