define(['jquery', 'test-lib'], function($){

	function ChartTestable(test) {
		this.test = test;
		this.chart = $('#chart');
		var self = this;

		this.left  = function(){ self.chart.chart('panMap',  20,  0); }
		this.right = function(){ self.chart.chart('panMap', -20,  0); }
		this.up    = function(){ self.chart.chart('panMap',   0, 20); }

		this.move_rand = function (n){
			var period = 14;
			var x =
				Math.sin((Math.PI / 2) * (n / period)) * 20 +
				Math.floor(Math.random()*11) - 5;
			var y =
				Math.sin((Math.PI / 2) * (n / 10)) * 20 +
				Math.floor(Math.random()*11) - 5;
			self.chart.chart('panMap', x, y);
		}

		this.change_product = function(product_name, on_done){
			console.log('________________________');
			var got_availability = false;

			this.bus.send('product-thumbnail-click', {name: product_name});
			this.bus.listen_once('chart-product-change', function(){
				if(!got_availability) console.warn('ChartTestable.change_product: no availability');
				on_done();
			});

			this.bus.listen_once('chart-product-availability', function(){
				got_availability = true;
			});
		}

		this.set_base_time = function(base_time_index, on_done){
			// TODO should use ChartTestable.DateSelector below
			var date_select = $('#time-navigator .date-select');
			var menu = date_select.data('uiDateSelect')._menu.menuWidget();
			date_select.find('.ui-selectmenu-button').click();
			Test.wait_for('.date-select-menu:visible').then(function(){

				self.test._listen_once('chart-time-select', function(event, msg){
					self.test.wait_for_remove('.date-select-menu:visible').then(function(){
						on_done();
					});
				});

				var basetime_count = date_select.find('select option').length;
				self.test.assert(base_time_index < basetime_count, 'basetime index out of range');

				console.log('selecting: ', (basetime_count - base_time_index - 1));
				menu.find('li:eq(' + (basetime_count - base_time_index - 1) + ')').click().orelse(function(){ self.test.fail('date selector not found'); });
			});
		}

		this.DateSelector = function(css){
			this.selector = $(css);
			this.test = null;
			this.select = function(idx, done){
				var self = this;
				return $.Deferred(function(df){
					var popup = self.selector.data('uiDateSelect')._menu.menuWidget();
					self.selector.find('.ui-selectmenu-button').click().orelse(function(){ self.test.fail('date selector not found'); });
					Test.wait_for(function(){ return popup.is(':visible'); }, 'menu visible').then(function(){
						console.log('select: got popup');
						var menu_size = popup.find('> li').length;
						if(idx < 0 || idx >= menu_size) return console.error('menu idx out of range: ', idx, menu_size);
						var menu_item = popup.find('> li:eq(' + idx + ')');
						if(!menu_item.length) console.error('cannot find menu item: idx=', idx);
						popup.menu('focus', {}, menu_item); // clicking does not work unless you focus it first
						menu_item.click();
						Test.wait_for(function(){ return !popup.is(':visible'); }, 'menu close').then(function(){
							done && done();
							df.resolve();
						});
					});
				}).promise();
			}
		}

		this._get_n_layers = function(){
			return $('#layer-list-pane #layer-list tr.layer-list-row:visible').length;
		}

		this.add_remove_layer = function(done){
			console.log('add_remove_layer___________________________');
			return $.Deferred(function(df){
				self.add_layer(function(){
					self.remove_layer(function(){
						done && console.log('add_remove_layer.done');
						done && done();
						df.resolve();
					});
				});
			}).promise();
		}

		this.add_remove_layers = function(done){
			return $.Deferred(function(df){
				self.add_layer(function(){
					self.add_layer(function(){
						self.add_layer(function(){
							//self.add_layer(function(){
							//	self.remove_layer(function(){
									self.remove_layer(function(){
										self.remove_layer(function(){
											self.remove_layer(function(){
												done && done();
												df.resolve();
											});
										});
									});
							//	});
							//});
						});
					});
				});
			}).promise();
		}

		// TODO move to forecaster
		this.open_product_catalogue = function(done){
			return $.Deferred(function(df){
				$.when((function(){
						$('#products-menu').click();
						return self.test.wait_for('#products:visible');
					})())
					.then(function(){
						return self.test.wait_for('#products .items > .thumbnail:visible', 'waiting for thumbnails');
					})
					.then(function(){
						$('#products > .more > a').click();
						return self.test.wait_for('#product-select:visible', 'selector dialogue visible');
					})
					.then(function(){
						return self.test.wait_for('#product-select .catalogue-grid:visible', 'grid visible');
					})
					.then(function(){
						return $.Deferred(function(df){
							if($('#product-select .catalogue-grid .item:visible').length > 20, '20 items') df.resolve();
						});
					})
					.then(function(){
						df.resolve();
					});
			}).promise();
		}

		this.close_product_catalogue = function(done){
			return $.Deferred(function(df){
				$.when((function(){
						$('#product-select .ui-icon-close').click();
						return self.test.wait_for_remove('#product-select', 'remove');
					})())
					.pipe(function(){
						$('#products-menu').click();
						return self.test.wait_for_remove('#products');
					})
					.then(function(){
						df.resolve();
					});
			}).promise();
		}

		this.add_product = function(done){
			function ensure_hide_own_products() {
				return $.Deferred(function(df){
					var checked = $('#product-select .check-options input')[0].checked
					if(checked){
						df.resolve()
					}else{
						checked = $('#product-select .check-options input').click()
						df.resolve()
					}
				})
			}

			function get_n_products(){
				return parseInt($('#product-select .status > div:first').text())
			}

			function get_n_user_products(){
				return $('#products .items > .thumbnail').length
			}

			return $.Deferred(function(df){
				var n_products;
				var n_user_products;
				$.when(self.open_product_catalogue())
					.then(function(){
						n_products = get_n_products();
						n_user_products = get_n_user_products();
						self.test.assert(n_products > 1, 'n_products');
						self.test.assert(n_user_products > 1, 'n_user_products');

						return $.Deferred(function(df){
							click($('#product-select .catalogue-grid .item:visible img:first'));
							df.resolve();
						});
					})
					.then(ensure_hide_own_products)
					.then(function(){
						return self.test.wait_for(function(){
							return get_n_products() === n_products - 1;
						}, function(){ return 'product thumbnail remove from selector: got ' + get_n_products() + ' expected ' + (n_products - 1)});
					})
					.then(function(){
						return self.test.wait_for(function(){
							return get_n_user_products() === n_user_products + 1;
						}, function(){ return 'product thumbnail add to user list: got ' + get_n_user_products() + ' expected ' + (n_products + 1)});
					})
					.then(function(){
						df.resolve();
					});
			}).promise();
		}

		this.remove_product = function(done){
			function get_n_products(){
				return parseInt($('#product-select .status > div:first').text())
			}

			function get_n_user_products(){
				return $('#products .items > .thumbnail').length
			}

			return $.Deferred(function(df){
				var n_products = get_n_products();
				var n_user_products = get_n_user_products();

				$.when((function(){
						return $.Deferred(function(df){
							self.test.assert($('#products .items > .thumbnail:first').length === 1, 'trash 1 ' + $('#products .items > .thumbnail:first').length)
							$('#products .items > .thumbnail:first').thumbnailTools('show')
							self.test.wait_for(function() { return $('#products .items > .thumbnail:first .ui-icon-trash').length === 1}, 'trash ' + $('#products .items > .thumbnail:first .ui-icon-trash').length)
								.then(function(){
									$('#products .items > .thumbnail:first .ui-icon-trash').click();
									df.resolve();
								})
						});
					})())
					.then(function(){
						return self.test.wait_for(function(){
							return get_n_user_products() === n_user_products - 1;
						}, function() { return 'product thumbnail remove: got ' + get_n_user_products() + ' expected ' + (n_user_products - 1)});
					})
					.then(function(){
						return self.test.wait_for(function(){
							return get_n_products() === n_products + 1;
						}, 'product thumbnail remove 2');
					})
					.then(self.close_product_catalogue)
					.then(function(){
						df.resolve();
					});
			}).promise();
		}

		this.add_layer = function(done){
			if(!self.test.running) return;
			self.test.set_timeout(40000);
			return $.Deferred(function(df){
				self.ensure_chart_loaded(function(){
					self.ensure_layer_dialogue(function(){
						self.test._listen_once('layer-dialogue-selection');
						self.test._listen_once('chart-content-update');
						var n_layers = self._get_n_layers();

						console.log('adding layer:', $('#layer-select .catalogue-grid .item img:eq(' + find_unused_layer() + ')').attr('name'));
						console.log('clicking:', $('#layer-select .catalogue-grid .item img:eq(' + find_unused_layer() + ')'));
						self.test.assert('#layer-select .catalogue-grid .item img:eq(' + find_unused_layer() + ')', 'layer exists');
						click($('#layer-select .catalogue-grid .item img:eq(' + find_unused_layer() + ')'));

						self.test._wait_for_listeners(function(){
							self.test.wait_for(function(){ return self._get_n_layers() === n_layers + 1; }, 'n_layers=' + (n_layers + 1)).then(function(){
								n_layers++;

								$('#layer-select .close').click();

								done && done();
								df.resolve();
							}, 'n_layers');
						}, 'messages');
					});
				});

				//-return the row idx in the dialogue box list.
				function find_unused_layer(){
					var state = $('#chart').chart('getState').state.layers;

					var idx = -1;
					$('#layer-select .catalogue-grid .item img').each(function(i, el){
						var found = false;
						$.each(state, function(j, l){
							if(l.name === $(el).attr('name')){
								found = true;
							}
						});
						(idx < 0) && (!found) && (idx = i);
					});
					console.log('>> idx=' + idx);
					return idx;
				}
			}).promise();
		}

		this.remove_layer = function(done){
			var self = this;
			console.log('*** removing layer...', $('#layer-list .layer-list-row:first td.title').text());
			self.test.set_timeout(40000);
			var n_layers = self._get_n_layers();

			this.test._listen_once('layer-delete');
			this.test._listen_once('chart-content-update');
			this.test._listen_once('chart-product-change');

			console.log('clicking...');
			$('#layer-list .layer-list-row:first .small-icon-trash').parent().click().triggerHandler('mouseup');

			this.test._wait_for_listeners(function(){
				console.log('checking layer count... waiting for n=' + (n_layers - 1));
				self.test.wait_for(
					function(){ return self._get_n_layers() === n_layers - 1; },
					function(){ return 'remove: expect n_layers=' + (n_layers - 1) + ' got ' + self._get_n_layers() }
				).then(function(){
					n_layers--;
					done && done();
				});
			}, 'listeners');
		}

		self.ensure_chart_loaded = function(done){
			var self = this;
			self.test.wait_for(function(){
				return !$('#chart .loader > div:visible').length;
			}, 'chart loaded')
			.then(done);
		}

		this.ensure_layer_dialogue = function(done){
			if(have_dialogue()) return done && done();
			var self = this;

			self.test.set_timeout(20000);
			$('#layer-list-pane:visible').length || $('#gadget .button.layers').click();
			self.test.wait_for('#layer-list-pane:visible', 'layer list').then(function(){
				self.test.wait_for(function(){ return $('#layer-list-pane #layer-list tbody > tr:visible').length > 1; }, 'layer items').then(function(){
					$('#layer-list-pane .add-layer').click();
					self.test.wait_for('#layer-select:visible', 'layer dialogue').then(function(){
						self.test.wait_for(have_dialogue).then(function(){
							done && done();
						});
					});
				});
			});

			function have_dialogue(){
				return $('#layer-select .catalogue-grid .item:visible').length > 10;
			}
		}

		this.close_layer_list = function(done){
			return $.Deferred(function(df){
				$('#layer-list-pane:visible').length && $('#gadget .button.layers').click();
				Test.wait_for(function(){ return !$('#layer-list-pane:visible').length }).then(function(){
					done && done();
					df.resolve();
				});
			}).promise();
		}
	}

	// provide a click with coordinates for cases where coords are needed by the event handler
	function click(el) {
		var position = el.offset()
		console.log('click: position=', position)
		var x = position.left + el.width() / 2;
		var y = position.top + el.height() / 2;

		var ev = new MouseEvent('click', {
			'view': window,
			'bubbles': true,
			'cancelable': true,
			'screenX': x,
			'screenY': y
		})

		el[0].dispatchEvent(ev);
	}

	return ChartTestable;
});
