define(['config', 'jquery', 'bus', 'ajax_queue', 'menu-pane', 'jquery-ui/dialog'], function(ECMWF, $, Bus, Ajaxqueue) {

ECMWF.MenuBar = function(bar_selector, panes_id){

	// MenuPane/MenuList wrapper for dynamically creating a top level application menu.
	// -see Dashboard application for usage.

	this.bar = $(bar_selector);
	this.panes = $('body').div({id: (panes_id || 'menu-panes'), css: {position: 'absolute'}});

	this.add = function(menus){
		var self = this;
		$.each(menus, function(i, item){
			var menu = item;
			if(menu.enabled === false) return;
			var css_name = self._css_name(menu.title);

			self.bar.li({id: css_name + '-menu', 'class': 'menu-item'})
				.menuList()
				.text(menu.title)
				.each(function(){
					if(menu.position > -1){
						self.bar.find('> li:eq(' + menu.position + ')').after($(this));
					}
				});

			var pane = self.panes.div('.' + css_name + ' main-menu menu-pane drop-shadow')

			menu.ul = pane.ul({'class': 'menu-list', name: menu.name});

			if(Array.isArray(menu.items)){
				$.each(menu.items, function(i, item){
					self.add_menu_item(menu, item);
				});
			}

			menu.ul
				.menuList()
				.on('select', function() {
					pane.trigger('close');
					return false;
				});

			pane.menuPane({
				group : '#' + panes_id + ' > div',
				owner : $('#' + css_name + '-menu')
			});

			if(Array.isArray(menu.items)){
				$.each(menu.items, function(i, item){
					if(item && item.disabled) menu.ul.find('li[name=' + item.name + ']').removeClass('enabled').addClass('disabled');
				});
			}
		});
		return this;
	}

	this.get_menu = function(menu_name){
		return this.panes.find('.' + menu_name + ' > ul:first');
	}

	this.add_menu_item = function(menu, args){
		if(args){
			if(args.selected){
				Bus().listen('menu-' + menu.name + '-' + args.name + '-select', function(){
					args.selected();
				});
			}

			var item = $.extend({}, args);
			delete item.selected;
			return menu.ul.li(item).text(args.title);
		}
	}

	this._css_name = function(name){
		return name.replace(' ', '-').toLowerCase();
	}
}


ECMWF.dialogue = function(options) {

	var ok    = options.ok;
	var title = options.title || "";
	options.dialogue || (options.dialogue = {})

	var div = $('<div title="' + title + '">');

	var content = ECMWF.remove(options, 'content') || '';
	content && div.div({'class': 'content'}).html(content);

	var is_modal = (options.dialogue.modal !== false);
	is_modal && (options.dialogue.dialogClass = (options.dialogue.dialogClass ? options.dialogue.dialogClass + ' ' : '') + 'z-8');

	options = $.extend(options, {
		ok : function(choices) {
			div.dialog('close');
			ok(choices);
			div.dialog('destroy');
			div.remove();
		},
		cancel : function() {
			div.dialog('close').dialog('destroy').remove();
		},
		title : null
	});

	return div
		.dialog($.extend({ autoOpen: false, modal: true, width: options.width }, options.dialogue))
		.shadowFix()
		.form(options)
		.dialog('open');
};

ECMWF.alert = function(options) {

    var ok    = options.ok || function() {};
    var title = options.title || "Alert";

    var div = $("<div title='" + title + "'/>");

    var content = ECMWF.remove(options, 'content') || '';
    content && div.div({'class': 'content font-size-medium'}).html(content);

    options = $.extend(options, {
                show_cancel : false,
                ok : function(choices) {
                    div.dialog('close');
                    ok(choices);
                    div.dialog('destroy');
                },
                title : null,
                help  : options.message
            });

    div.dialog({
            autoOpen: false, 
            modal : true,
            dialogClass: 'alert'
        })
        .form(options);

    return div.dialog('open');
};

ECMWF.confirm = function(options) {

    var title = options.title || "Alert";

    var div = $("<div title='" + title + "'/>");
    var yes = options.yes || $.noop;
    var no  = options.no  || $.noop;

    if(options.text) div.div(options.text).addClass('content font-size-medium');

    options = $.extend({
                    cancel_label : "No",
                    ok_label : "Yes"
                },
                options, {
                    ok     : function(choices) { yes(choices); div.dialog('destroy').remove(); },
                    cancel : function(choices) { div.dialog('close'); no(choices); div.dialog('destroy').remove(); },
                    title  : null,
                    help   : options.message
                });

    div.dialog({
        autoOpen: false, 
        modal : true,
        resizable: false, 
        dialogClass: 'alert'
    }).form(options)
        .shadowFix();

    $('<div>').prependTo(div).html(options.preamble);

    return div.dialog('open');
};


ECMWF.inform = function(parent, options) {

    options = $.extend({
            offset_top  : 20,
            offset_left : 20,
            width       : "auto",
            auto_dismiss : 0,
            actions     : {},
            classes     : {},
            titles      : {}
        }, options);


    var pos = parent.position();

    var div = $('body').div('.inform-dialogue drop-shadow-small z-5')
        .css({
            position : 'absolute',
            top      : pos.top + options.offset_top,
            left     : pos.left + options.offset_left,
            width    : options.width
        })

    var text = options.text;

    text += "\n\n{dismiss}"
    options.actions.dismiss = options.actions.dismiss || function() { };
    options.titles.dismiss   = "Dismiss"
    options.classes.dismiss  = "right"

    var re = /\{(\w+)\}/g;
    text = text.replace(/\n/g, "<br>");

    var opened = true;

    if(options.auto_dismiss) {
        setTimeout(function() {
            if(opened) {
                div.remove();
            }
        },options.auto_dismiss * 1000);
    }

    // Replacement for string.split(re) that is broken on IE
    var vars = [], m; pos = 0;

    while((m = re.exec(text))) {
        vars.push(text.substr(pos,m.index-pos));
        pos = m.index + m[0].length;
        vars.push(m[1]);
    }

    if(pos < text.length) {
        vars.push(text.substr(pos,text.length-pos));
    }

    $.each(vars, function(i, txt) {
        if((i%2) === 0) {
            $("<span>").html(txt).appendTo(div);
        }
        else {

            var c = options.classes[txt];
            var w;

            if(options.actions[txt]) {
                var title = options.titles[txt] || txt;
                w = $("<a>").addClass("text-link").attr("href","#")
                    .text(title).appendTo(div).click(function(event){
                        opened = false;
                        div.remove();
                        options.actions[txt]();
                        //event.preventDefault();
                        return false;
                    });
            }
            else {
                title = options.titles[txt] || txt;
                w =  $("<span>").html(title).appendTo(div);
            }

            if(c) {
                w.addClass(c);
            }
        }
    });
}

ECMWF.get_user_catalogue = function(args) {
	Ajaxqueue($.extend(args, {
		url: '/backend/catalogue-items/',
		json : {
			type          : args.kind || 'product',
			user_items    : true,
			standard_items: false
		}
	}));
}

});
