define(['config', 'jquery', 'bus', 'jquery-ui/widget', 'city-autocomplete'], function(ECMWF, $, Bus) {

	var count = 0;

	$.widget('ui.cities', {

		options: {
			messageBus: Bus(),
			showSmallMap: false,
			selection: $.noop
		},

		_init: function() {
			var self = this, o = this.options;

			count++;

			self.name = 'cities_' + count;

			self.element.addClass('cities');

			self.cities = self.element.div().addClass('cities-inner');

			/* We do that so that the marker is visible when the cities are visible */
			self.element
				.on('show', function() { $.openProbe(self); })
				.on('hide', function() { $.closeProbe(self); });

			var input = self.cities
				.input()
				.cityAutocomplete({
					name: 'cities_view',
					source: function(event, result) {
						console.log('cities.source response=', result);
						self.small_map.smallMap('removeMarkers');
						if (result.matches) {
							self.message.text(result.matches + ' match' + (result.matches > 1 ? 'es' : ''));

							var last = result.results.length - 1;
							$.each($.merge([], result.results).reverse(), function(i, p) {
								self.small_map.smallMap('addMarker', p, (i === last));
							});

							self.view.show();
						} else {
							self.message.text('No match');
							if (self.view.text() === 'view') self.view.hide();
						}
					},
					select: function(event, ui) {
						console.log('cities.select');
						var city = ui.item;
						console.log('cities.select city=', ui.item);
						var pt = {
							from: 'cities',
							lat: parseFloat(city.lat),
							lon: parseFloat(city.lon),
							title: city.title
						}
						o.messageBus.send('latlon-location-select', pt);
						self.small_map.smallMap('selectMarker', city);
						o.selection(pt);
					},
					open: function(event, ui) {
						console.log('cities.open', ui);
					},
					close: function(event, ui) {
						console.log('cities.close');
					}
				});

			if ($.fn.onScreenKeyboard) {
				input.onScreenKeyboard({});
			}

			//self.loader = $("<span>").appendTo(self.cities).attr("id", "loader").hide();
			self.message = self.cities.span();
			self.view = self.cities.span()
				.hide()
				.css({paddingLeft: 3, cursor: 'pointer'});
			self.link = self.view.a('.text-link').text('view').click(function() {
				self.small_map.toggle();
				self.link.text(self.small_map.is(':visible') ? 'hide' : 'view');
				return false;
			});

			self.small_map = self.element.div().smallMap({messageBus: o.messageBus});

			o.showSmallMap || self.small_map.hide();
		},

		_destroy: function() {
			this.element.removeClass('cities').empty();
		}
	});


	$.widget('ui.smallMap', {

		options: {
			messageBus: Bus()
		},

		_init: function() {
			var self = this,
				o = this.options;

			self.element.addClass('small-map');

			self.small_map_img = self.element.img({
				json_url: '/backend/small-map/'
			});

			self.element.div('Matches may be very close to each other and show as one point on the map.').addClass('comment');

			// TODO: Share with other widgets
			var tip = $('<div>').addClass('tip').css('position', 'absolute').hide().appendTo('body');

			$('.marker', self.element.get(0))
				.on('hover', function(e) {
					var d = $(this).data();
					if (d && (e.type === 'mouseover' || e.type === 'mouseenter')) {
						var o = $(this).offset();
						tip.html(title(d)).css({
							top: o.top - tip.outerHeight(true) - 3,
							left: o.left - 10
						}).show();
						$(this).addClass('hover');
					} else {
						tip.hide();
						$(this).removeClass('hover');
					}
				})
				.on('click', function() {
					var pt = point($(this).data());
					o.messageBus.send('latlon-location-select', pt);

					$('.marker', self.element.get(0)).removeClass('current');
					$(this).addClass('current');
					o.messageBus.send('city-select', pt);
				});

			function point(p) {
				var city = p.city;

				var pt = {
					lat: city[2],
					lon: city[3]
				};

				if (city[0]) {
					pt.title = city[0]
				}
				if (city[0] && city[1]) {
					pt.title = city[0] + ', ' + city[1];
				}

				return pt;
			}

			function title(p) {
				var pt = point(p);
				return pt.title ? (pt.title + ' ' + $.formatGeoLocation(pt)) : $.formatGeoLocation(pt);
			}
		},

		destroy: function() {
			var self = this;

			self.element.removeClass('small-map');
		},

		addMarker: function(pt, select) {
			var self = this;

			var lat = parseFloat(pt[2]);
			var lon = parseFloat(pt[3]);

			while (lon <= -180) {
				lon += 360;
			}
			while (lon > 180) {
				lon -= 360;
			}

			self.element
				.div({'class': 'marker'})
				.data('city', pt)
				.css({top: 90 - lat, left: lon + 180})
				.map(function() {
					return select ? this : null;
				})
				.addClass('current');
		},

		removeMarkers: function() {
			var self = this;

			self.element.find('.marker').remove();
		},

		selectMarker: function(item) {
			var self = this;
			console.log('selectMarker', item);

			self.element.find('.marker')
				.removeClass('current')
				.each(function() {
					var pt = $(this).data().city;
					if (pt[2] === item.lat && pt[3] === item.lon) {
						$(this).addClass('current');
					}
				});
		}
	});

});
