/*
 * ECMWF City Autocomplete
 * Copyright ECMWF http://www.ecmwf.int
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 *
 */
define(['jquery', 'ajax_queue', 'jquery-ui/autocomplete'], function($, Ajaxqueue) {

	var pos_re = /([0-9\.\,]+)\s*([nsew])\s*([0-9\.\,]+)\s*([nsew])/i;
	var loc_re = /^\s*current\s*location\s*$/i;

	$.widget('ui.cityAutocomplete', {

		options: {
			name: '',
			is_country : false,
			source: $.noop,
			select: $.noop,
			open: $.noop,
			close: $.noop,
			focus: $.noop
		},

		_init: function () {
			var self = this,
			o = this.options;

			self.element.addClass('city-autocomplete');

			if (o.is_country)
				placeholder = 'Enter a country name';
			self.element.attr('placeholder', placeholder);

			/*
			var canceller = new Canceller(function () {
				console.log('!! cancelled');
			});
			*/

			self.debug = {n_pending: 0};

			self.element.autocomplete({
				minLength: 3,
				select: function (e, ui) {
					o.select(e, ui);
				},
				open: function (e, ui) {
					o.open(e, ui);
				},
				close: function (e, ui) {
					o.close(e, ui);
				},
				change: function (e, ui) {},
				focus: function (e, ui) {
					o.focus(e, ui);
				},
				source: function (request, response) {
					self.debug.n_pending++;
					console.log('request:', request);
					if (navigator.geolocation) {
						if (loc_re.exec(request.term)) {
							console.warn("Current location");
							navigator.geolocation.getCurrentPosition(
							function (position) {
								var pt = {
									lat: position.coords.latitude,
									lon: position.coords.longitude,
									title: 'Current location',
									long_title: 'Current location ' + $.formatGeoLocationPlain(position.coords)
								};
								pt.label = pt.long_title;
								pt.value = pt.title;
								console.warn('Current location', pt);
								o.source(request, {
									matches: 1,
									results: [
										['Current location', undefined, pt.lat, pt.lon]]
								});
								response([pt]);
							},
							function (error) {
								console.warn(error);
								response({});
								if (error.message) {
									$.logError(error.message);
								}
								else {
									switch (error.code) {
									case error.PERMISSION_DENIED:
										$.logError('Get current position: permission denied');
										break;
									case error.POSITION_UNAVAILABLE:
										$.logError('Get current position: position unavailable');
										break;
									case error.TIMEOUT:
										$.logError('Get current position: time out');
										break;
									default:
										$.logError('Get current position: unknown error');
										break;
									}
								}
							});
							return;
						}
					}

					var match = pos_re.exec(request.term);
					if (match) {
						console.warn("MATCH ", match);
						var pt = {};
						function parse(a, b) {
							a = a.replace(',', '.');
							if (b === 'N' || b === 'n') {
								pt.lat = parseFloat(a);
							}
							if (b === 'S' || b === 's') {
								pt.lat = -parseFloat(a);
							}
							if (b === 'E' || b === 'e') {
								pt.lon = parseFloat(a);
							}
							if (b === 'W' || b === 'w') {
								pt.lon = -parseFloat(a);
							}
						}

						parse(match[1], match[2]);
						parse(match[3], match[4]);
						console.warn("MATCH ", pt);

						if (pt.lat !== undefined && pt.lon !== undefined) {
							while (pt.lon <= -180) {
								pt.lon += 360;
							}
							while (pt.lon > 180) {
								pt.lon -= 360;
							}

							pt.label = $.formatGeoLocationPlain(pt);
							pt.value = $.formatGeoLocationPlain(pt);
							o.source(request, {
								matches: 1,
								results: [
									[undefined, undefined, pt.lat, pt.lon]]
							});
							response([pt]);
							return;
						}
					}
					//TODO: To be implemented. Set is_wmo:True to search in WMO database
					Ajaxqueue({
						url: '/backend/city-lookup/',
						name: 'autocomplete_' + o.name,
						json: {
							name: request.term,
							members: false,
							is_country: o.is_country
						},
						logErrors: 'City lookup',
						success: function (json) {
							self.debug.n_pending--;
							//console.log('cities SUCCESS pending=' + self.debug.n_pending);
							o.source(request, json);

							if (json) {
								if (o.is_country) {
									var countries= [];
									for (var cnt in json.results) {
										countries.push({
											title: cnt,
											long_title: cnt,
											label : cnt,
											value: cnt,
											result : json.results[cnt]
										});
									}
									console.log('countries: ', countries);
									response(countries);
								}
								else {
									response($.map(json.results, function (city) {
										var pt = {
											lat: city[2],
											lon: city[3],
											title: city[0] + ", " + city[1],
											long_title: city[0] + (city[6] ? (", " + city[6]) : "")  + ",  " + city[1],
											altitude: city[5]
										};
										pt.label = pt.long_title;
										pt.value = pt.title;
										console.log('pt: ', pt);
										return pt;
									}));
								}
							}
							else {
								console.log("cities AJAX EMPTY");
								response([]);
							}
						},
						canceled: function() {
							self.debug.n_pending--;
							console.log('cities CANCELLED <<<<<<<<<<<<<<<<<< pending=' + self.debug.n_pending);
							response([]);
						},
						error: function (error) {
							self.debug.n_pending--;
							console.log('cities ERROR pending=' + self.debug.n_pending);
							$.logError($.formatError(error));
							response([]);
						}
					});
				}
			});
		},

		destroy: function () {
			var self = this;

			self.element.removeClass('city-autocomplete');
		}

	});

	var placeholder = "Enter a city name";

	function Canceller(_on_cancel) {
		this.on_cancel = _on_cancel;
		this.timeout = null;

		this.reset = function () {}

		this.start = function () {
			this.timeout = setTimeout(function () {
				//this.on_cancel();
				_on_cancel();
			},
			20 * 10);
		}

		this.stop = function () {
			clearTimeout(this.timeout);
			this.timeout = 0;
		}
	}

});
