define(['config', 'jquery', 'ajax_queue', 'storage', 'filterable-list', 'mixins', 'utils', 'throttle-debounce'], function(ECMWF, $, Ajaxqueue, localStorage, List, Mixins){

	/*
	 * Facets are formatted as below:
	 *
	 * product: {
	 *     tags: {
	 *         facet_name1: value1,
	 *         facet_name2: [value2, value3]
	 *     }
	 * }
	 *
	 */

	var settings = {
		cache_expire_mins: ECMWF.options.thumbnail_cache || 10
	}

	var mixins = {
		validate: function(tests) {
			for(var t in tests) if(!tests[t][1]()){
				var message = tests[t][0];
				console.warn('validate: "' + message + '" failed.');
				this.error_msg = "test '" + message + "' failed";
				return false;
			}
			return true;
		}
	}

	function Collection(){
	}
	Collection.prototype = $.extend(new List(), mixins, {bind: Mixins.bind}, {
		product_adaptor: function(result){
			return result;
		},

		set_package: function(pkg){
			this._products = null;
			this._facets = null;
			$.extend(this.query, {package: pkg})
		},

		set_list: function(){
			var self = this;
			this.products().then(function(items){
				self.on_load(items)
			})
		},

		product_by_name: function(name) {
			for(var i in this._products){
				if(this._products[i].data.name === name) return this._products[i];
			}
		},

		/*
		 *  Get html text content associated with the given product or category.
		 */
		content: function(product_name, done) {
			Ajaxqueue({
				url: '/plots/get-product-text/',
				json: {product: {name: product_name}},
				success: done
			});
		},

		/*
		 *  Generate a list of facets from the product list.
		 *  The method is synchronous and relies only on product data, not package data.
		 */
		facets: function(){
			var self = this;
			if(!this._facets){
				this._facets = {};

				$.each(this._products, function(i, product){
					if(product.data.tags){
						$.each(product.data.tags, function(i, v){

							self._facets[i] || (self._facets[i] = {});

							if(Array.isArray(v))
								$.each(v, function(j, v){ self._facets[i][v] = true; });
							else
								self._facets[i][v] = true;
						});
					}
				});
				$.each(self._facets, function(i, v){
					self._facets[i] = $.map(v, function(_, i){ return i; }).sort();
				});
			}

			return this._facets;
		},

		/*
		 * Now that products can be in more than one package, the pkg needs to be specified
		 */
		sorted_facets: function(pkg, blacklist, done) {
			var self = this;
			if(arguments.length === 2){
				done = blacklist;
				blacklist = [];
			}

			this._products.length && this.packages.then(function() {
				self.facets();

				var sorted = pkg.facets || [];

				var keys = sorted.length
					? $.map(self._facets, function(i, key){ if(!~$.inArray(key, blacklist)) return {name: key, values: i, order: $.inArray(key, sorted)}; })
					: $.map(self._facets, function(i, key){ if(!~$.inArray(key, blacklist)) return {name: key, values: i, order: 0}; });

				done(keys.sort(function(a, b) {
					return a.order - b.order;
				}));
			});
		},

		hierarchy: {
			products: this,
			tree: {},

			update: function(categories) {
				var self = this;
				/*
				this.tree = {};
				$.each(this.products.products, function(i, p){
					for(var k in p.data.path){
						var path = p.data.path[k].split('/');
						var cursor = self.tree;
						for(var j in path){
							if(path[j]){
								cursor = cursor[path[j]] = cursor[path[j]] || {};
							}
						}
					}
				});
				*/
				this.tree = categories;

				// fill in missing categories TODO move to server.
				for(var i in this.products._packages){
					var pkg = this.products._packages[i];
					if(!$.findWhere(this.tree, {name: pkg.name})){
						this.tree.push({name: pkg.name, title: pkg.title});
					}
				}
			},

			children: function(path) {
				var node = this._child(path, 0, this.tree);
				return node ? node.children : [];
			},

			lookup: function(path) {
				if(!path || path === 'all') return {title: 'ECMWF charts'}; // special case because the tree does not currently have a root item

				$.isEmptyObject(this.tree) && console.warn('hierarchy.lookup: no package list');
				if(!Array.isArray(path)) path = path.replace(/\/#.*/, '').split('/');
				return this._child(path, 0, this.tree) || $.findWhere(products._packages, {name: ECMWF.options.default_package})
			},

			_child: function(path, p, tree){
				for(var i in tree){
					var node = tree[i];
					if(node.name === path[p]){
						if(p === path.length - 1){
							return node;
						}else{
							p++;
							return this._child(path, p, node.children);
						}
					}
				}
			},

			path_is_valid: function(path) {
				return true;
			}
		},

		store: localStorage ? function(name, item) {
			this.__cache[name] = item;
			this._store();
		} : $.noop,

		cancel_thumbnails: function() {
			//Ajaxqueue.cancelByTag(new RegExp(self.eventNamespace + '$')); - does not work when batching requests
			this._cancel_warning_done || console.log('products.cancel_thumbnails TODO');
			this._cancel_warning_done = true;
		}
	})

	return Collection
})

