define(['config', 'jquery', 'ajax_queue', 'storage', 'bus', 'throttle-debounce'], function(ECMWF, $, Ajaxqueue, localStorage, Bus){

	var settings = {
		score: ECMWF.options.product_scoring || false,   // reject misbehaving products
	}

	//------------------------------------

	// TODO review caching - which collection types need caching?

	var _cache =
		localStorage ? function(name) {
			if(!this.__cache){
				this.__cache = JSON.parse(localStorage.products || '{}');
			}
			return name ? this.__cache[name] : this.__cache;
		}
		: function(){ return {}; }

	function expired(item) {
		return item && ((!item.timestamp) || (new Date().getTime() - item.timestamp > 1000 * 60 * settings.cache_expire_mins));
	}

	setTimeout(function clean_cache() {
		$.each(_cache(), function(i, item) {
			if(expired(item)){
				delete _cache()[i]
				products._store();
			}
		});
		//console.log('products.clean_cache: size=', ECMWF.n_properties(_cache()));
	}, 5000);

	//------------------------------------

	var mixins = {
		validate: function(tests) {
			for(var t in tests) if(!tests[t][1]()){
				var message = tests[t][0];
				console.warn('validate: "' + message + '" failed.');
				this.error_msg = "test '" + message + "' failed";
				return false;
			}
			return true;
		}
	}

	var ChartListItem = function(item) {
		if(!item) return console.warn('ChartListItem: no data');

		this.data = item;

		// tags are forced to be arrays for efficient filtering.
		for(var tags in item.tags){
			var t = item[tags];

			if(t) this.data.tags[tags] = [t]; // for 'title'. can probably be remove later.
			if(typeof item.tags[tags] === 'string') this.data.tags[tags] = [item.tags[tags]];
		}

		this.cache = {
			thumbnail: function(val) {
				return this.cache._prop('thumbnail', val);
			},

			score: function(val) {
				return settings.score
					? (this.cache._prop('score', val) || 0) // TODO is the zero needed?
					: 0;
			},

			_prop: function(name, val) {

				this._item = this._item || _cache(this.product.name()) || this._empty();
				if(val){
					this._item[name] = val;
					products.store(this.product.name(), this._item);
				}else{
					expired(this._item) && this._clear();
				}

				if(this._item[name]){
					return this._item[name];
				}
			},

			_empty: function() {
				return this._item = {timestamp: new Date().getTime()};
			},

			_clear: function() {
				this._empty();
				products.store(this.product.name(), this._item);
			},

			_item: null,
			product: this
		}
	}

	ChartListItem.prototype = $.extend({
		name: function() {
			return this.data.name;
		},

		title_: function() {
			return this.data.title;
		},

		long_title: function() {
			return this.data.long_title || this.data.title;
		},

		keywords: function(order) {
			return '';
		},

		has_widget: function(widget) {
			return !!~$.inArray(widget, this.data.widgets)
		},

		thumbnail: function(args, done) {
			done = done || $.noop;

			var tn = this.cache.thumbnail.call(this);
			if(tn){
				return done(tn);
			}

			Thumbnail_queue.add(this, args, done);
		},

		/*
		 *   Temporary method TODO integrate with ChartListItem.thumbnail
		 */
		large_thumbnail: function(args, done) {
			var viewport = {width: 480}
			var json = this.has_widget('interactive')
				? {product: this.name(), viewport: $.extend(viewport, {height: 270})}
				: {product: {name: this.name()}, viewport: $.extend(viewport, {thumbnail: true})}
			Ajaxqueue($.extend({
				url        : '/plots/get-map/',
				json       : {product: json},
				tag        : 'grid-large-' + this.name(),
				priority   : products.thumbnail_priority,
				logErrors  : 'Get thumbnail (' +  this.title_() + ')',
				complete: function(result){
					done(result)
				}
			}, args));
		}
	}, mixins)

	// In the absence of generic ajax bundling, thumbnail requests are bundled together here
	Thumbnail_queue = {
		max_bundle: 1,
		items: [],
		in_progress: [],
		timer: null,
		add: function(item, args, done) {
			var self = this, q = Thumbnail_queue, lists;

			for(var ai in lists = [q.items, q.in_progress]){
				for(var i in lists[ai]){
					var l = lists[ai][i];
					if(item.name() === l.item.name()){
						return l.done.push(done);
					}
				}
			}

			q.items.push({item: item, args: args, done: [done]});
			q.timer = q.timer || setInterval(function() {
				if(q.items.length){
					do q._fetch(q.items.splice(0, q.max_bundle)); while(q.items.length >= q.max_bundle);
				}else{
					q.timer = clearInterval(q.timer);
				}
			}, 100);
		},
		_fetch: function(items) {
			var self = this, q = Thumbnail_queue;

			q.in_progress = q.in_progress.concat(items);

			var args = items[0].args; // args (eg, size) are assumed to be common to all requests
			var first = items[0].item;
			var names = $.map(items, function(v){ return v.item.name() });

			function _fail(error) {
				$.each(items, function(i, item){
					var product = item.item;
					settings.score && product.cache.score.call(product, product.cache.score.call(product) - 1);
					$.each(item.done, function(i, done){ done(null, null, error || 'failed'); });
				});
			}

			Ajaxqueue($.extend(
				{
					url        : '/backend/product-thumbnail-inline/',
					json       : {name: names},
					tag        : 'grid-' + names.join(','),
					priority   : products.thumbnail_priority,
					logErrors  : 'Get thumbnail (' +  first.title_() + ')',
					complete: function(result){
						var indexes = [];
						for(var i in items){
							var item = items[i].item
							for(var j in q.in_progress){
								var l = q.in_progress[j];
								if(l.item.name() === item.name()){
									indexes.push(+j)
									break
								}
							}
						}
						indexes = indexes.sort(function(a, b){ return b - a });
						for(i in indexes){
							q.in_progress.splice(indexes[i], 1);
						}

						$.each(items, function(i, item) {
							var product = item.item;
							if(product.validate([
								['has result', function(){ return result; }],
								['has url', function(){ return result.thumbnails || result.url; }]
							])){
								product.cache.thumbnail.call(product, result.url)
								$.each(item.done, function(j, done){ done(result.thumbnails[i].url, result.thumbnails[i].img); });
							}
							else _fail('validation failed');
						});
					},
					error: _fail
				},
				args)
			);
		}
	}

	return ChartListItem
})
