/** @license
 *  ECMWF Ajax-queue
 *  (C) Copyright ECMWF http://www.ecmwf.int
 *
 *  This software is licensed under the terms of the Apache Licence Version 2.0
 *  which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 *  In applying this licence, ECMWF does not waive the privileges and immunities
 *  granted to it by virtue of its status as an intergovernmental organisation nor
 *  does it submit to any jurisdiction.
 *
 */
define(['config', 'jquery', 'core/clone', 'utils', 'core/browser'], function(ECMWF, $, clone, Utils) {

    $.ajaxQueuePOST = 1;

    var PRIORITIES = {
        'vip': 0,
        'one': -1,
        'very-high': 1,
        'high': 10,
        'normal': 100,
        'low': 1000,
        'very-low': 10000
    };

    var queue = [];
    var active = [];
    var seqno = 0;
    //var timeout     = 5;

    var queue_locked = false;
    var pre_queue = []; //probably temporary, pending review of queue locking.

    var outstanding = {
        'vip': 0,
        'very-high': 0,
        'high': 0,
        'normal': 0,
        'low': 0,
        'very-low': 0,
        'one': 0
    };

    var order = {
        'very-low': false,
        'low': 'very-low',
        'normal': 'low',
        'high': 'normal',
        'very-high': 'high',
        'vip': 'very-high',
        'one': false
    };

    // Because a "very-high" request can be scheduled as a "high" request
    // or lower, there are in effect 5 very-high, 4 high and so on...

    var max = {
        'vip': 1,
        'very-high': 1,
        'high': 1,
        'normal': 1,
        'low': 1,
        'very-low': 1,
        'one': 1
    };

    var ignore = $.noop;
    var queueStart = ignore;
    var queueStop = ignore;

    $(window).on('unload', function() {
        queue = null;
        active = null;
        $(document).off('debug');
    });

    var handlers = [];

    function log(msg, args) {
        Ajaxqueue.options.log &&
            console.log(msg + ' ' + args._seqno + ': ' + Ajaxqueue.options.host + args.url +
                (args.xhr ? ' ' + args.xhr.status : '') +
                (args.name ? (' name:' + args.name) : '') +
                (args.cancelled_by && !$.isEmptyObject(args.cancelled_by)? ' cancel:' + clone.serialise(args.cancelled_by) : '') +
                (args.image_load ? ' image: ' + args.image_load : ''));
    }

    var _ajaxQueue = function(args) {

        args = $.extend({
            'type': Ajaxqueue.options.type,
            dataType: Ajaxqueue.options.dataType,
            _seqno: seqno++,
            _valid: true,
            name: null,
            priority: 'normal',
            'status': ignore,
            statistics: false,
            cache: undefined, // time in milliseconds
            tag: undefined, // for cancellation only
            image_load: false,
            logErrors: undefined, // context string passed to $.logError()
            cancelled: undefined, // callback function
            cancelled_by: {},
            timers: {
                queued: new Date()
            },
            timeout: Ajaxqueue.options.timeout,
            start: (new Date()).getTime() / 1000.0,
            xhrFields: {
                withCredentials: true // always send cookies
            },
            contentType: 'application/json'
        }, args);

        if (ECMWF.token) {
            args.headers = {
                'X-ECMWF-KEY': ECMWF.token
            };
        }

        if (__DEV__ && !args.url) {
            if (args.error) {
                args.error(null, 'ajaxQueue: URL is null', null);
            } else {
                console.error('ajaxQueue: URL is null', args);
            }
            console.trace();
            try {
                throw Error();
            } catch (e) {
                for (var x in e) {
                    console.error(e[x]);
                }
            }
            throw Error('ajaxQueue: URL is null');
        }

        if (args.json) {
            if ($.ajaxQueuePOST) {
                args.type = 'POST';
                args.contentType = 'application/json';
                args.data = clone.serialise(args.json);
            } else {
                args.type = 'GET';
                if (args.url.indexOf('?') === -1) {
                    args.url += '?json=' + clone.serialise(args.json);
                } else {
                    args.url += '&json=' + clone.serialise(args.json);
                }
            }
        }

        if (Ajaxqueue.options.noStreaming) {
            if (args.url.indexOf('?') === -1) {
                args.url += '?_no_streaming=1';
            } else {
                args.url += '&_no_streaming=1';
            }
        }

        /* Make sure jquery does not try to cache the request itself */
        if (args.cache) {
            args._cache = args.cache;
            args.cache = undefined;
        }

        args.queue = args.priority;
        args.priority = PRIORITIES[args.priority];

        if (typeof(args.cancelled_by) == 'object') {
            if (args.cancelled_by instanceof Array) {
                var c = {};
                for (var n in args.cancelled_by) {
                    c[n] = true;
                }
                args.cancelled_by = c;
            }
        }

        if (typeof(args.cancelled_by) == 'string') {
            var s = args.cancelled_by;
            args.cancelled_by = {};
            args.cancelled_by[s] = true;
        }

        if (args.name) {
            args.cancelled_by[args.name] = true;
        }

        enqueue(args);

        return args;
    };

    function cancel(why, args) {
        __DEV__ && log('cancel ' + why, args);

        if (!args._canceled) {
            if (args.canceled) {
                try {
                    args.canceled(why);
                } catch (e) {
                    console.error(e);
                    args.exception = e;
                }
            }
            status(args, 'canceled');
        }

        args._canceled = true;

        // Tell event handlers to giveup
        if (args.giveup) {
            args.giveup();
        }
    }

    var check_cancelations = function(args, other, status) {
        if (args.name && other.cancelled_by[args.name]) {
            cancel('CANCEL by newer [' + args._seqno + '] request (' + status + ') :', other);
            other._canceled = true;
        }
    };

    var shown = false;

    function stopStartQueue() {
        try {
            if (active.length) {
                if (!shown) queueStart();
                shown = true;
            } else {
                if (shown) queueStop();
                shown = false;
            }
        } catch (e) {
            console.error(e);
        }
    }

    // If the Internet connection is lost with FF, all the outstanding requests are lost and the callbacks are not called
    /*
    setInterval(function() {
        $.each(active, function(_, args) {
            if (args.xhr && args.xhr.readyState === 4 && (!args.xhr.responseText)) {
                if (!args._valid) {
                    console.warn('Invalid state for request', args.url, args.xhr.status);
                    args.error(args.xhr, 'Invalid request state, Internet connection may be lost', null);

                } else {
                    args._valid = false;
                    args._status = 'invalid';
                }
            }
        });
    }, 5000);
    */

    var enqueue = function(args) {

        __DEV__ && log('enqueue', args);

        // Cancel previous request with the same name
        if (args.name) {
            $.each(queue, function(_, other) {
                check_cancelations(args, other, 'queue');
            });
            $.each(active, function(_, other) {
                check_cancelations(args, other, 'active');
            });
        }

        status(args, 'queued');

        if (queue_locked) {
            pre_queue.push(args);
        } else {
            queue.push(args);
        }

        // Sort by priority first, then by seqno

        queue = queue.sort(function(a, b) {

            if (a.priority != b.priority)
                return a.priority - b.priority;

            return a._seqno - b._seqno;
        });

        if (args.priority === 0)
            check_queue();
        else
            setTimeout(check_queue, 0);
    };


    function notify(args, fn, result, what) {

        // DO NOT REMOVE THE TRY/CATCH!!
        try {
            // console.log("NOTIFY " + what + ": " + result);
            if (!args._canceled) {
                __DEV__ && log('AJAX ' + what, args);
                status(args, what);
                fn(result);
                args._complete(result);
            } else {
                // TODO: Notify user that the request was ignored
                // setTimeout(function() { c(args, result);} , 5);
                __DEV__ && log('IGNORE CANCELED', args);
            }
        } catch (e) {

            console.error(e);

            args.exception = e;
            try {
                console.trace();
            } catch (e) {}
        }

        active = $.grep(active, function(other) {
            return other !== args;
        });
        stopStartQueue();
        outstanding[args.queue]--;
        setTimeout(check_queue, 0);

        /*
        if (ECMWF.options && ECMWF.options.settings && ECMWF.options.settings.collect_statistics && args.statistics && !args.from_cache) {

            setTimeout(function() {
                var data = $.extend($.cloneObject(args, true), {
                    framework: {
                        loaded_at: ECMWF.timestamp.format('yyyymmddhhMMss')
                    }
                });

                $.ajaxQueue({
                    url: ECMWF.options.ajax.backend.statistics,
                    json: data,
                    type: 'POST',
                    statistics: false,
                    priority: 'very-low'
                });
            }, 10);
        }
        */
    }

    var can_run = function(args) {
        var q = args.queue;
        while (q) {
            if (outstanding[q] < max[q]) {
                args.queue = q;
                return true;
            }
            q = order[q];
        }
        return false;
    };

    var check_queue = function _check_queue() {
        queue_locked = true;

        var cannot_run = [];

        $.each(queue, function(_, args) {

            if (args._canceled) {
                __DEV__ && log('canceled request not dispatched', args);
                return;
            }

            if (!can_run(args)) {
                cannot_run.push(args);
                return;
            }

            var success = args.success || ignore;
            var error = args.error || ignore;

            // manage complete() ourselves in order to return the json response
            args._complete = args.complete || ignore;
            delete args.complete;

            args.success = function _success(result) {

                args.result = result;

                if (result && result.error) {
                    args.error(result.error, result.error);
                }

                if (result && !args._canceled) {

                    if (__DEV__ && (ECMWF.uid && result.uid) && (ECMWF.uid !== result.uid)) {
                        setTimeout(function() {

                            //TODO: Make that an exception later...
                            $.logError("User mismatch ui='" + ECMWF.uid + "', ajax='" + result.uid + "' (" + args.url + ')', args.log_error);
                        }, 50);
                    }

                    var a = $.extend([], arguments, [args]);

                    $.each(handlers, function(i, fn) {
                        setTimeout(function() {
                            fn(args, result);
                        }, 0);
                    });

                    if (args.timers && !args.timers.ajax) {
                        args.timers.ajax = (new Date() - args.timers.ajax_start) / 1000.0;
                    }

                    // Cache settings from backend are more important
                    var cache = result && (result.cache || args._cache);

                    if (cache && Ajaxqueue.options.cache) {
                        try {
                            Ajaxqueue.options.cache.put('ajax-queue', {
                                'url': args.url,
                                'data': args.json ? null : args.data,
                                'json': args.json
                            }, result, cache);
                        } catch (e) {
                            args.exception = e;
                            console.warn(e);
                        }
                    }
                }

                notify(args, success, result, 'complete');
            };

            args.error = function(request, textStatus, errorThrown) {
                var json = {
                    status: textStatus,
                    exception: errorThrown
                };
                if (request) {
                    args.result = json;
                    try {
                        json = $.extend(json, JSON.parse(request.responseText));
                        args.result = json;
                    } catch (e) {
                        args.exception = e;
                    }

                    //override unhelpful status text:
                    try {
                        if (request.statusText && json.status === 'error') {
                            json.status = request.status + ' ' + ECMWF.ucfirst(request.statusText.toLowerCase());
                        }
                    } catch (e) {}
                }

                try {
                    args._error = $.formatError(json);
                } catch (e) {
                    args.exception = e;
                }

                if (args.logErrors) {
                    try {
                        $.logError(json, args.logErrors, args.log_bus);
                    } catch (e) {
                        args.exception = e;
                    }
                }
                notify(args, error, json, 'aborted');

                // if using jsonp, error will always be 404
                if(Ajaxqueue.options.on_error) Ajaxqueue.options.on_error('backend failure: ' + args.url + ' '+ textStatus + ' ' + args.xhr.status);
            };

            if (args.img) {
                // If the image provided, assume that there is an URL in the reply
                var save_success = args.success;
                var save_error = args.error;

                args.success = function _on_img_success(json) {

                    if (args._canceled) {
                        save_success(json);
                        return;
                    }

                    if (!args.timers.ajax) {
                        args.timers.ajax = (new Date() - args.timers.ajax_start) / 1000.0;
                    }

                    if (!json) {
                        return save_error.apply(json, [null, 'Failed to get image url: empty response']);
                    }

                    if (json.error) {
                        return save_error.apply(json, [null, 'Failed to get image url: ' + json.error]);
                    }

                    if (json.deferred) {
                        save_success(json);
                        return;
                    }

                    var url = json.result ?
                        json.result.url :
                        json.url; // + "?seqno=" + args._seqno;

                    if (!url) {
                        return save_error.apply(json, [null, 'Failed to get image url']);
                    }

                    var prev = args.img.attr('src');
                    __DEV__ && log('IMAGE URL' + url, json);

                    if (url === prev) {
                        // Some browser will not trigger "load" if the url is the same
                        save_success(json);
                        return;
                    }

                    var img_done = false;

                    args.img
                        .each(function() {
                            __DEV__ && log('BIND ' + $(this).get(0).onload, args);
                        })
                        .off('.ajaxqueue')
                        .one('load.ajaxqueue', function() {
                            img_done = true;

                            clearTimeout(timeout_timer);
                            timeout_timer = null;

                            __DEV__ && log('IMAGE IN-LOAD ', args);

                            if (args.ignore_image) {
                                __DEV__ && log('IGNORE IMAGE  ', args);
                                return;
                            }

                            __DEV__ && log('ajaxqueue: IMAGE LOAD done: ', args);
                            args.timers.image = (new Date() - args.timers.image_start) / 1000.0;
                            args.image_load = false;
                            args.ignore_image = true;
                            save_success(json);
                        })
                        .one('error.ajaxqueue', function(event) {

                            __DEV__ && console.log('IMG ERROR EVENT RECEIVED');

                            if (args.ignore_image) {
                                return;
                            }

                            if (!args._canceled) {
                                args.img.attr('src', Utils.blank_gif).addClass('failed ui-icon ui-icon-alert');
                            }
                            __DEV__ && log('ajaxqueue: IMAGE LOAD failed: ', args);
                            args.image_load = false;
                            args.ignore_image = true;
                            save_error.apply(json, [null, 'Failed to load image ' + url]);
                        });

                    // a long timeout is not used with MSIE, as it causes an unexplained memory leak.
                    var max_timeouts = ECMWF.browser.msie ? 100 : 1;
                    var n_timeouts = 0;

                    function check_for_timeout() {
                        if (!img_done) {
                            if (n_timeouts++ >= max_timeouts) giveup();
                            else set_timeout();
                        }
                    }

                    function set_timeout() {
                        timeout_timer = setTimeout(check_for_timeout, args.timeout / max_timeouts);
                    }

                    function giveup() {

                        if (args.ignore_image) {
                            return;
                        }

                        if (args.image_load) {
                            __DEV__ && log('GIVING UP on ' + url, args);
                            args.ignore_image = true;
                            if (args._canceled) {
                                save_success(json);
                            } else {
                                save_error.apply(json, [null, 'Timeout on ' + url]);
                            }
                        }
                    }

                    __DEV__ && args.img.each(function(){ log('BIND 2 ' + $(this).get(0).onload, args); });
                    args.timers.image_start = new Date();
                    args.image_load = url;
                    args.giveup = giveup;
                    __DEV__ && log('IMAGE set to ' + url, args);
                    args.img.attr('src', url);

                    if (0) {

                        var dom_img = args.img.get(0);
                        //console.warn("IMAGE LOAD:",dom_img.complete, " " , dom_img.readyState);

                        // Some browser do not trigger a onload if the image is already
                        // in the their cache. We can trigger
                        if ((dom_img.readyState === undefined && dom_img.complete) || (dom_img.readyState === 4 || dom_img.readyState === 'complete')) {
                            //console.warn("IMAGE IS READY");
                            // It is OK to trigger the handler,
                            // even if it triggered also by the browser
                            // as it is registered with "one()"
                            __DEV__ && log('IMAGE READY', args);
                            setTimeout(function() {
                                __DEV__ && log('-> IMAGE TROGGER LOAD', args);
                                args.img.trigger('load');
                                __DEV__ && log('<- IMAGE TROGGER LOAD', args);
                            }, 500);
                        }
                    }

                    // If the DOM object is destroyed, the event handlers are not called

                    var timeout_timer;
                    set_timeout();
                };
            }

            outstanding[args.queue]++;

            if (args._cache && Ajaxqueue.options.cache) {
                var json = null;
                try {
                    json = Ajaxqueue.options.cache.get('ajax-queue', {
                        url: args.url,
                        data: args.json ? null : args.data,
                        json: args.json
                    });
                } catch (e) {
                    args.exception = e;
                    console.warn(e);
                }
                if (json) {
                    args.from_cache = true;
                    args.success(json);
                    return;
                }
            }

            status(args, 'active');
            active.push(args);
            stopStartQueue();

            var now = new Date();

            args.timers.queued = (now - args.timers.queued) / 1000.0;
            args.timers.ajax_start = now;

            var req = args;

            if (Ajaxqueue.options.host && req.url[0] === '/') {
                req = $.extend({}, req, {url: Ajaxqueue.options.host + req.url});
            }

            if (req.contentType === 'application/json' && req.dataType === 'json') {
                delete req.contentType; // fix for cross domain requests.
            }

            args.xhr = $.ajax(req);
        });

        queue = cannot_run;

        queue_locked = false;
        if (pre_queue.length) {
            var len = pre_queue.length;
            for (var i = 0; i < len; i++) {
                queue.push(pre_queue.shift());
            }
        }

        cannot_run = [];
    };

    var status = function(args, what) {
        args.status && setTimeout(function() {
            args._status = what;
            args.status(what, args);
        }, 0)
    };

    /*--------------------------------------------------------------------*/
    /* Support for batch request */

    var _batchQueue = function(request) {

        var success = request.success || ignore;
        var error = request.error || ignore;
        var img = request.img;
        var status = request.status || ignore;
        var errorLog = request.errorLog || ignore;

        function wait(json) {

            var retry = json.retry || 5;

            status(json.status);

            setTimeout(function() {

                _ajaxQueue({
                    url: json.callback,
                    json: json,
                    priority: 'vip',
                    error: error,
                    errorLog: errorLog,
                    status: status,
                    img: img,
                    success: function(data) {
                        status(data.status);
                        if (data && data.deferred) {
                            wait(data);
                        } else {
                            if (data.status === 'complete') {
                                success(data);
                            } else {
                                error(data);
                            }
                        }
                    }
                });

            }, retry * 1000);
        }

        var req = $.extend({}, request, {
            success: function(result) {
                if (result && result.deferred) {
                    wait(result);
                } else {
                    success(result);
                }
            }
        });

        return _ajaxQueue(req);
    };

    var Ajaxqueue = _batchQueue;

    Ajaxqueue.options = {
        type: 'POST',
        dataType: 'json',
        timeout: 60 * 1000, // milliseconds,
        host: '',           // for cross-domain requests. For performance reasons do not set unless needed
        noStreaming: false  // Use to bypass streaming from internal hosts
    };

    $.extend(Ajaxqueue, {
        info: function() {
            return {
                queue: queue,
                active: active,
                outstanding: outstanding
            };
        },

        start: function(proc) {
            queueStart = proc ? proc : ignore;
        },

        stop: function(proc) {
            queueStop = proc ? proc : ignore;
        },

        cancelRequest: function(args) {
            cancel('by request', args);
        },

        cancelAll: function() {

            $.each(queue, function(i, args) {
                cancel('Cancel by cancel all', args);
            });

            $.each(active, function(i, args) {
                cancel('Cancel by cancel all', args);
            });

            setTimeout(check_queue, 0);
        },

        /*
         *  tag can be string or regex
         */
        cancelByTag: function(tag) {
            if(__DEV__ && !tag) return console.warn('cancelByTag: ignoring empty tag');

            if(typeof tag !== 'object') {
                tag = new RegExp(tag);
            }

            $.each(queue, function(i, args) {
                if(tag.test(args.tag)) {
                    cancel('Cancel by tag ' + tag, args);
                }
            });

            $.each(active, function(i, args) {
                if(tag.test(args.tag)) {
                    cancel('Cancel by tag ' + tag, args);
                }
            });

            setTimeout(check_queue, 0);
        },

        add_callback: function(fn) {
            fn && handlers.push(fn);
        }
    });

    return Ajaxqueue;
});
