/*
 *   The Animator maintains a list of frames and can iterate over them.
 *   The list is the intersection of the frames of each subscriber.
 *   Animation is started automatically upon subscription.
 *
 *   Use the $.animationAdd and $.animationRemove functions to subscribe and unsubscribe.
 *   Use the "animation-frame-set" message to control the position.
 */
define(['config', 'jquery', 'bus', 'settings', 'utils'/* FIXME core file should not have non-core dependency */], function(ECMWF, $, Bus, Settings) {
    var period = 500;
    var speed_factor = 0;

    var animationQueue = [];
    var filters = [];
    var animationTimer = null;
    var animationLength = 0;
    var frame = 0;

    var waitingTimer = new ECMWF.timer(3500);

    var bus = Bus();


    var step_increments = [24, 12, 6, 3, 1];
    var increment = 0;


    // This is used to synchronise animation between browser windows
    // Only tested on FF 3.6


    // IE wants the handler on "body", webkit on "document", FF does not care
    // Unfortunately, we don't know what changed
    //

    // Does not seem to work on Chrome...

    /*
    var lastFrameKey;

    $(document).ready(function() {
        //var s = (ECMWF.browser.webkit) ? $("body") : $(document);
        var s = $(document);

        if(ECMWF.browser.webkit) {
            s = $(window);
        }

        s.on("storage", function() {

            var key = window.localStorage.getItem("lastFrameKey");

            if(key != lastFrameKey) { //only true for slave windows.
                console.log("lastFrameKey: " , key);

                if(animationTimer) {
                    $.each(animationFrames, function(i,frame) {
                        if(frame.key === key) {
                            console.log("Sync frame :", lastFrameKey ," -> ", key, " " , frame , " -> " , i);
                            frame = i;
                            show_frame(frame);
                        }
                    });
                }

            }

        });
    });
    */

    function set_speed(factor) {
        speed_factor = factor;
        period = 439 * Math.pow(1.3, speed_factor);
        // $.logMessage('Animation speed ' + (1000 / period).toPrecision(2) + ' frames/sec');
    }

    bus
        /*
        .listen("animation-frame-set", function(_, msg) {
            console.warn(">>>>>>>>>>>>>>>>> NEVER GET HERE ?");
            clear_timer();
            frame = msg.frame % animationFrames.length;
            animation_step();
        })
        */
        .listen('animation-speed-down', function(_, msg) {
            set_speed(Math.min(11, ++speed_factor));
        })
        .listen('animation-speed-up', function(_, msg) {
            set_speed(Math.max(-8, --speed_factor));
        })
        .listen('animation-speed-set', function(_, _speed_factor) {
            set_speed(_speed_factor);
        })
        .listen('animation-set', function(_, change) {
            if (change.filter) {
                update_animation();
            }
        });

    function frame_ready(frame, callback) {

        var ready = true;

        $.each(animationQueue, function(_, o) {
            try {
                if (!o.animator.frameReady(o.frames[frame]))
                {
                    ready = false;
                }
            }
            catch (e)
            {
                console.warn(e);
            }
        });

        if (ready) {
            callback();
        }
        else {
            waitingTimer.start(callback);
        }
    }

    function show_frame(frame) {

        $.each(animationQueue, function(_, o) {
            try {
                o.animator.showFrame(o.frames[frame]);
            }
            catch (e)
            {
                console.warn(e);
            }
        });
    }

            /*
            if(window.localStorage) {
                if(lastFrameKey != frame.key){
                    lastFrameKey = frame.key;
                    window.localStorage.setItem("lastFrameKey", lastFrameKey);
                }
            }
            */

    var step_timer;
    function animation_step() {

        frame_ready(frame, function() {
            //console.log("ANIMATION ", frame, '/', animationLength, animationQueue.length);
            show_frame(frame);

            // Check next frame

            if (false && step_increments[increment]) {
                for (;;) {
                    frame = (++frame) % animationLength;
                    /*
                    var x = (animator.frameList[frame].valid_time.getTime() / 3600/1000) % step_increments[increment];

                    if(frame === 0) {
                        increment++;
                    }
                    if(+x === 0) break;
                    */
                }
            }
            else
            {
                frame = (++frame) % animationLength;
            }

            step_timer || (step_timer = setTimeout(function() {
                step_timer = null;
                animationTimer && (animationTimer = timer.requestAnimFrame(animation_step, period));
            }, period));
        });
    }

    var animatables = {
        time: {
            sort: function(a, b) {
                return a.valid_time.getTime() - b.valid_time.getTime();
            },
            make_key: function(frame) {
                return frame.valid_time.yyyymmddHHMM();
            }
        },
        parameter: {
            sort: function(a, b) {
                return -1; // already sorted
            },
            make_key: function(frame) {
                return frame + ''; // do they need to be strings?
            }
        }
    }
    var animatable = animatables.time;

    function makeFrameList()
    {
            var common_frames = new KeyedList();
            var preferred = { animationFrame: 1};

            var clients = $.merge($.merge([], filters), animationQueue);

            animationLength = frame = 0;

            $.each(clients, function(_, o) {
                if (!o.animator.getFrames) return;

                var p = $.isFunction(o.animator.getPreferredFrame) && o.animator.getPreferredFrame();
                o.frames = o.animator.getFrames().sort(animatable.sort);
                //console.warn($.map(o.frames, function(x) { return x.key; }));
                //console.warn($.map(o.frames, function(x) { return x.valid_time; }));

                if (p) {
                    o.preferred = animatable.make_key(p);
                    preferred[o.preferred] = preferred[o.preferred] || 2;
                    preferred[o.preferred]++;
                }

                common_frames.intersect(o.frames);
            });

            // Apply reject filters
            $.each(filters, function(_, f) {
                common_frames.filter(f.animator);
            });

            animationLength = common_frames.length();

            // Update each client's frame list for common valid time intersection.
            $.each(clients, function(_, o) {
                o.frames = $.grep(o.frames, function(f) { return common_frames.list[animatable.make_key(f)]; });
            });

            // Select preferred frame to restart from
            frame = 0;
            var best = 0;
            var choice;

            $.each(preferred, function(valid, score) {
                if (score > best) {
                    best = score;
                    choice = valid;
                }
            });

            if (choice) {
                $.each(clients, function(_, o) {
                    $.each(o.frames, function(i, f) {
                       if (animatable.make_key(f) === choice) {
                            frame = i;
                       }
                    });
                });
            }

            // Tell objects
            $.each(clients, function(_, o) {
                if (o.animator.frameList) {
                    o.animator.frameList(
                        (o.frames && o.frames.length)
                            ? o.frames
                            : common_frames.to_array(),
                        frame);
                }
            });

            increment = 0;

            //console.log("ANIMATION: ", frame, animationLength);
    }

    /*
    function clear_timer() {
        animationTimer = null;
    }
    */

    function update_animation() {

        bus.send('animation-stop');

        if (animationTimer) {
            animationTimer = timer.cancel(animationTimer);
        }

        waitingTimer.clear();

        makeFrameList();

        if (animationQueue.length)
        {
            if (animationLength) {
                bus.send('animation-start', { current: frame, count: animationLength});
                animationTimer = setTimeout(animation_step, 500); //FIXME 500 ?!
            }
            else {
                alert('Cannot establish a list of common valid time for the selected products');
            }
        }
    }

    // TODO this appears to be unneccesary. It can be done automatically in $.animationAdd
    $.animationAddFilter = function(o) {
        filters.push({ animator: o, frames: [], preferred: 0});
        update_animation();
    };

    $.animationRemoveFilter = function(o) {
        filters = $.grep(filters, function(x) { return x.animator !== o; });
        update_animation();
    };

    $.animationUpdateFilter = function(o) {
        update_animation();
    };

    $.animationAdd = function(o, animatable_type) {

        console.log('ANIMATION ADD ');
        animatable = animatables[animatable_type || 'time'];

        for (var i in animationQueue) {
            if (animationQueue[i].animator === o) {
                console.log('ANIMATION ADD - DUPLICATED ', o);
                return;
            }
        }

        animationQueue.push({animator: o, frames: [], preferred: 0});

        update_animation();
    }

    $.animationRemove = function(o) {

        console.log('ANIMATION REMOVE');

        var before = animationQueue.length;
        animationQueue = $.grep(animationQueue, function(x) { return x.animator !== o; });

        // Nothing was removed
        if (before === animationQueue.length)
            return;

        update_animation();
    }

    Settings.data.then(function() {
        Settings.get_pref('animation_speed') && set_speed(Settings.get('animation_speed'));
    })

    var timer = (function setup_timer() {
        //allows browser to stop the animation when application is backgrounded. If RequestAnimationFrame available, max fps is 60, otherwise ~250

        function wait_for_timeout(callback, element) {
            return setTimeout(callback, 1);
        }

        function wait_for_frame(callback, element) {
            __anim(callback);
            return -1; //return non-zero to flag to the Animator that the animation is active.
        }

        var __anim;
        var requestAnimFrame = (function() {
            return (__anim = window.requestAnimationFrame)
                ? wait_for_frame
                : wait_for_timeout;
        })();

        var __cancel;
        var cancelRequestAnimFrame = (__cancel = window.cancelAnimationFrame)
            ? function(id) { __cancel(id); }
            : function _clear_timeout(id) {
                clearTimeout(id);
            };
        return {requestAnimFrame: requestAnimFrame, cancel: cancelRequestAnimFrame};
    })();

    var KeyedList = function(list) {
        this.list = list || {};
        this.length = function() {
            return ECMWF.n_properties(this.list);
        };
        this.intersect = function(other) {
            var kl = this;
            var c = {};
            var empty = !this.length(); // assume empty list means first iteration.
            $.each(other, function(_, frame) {
                var key = animatable.make_key(frame);
                if (empty || kl.list[key]) {
                    c[key] = frame;
                }
            });
            this.list = c;
        };
        this.filter = function(animator) {
            if (animator && animator.filter) {
                var filtered = {};
                $.each(this.list, function(k, t) {
                    animator.filter.call(animator, t) && (filtered[k] = t);
                });
                this.list = filtered;
            }
            return this.list;
        };
        this.to_array = function() {
            return $.map(this.list, function(i) { return i; });
        };
    };


    $.AnimatorClient = function(animatable_type) {
        this.showFrame = function(time)        { /* ignore */ };
        this.frameReady = function(time)       { return true; };
        this.frameList = function(list, first) { /* ignore */ }; // notifies clients of a new combined framelist once generation is complete.
        this.getFrames = function()            { return [];   }; // as well as asking clients to provide a framelist for the animator, it also acts as change notification to the client *before* the combined framelist is generated.
        this.getPreferredFrame = function()    { return null; };

        this._on_start = $.noop;
        this._on_stop = $.noop;
        this._on_pause = $.noop;
        this._on_resume = $.noop;

        this.toggle = function() {
            return (!this._started)
                ? (this.start(), true)
                : (this._paused
                    ? (this.resume(), true)
                    : (this.pause(), false))
        }

        this.start = function() {
            var self = this;
            if (self._started) {
                if (self._paused) {
                    self.resume();
                }
                return true;
            }

            self._active = true;
            self._started = true;
            self._paused = false;

            self._on_start(function(){
                $.animationAdd(self, animatable_type);
            });
        }

        this.stop = function() {
            this._started = false;
            this._on_stop();
        }

        this.pause = function() {
            if(this._started && !this._paused && !this.__pausing){
                this.__pausing = true; // recursion protection
                $.animationRemove(this);
                this._on_pause();
                this._paused = true;
                this.__pausing = false;
            }
        }

        this.resume = function() {
            var self = this;
            self._active = true;

            if (!self._started) {
                self.start();
            }

            if(self._paused) {
                self._paused = false;
                $.animationAdd(self, animatable_type);
            }
            self._on_resume();
        }

        this._destroy = function() {
            var self = this;

            self.pause();

            self._started = false;
            self._paused = false;
            self._active = false;
            self._complete = false;

            $.animationRemove(self);
        }
    }
});
