define(['config'], function(ECMWF){
    function uaMatch(ua) {
        ua = ua.toLowerCase();

        var match = /(chrome)[ \/]([\w.]+)/.exec(ua) || /(webkit)[ \/]([\w.]+)/.exec(ua) || /(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) || /(msie) ([\w.]+)/.exec(ua) || ua.indexOf('compatible') < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) || [];

        return { browser: match[1] || '', version: match[2] || '0' };
    }

    var matched, browser;
    window.ECMWF || (window.ECMWF = {});
    ECMWF.browser || (function() {

        matched = uaMatch(navigator.userAgent);
        browser = {};

        if (matched.browser) {
            browser[matched.browser] = true;
            browser.version = matched.version;
        }

        // Chrome is Webkit, but Webkit is also Safari.
        if (browser.chrome) {
            browser.webkit = true;
        } else if (browser.webkit) {
            browser.safari = true;
        }

        window.navigator.userAgent = window.navigator.userAgent || 'node'

        ECMWF.browser = browser;
    })();

    ECMWF.browser.supports_svg = function() {
        return !!document.createElementNS && !!document.createElementNS('http://www.w3.org/2000/svg', 'svg').createSVGRect;
    };

    ECMWF.browser.supports_svg_img = function(is_supported_callback) {

        if (typeof ECMWF.browser._have_svg_img_support !== 'undefined') {
            console.log('not undefined');
            is_supported_callback(ECMWF.browser._have_svg_img_support);
            return;
        }

        var timer = setTimeout(function() {
            is_supported_callback(ECMWF.browser._have_svg_img_support);
        }, 4000);

        var testImg = 'data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNzUiIGhlaWdodD0iMjc1Ij48L3N2Zz4%3D';
        var img = document.createElement('img');
        img.setAttribute('src', testImg);
        img.addEventListener('load', function() { clearTimeout(timer); ECMWF.browser._have_svg_img_support = true; is_supported_callback(true); });
    };

    ECMWF.browser.engine = function() {
        //code from https://github.com/faisalman/ua-parser-js/blob/master/src/ua-parser.js, license MIT/GPL

        //temp
        var ua = ((window && window.navigator && window.navigator.userAgent) ? window.navigator.userAgent : '');

        var engine_regex = [
            [
                /(presto)\/([\w\.]+)/i,                                             // Presto
                /(webkit|trident|netfront|netsurf|amaya|lynx|w3m)\/([\w\.]+)/i,     // WebKit/Trident/NetFront/NetSurf/Amaya/Lynx/w3m
                /(khtml|tasman|links)[\/\s]\(?([\w\.]+)/i,                          // KHTML/Tasman/Links
                /(icab)[\/\s]([23]\.[\d\.]+)/i                                      // iCab
            ],
            ['name', 'version'],
            [/rv\:([\w\.]+).*(gecko)/i],                                           // Gecko
            ['version', 'name']
        ];

        return rgx.apply(this, engine_regex);

        function rgx() {

            // loop through all regexes maps
            for (var result, i = 0, j, k, p, q, matches, match, args = arguments; i < args.length; i += 2) {

                var regex = args[i],       // even sequence (0,2,4,..)
                    props = args[i + 1];   // odd sequence (1,3,5,..)

                // construct object barebones
                if (typeof(result) === 'undefined') {
                    result = {};
                    for (p in props) {
                        q = props[p];
                        if (typeof(q) === 'object') {
                            result[q[0]] = undefined;
                        } else {
                            result[q] = undefined;
                        }
                    }
                }

                // try matching uastring with regexes
                for (j = k = 0; j < regex.length; j++) {
                    matches = regex[j].exec(ua);
                    if (!!matches) {
                        for (p in props) {
                            match = matches[++k];
                            q = props[p];
                            // check if given property is actually array
                            if (typeof(q) === 'object' && q.length > 0) {
                                if (q.length == 2) {
                                    if (typeof(q[1]) == 'function') {
                                        // assign modified match
                                        result[q[0]] = q[1].call(this, match);
                                    } else {
                                        // assign given value, ignore regex match
                                        result[q[0]] = q[1];
                                    }
                                } else if (q.length == 3) {
                                    // check whether function or regex
                                    if (typeof(q[1]) === 'function' && !(q[1].exec && q[1].test)) {
                                        // call function (usually string mapper)
                                        result[q[0]] = match ? q[1].call(this, match, q[2]) : undefined;
                                    } else {
                                        // sanitize match using given regex
                                        result[q[0]] = match ? match.replace(q[1], q[2]) : undefined;
                                    }
                                } else if (q.length == 4) {
                                        result[q[0]] = match ? q[3].call(this, match.replace(q[1], q[2])) : undefined;
                                }
                            } else {
                                result[q] = match ? match : undefined;
                            }
                        }
                        break;
                    }
                }

                if (!!matches) break; // break the loop immediately if match found
            }
            return result;
        }
    };

    if(ECMWF.browser.msie){
        (!document.documentMode) && (ECMWF.browser.msie7 = true);
        ((document.documentMode || 100) < 9) && (ECMWF.browser.msie8 = true);
        ((document.documentMode || 100) === 9) && (ECMWF.browser.msie9 = true);
        console.log('documentMode=' + document.documentMode + ' msie8=' + ECMWF.browser.msie8 + ' msie9=' + ECMWF.browser.msie9);
    }

    return ECMWF.browser;
});

