define(['jquery', 'utils'], function($, Utils) {

var caches = new (function CacheStore() {})(); //give the cache its own type name for profiling.

function cleanup() {
	var now = new Date();
	var empty = [];

	for (var name in caches) {
		var del = [];
		var n = 0;
		for (var key in caches[name]) {
			var k = caches[name][key];
			n++;

			if (k.timeout && k.timeout !== 'forever') {
				var age = (now.getTime() - k.inserted.getTime()) / 1000;
				if (age > k.timeout) {
					del.push(key);
				}
			}

			if (del.length == n) {
				empty.push(name);
			}

			for (var i in del) {
				//console.warn("CACHE - cleanup : " + name + " delete " + del[i] );
				delete caches[name][del[i]];
			}
		}

		for (i in empty) {
			//console.warn("CACHE - cleanup : delete " + empty[i] );
			delete caches[empty[i]];
		}
	}
}

function debug_get_object_size(obj) {
	return (typeof obj === 'undefined' || $.isEmptyObject(obj)) ? 0 : JSON.stringify(obj).length;
}

setInterval(cleanup, 1000 * 60 * 5);

function hash_key(key) {
	if (typeof key === 'string') {
		return key;
	}
	return $.makeCacheKey(key);
}

__DEV__ && $(document).on('debug', function() {
	console.log('Caches: ' + Utils.prettyJSON(caches));
});

return {
	get: function(name, key) {

		if (!caches[name])
			caches[name] = {};

		key = hash_key(key);

		var k = caches[name][key];
		if (!k) {
			//console.warn("CACHING(" + name + "," + key + ") -> MISS");
			return undefined;
		}

		if (k.timeout && k.timeout !== 'forever') {
			var now = new Date();
			var age = (now.getTime() - k.inserted.getTime()) / 1000;
			if (age > k.timeout) {
				//console.warn("CACHING(" + name + "," + key + ") -> OBSOLETE");
				delete caches[name][key];
				return undefined;
			}
		}

		//console.warn("CACHING(" + name + "," + key + ") -> HIT");
		//
		// Return a deep copy, so that user can modify original
		//
		//console.log("cacheFetch " + key + ": " + JSON.stringify(k.value));

		return $.cloneObject(k.value);
	},

	put: function(name, key, value, timeout) {

		if (!caches[name])
			caches[name] = {};

		key = hash_key(key);
		//console.log("cacheStore: " + key + ": " + JSON.stringify(value));
		//console.log('cacheStore: ' + (caches['ajax-queue'] ? ECMWF.n_properties(caches['ajax-queue']) : 0) + ' (' + (Math.round(debug_get_object_size(caches['ajax-queue'])/1000)) + 'k) key=' + key.substring(0, 20));

		// Store a deep copy, so that user can modify original
		//
		//console.warn("CACHING(" + name + "," + key + ") -> STORE");

		caches[name][key] = {
			value: $.cloneObject(value),
			timeout: timeout,
			inserted: new Date()
		};
	},

	clear: function() {
		var size = debug_get_object_size(caches);
		caches = {};
		return size;
	}
}

});
