/** @license
 *  ECMWF Clone
 *  (C) Copyright ECMWF http://www.ecmwf.int
 *
 *  This software is licensed under the terms of the Apache Licence Version 2.0
 *  which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 *  In applying this licence, ECMWF does not waive the privileges and immunities
 *  granted to it by virtue of its status as an intergovernmental organisation nor
 *  does it submit to any jurisdiction.
 *
 */
define(['jquery', 'time'], function($, Time) {

 function clone(obj, check_loops, depth) {
    /*
     *  check_loops - an array of the objects already tested.
     *  depth       - maximum number of allowed recursion levels.
     */
    if (obj === null) return null;
    if (obj === undefined) return undefined;
    if (depth === 0) { return '<CLONE: max depth reached.>'; }

    if (obj instanceof Date) {
        return new Date(obj);
    }

    if (obj instanceof Time) {
        return new Time(obj);
    }

    if (!$.isPlainObject(obj) && !Array.isArray(obj)) {
        var t = typeof obj;
        if (t === 'number' || t === 'string' || t === 'boolean') { return obj; }
        if ($.isFunction(obj)) { return undefined; }
        return undefined;
    }

    for (var i in check_loops) {
        if (obj === check_loops[i]) {
            var s = '<CLONE: loop detected: ' + obj + '>';
            console.warn(s);
            return s;
        }
    }

    if (Array.isArray(obj)) {

        var result = [];
        check_loops.push(obj);
        for (i in obj) {
            result[i] = clone(obj[i], check_loops, depth - 1);
        }
        check_loops.pop();
        return result;
    }

    if (isPlainObject(obj)) {
        result = {};
        check_loops.push(obj);
        for (i in obj) {
            var c = clone(obj[i], check_loops, depth - 1);
            if (c !== undefined) {
                result[i] = c;
            }
        }
        check_loops.pop();
        return result;
    }

    //throw "clone: ooops!";
    //return obj;
    return '[un-clonable object]';
 }

 $.cloneObject = function(object) { return clone(object, [], 50); };

 $.cleanupObject = function(object) { return clone(object, [], 50); };

 $.makeCacheKey = function(object) {

     var result = [];

     function process(o) {
         if (Array.isArray(o)) {
             result.push('[');
             $.each(o, function(_, v) { process(v); });
             result.push(']');
         }
         else if ($.isPlainObject(o)) {
             var keys = [];
             $.each(o, function(k, _) { keys.push(k); });

             keys = keys.sort();

             result.push('{');
             $.each(keys, function(_, k) {
                     result.push(k);
                     process(o[k]);
             });
             result.push('}');
         }
         else {
             result.push(o);
         }
     }

     process($.cleanupObject(object));

     return result.join(':');

 };

 function isPlainObject(obj) {
    if (obj.readyState) return false;
    return $.isPlainObject(obj);
 }

 function deepMerge(into, obj) {

    if (into === undefined) {
        return obj;
    }

    if (obj instanceof Object) {
        for (var i in obj) {
            into[i] = deepMerge(into[i], obj[i]);
        }
    }
    else {
        return obj;
    }

    return into;
 }

 $.deepMerge = deepMerge;

	return {
		serialise: function(object) {
			return JSON.stringify($.cleanupObject(object));
		}
	}
});
