
/** @license
 *  ECMWF Framework
 *  Copyright ECMWF http://www.ecmwf.int
 *  This software is licensed under the terms of the Apache Licence Version 2.0
 *  which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 *  In applying this licence, ECMWF does not waive the privileges and immunities
 *  granted to it by virtue of its status as an intergovernmental organisation nor
 *  does it submit to any jurisdiction.
 */

/*
 * $.framework.onReady(callback)
 *    Use this method to be notified when the framework is loaded.
 *    The callback will never be called before the dom has loaded.
 *
 * $.frameworkRegister
 *    Use this method to add functions which must be completed before
 *    the framework can be considered ready.
 */

define(['config', 'jquery', 'async'], function(ECMWF, $, async) {

    var framework_loaded = false;
    var tasks = [];
    var callbacks = [];
    var has_run = false;

    $.frameworkRegister = function(proc, title) {
        framework_loaded && (proc($.noop), true) ||
        tasks.push(
            function(next) {
                proc(function() {
                    //console.log("Ready: ", title);
                    next();
                });
            }
        );
    };

    ECMWF.timestamp = new Date();

    $.framework = {
        _run: function() {
            has_run = true;
            var _tasks;

            async.whilst(
                function _test(){
                    _tasks = tasks; // allow new tasks to be added before existing tasks have completed.
                    tasks = [];
                    return !!_tasks.length; // returning false will end the whilst loop
                },
                function execute_tasks(next_task_list){
                    async.parallel(_tasks, function() {
                        _tasks = null;

                        next_task_list();
                    });
               },
               function done() {
                   $.each(callbacks, function() { this(); });
                   callbacks = [];
                   framework_loaded = true;

                   /*
                   var now = new Date();
                   console.log("FRAMEWORK READY", (now - ECMWF.timestamp) / 1000);
                   */
               }
            );
        },
        onReady: function(callback) {
            if(framework_loaded) return callback();

            callbacks.push(callback);
            has_run || $.framework._run();
        }
    };

    $.frontendReady = $.framework.onReady;

    $.frameworkRegister($(document).ready, 'Waiting for DOM');

});
