/*jslint white: false */
/*global console: true, jQuery: true, setInterval: true */

/** @license
 *  ECMWF Log
 *  (C) Copyright ECMWF http://www.ecmwf.int
 *
 *  This software is licensed under the terms of the Apache Licence Version 2.0
 *  which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 *  In applying this licence, ECMWF does not waive the privileges and immunities
 *  granted to it by virtue of its status as an intergovernmental organisation nor
 *  does it submit to any jurisdiction.
 *
 */

define(['jquery', 'bus', 'utils'], function($, Bus, Utils) {

 var bus = Bus();

 function formatError(msg) {
    if(!msg) return;

    if (typeof msg === 'string')
    {
        // special case, should be in the backend
        msg = msg.replace(/Error in http.*\/(\w+):/gi, '$1:');
        return msg;
    }

    if (msg instanceof Array)
    {
        return formatError(msg[0]);
    }

    if (msg.children && msg.children.length)
    {
       var m;
       $.each(msg.children, function(_, child) {
           m = formatError(child);
       });

       return m;
    }

    if (msg.message)
    {
        if (msg.service) {
            return formatError(msg.service + ': ' + msg.message);
        }
        return formatError(msg.message);
    }

    if (msg.error)
    {
        return formatError(msg.error);
    }

    if (msg.exception)
    {
        return formatError(msg.exception);
    }

    if (msg.status)
    {
        return formatError(msg.status);
    }

    return Utils.prettyJSON(msg);
 }

 $.formatError = formatError;

 var errors = [];

 $.lastErrors = function() {
     return errors;
 };

 $.logInfo = function(msg, context) {
    var t = formatError(msg);
    console.log(t);
    var args = { title: t, message: msg};
    bus.send('log-info', args);
    /* $.notify && $.notify($.extend({message:t})); */
 };

 $.logDebug = function(msg, context) {
    var t = formatError(msg);
    console.log(t);
    var args = { title: t, message: msg};
    bus.send('log-debug', args);
 };

 $.logMessage = function(msg, context, o) {
    var t = formatError(msg);
    console.log(t);
    var args = { title: t, message: msg};
    bus.send('log-message', args);
    /* $.notify && $.notify($.extend(o||{},{message:t})); */
 };

 $.logWarning = function(msg, context) {
    var t = formatError(msg);
    console.warn(t);
    var args = { title: t, message: msg};
    bus.send('log-warning', args);
    /* $.notify && $.notify({message:t}); */
 };

 $.logError = function(msg, context) {
    var t = formatError(msg);
    if (context) {
        t = context + ': ' + t;
    }
    console.warn('ERROR: ' + t);
    var args = { title: t, message: msg};
    bus.send('log-error', args);
    errors.push($.extend(args, {when: new Date()}));
    while (errors.length > 100) {
        errors.shift();
    }
    /* $.notify && $.notify({message:t, mode: 'error'}); */
 };

});
