define(['jquery'], function($) {

    function julian_to_date(jdate) {
        var x, y, d, m, e, day, month, year;

        x = 4 * jdate - 6884477;
        y = Math.floor(x / 146097) * 100;
        e = x % 146097;
        d = Math.floor(e / 4);

        x = 4 * d + 3;
        y = Math.floor(x / 1461) + y;
        e = x % 1461;
        d = Math.floor(e / 4) + 1;

        x = 5 * d - 3;
        m = Math.floor(x / 153) + 1;
        e = x % 153;
        d = Math.floor(e / 5) + 1;

        if (m < 11)
            month = m + 2;
        else
            month = m - 10;

        day = d;
        year = y + Math.floor(m / 11);

        return [year, month, day];
    }

    function date_to_julian(year, month, day) {
        var m1, y1, a, b, c, d, j1, ddate;

        if (month === undefined) {
            ddate = year;
            year = Math.floor(ddate / 10000);
            ddate %= 10000;
            month = Math.floor(ddate / 100);
            ddate %= 100;
            day = ddate;
        }


        if (month > 2) {
            m1 = month - 3;
            y1 = year;
        } else {
            m1 = month + 9;
            y1 = year - 1;
        }
        a = Math.floor(146097 * Math.floor(y1 / 100) / 4);
        d = y1 % 100;
        b = Math.floor(1461 * d / 4);
        c = Math.floor((153 * m1 + 2) / 5) + day + 1721119;
        j1 = a + b + c;

        return j1;
    }

    var ref = date_to_julian(19700101);

    function UTCDate(y, m, d, H, M, S) {

        this.year = y || 0;
        this.month = m || 0;
        this.day = d || 0;
        this.hour = H || 0;
        this.minute = M || 0;
        this.second = S || 0;

        var j = date_to_julian(this.year * 10000 + (this.month + 1) * 100 + this.day) - ref;

        this.value = (((j * 24 + this.hour) * 60 + this.minute) * 60 + this.second) * 1000;

        return this;
    }

    UTCDate.prototype.format = function(mask, utc) {

        /*
        if(utc === undefined) {
            utc = true;
        }
        */
        // TODO: Review me.... need to adjust for TZ
        //FIXME: Date is falling back to 1969123123 for the 1970010100) FIREFOX bug

        if (this.year <= 1970 ) {
            var f = this.year + "/" + (this.month+1) + "/" + this.day + "  " + this.hour + ":"+ this.minute+":"+ this.second +" UTC";

            return new Date(f).format(mask, true);
        }
        var d = new Date(this.year, this.month, this.day, this.hour, this.minute, this.second).format(mask, utc);
        return d;
    };

    UTCDate.prototype.yyyymmddHHMM = function() {
        var ar = [this.year, null, null, null, null];

        var month = this.month + 1;
        ar[1] = month < 10 ? '0' + month : month;
        var day = this.day;
        ar[2] = day < 10 ? '0' + day : day;
        var hours = this.hour;
        ar[3] = hours < 10 ? '0' + hours : hours;
        var mins = this.minute;
        ar[4] = mins < 10 ? '0' + mins : mins;

        return ar.join('');
    };

    UTCDate.prototype.addSeconds = function(s) {
        var val = this.value / 1000 + s;
        var days = Math.floor(val / (24 * 60 * 60));
        var secs = val % (24 * 60 * 60);

        var a = julian_to_date(days + ref);
        return new UTCDate(a[0], a[1] - 1, a[2], Math.floor(secs / (60 * 60)), Math.floor((secs % (60 * 60)) / 60), secs % 60);

    };

    UTCDate.prototype.getTime = function() {
        return this.value;
    };
    UTCDate.prototype.valueOf = function() {
        return this.value;
    };
    UTCDate.prototype.equals = function() {
        throw new Error('UTCDate.equals() is not defined');
    };

    UTCDate.prototype.getFullYear = function() {
        return this.year;
    };
    UTCDate.prototype.getMonth = function() {
        return this.month;
    };
    UTCDate.prototype.getHours = function() {
        return this.hour;
    };
    UTCDate.prototype.getMinutes = function() {
        return this.minute;
    };
    UTCDate.prototype.getSeconds = function() {
        return this.second;
    };

    UTCDate.prototype.toString = function() {
        var ar1 = [this.year, null, null];

        var month = this.month + 1;
        ar1[1] = month < 10 ? '0' + month : month;
        var day = this.day;
        ar1[2] = day < 10 ? '0' + day : day;


        var ar2 = [null, null, null];
        var hours = this.hour;
        ar2[0] = hours < 10 ? '0' + hours : hours;

        var mins = this.minute;
        ar2[1] = mins < 10 ? '0' + mins : mins;

        var sec = this.second;
        ar2[2] = sec < 10 ? '0' + sec : sec;

        return [ar1.join('-'), 'T', ar2.join(':'), 'Z'].join('');
    };

    UTCDate.prototype.toJSON = UTCDate.prototype.toString;

    return UTCDate;
});

