define(['config', 'jquery', 'user_info', 'dashboard/model', 'bus', 'dashboard/widgets/widget', 'app-menu', 'jquery-ui/widget', 'messages', 'utils', 'ajax-indicator', 'dashboard', 'shortcuts', 'theme-ecmwf-2017/bootstrap/index'], function(ECMWF, $, User, Tabs, Bus, DashboardWidget, AppMenu) {

var wrapper, bus = Bus();
var old_template;

ECMWF.timestamp = new Date()

var Application = {
	ready: $.Deferred(function(df) {
		setTimeout(df.resolve, 10000);
		bus.listen_once('dashboard-set-loaded', df.resolve);
	})
}

$(function() {
	$('#ajax-loader').ajaxIndicator();
	$('#messages').messages();

	if(__HAVE_APPMENU__)
		$('body').addClass('template-2017');
	else
		old_template = true;

	Application.ready.then(function(){
		ECMWF.wheelspeed = ECMWF.browser.mozilla ? 10 : 1;

		$('#main > .loader').fadeOut(function(){
			$('#main').loader('destroy');
		});
		User.info.then(function(user) {
			if(__HAVE_APPMENU__)
				user.user_name && make_menus_2017();
			else
				user.user_name && make_menus_old();
		})
	});
});

User.info.then(function(user) {

	user.user_name && Tabs.load.then(function(dashboard_set){
		$(function() {
			var main = $(old_template ? '#main' : '#main > .container-fluid'); // TODO will not work if not using fluid layout

			wrapper = main
				.div('#dashboard-set')
					.dashboardSet({dashboard_set: dashboard_set, focused: ECMWF.request.widget, wrapper: '#main'});
					//.selectable({filter: 'li', cancel: '.widget-content', stop: function(e, ui){}});

			User.is_forecaster() && main.add('#header, #footer').css({maxWidth: 10000})

			update_layout_menu();

			bus
				.listen('dashboard-set-changed', function(_, set){
					Tabs.save_dashboard_set(set, function(){
						console.log('set saved.');
					});
					update_layout_menu();
				})
				.listen('dashboard-changed', function(_, dashboard){
					dashboard.get_state_async().then(function(get_state) {
						Tabs.save_dashboard(get_state(), function(){
							console.log('tab saved.');
						});
					});
				})
				.listen('dashboard-animation-toggle', function(){
					//$.animationAdd();
				});

			$(document)
				.shortcuts({shortcuts: [
					{
						key    : $.ui.keyCode.SPACE,
						signal : 'dashboard-animation-toggle'
					},
					{
						key    : 70, // 'f'
						signal : 'fullscreen'
					}
				]});

			require(['forecaster/reporter'], function(Reporter) { Reporter(); });
		})
	})

	old_template && bus
		.listen('menu-layout-radio-change', function(_, set){
			wrapper.dashboardSet('reflow', set.value.substring(13, 14));
		})
})

function update_layout_menu(){
	if(wrapper){
		User.info.then(function() {
			$('#menu-panes .layout input').prop('checked', false);

			$('#menu-panes .layout .menu-list li:eq(' + (wrapper.dashboardSet('n_columns') - 1) + ') input')
				.prop('checked', true);
		})
	}
}

function make_menus_old(){

	if($('#menu-bar .menu-item').length) return;

	User.info.then(function(user) {
		var layout_items = [];
		for(var i=1;i<=6;i++){
			layout_items.push({'class': 'menu-radio', name: 'radio-' + i, title: i +' Column' + (i - 1 ? 's' : '')});
		}

		var exclude = User.is_forecaster() ? ['basic_chart'] : ['chart', 'meteogram_10_days', 'meteogram_15_days', 'meteogram_efi', 'meteogram_extended']

		var add_widget_items = $.map(DashboardWidget.registry(), function(widget){
			if(!widget.is_private && !~$.inArray(widget.kind, exclude)){
				return {name: widget.kind, title: widget.title, selected: function(){
					wrapper.dashboardSet('add_widget', {widget: widget, top: true}, function(){
						$.logMessage('New widget added');
					});
				}};
			}
		});

		var enable_menubar = ECMWF.options.enable_experimental;
		var menu = enable_menubar && new ECMWF.MenuBar('ul#menu-bar', 'menu-panes')
			.add([
				{
					name: 'addwidget',
					title: 'Add widget',
					items: add_widget_items
				},
				{
					name: 'layout',
					title: 'Layout',
					items: layout_items,
					enabled: User.is_forecaster()
				},
				{
					name: 'help',
					title: 'Help',
					items: [
						{name: 'help',              title: 'EcCharts help',   selected: function(){ window.open(ECMWF.options.site.eccharts_help); }},
						{name: 'preferences',       title: 'Preferences',     selected: function(){ require(['preferences'], function(){ $(window).preferences() })}},
						{name: 'report',            title: 'Report issue'     }
					],
					// TODO this menu is currently visible to non-forecasters
					enabled: User.is_forecaster()
				/* TODO its not clear if site navigation should go in the app menu
				},
				{
					name: 'go',
					title: 'Go',
					items: [
						{name: 'forecaster', title: 'Forecaster', selected: function(){ location.href = ECMWF.options.site.forecaster; }},
						//{name: 'dashboard',  title: 'Dashboard', disabled: true}
					]
				*/
				}
			]);

		//$('ul#menu-bar').workspaceMenu({menu: menu, position: 2})
	})
}

/*
 *  Non-forecasters see a very cut-down 'Add widget' menu, and nothing else.
 */
function make_menus_2017(){
	User.info.then(function(user) {
		var exclude = User.is_forecaster() ? ['basic_chart'] : ['chart', 'meteogram_10_days', 'meteogram_15_days', 'meteogram_efi', 'meteogram_extended']

		var items = [
			{
				name: 'addwidget',
				title: 'Add widget',
				items: $.map(DashboardWidget.registry(), function(widget){
					if(!widget.is_private && !~$.inArray(widget.kind, exclude)){
						return {
							name: widget.kind,
							title: widget.title,
							selected: function(){
								wrapper.dashboardSet('add_widget', {widget: widget, top: true}, function(){
									$.logMessage('New widget added');
								});
							}
						}
					}
				})
			},
			{
				name: 'layout',
				title: 'Layout',
				items: $.map(new Array(6), function(v, i){ return {
					name: 'radio-' + (i + 1),
					title: (i + 1) +' Column' + (i ? 's' : ''),
					selected: function(e) {
						wrapper.dashboardSet('reflow', i + 1);
					}
				}}),
				enabled: User.is_forecaster()

				// TODO how does bootstrap do select-menus?

				// TODO example message: menu-layout-radio-change {"value":"layout-radio-2-9"}
			}
		]

		// TODO what should help menu contents be for non-forecasters?
		var help = [{
			name: 'help',
			title: 'Help',
			items: [
				{name: 'help',        title: 'EcCharts help', selected: function(){ window.open(ECMWF.options.site.eccharts_help); }},
				{name: 'preferences', title: 'Preferences',   selected: function(){ require(['preferences'], function(){ $(window).preferences() })}},
				{name: 'report',      title: 'Report issue'   }
			],
			enabled: User.is_forecaster()
		}]

		var el = $('#app-menu .row')

		var menu = new AppMenu({
			el: el,
			items: items,
			help: help
		})
		menu.render();
	})
}

return Application
});
