require(['jquery', 'bus', 'test_lib', 'dashboard/test/dashboard_set_testable', 'dashboard/test/dashboard_testable'], function($, Bus, Tests, DashboardSetTestable, DashboardTestable){

$.frameworkRegister(function(done){

	var dont_run = [
		/*
		'Add Dashboard',
		'Add Widget',
		'Preview',
		'Edit widget',
		'Add Remove Widget',
		'Add Remove Many Widgets',
		'Remove Dashboard',
		'Move Widget',
		'Change Layout'
		*/
	]

	var bus = Bus();
	bus.listen('application-loaded', function(){
		var dst = new DashboardSetTestable();
		bus.listen('dashboard-set-loaded', function(){
			var tests = new Tests([
				{
					name: 'Add Dashboard',
					start: function(){
						var test = dst.test = this;
						this.bus = bus;
						$.when(dst.add_dashboard()).then(test.finished);
					}
				},
				{
					name: 'Add Widget',
					start: function(){
						dst.test = this;
						$.when(new DashboardTestable(this).add_widget()).then(dst.test.finished);
					}
				},
				{
					name: 'Preview',
					start: function(){
						var test = dst.test = this;
						test.set_timeout(20000);
						var dt = new DashboardTestable(this);

						var dashboard = $('.dashboard .ui-tabs-panel:visible');
						// TODO for this test, the widget must be an interactive chart.
						dt.ensure_widget(function(widget){
							$.when(dt.multiple_previews(widget.attr('id'))).then(function(){
								test.assert(!$('.chart-controls').length);
								$.when(dt.multiple_previews(widget.attr('id'))).then(function(){
									dst.change_dashboard($('.ui-tabs-nav > li:first a').attr('href').slice(1)).then(test.bind(test.finished));
								});
							});
						});
					}
				},
				{
					/*
					 *   This test is currently very simple but serves to check that the form widget is loaded.
					 */
					name: 'Edit widget',
					start: function(){
						var test = dst.test = this;
						var dt = new DashboardTestable(this);

						dt.ensure_editable_widget(function(widget){
							$.when(dt.edit_start(widget.attr('id'))).then(test.finished);
						});
					}
				},
				{
					name: 'Add Remove Widget',
					start: function(){
						var test = dst.test = this;
						var dt = new DashboardTestable(this);
						$.when(dt.add_widget())
							.pipe(function(widget){
								console.log('>>> on_added: widget=', widget);
								return dt.remove_widget(widget.options.name);
							})
							.then(test.finished);
					}
				},
				{
					name: 'Add Remove Many Widgets',
					start: function(){
						var n_widgets_to_add = 3;

						var test = dst.test = this;
						var dt = new DashboardTestable(this);

						var fnlist = [];
						for(var j = 0; j < n_widgets_to_add; j++) fnlist.push(dt_bind(dt.add_widget));

						var n_widgets = dt.n_widgets();

						console.log('_____________________________');
						test.series(fnlist, function(err){
							if(err) console.warn('err=', err); // TODO
							console.log('series A finished. n_widgets=', dt.n_widgets());

							var ok = dt.n_widgets() === n_widgets + n_widgets_to_add;
							test.assert(ok, 'n_widgets=' + dt.n_widgets() + ' expected ' + (n_widgets + n_widgets_to_add));
							if(!ok) return;

							var fnlist = [];
							for(var j = 0; j < n_widgets_to_add; j++) fnlist.push(dt_bind(dt.remove_widget));
							test.series(fnlist, function(err){
								console.log('series B finished. n_widgets=', dt.n_widgets());
								test.assert(dt.n_widgets() === n_widgets, 'n_widgets=' + dt.n_widgets() + ' expected ' + (n_widgets));
							});

							return false;
						});

						function dt_bind(fn){
							return function(){ return fn.apply(dt, arguments); };
						}
					}
				},
				{
					name: 'Remove Dashboard',
					start: function(){
						var test = dst.test = this;
						var dt = new DashboardTestable(this);
						$.when(dst.remove_dashboard(dt.dashboard.name)).then(test.finished);
					}
				},
				{
					name: 'Move Widget',
					start: function(){
						var test = dst.test = this;
						this.bus = bus;
						var db1, db2;
						$.when(Test.sleep)
							.pipe(bind(dst.add_dashboard))
							.pipe(function(dashboard){ db1 = new DashboardTestable(test); return db1.add_widget(); })
							.pipe(bind(dst.add_dashboard))
							.pipe(function(dashboard){ return (db2 = new DashboardTestable(test)).add_widget(); })
							.pipe(function(value){
								var widget = value;
								return dst.move_widget(widget, db2, db1);
							})
							.then(test.finished);
					}
				},
				{
					name: 'Change Layout',
					start: function(){
						// TODO so that test is repeatable, we should create a new dashboard with a known number of widgets
						var test = dst.test = this;
						var dt = new DashboardTestable(this);
						$.when(dt.set_n_columns(4))
							.pipe(function(){ return dt.set_n_columns(6); })
							.pipe(function(){ return dt.set_n_columns(1); })
							.pipe(function(){ return dt.set_n_columns(3); })
							.then(test.finished);
					}
				}
			]);

			tests.dont_run(dont_run).run();
		});

		function bind(fn){
			return function(){ return fn.apply(dst, arguments); };
		}
	});

	done();

}, "Load test script");

});
