/*jslint white: false, nomen: false, onevar: false, forin: false, browser: true, devel: true, immed: false */
/*global console: true, jQuery: true */
/*
 *  sizing:
 *     outgoing:
 *        - dashboardResized - sent to each widget following a resize
 *
 */
define(['config', 'jquery', './widgets/widget', 'user_info', './model', 'bus', 'help-panel', 'throttle-debounce', 'show_on_mouseover', 'settings', 'icon', 'thumbnail-tools', 'undo', './dashboard_set', './sortable', './empty', './widgets/chart', './widgets/basic_chart', './widgets/meteogram', './widgets/text', './widgets/controls', './dashboard.css'], function(ECMWF, $, DashboardWidget, User, Tabs, Bus, HelpPanel) {

var box = {
	offset: null,
	widget_padding: {x: 30, y: 30}
};

var old_template;

$.widget('ui.dashboard', {

	options: {
		dashboard: null,
		handleSelector: '.widget-head'
	},

	_init: function () {
		var self = this, o = this.options;

		old_template = !$('#app-menu').length;

		self.active = false;
		var dashboard_set = $('#dashboard-set');
		self.dashboard_set = dashboard_set.dashboardSet('instance');

		self.element.attr({id: o.dashboard.name});
		self.element.addClass('managed');
		self.box = {
			width: o.width // not using global box so we can detect size changes
		}
		self.widgets = new ColumnList(o.dashboard.columns.length); // perhaps rename to 'columns'?

		self.bus = Bus(self.element, Bus())
			.listen('tab-activated', function(_, size){
				console.log(o.dashboard.name + '.active!');
				box.offset || self._update_offset();

				self.active = true;

				if(self._fill_done){
					if(size.width !== self.box.width){
						self.set_width(size.width)
					}
					if(self.__ready) self.__ready.resolve(self.__ready._get_state)
				}else{
					self.fill(function() {
						console.log('fill done');
						// send activation _again_ for the benefit of newly created widgets
						self.widgets.all(function(w, c, r){
							w.instance.activate({width: self._widget_width()});
						})
						if(self.__ready) self.__ready.resolve(self.__ready._get_state)
					});
				}

				if(self.is_loading) self._show_loader();

				$('body > .dashboard-help[name="' + o.dashboard.name + '"]')
					.show()
					.each(function(){
						if(ECMWF.browser.msie){
							setTimeout(function(){
								$(this).css({left: $(this).attr('left'), top: $(this).attr('top')});
							}, 200);
						}
					});

				self.widgets.show(),
				self._make_sortable()

				// note that the first time this is run, the widgets have not yet been created
				self.widgets.all(function(w, c, r){
					w.instance.activate({width: self._widget_width()});
				})
			})
			.listen('tab-deactivated', function() {
				self.active = false;

				$('body > .dashboard-help[name="' + o.dashboard.name + '"]').hide();

				self.widgets.hide();
			});

		self.changed = {};

		self._on_dimension_change = $.throttle(1000, function() {
			/*
			 *  This serves 2 main purposes:
			 *      1- to update widget y positions and set total dashboard height following content changes
			 *      2- to set widget widths.
			 */
			var self = this, o = this.options;

			if(!self.active || self.deleted) return;

			//console.log(o.dashboard.name + '._on_dimension_change: width=' + self.width + ' ' + self._widget_width());

			self._update_offset();

			var dashboard_height = 0;
			self.widgets.each(function(column, c){
				var column_height = 0;

				if(self.changed.width){
					$.extend(column, {
						x: self._x_from_column(c) + box.offset.x,
						width: self._widget_width(),
						widget_margin: {left: parseInt((column.rows.length && column.rows[0].li) ? column.rows[0].li.css('marginLeft') : 0, 10)} // TODO not ideal place to measure widget margin
					});
				}

				column.y = box.offset.y;

				column.each(function(w, r){
					if(w.widget){
						var widgetClass = DashboardWidget.findClass(w.widget.kind);
						var collapsed = w.instance ? w.instance.options.collapsed : w.widget.collapsed;
						var height = collapsed
							? 0
							: (w.instance ? w.instance.content_height() : 100); // TODO what to do if widget not yet instantiated?
						//console.log('dim_change: [' + c + ',' + r +'] width=', self._widget_width());
						var css = {
							top: box.offset.y + column_height
						};
						if(self.changed.width){
							$.extend(css, {
								left : box.offset.x + self._x_from_column(c) - column.widget_margin.left,
								width: self._widget_width()
							});
						}
						w.li.css(css);
							//.triggerHandler('dashboardResized');

						column_height += height + 24/*header*/ + 2/*border*/ + box.widget_padding.y;
						dashboard_height = Math.max(dashboard_height, column_height);
					}
				});
				self.height = Math.max(dashboard_height, column_height);
			});
			self.element.css({height: self.height + 26/*statusbar height*/});

			self.changed.width = false;
			self.changed.height = false;

			self.element.triggerHandler('dashboardResized'); // (re-)notify parent that resize is complete

			self._check_scrollbar = self._check_scrollbar || $.debounce(500, function(){ // TODO perhaps should not be throttled but needs widgets to be created
				if(self.element.is(':visible')){
					var _width = $('body').width();
					if(self._on_dimension_change.width !== _width){
						// careful setting box.width because is used to detect changes in set_width()
						self.box.width = self._width();

						self.changed.width = true;
						self._send_resize();
						self._on_dimension_change();
						self.set_x();
					}
					self._on_dimension_change.width = _width;
				}
			});

			self._check_scrollbar(); // changing the height may have changed the visibility of the page vertical scrollbar
			self._layout_done();
		});

		self._layout_done = function(){
			self._layout_done = $.noop;
		}

		self.get_sortable_items = function(){
			//note: the first time we call this is too early and the widgets have not yet been created.
			return $($.map(self._widget_list(), function(w){ return '#' + w }).join(', '));
		}
	},

	_send_resize: function() {
		var self = this;

		self.widgets.all(function(w, c, r){
			if(w.widget && w.li){
				w.li.triggerHandler('dashboardResized', [{width: self.box.width, height: self.height}, {width: self._widget_width()}]);
			}
		});
	},

	// called by the DashboardSet
	// the dashboard widget must change to be new_width pixels wide
	set_width: function(new_width){
		var self = this;

		var width_changed = new_width !== self.box.width;
		self.box.width = new_width;
		var height_changed = true; // TODO the height is set by the contents, not the container, so this should not be needed.

		if(width_changed || height_changed){
			width_changed && (self.changed.width = true);
			height_changed && (self.changed.height = true);
		}

		if(width_changed){ // widgets dont need to know about height changes.
			self._send_resize();
			self._on_dimension_change();
		}

		// with fixed width page, x positions need to be set, even if width is changed. remove this once changed to full-screen layout for www
		// TODO integrate this special case into normal handling
		self.set_x();
	},

	set_x: function(){
		var self = this;

		if((box.offset.x = this.element.position().left + this.element.offset().left)){
			this.widgets.each(function(column, c){
				column.x = self._x_from_column(c) + box.offset.x;
				column.layout();
			});
		}
	},

	fetch: function () {
		return this.options.dashboard;
	},

	destroy: function () {
		var self = this, o = this.options;

		self.widgets.each(function(column, c){
			column.each(function(w, r){
				w.li.remove();
			});
		});
		self.widgets = null;
		self.deleted = true;

		self.element.removeAttr('id');
		$('body > .dashboard-help[name="' + o.dashboard.name + '"]').remove();
	},

	setTitle: function (title) {
		this.options.dashboard.title = title;
		this._emit_changed();
	},

	_show_loader: function () {
		var self = this;
		setTimeout(function(){
			if(self.is_loading){
				self.element.show().loader('active', 'Dashboard Loading...');
			}
		}, 500);
	},

	_hide_loader: function () {
		var self = this;
		self.element.loader('destroy');
	},

	_loading: function (is_loading) {
		var self = this, o = this.options;

		is_loading = (is_loading !== false) && self._widget_list().length;

		if(self.active){
			if(is_loading){
				self._show_loader();
			}else{
				console.log('dashboard.loading: false');
				self._hide_loader();
			}
		}

		if(!is_loading && self.is_loading) self.bus.send('dashboard-loaded');

		self.is_loading = is_loading;
	},

	get_dashboard: function(done) {
		var self = this, o = self.options;

		if(o.dashboard.columns){
			setTimeout(done, 1);
		}else{
			Tabs.get_dashboard(o.dashboard.name, function(result){
				$.extend(o.dashboard, result);
				self._conform();
				done();
			});
		}
	},

	fill: function(done) {
		var self = this, o = this.options;
		done || (done = $.noop);

		self._loading();

		self._fill_done = true; // TODO try and remove this.

		self.get_dashboard(function(){
			if(!self._widget_list().length) self.element.emptyDashboardContent();

			self.widgets.count() || $.each(o.dashboard.columns, function(c, _col){
				var col = self.widgets.at(c);
				$.each(_col, function(i, widget_name){
					var r = col.add();
					col.rows[r].widget = {name: widget_name};
				});
			});

			var widget_loaders = [];
			self.widgets.all_by_row(function(w, c, r){
				w.li || widget_loaders.push(Tabs.Widget.get(w.widget.name, function(widget) {
					self.is_loading && self._loading(false);
					self._add_widget(widget, {c: c, r: r});
				}));
			});

			$.when.apply($, widget_loaders).then(function(){
				($('body').dashboardSortable('option', 'items', self.get_sortable_items).dashboardSortable('refresh'));
				done();
			});
		});
	},

	set_n_columns: function (n_columns, done) {
		var self = this, o = this.options;
		console.log('dashboard.set_n_columns', o.dashboard.columns.length, ' --> ', n_columns);

		if(n_columns === o.dashboard.columns.length) return;

		// stop animations and the like
		self.bus.send('tab-deactivated');

		self.widgets.all(function(w, c, r){
			w.li.remove();
		});

		self.widgets.clear();

		// TODO we modify o.dashboard and then update self.widgets.
		//      It would be better to modify self.widgets and then use on_layout_changed to update o.dashboard
		//      but self.fill needs to be modified to facilitate this
		if(n_columns < o.dashboard.columns.length){
			for(var i = o.dashboard.columns.length - n_columns; i; i--){
				self.widgets.pop();
				$.each(o.dashboard.columns.pop(), function(i, v){
					o.dashboard.columns[0].push(v);
				});
			}
		}else if(n_columns > o.dashboard.columns.length){
			var n_to_add = n_columns - o.dashboard.columns.length;
			for(i = 0; i < n_to_add; i++){
				o.dashboard.columns.push([]);
				self.widgets.push();
			}
		}

		self.fill(function() {
			self.bus.send('tab-activated', self.dashboard_set._measure(), {width: self._widget_width()});
			done && done();
		});
	},

	_start_change: function() {
		if(!this._is_changing){
			this._is_changing = true;
			if(this.__ready){
								if(this.__ready.state() === 'resolved') console.log('*** start_change: removing old promise...')
								else console.log('*** start_change: keeping existing unresolved promise')
				if(this.__ready.state() === 'resolved') this.__ready = null;
			}
		}
	},

	_end_change: function() {
		if(this.__ready){
			var complete = true;
			this.widgets.all(function(w, c, r){
				w.widget || (complete = false);
			});
			complete && this.__ready.resolve(this.__ready._get_state)
		}
	},

	add_widget: function (args, done) {
		var self = this, o = this.options;

		this._start_change()

		self.element.data('uiEmptyDashboardContent') && self.element.emptyDashboardContent('destroy');

		Tabs.Widget.add(o.dashboard.name, args.widget, function(result){
			console.log('dashboard.add: got new widget: name=', result.name, result);
			var row = self.widgets.at(args.column_num || 0).add(args.row || (args.top ? 0 : null));
			self._add_widget(result, {c: args.column_num, r: row, notify: true});

			self._end_change()

			done && done(result.name);
		});
	},

	_add_widget: function (widget, args) {
		//update the user interface following a model change.
		var self = this, o = this.options;

		// TODO c, r defaults are dangerous. may already be a widget at this position
		var _o = $.extend({}, {notify: false, c: 0, r: 0}, args);

		//console.log(o.dashboard.title + '._add_widget: widget=' + widget.name, widget);
		//console.log(o.dashboard.title + '._add_widget: dashboard=', o.dashboard);

		// temporary workaround.
		if(o.dashboard.columns.length <= _o.c){
			for(var i=0;i<Math.max(3, _o.c + 1);i++) o.dashboard.columns.push([]);
		}

		($.inArray(widget.name, o.dashboard.columns[_o.c]) > -1) || o.dashboard.columns[_o.c].push(widget.name);

		var widgetClass = DashboardWidget.findClass(widget.kind);
		if(__DEV__ && !widgetClass) return console.warn('dashboard: no widget for content type: ' + widget.kind);

		var w = self.widgets.at(_o.c).rows[_o.r];
		$.extend(w, {
			li: $('body')
				.div('.managed')
				.css({left: box.offset.x + self._x_from_column(_o.c)}),
			widget: widget
		});

		self.changed.width = true; //TODO
		self._on_dimension_change(); //set widget widths. FIXME this is throttled, so we need to delay subsequent code until it has run.
		w.li.css({
			width: self._widget_width(),
			top: box.offset.y + self._y_from_row(_o.r, _o.c) //too early to get accurate y position.
		});

		//console.log('active?', o.dashboard.name, self.dashboard_set.get_selected_dashboard().name, self._active());
		self._active() || w.li.hide();

		if(__DEV__ && !$.fn[widgetClass]) console.warn('no such widget type: ' + widgetClass);

		w.instance = w.li[widgetClass]($.extend(widget, {
			_owner: self, _dashboard: o.dashboard, dashboard: o.dashboard.name, messageBus: self.bus, active: self._active(), width: self._widget_width()
		}))[widgetClass]('instance');

		if(self._active()) self._make_sortable();
		if(_o.notify) self._emit_changed();

		w.instance.options.has_help && w.li.one('click', '.ui-icon-help', function() {
			HelpPanel({el: w.li});
			$(this).click();
		});

		self.changed.width = true;   // TODO
		self._on_dimension_change(); // TODO check we need this here
		w.li.on('contentChanged', function(){
			self._on_dimension_change();
		});

		/*
		setTimeout(function(){
			self._verify(); //verification may fail if another operation is started before it runs.
		}, 4000);
		*/

		Bus(w.li).listen('height-changed', function (_, height) {
			w.height = height;

			var pos = self.widgets.find(w.widget);
			// TODO during bootstrap, is not found
			if(pos.r > -1 && pos.c > -1){
				self.widgets.at(pos.c).layout();
			}
			self._on_dimension_change();

			$('#dnd-constrainer').triggerHandler('resize');
		});
	},

	remove_widget: function (widget_name, done) {
		var self = this, o = this.options;

		this._start_change()

		Tabs.Widget.remove(widget_name, function () {
			self._on_widget_removed(widget_name, done);
			self._end_change() // TODO check this is guaranteed not to be too early
		});
	},

	_on_widget_removed: function(widget_name, done) {
		var self = this, o = this.options;
		console.log('_on_widget_removed: ' + widget_name);

		var widget = $('#' + widget_name).orelse(function(){ console.warn('on_widget_removed: no such widget:', widget_name) });

		function on_fade_out(){
			widget.wrap('<div>').parent().slideUp(function () {
				$(this).remove();
				$('#dnd-constrainer').triggerHandler('resize');
			});
		}

		//console.log('_on_widget_removed: widget=' + widget_name +' hiding... opacity=' + widget.css('opacity') + ' hidden=' + !widget.is(':visible'));
		if(widget.css('opacity') && !widget.is(':hidden')){
			widget.animate({
					opacity: 0
				},
				function () {
					on_fade_out();
				});
		}
		else on_fade_out();

		var n_widgets = self.widgets.count();

		// o.dashboard.columns is deprecated - use self.widgets instead
		self.widgets.each(function(column, c){
			if(o.dashboard.columns[c]){
				o.dashboard.columns[c] = $.map(o.dashboard.columns[c], function(v){ if(v !== widget_name) return v; });
			}
		});

		self.widgets.remove_widget(widget_name);

		console.log(o.dashboard.title + ' REMOVE WIDGET ', n_widgets, '-->', self.widgets.count());

		if(done) done();

		self._emit_changed();

		self._widget_list().length || self.dashboard_set.element.find('#' + o.dashboard.name).emptyDashboardContent();

		setTimeout(function() {
			self._on_dimension_change(); // move widgets below up
		}, 1000);
	},

	move_widget: function(widget_name, dest_column, done){
		var self = this;

		if(typeof dest_column === 'number') dest_column = self.element.find('> ul:eq(' + dest_column + ')');

		$('#' + widget_name).prependTo(dest_column);

		done && done();
		self.on_layout_changed();
	},

	transmute_widget: function(widget, product) {
		// TODO would it be better to keep the same widget but update it?
		// TODO interactive www widgets should use the new edit form
		var self = this;
		var position = this.widgets.find({name: widget.options.name});
		var Type;
		if(product.data.widgets){
			if(product.has_widget('interactive')){
				Type = 'chart'
			}
		}

		if(Type){
			this.remove_widget(widget.options.name, function(){
				var w = $.extend({},
					// TODO use the correct type
					DashboardWidget.registry()[Type],
					{data: {product: product.name()}, title: widget.options.title}
				)
				self.add_widget({widget: w, column_num: position.c, row: position.r})
			})
		}else{
			console.error('cannot determine widget type. product=', product)
		}
	},

	_widget_list: function() {
		return Array.prototype.concat.apply([], this.options.dashboard.columns);
	},

	_make_sortable: function () {
		var self = this, o = this.options;

		var sortable = {
			drop_to_tab: null,
			need_update: false,
			collapse: function (e, ui) {
				if(!sortable.collapsed){
					ui.helper.css({height: 22, overflow: 'hidden'});
				}
				sortable.collapsed = true;
			},
			uncollapse: function (e, ui) {
				if(sortable.collapsed){
					ui.helper && ui.helper.css({height: 'auto', overflow: 'visible'});
					sortable.collapsed = false;
				}
			}
		};

		var placeholder_position = {c: 0, r: 0};
		var previous_placeholder_position = null;

		self.sortable = $('body').dashboardSortable({
			items: self.get_sortable_items,
			containment: '#dnd-constrainer',
			handle: o.handleSelector,
			connectWith: '.ui-tabs-nav li',
			revert: 200,
			helper: function(ev, el){
				var div = $('<div>', {'class': 'widget managed dashboardWidget', css: {width: self._widget_width()}})
					.html(el.html());
				div.find('.edit-box, .buttons').remove();
				return div;
			},
			//helper: 'original', // need to amend layout fn if using this.
			start: function (e, ui) {
				console.log('%cstart', 'color: #f63');
				//console.log('start: placeholder=', ui.placeholder.parent(), ui.placeholder);
				//$(ui.helper).addClass('dragging');
				ui.placeholder.css({top: ui.originalPosition.top, left: ui.originalPosition.left, visibility: 'visible', border: 'dotted 1px #aaa', backgroundColor: '#fff'});

				//$('body').div({'class': 'widget-placeholder', width: self._widget_width(), height: ui.item.height(), css: {border: 'dotted 1px #aaa', position: 'absolute', top: ui.originalPosition.top, left: ui.originalPosition.left, marginLeft: 20, marginTop: box.widget_padding.y, zIndex: 1000}});

				var widget = ui.item.attr('id');
				var pos = self.widgets.find({name: widget});
				var item;
				for(var c=0;c<self.widgets.columns.length;c++){
					for(var r=0;r<self.widgets.columns[c].rows.length;r++){
						var w = self.widgets.columns[c].rows[r];
							if(w.widget && w.widget.name === widget){
								item = {c: c, r: r};
								previous_placeholder_position = {c: c, r: r};
							}
					}
				}
			},
			stop: function (e, ui) {
				console.log('stop: item=', (self.__drag ? self.__drag.item : null), (self.__drag ? self.__drag.item.attr('id') : null));
				if(!self.__drag) return;

				if(sortable.drop_to_tab){
					console.log('sortable.stop: moving...');
					var widget_name = ui.item.attr('id');
					ui.item.hide();
					var dest_name = sortable.drop_to_tab.find('a').attr('href').substring(1);
					self.move_widget_to_other_tab(widget_name, dest_name);
					sortable.drop_to_tab = null;
					sortable.uncollapse(e, ui);

				}else{
					var widget_to_move = self.__drag.item.attr('id');
					var pos = self.widgets.find({name: widget_to_move});
					console.log('stop: top=', Math.round(ui.position.top), ' old_pos=', pos);
					if(pos.c === self.__drag.new_position.c && pos.r === self.__drag.new_position.r){
						console.log('stop: position is unchanged');
					}else if(pos.c > -1){
						var old_column = self.widgets.at(pos.c);
						var item = old_column.rows.splice(pos.r, 1); // remove the cell from the old column
						if(item && (item = item[0])){
							var new_column = self.widgets.at(self.__drag.new_position.c);

							// re-order the widgets in the column
							new_column.rows.splice(self.__drag.new_position.r > -1 ? self.__drag.new_position.r : new_column.rows.length, 0, item); //TODO index wrong if moved within same column

							// move the cell to the drag position
							item.li.css({transition: 'none'});
							item.li.css(ui.position);
							item.li[0].offsetHeight; // trigger a reflow
							item.li.css({transition: ''});

							console.log('stop: c:', pos.c + ' -->' + self.__drag.new_position.c);
							(self.__drag.new_position.c >= 0) && new_column.layout();                    // layout new column
							if(pos.c != self.__drag.new_position.c && (pos.c >= 0)) old_column.layout(); // layout old column if different

							o.dashboard.columns = $.map(Array.apply(null, Array(+o.dashboard.columns.length)), function(){ return [[]]; });
							self.widgets.all(function(w, c, r){
								w.widget && o.dashboard.columns[c].push(w.widget.name);
							});

							self._on_dimension_change(); // update height
							self._emit_changed();
						}
					}else{
						console.warn('stop: column not set:', pos.c);
					}
				}
			},
			sort: function (e, ui) {
				function get_column(x){
					return Math.floor(x * 3 / self.box.width, 10);
				}
			},
			update: function (e, ui) {
				console.log('dashboard.update');
				//update event happens before stop.
			},
			change: function (e, ui) {
				self.rearrange(ui);

				if(ui.target && ui.target.element.parent().hasClass('ui-tabs-nav') && (ui.target !== self.sortable)){
					var li = sortable.drop_to_tab = ui.placeholder.parent();
					ui.placeholder.addClass('drop-shadow-small move-to-tab').removeClass('widget').span().html('Move to dashboard <b>' + li.find('a').text() +'</b>');
					li.parent().find('> li').removeClass('ui-state-hover');
					li.addClass('ui-state-hover')
					self.__drag = self.__drag || {item: ui.item};
					ui.placeholder.css({top: 10, left: 30});

				}else{
					if(sortable.drop_to_tab){
						sortable.drop_to_tab = null;
						ui.placeholder.text('');
						$('.ui-tabs-nav').find('li').removeClass('ui-state-hover');
						ui.placeholder.removeClass('drop-shadow-small move-to-tab').addClass('widget');
						ui.placeholder.appendTo('body'); // this should really be done by the sortable.
					}

					//console.log('sortable.change: left=', Math.round(ui.position.left), '[' + ui.c + ',' + ui.r + ']');
					self.__drag = {item: ui.item, new_position: {c: ui.c, r: ui.r}};
					placeholder_position.c = ui.c;
					placeholder_position.r = ui.r;
					//console.log('sortable.change: originalPosition.top=', Math.round(ui.originalPosition.top));
					//console.log('sortable.change: setting placeholder: row=', ui.r, ' top=', Math.round(ui.originalPosition.top) + self._y_from_row(ui.r, ui.c));
					ui.placeholder.css({
						left: box.offset.x + self._x_from_column(placeholder_position.c),
						top: box.offset.y + self._y_from_row(ui.r, ui.c)
					});
					self.widgets.at(placeholder_position.c).layout(placeholder_position.r);
					previous_placeholder_position && (previous_placeholder_position.c !== placeholder_position.c) && self.widgets.at(previous_placeholder_position.c).layout();
					previous_placeholder_position = $.extend({}, placeholder_position);
				}
			},
			get_item_at_position: function(ui, position){
				//console.log('dashboard.get_item_at_position: top=', position.top, ' left=', position.left);
				var sortable = self.sortable;

				var c = self._column_from_x(position.left + self._widget_width() / 2);
				var r = self._row_from_y(position.top, c);

				var item = (c < self.widgets.columns.length && r > -1 && r < self.widgets.at(c).rows.length) && self.widgets.at(c).rows[r].li;
				// compensate for the fact that the positions dont reflect the transient conditions during drag
				if(item && (item[0] === sortable.currentItem[0])){
					if(!(sortable.__placeholder_position.c === c && sortable.__placeholder_position.r === r)){
						if(self.widgets.at(c).rows[r + 1]){
							item = self.widgets.at(c).rows[r + 1].li;
						}else{
							item = null;
						}
					}
				}

				return $.extend({}, {
					c: c,
					r: r,
					helper_position: {left: c * self._widget_width()},
					item: item
				});
			}
		}).dashboardSortable('instance');

		self.sortable.options.items = self.get_sortable_items;
	},

	move_widget_to_other_tab: function (widget_name, dest_name, done){
		var self = this, o = this.options;

		self.dashboard_set.move_widget(widget_name, o.dashboard.name, dest_name, function(result){
			console.log('move_widget_to_other_tab: done');

			//update this dashboard:
			o.dashboard = result.dashboards.src;

			self._on_widget_removed(widget_name);

			if(__DEV__ && result.widget.name != widget_name) console.warn('move_widget_to_other_tab: name mismatch!');

			var dest = self.dashboard_set.dashboard_lookup_by_name(dest_name);

			(function add_widget_to_foreign_dashboard(dest, widget, done){
				dest.element.data('uiEmptyDashboardContent') && dest.element.emptyDashboardContent('destroy');
				if(dest._fill_done) {
					var column = dest.widgets.at(0)
					column.add(0);
					column.rows[0].widget = {name: widget_name}
					dest._active() && dest._add_widget.call(dest, widget);
					dest._active() || (dest._fill_done = false);
					done();
				}else{
					Tabs.get_dashboard(dest.options.dashboard.name, function(result){
						$.extend(dest.options.dashboard, result);
						done();
					});
				}
			})(dest, result.widget, function(){
				$.logMessage('Widget moved to dashboard \'' + dest.options.dashboard.title + '\'');
				done && done();
			});
		});
	},

	rearrange: function(i){
	},

	get_state_async: function(){
		var self = this, o = self.options;

		function get_state(){

			o.dashboard.columns = $.map(Array.apply(null, Array(+o.dashboard.columns.length)), function(){ return [[]]; });
			self.widgets.all(function(w, c, r){
				o.dashboard.columns[c].push(w.widget.name);
			});

			return o.dashboard;
		}

		if(!this.__ready){
			this.__ready = new $.Deferred()
			this.__ready._get_state = get_state;
			this.__ready.then(function(get_state){ console.log('> ready.resolved: ', get_state()) })
			if(this._fill_done && !this._is_changing) this.__ready.resolve(this.__ready._get_state)
		}

		return this.__ready;
	},

	_active: function(){
		var o = this.options;
		if(__DEV__ && (o.dashboard.name === this.dashboard_set.get_selected_dashboard().name) !== this.active) console.warn(o.dashboard.name + ': active flag incorrectly set. active=', this.active, ' selectected=', this.dashboard_set.get_selected_dashboard().name);
		return (o.dashboard.name === this.dashboard_set.get_selected_dashboard().name);
	},

	_update_offset: function(){
		var self = this, el = self.dashboard_set.element;

		if(old_template){
			$('body.page-en').each(function() { // www
				box.offset = {
					x: self.element.position().left + self.element.offset().left,
					y: el.offset().top + self.element.parent().find('.ui-tabs-nav').outerHeight()
				};
			}).orelse(function() { // apps
				box.offset = {
					x: el.position().left + el.offset().left,
					y: parseInt($('body').css('padding-top'), 10) + $('#header').css({minHeight: 28}).outerHeight() + self.element.parent().find('.ui-tabs-nav').css({minHeight: 27}).outerHeight()
				}
			});
		}else{
			box.offset = {
				x: 30, //self.element.position().left + self.element.offset().left,
				y: el.offset().top + self.element.parent().find('.ui-tabs-nav').outerHeight()
			};
		}
	},

	_width: function(){
		var self = this;
		return old_template
			? (Math.round(self.dashboard_set.element.width() - parseInt(self.element.parent().css('marginLeft'), 10) - parseInt(self.element.parent().css('marginRight'), 10)))
			: window.outerWidth - 2 * box.offset.x;
	},

	_widget_width: function(){
		var o = this.options;
		var border = 2 * 2;
		return Math.round((this.box.width + box.widget_padding.x) / o.dashboard.columns.length - box.widget_padding.x - border);
	},

	_column_from_x: function(x){
		// x is page-relative not dashboard-relative
		var o = this.options;

		return Math.max(0, Math.floor((x - box.offset.x) * o.dashboard.columns.length / this.box.width));
	},

	_x_from_column: function(c){
		var self = this, o = this.options;

		return c * ((this.box.width + box.widget_padding.x) / o.dashboard.columns.length);
	},

	_row_from_y: function(y, c){
		var self = this;
		if(c >= 0 && c < self.widgets.columns.length){
			var _y = 0;
			var column = self.widgets.at(c);
			if(!column.rows.length) return 0;
			for(var r=0;r<column.rows.length;r++){
				var w = column.rows[r];
				//console.log('   ', w);
				_y += w.height;
				if(y < _y) return r;
			}
		}
		return -1;
	},

	_y_from_row: function(r, c){
		var self = this;

		(typeof r === 'undefined') && (r = 0);
		(r < 0) && (r = 1000);
		var y = 0;
		if((c >= 0) && (c < self.widgets.columns.length)){
			self.widgets.at(c).each(function(w, r_){
				if(r_ >= r) return false;
				if(w.widget){
					var widgetClass = DashboardWidget.findClass(w.widget.kind);
					var height = (w.instance ? w.instance.widget_height() : w.height) + box.widget_padding.y; // TODO shouldnt this just use w.height ?
					//console.log(w.widget.name + '._y_from_row:   height:', w.height, height);
					y += height;
				}
			});
		}
		//console.log('_y_from_row: [', r, c, '] y=' + y);
		return y;
	},

	/*
	_verify: function(){
		var self = this, o = this.options;

		function warn(str){
			console.warn('dashboard (' + o.dashboard.title + '): ' + str);
		}

		var n_widgets = o.dashboard.widgets.length;
		var n_dom_columns = self.element.find('> .column').length;
		var n_dom_widgets = self.element.find('> .column > .dashboardWidget').length //excludes any wrapped for slideup
		var header = 'dashboard (' + o.dashboard.title + '): ';
		if(ECMWF.n_properties(o.dashboard.columns) != n_widgets) console.warn(o.dashboard.title + ': dashboard.columns incorrect: ' + ECMWF.n_properties(o.dashboard.columns) + ' expected ' + n_widgets + '. columns=', o.dashboard.columns);
	},
	*/

	// update old dashboard formats
	_conform: function(){
		var self = this, o = this.options;
		// columns has been changed to an array so that order is preserved
		/*
		if(!Array.isArray(o.dashboard.columns)){
			var c = self._count_columns();
			if(!o.dashboard.n_columns || (c != o.dashboard.n_columns)){ // not set in some dashboards!
				console.warn('n_columns not correctly set: ', o.dashboard.n_columns, c);
				o.dashboard.n_columns = c;
			}
			var columns = $.map(Array.apply(null, Array(+o.dashboard.n_columns)), function(){ return [[]]; });
			$.each(o.dashboard.widgets, function(k, v) {
				var column = o.dashboard.columns[v] || '0';
				columns[+column.replace(/column-/, '')].push(v);
			});
			o.dashboard.columns = columns;
		}
		*/

		delete o.dashboard.widgets;
	},

	_count_columns: function() {
		var self = this, o = this.options;

		if(Array.isArray(o.dashboard.columns)){
			console.log('count_columns:', o.dashboard.columns.length);
			return o.dashboard.columns.length;
		}else{
			// should not get here
			var columns = {};
			$.each(o.dashboard.columns, function(i, v) {
				columns[v] = true;
			});
			console.warn('count_columns:', ECMWF.n_properties(columns));
			return ECMWF.n_properties(columns);
		}
	},

	on_layout_changed: function(){
		var self = this, o = this.options;

		o.dashboard.columns = $.map(Array.apply(null, Array(+o.dashboard.columns.length)), function(){ return [[]]; });

		self.widgets.all(function(w, c, r){
			o.dashboard.columns[c].push(w.widget.name);
		});

		self._emit_changed();
		self._send_resize();
	},

	_emit_changed: $.throttle(1000, function(){
		Bus().send('dashboard-changed', this);
	})
});

function ColumnList(n){
	this.columns = [];

	if(n) for(var i=0;i<n;i++) this.columns.push(new Column(i));
}

ColumnList.prototype = {
	each: function(fn){
		for(var c=0;c<this.columns.length;c++){
			if(fn(this.columns[c], c) === false) break;
		}
	},

	all: function(fn){
		var ongoing = true;
		for(var c=0;c<this.columns.length;c++){
			this.columns[c].each_(function(w, r){
				return (ongoing = fn(w, c, r));
			});
			if(ongoing === false) break;
		}
	},

	all_by_row: function(fn){
		var ongoing = true;
		var r = 0;
		while(true){
			var found = 0;
			for(var c=0;c<this.columns.length;c++){
				var w = this.columns[c].rows[r];
				if(w){
					if(fn(w, c, r) === false){ ongoing = false; break; }
					found++;
				}
			}
			if(!found || !ongoing) break;
			r++;
		}
	},

	count: function() {
		return this.columns.reduce(function(prev, current) { return prev + current.rows.length; }, 0);
	},

	// param match is an object whose keys must match those of the sought widget.
	// It can either be a complete widget, or e.g. can just contain the name.
	find: function(match){
		var result = {c: -1, r: -1};
		this.all(function(w, c, r){
			if(w.widget){
				var found = true;
				for(var i in match){
					var _val = w.widget[i];
					if(_val !== match[i]){
						found = false;
					}
				}
				if(found){
					result = {c: c, r: r}
					return false;
				}
			}
		});
		(result.c === -1) && console.log('ColumnList.find: not found:', match);
		return result;
	},

	push: function(column){
		this.columns.push(column || new Column(this.columns.length));
	},

	pop: function(){
		return this.columns.pop();
	},

	at: function(i){
		return this.columns[i];
	},

	show: function(){
		this.each(function(column){
			column.show();
		});
	},

	hide: function(){
		this.each(function(column){
			column.hide();
		});
	},

	clear: function(){
		this.each(function(column) {
			column.rows = [];
		});
	},

	remove_widget: function(name){
		var self = this;
		this.all(function(w, c, r) {
			if(w.widget.name === name){
				self.columns[c].rows.splice(r, 1);
				return false;
			}
		});
	}
}

function Column(i){
	this.rows = [];
	this.x = 0;
	this._idx = i;
	this.width = 0;
	this.widget_margin = {left: 0};
}

Column.prototype = {
	each: function(fn){
		var y = 0; //count of non-hidden rows.
		for(var r=0;r<this.rows.length;r++){
			if(this.rows[r].li && (!this.rows[r].widget || this.rows[r].li.is(':visible'))){ // ignore rows that have widgets but are invisible
				if(fn(this.rows[r], y) === false) break;
				y++;
			}
		}
	},

	// like above but ignore visibility TODO why does above check for visibility?
	each_: function(fn){
		var y = 0; //count of non-hidden rows.
		for(var r=0;r<this.rows.length;r++){
			if(fn(this.rows[r], y) === false) break;
			y++;
		}
	},

	idx: function(widgets){
		for(var c=0;c<widgets.length;c++){
			if(widgets.at(c) === this) return c;
		}
		console.log('not found');
	},

	layout: function(placeholder_row){
		var column = this;

		var y = column.y;
		var c = this._idx;
		this.each(function(w, r){
			if(r === placeholder_row) y += $('.ui-sortable-placeholder').innerHeight() + box.widget_padding.y;
			if(__DEV__ && r === placeholder_row) console.log('column.layout: [' + c + ',' + r + ']: adding ' + $('.ui-sortable-placeholder').innerHeight() + ' for placeholder');
			if(w.instance){
				w.li.hasClass('ui-sortable-helper') && console.log('column.layout: *** widget is helper');
				if(w.li.is(':visible')){
					//console.log('column.layout: [' + c + ',' + r +'] widget_height=', height, ' top=', y, w.widget.name);
					w.li.css({
						left : column.x - column.widget_margin.left,
						top  : y,
						width: column.width
					});
					y += w.instance.widget_height() + box.widget_padding.y;
				}
			}
			else console.warn('layout: bad array entry: item has no widget', c, r, w);
		});
	},

	add: function(row){
		if(arguments.length && row !== null)
			this.rows.splice(row, 0, {height: 200});
		else
			this.rows.push({height: 200});
		return (arguments.length && row !== null) ? row : this.rows.length - 1;
	},

	show: function(){
		this.each_(function(w, r){
			w.li && w.li.show();
		});
	},

	hide: function(){
		this.each(function(w, r){
			w.li && w.li.hide();
		});
	}
}

});
