define(['config', 'jquery', './model', 'bus', 'user_info', 'jquery-ui/tabs', 'jquery-ui/button', 'jquery-ui/sortable', 'jquery-ui/dialog', 'throttle-debounce'], function(ECMWF, $, Tabs, Bus, User) {

setTimeout(function() {
	require(['x-editable'], function() {
		$.fn.editable.defaults.mode = 'popup';
	})
}, 0)

var old_template;
var hmargin = 40;

$.widget('ui.dashboardSet', {

	options: {
		dashboard_set: { dashboards : [], selected : 0 },
		widgetSelector: '.widget',
		messageBus: Bus(),
		currentBus: null,
		wrapper: '#wrapper' // dom element to use for dnd constraining
	},

	_init : function () {
		var self = this, o = this.options;
		self.element.addClass('dashboard');
		self.element.addClass('managed');
		self.queue = $({});
		$('body').div('#preview');

		old_template = !$('#app-menu').length;
		if(!old_template) hmargin = 15;

		var save_selected = o.dashboard_set.selected;

		//add an invisible div to set constraints on widget dragging
		var dnd_constrainer = $('body').div({id: 'dnd-constrainer', css: {position: 'absolute'}}).prependTo('body')
			.resize(function(){
				dnd_constrainer.css($.extend({}, self._measure(), {top: 40, height: $(o.wrapper).height()}));
				if(old_template){
					$('.top-bar').height($(o.wrapper).height() + 100);
				}else{
					var h = Math.max(document.documentElement.clientHeight, window.innerHeight || 0);
					$('footer').css({bottom: 'auto', top: Math.max(h - 50, $(o.wrapper).height() + 50)});
				}
			});

		$(window).resize($.throttle(2000, resize));

		resize();

		self.tabs = self.element.div();
		self.nav = self.tabs.ul();

		// subclass and overwrite ui.tabs to remove hard-coded keyboard shortcuts.
		$.widget('ui.tabs', $.ui.tabs, {
			_tabKeydown: function (event) {
				if (!~$.inArray(event.keyCode, [32, 37, 39])) {
					this._super(event);
				}
			}
		});

		$.each(o.dashboard_set.dashboards, function(i, dashboard) {
			self._add_tab_nav(dashboard);
		});

		self.tabs
			.tabs({
				activate: function(e, ui){
					ui.newPanel.find('.column').each(function(c){
						var column = $(this);
						column.find('li.widget').each(function(){
							var el = $(this);
							el[el.findDashboardWidgetClass().klass]('on_show');
						});
					});

					o.dashboard_set.selected = ui.newTab.index();

					self._emit_dashboard_set_changed(o.dashboard_set);

					self._on_tab_nav_changed()
				}
			})
			.removeClass('ui-widget-content ui-corner-all')
			.find('.ui-tabs-nav')
				.removeClass('ui-corner-all')
				.sortable({ // tab re-ordering
					axis: 'x',
					forcePlaceholderSize: true,
					start: function(event, ui){
						ui.item.find('a').off('click');
						ui.item.find('.ui-button').hide();
					},
					update: function(event, ui){
						var active = self.nav.find('> li.ui-state-active').index()
						var out = []
						self.nav.find('> li').each(function(i, li){
							var d = $.findWhere(o.dashboard_set.dashboards, {name: $(li).attr('aria-controls')})
							d && out.push(d)
						})
						if(out.length === o.dashboard_set.dashboards.length){
							o.dashboard_set.dashboards = out;
							o.dashboard_set.selected = active;
							self._emit_dashboard_set_changed(o.dashboard_set);
						}
						ui.item.hasClass('ui-state-active') && setTimeout(function(){ self._bind_rename(ui.item.find('a')) }, 0);
					}
				});

		self.element.div({className: 'last', css: {position: 'absolute', top: 1, right: 0}})
			.button({label: 'Add Tab', icons: {primary: 'ui-icon-plus'}}).addClass('fff').click(function() {
				console.log('add.clicked');
				self.addTab();
				return false;
			});

		if(__DEV__){ // columns menu - temporary implementation pending template change

			/*
			var layout_items = [];
			for(var i=1;i<=6;i++){
				layout_items.push({name: i +' Column' + (i - 1 ? 's' : '')});
			}

			self.button = self.element
				.optionsMenu({menu_class: 'layout-menu application-menu', menu_items: $.map(layout_items, function(v){ return v.needs_login ? (user.uid ? v : null) : v; })})
				.find('.options')
				.css({position: 'absolute', top: 1, right: 90, height: 26})
			*/
		}

		self.dashboards = $.map(o.dashboard_set.dashboards, function(dashboard, i) {
			var content = self.tabs.div().dashboard({dashboard: dashboard, width: self._measure().width});
			o.dashboard_set.dashboards[i] = content.dashboard('fetch');

			content.on('dashboardResized', function(){
				dnd_constrainer.triggerHandler('resize');
			});
			return content.dashboard('instance');
		});

		self.get_selected_dashboard2().bus.listen('tab-activated', function(){
			o.messageBus.send('dashboard-set-loaded', []);
		});

		if(o.dashboard_set.dashboards.length){
			// This may have been changed when creating the tabs
			o.dashboard_set.selected = save_selected; 

			var tab_to_select = $('#' + ECMWF.request.dashboard).orelse(function(){ return $('#' + self.get_selected_dashboard().name); });
			var tab_index = tab_to_select.index() - 1;

			self.tabs.tabs('option', 'active', tab_index);

			on_tab_select(self.get_selected_dashboard2());

			//only now can we attach the select handler
			self.tabs.tabs('option', 'beforeActivate', function(_, ui) {
				on_tab_select(self.dashboard_lookup_by_name(ui.newPanel.attr('id')));
			});
			self._on_tab_nav_changed();
		}

		if(o.focused){
			setTimeout(function(){
				$('#' + o.focused).addClass('focused');
			}, 4000);
		}

		function resize(){
			self._size = null;

			var size = self._measure();
			dnd_constrainer.triggerHandler('resize');
			if(old_template){
				$('#main, #header').css(size)
				$('#header').css({display: 'block'});
			}
			self.dashboards && self.get_selected_dashboard2().set_width(size.width); // notify visible child of width changes
		}

		function on_tab_select(dashboard){

			o.currentBus && o.currentBus.send('tab-deactivated');

			o.currentBus = dashboard.bus;

			o.currentBus && o.currentBus.send('tab-activated', self._measure());

			require(['x-editable'], function() {
				self.nav.find('li a.editable').editable('disable');
			})

			$('.inform-dialogue').remove();
		}

		/*
		function verify_html(){
			var ok = true;
			$('#dashboard-set .ui-tabs-nav > li').each(function(){
				var id = $(this).find('a:first').attr('href').substring(1);
				$('#' + id).orelse(function(){ ok = false; console.warn('tab panel not found: ' + id); });
			});
			return ok;
		}
		if(!verify_html()) return; //prevent jquery ui throwing unhelpful error.
		*/

		self.element.find('.edit-box').each(function() {
			$('input', this).keyup(function() {
				//FIXME instant update no longer working
				$(this).parents(o.widgetSelector).find('h3').text(ECMWF.truncate($(this).val()), 20);
				$('#' + this.get_selected_dashboard().name).data('dashboard')._emit_changed();
			});

			$('ul.colors li', this).click(function() {
				var colorStylePattern = /\bcolor-[\w]{1,}\b/,
				thisWidgetColorClass = $(this).parents(o.widgetSelector).attr('class').match(colorStylePattern)
				if (thisWidgetColorClass) {
					$(this).parents(o.widgetSelector)
						.removeClass(thisWidgetColorClass[0])
						.addClass($(this).attr('class').match(colorStylePattern)[0]);
				}
				return false;
			});
		});
	},

	_measure: function() {
		if(this._size) return this._size;

		var page_width = window.outerWidth;
		var width = Math.min(User.is_forecaster() ? 10000 : 1040, page_width - 2 * hmargin);
		return this._size = {
			width: width,
			left: (page_width - width - 20) / 2
		}
	},

	get_selected_dashboard: function() {
		return this.options.dashboard_set.dashboards[this.options.dashboard_set.selected];
	},

	get_selected_dashboard2: function() {
		return this.dashboards[this.options.dashboard_set.selected];
	},

	get_selected_dashboard_element: function() {
		return this.tabs.find('#' + this.options.dashboard_set.dashboards[this.options.dashboard_set.selected].name);
	},

	destroy : function () {
		this.element.removeClass('dashboard');
	},

	addTab: function(done){
		var self = this, o = this.options;

		Tabs.add_dashboard(o.dashboard_set.name, function(dashboard) {
			console.log('dashboardSet.addTab: done.', dashboard);

			//dashboard.n_columns || (dashboard.n_columns = 3);

			o.dashboard_set.selected = o.dashboard_set.dashboards.length;
			o.dashboard_set.dashboards.push(dashboard);

			var content = self.tabs.div().dashboard({dashboard: dashboard});
			self.dashboards.push(content.data('uiDashboard'));

			dashboard.messageBus = Bus(content, self.options.messageBus);

			self._add_tab_nav(dashboard);

			self.tabs
				.tabs('refresh')
				.tabs('option', 'active', self.nav.find('li').length - 1);

			self._on_tab_nav_changed();
			self._emit_dashboard_set_changed(o.dashboard_set);

			if(done) done(dashboard);
		});
	},

	removeTab: function(name, done) {
		var self = this, o = this.options;

		var nav = self._nav_lookup(name);

		var widgets = this.dashboard_lookup_by_name(name).widgets;

		Tabs.remove_dashboard(name, function(){

			var index = nav.index();
			try{
				console.log('removeTab: removing ' + index, ' li=', nav);
				$(nav).remove();
				self.tabs.tabs('refresh');
			}
			catch(e) 
			{
				console.error(e);
			}

			$('#' + name).remove();

			/*
			$.each(self.dashboards, function(i, v) {
				if(v.options.dashboard.name === name){
					self.dashboards.splice(i, 1);
					return false;
				}
			});
			*/

			// TODO: Select previous ?
			o.dashboard_set.selected = index - 1 >= 0 ? index -1 : 0;
			o.dashboard_set.dashboards = $.grep(o.dashboard_set.dashboards, function(x) {
				return x.name !== name;
			});
			self.dashboards = $.grep(self.dashboards, function(v){ return v.options.dashboard.name !== name; });
			self._emit_dashboard_set_changed(o.dashboard_set);

			widgets.all(function(w, c, r){
				w.li.remove();
			})

			done && done();
		});
	},

	add_widget: function (args, done) {
		this.get_selected_dashboard_element().dashboard('add_widget', args, done);
	},

	remove_widget: function (dashboard_name, widget_name, done) {
		var dashboard = this.dashboard_lookup_by_name(dashboard_name);
		dashboard.remove_widget(widget_name, done);
	},

	move_widget: function (widget_name, src_name, dest_name, done) {
		var self = this, o = this.options;

		if(__DEV__ && src_name === dest_name) return console.error('cannot move widget to same dashboard');

		Tabs.Widget.move(widget_name, src_name, dest_name, function(result){
			console.log('dashboardSet: move done!');
			if(result.dashboards.src.name !== src_name) console.warn('move: src dashboard name mismatch');

			var src_dashboard_idx = self.lookup_dashboard_idx_by_name(src_name);
			var src_dashboard = o.dashboard_set.dashboards[src_dashboard_idx];

			if(src_dashboard){
				result.dashboards.src.messageBus = src_dashboard.messageBus;
				console.log('dashboardSet.move_widget: src: orig:', src_dashboard.name, src_dashboard);
				console.log('dashboardSet.move_widget: src: new:', result.dashboards.src.name, result.dashboards.src);

				//update the dashboard set:
				o.dashboard_set.dashboards[src_dashboard_idx] = result.dashboards.src;
			}

			done(result);
		});
	},

	_add_tab_nav: function(dashboard){
		var self = this;

		var li = this.nav.li()
			.a({href: '#' + dashboard.name, css: {display: 'inline-block'}})
			.text(dashboard.title)
			.each(function() {
				// xeditable is no longer fully compatible with jquery ui - as a workaround, widget is recreated on each click
				self._bind_rename($(this), dashboard);
			})
			.parent();

		this._add_close_button(dashboard, li);

		return li;
	},

	_bind_rename: function(label, dashboard) {
		dashboard  = dashboard || this.get_selected_dashboard();
		label.click(function(){
			require(['x-editable'], function() {
				label.parent().hasClass('ui-state-active') &&
					label
						.editable({
							title: 'Edit dashboard title',
							success: function(_, title) {
								(title !== dashboard.title) && $('#' + dashboard.name).dashboard('setTitle', title);
							}
						})
						.editable('show')
						.one('hidden', function() {
							label.editable('destroy');
						});
			});
		});
	},

	_on_tab_nav_changed: function() {
		//either a tab has been added/removed, or the selection has changed.

		var self = this, o = this.options;

		self.nav.find('li').each(function(){
			var tab = $(this);
			if(tab.hasClass('ui-state-active')){
				console.log('dashboardSet._on_tab_nav_changed: active=', tab.text());
				tab.data('ui-sortable') && tab.sortable('destroy');
			}
		});
	},

	reflow: function (cols) {
		var self = this;
		console.log('dashboardSet.reflow: n_columns=' + cols);
		if(cols < 1 || cols > 6) return;
		var dashboard = self.get_selected_dashboard2();

		$.logMessage('Changing to ' + cols + ' columns...');

		dashboard.set_n_columns(cols, function(){
			$.logMessage('Layout changed to ' + dashboard.options.dashboard.columns.length + ' columns');
			dashboard.on_layout_changed();
		});
	},

	n_columns: function (dashboard) {
		dashboard = dashboard || this.get_selected_dashboard();
		return dashboard ? dashboard.columns.length : 0;
	},

	_add_close_button: function(dashboard, li){
		var self = this, o = this.options;

		li
			.mouseenter(function(){
				if(o.dashboard_set.dashboards.length > 1){
					$(this).find('.ui-button').show();
				}
			})
			.mouseleave(function(){
				$(this).find('.ui-button').hide();
			})
			.div('.tab-nav-item').div()
				.button({text: false, icons: {primary: 'ui-icon-close-222'}}).removeClass('ui-corner-all ui-widget-content ui-state-default')
					.click(function(){
						if(dashboard.name === self.get_selected_dashboard().name){
							if(o.dashboard_set.dashboards.length > 1){ 

								ECMWF.confirm({
									title: 'Remove Dashboard',
									text: 'Dashboard <b>' + dashboard.title + '</b> will be removed, ok?',
									yes: function() {
										self.removeTab(dashboard.name);
									}
								});
							}
							return false;
						}

					}).hide();
	},

	_nav_lookup: function(name){
		//return the tab li element for the dashboard with the given name.
		return this.nav.find('> li a[href$=' + name + ']').parent();
	},

	dashboard_lookup_by_name: function(name){
		for(var i in this.dashboards){
			if(this.dashboards[i].options.dashboard.name === name) return this.dashboards[i];
		}
	},

	lookup_dashboard_idx_by_name: function(name){
		var o = this.options;

		for(var i in o.dashboard_set.dashboards){
			if(o.dashboard_set.dashboards[i].name === name) return i;
		}
		return -1;
	},

	_emit_dashboard_set_changed: $.throttle(1000, function(msg){
		this.options.messageBus.send('dashboard-set-changed', msg);
	})

});

	// used temporarily pending possible addition of side-wide application menu
	// The old dashboard menu uses the MenuPane widget - we change to using a more recent widget set, but which one?
	$.widget('ui.optionsMenu', {
		options: {
			menu_items: [],
			menu_class: ''
		},

		_init: function(){
			var self = this, o = self.options;

			function make_menu(){
				return $.map(o.menu_items, function(i){
					if(i.enabled && !i.enabled()) return;
					var classes = i.classes || [];
					if(i.activatable && !i.activatable()) classes.push('ui-state-disabled');
					return '<li data-value="' + i.name + '" class="' + classes.join(' ') + '"><a href="#">' + i.name + '</a>' + (i.icon ? '<span class="ui-icon ' + i.icon + '"/>' : '') + '</li>';
				}).join('');
			}

			self.element.div('.options ui-button keep')
				.button({label: 'Actions', icons: {primary: 'ui-icon-gear', secondary: 'ui-icon-triangle-1-s'}, text: false})
				.click(function(e) {
					if(!self.menu){
						self.menu = $('<ul>' + make_menu() + '</ul>')
							.addClass(o.menu_class)
							.appendTo($(this))
							.menu({
								select: function(e, ui){
									var ret; // callbacks should return false to keep the menu open

									var item = o.menu_items[ui.item.index()];
									item.select && (ret = item.select(e, item));
									(ret === false) && e.stopPropagation();
									e.preventDefault();
								}
							});

						$('body').click(function() {
							self.menu.hide();
						});
						return false;
					}else{
						if($(e.target).attr('role') !== 'menuitem'){
							self.menu.toggle();
							return false;
						}else{
							// jquery.ui menu does not select items that are already focused so it is done manually
							if(self.menu.is(':visible') && !$(e.target).parent().hasClass('ui-state-disabled')) self.menu.menu('select');
							e.preventDefault();
						}
					}
				});
		},

		addItems: function(items) {
			if(Array.isArray(items))
				$.merge(this.options.menu_items, items)
			else
				this.options.menu_items.push(items);
		},

		disableItem: function(name, disable) {
			this.menu && this.menu.find('[data-value="' + name + '"]')[disable ? 'addClass' : 'removeClass']('ui-state-disabled');
		}
	});

});
