define(['jquery', 'ajax_queue'], function($, Ajaxqueue) {

	// this is not a model proxy, it just provides convenience fns for retreiving and saving.
	// No data is stored here. Consumers must store the data themselves, or rely on the AjaxQueue cache.

	var Tabs = {

		_call: function(url, data, done) {
			done = done || $.noop;
			var dfd = $.Deferred();
			if(__DEV__ && !url){ // TODO do this using ajaxQueue.logErrors ?
				done({error: 'missing url'});
				dfd.resolve({error: 'missing url'});
				return dfd.promise();
			}

			Ajaxqueue({
				url      : url,
				json     : data,
				priority : 'very-high',
				success  : function(result) {
					done(result);
					dfd.resolve();
				},
				logErrors: url, //TODO: give a sensible message
				error    : function(result) {
					console.warn('Tabs.' + url + ': ', result);
					if(result.error){
						for(var i in result.error) console.log(result.error[i]);
						done(result);
						dfd.resolve({error: result});
					}
				}
			});
			return dfd.promise();
		},

		load: $.Deferred(function(df) {
			Ajaxqueue({
				url: '/backend/get-dashboard-set/',
				priority: 'vip',
				complete: df.resolve
			})
		}).promise(),

		add_dashboard: function(set_name, done) {
			console.log('Tabs.add_dashboard');
			Tabs._call('/backend/add-dashboard/', {set: set_name}, done);
		},

		remove_dashboard: function(name, done) {
			console.log('Tabs.remove_dashboard: ' + name);
			Tabs._call('/backend/remove-dashboard/', {name: name}, done);
		},

		get_dashboard: function(name, done) {
			Tabs._call('/backend/get-dashboard/', {name: name}, done);
		},

		save_dashboard: function(data, done) {
			console.log('save_dashboard: ' + data.title);
			Tabs._call('/backend/save-dashboard/', data, done);
		},

		save_dashboard_set: function(data, done) {
			console.log('Tabs.save_dashboard_set');
			Tabs._call('/backend/save-dashboard-set/', data, done);
		},


		Widget: {
			add: function(dashboard, widget, done) {
				console.log('Tabs.add_widget', widget);
				widget.dashboard = dashboard;
				Tabs._call('/backend/add-widget/', widget, done);
			},

			remove: function(name, done) {
				console.log('Tabs.remove_widget: ' + name);
				Tabs._call('/backend/remove-widget/', {name: name}, done);
			},

			move: function(widget_name, src, dest, done) {
				console.log('Tabs.move_widget: ' + widget_name, '. ', src + ' --> ' + dest);
				if(src !== dest){
					Tabs._call('/backend/move-widget/', {widget: widget_name, src: src, dest: dest}, function(result){
						done(result);
					});
				}else{
					console.warn('ignoring request to move to same dashboard.');
				}
			},

			save: function(data, done) {
				console.log('Tabs.Widget.save: ' + data.name);
				Tabs._call('/backend/save-widget/', data, done);
			},

			get: function(name, done) {
				//console.log('Tabs.get_widget: ' + name);
				return Tabs._call('/backend/get-widget/', {name: name}, done);
			}
		}
	};

	return Tabs;
});
