/*
 *  Demonstration application for the dashboard widget
 *  --------------------------------------------------
 *
 *  TODO automated tests should be moved to dashboard/test/test.js
 */
define(['./widgets/widget', './model', 'test-lib'], function(DashboardWidget, Tabs, Test) {
	$.framework.onReady(function dashboardTest(){

		var el = $('#content');
		var status = el.div({id: 'status'});

		Tabs.load.then(function(dashboard_set){
			var dashboard =
				el.div({'class': 'content', id: 'dashboard', css: {}}).prependTo(el)
					.dashboardSet({dashboard_set: dashboard_set});

			$('#dnd-constrainer').remove();

			new Test('Add/remove tabs', el, function start(){
				var n_iterations = 50;

				var test = this;
				var iter = 0;
				var add_remove_tabs_running = true;
				test.set_timeout(80000);

				var fnlist = [];
				for(var i = 0; i < n_iterations; i++) fnlist.push(add_remove);

				async.series(fnlist, function(){
					console.log('******** finished.');
					status.text('finished');
					test.finished();
				});

				function add_remove(done){
					if(add_remove_tabs_running){
						add_tab(function(){
							var tab_name = dashboard.find('.ui-tabs-panel:last').attr('id');
							remove_tab(tab_name, function(){
								iter++;
								done();
							});
						});
					}
				}

				function add_tab(done){
					status.text(iter + '/' + n_iterations + ' adding tab...');
					dashboard.dashboardSet('addTab', function(){
						setTimeout(function(){
							done();
						}, 200)
					});
				}

				function remove_tab(tab_name, done){
					status.text(iter + '/' + n_iterations + ' removing tab...');
					dashboard.dashboardSet('removeTab', tab_name, function(){
						setTimeout(function(){
							done();
						}, 500)
					});
				}
			});

			new Test('Add/remove widgets', el, function start2(){
				var n_iterations = 25;
				var i = 0;

				var test = this;

				var add_remove_widgets_running = true;

				var fnlist = [];
				for(var f = 0; f < n_iterations; f++) fnlist.push(add_remove);

				async.series(fnlist, function(){
					console.log('******** finished.');
					status.text('finished');
					test.finished();
				});

				function add_remove(done){
					i++;
					if(add_remove_widgets_running){
						test.set_timeout(15000);
						add_widget(function(widget_name){
							remove_widget(widget_name, function(){
								//status.text(i + ': waiting...');
								//setTimeout(function(){
									done();
								//}, 5000);
							});
						});
					}
				}

				function add_widget(done){
					//TODO the chart widget product triggers a memory leak (only if it is forwarded to the ajaxqueue).

					status.text(i + ': adding widget...');
					//dashboard.dashboard('add_widget', {widget: dashboardWidget.registry().chart}, function(widget){
					dashboard.dashboardSet('add_widget', {widget: $.extend({}, dashboardWidget.registry().chart, {blah: 'blah'
						//product: ECMWF.product_fixture <--- we cannot pass products into the dashboard - they come via the Tabs object. redefine it?
					})}, function(widget_name){
					//dashboard.dashboardSet('add_widget', {widget: dashboardWidget.registry().text}, function(widget){
						console.log('------------add_widget: name=', widget_name);
						setTimeout(function(){
							done(widget_name);
						}, 200)
					});
				}

				function remove_widget(widget_name, done){
					status.text(i + ': removing widget...');
					var widget = $('#' + widget_name);
					var dashboard = widget.parent().parent();
					var _n_widgets = n_widgets(dashboard);
					widget.find('.ui-icon-close').click();
					setTimeout(function(){
						$('.alert #_ok').click();
						Test.wait_for(function(){
							return (n_widgets(dashboard) === _n_widgets - 1);
						}).then(function(){
							done();
						});
					}, 1000)
				}

				function n_widgets(dashboard){
					return dashboard.find('.dashboardWidget').length;
				}
			});

			new Test('Make large dashboard', el, function start3(){
					var config = {
						n_widgets: 64
					}
					var test = this;
					test._clear_timeout();

					var fnlist = [];
					for(var i = 0; i < config.n_widgets; i++) fnlist.push(do_test);

					async.series(fnlist, function(){
						console.log('******** finished.');
						test.finished();
					});

					i = 0;
					function do_test(done){
						if(test.running){
							i++;
							test.status.text('' + i + ' of ' + config.n_widgets);
							add_widget(function(){
								done();
							});
						}
					}

					function add_widget(done){
						console.log('test.add_widget');

						dashboard.dashboardSet('add_widget', {widget: $.extend({}, dashboardWidget.registry().chart, {blah: 'blah'
							//product: ECMWF.product_fixture <--- we cannot pass products into the dashboard - they come via the Tabs object. redefine it?
						})}, function(widget_name){
							console.log('------------add_widget: name=', widget_name);
							setTimeout(function(){
								done(widget_name);
							}, 200)
						});
					}
				},
				function stop(){
				}
			);

			new Test('DashboardWidget create/destroy', el, function start_(){
					// test creation and destruction of a widget outside of the dashboard container.
					var config = {
						widget_type: 'chart'
						//widget_type: 'text'
						//widget_type: 'null'
					}
					var n_iterations = 25;
					var test = this;

					//clear previous tests:
					$('#dashboard').remove();

					console.log('_____________________________________');

					var fnlist = [];
					for(var j = 0; j < n_iterations; j++) fnlist.push(do_test);

					var options = {
						text: {data: {text: 'hello'}}
						//_owner: self, _dashboard: dashboard, dashboard: dashboard.name, messageBus: dashboard.messageBus, active: (dashboard === self._get_selected_dashboard())
					}

					var widget_class = DashboardWidget.findClass(config.widget_type);
					if(!test.assert((typeof $.fn[widget_class] !== 'undefined'), ('no such widget: ' + widget_class))) return;

					var container = el.div({'class': 'widget-container'});

					test.series(fnlist, function(err){
					});

					function do_test(done){
						if(test.running){
							test.iter++;
							test.status.text('' + test.iter + ' of ' + n_iterations);
							add_widget(function(){
								remove_widget(function(){
									test.assert(!container.data('dashboard'), 'element dashboard data not cleared');
									test.assert((ECMWF.n_properties(container.data()) < 2), 'element data not cleared ' + ECMWF.n_properties(container.data())); //there may be a jQueryNNN object.
									if(!test.running){
										console.log('data=', container.data());
									}
									done((!test.running && test.iter < n_iterations) ? {error: 'error'} : null);
								});
							});
						}
					}

					function add_widget(done){

						console.log('test.add_widget');

						//container = el.div({'class': 'widget-container'});

						container[widget_class](options[config.widget_type]);

						setTimeout(function(){ done(); }, 2000); //dont make too short or the widget wont get removed.
					}

					function remove_widget(done){
						container[widget_class]('destroy');
						//$('.widget-container').remove();
						setTimeout(function(){ done(); }, 100);
					}
				},
				function stop_(){
				}
			);

			new Test('Move widget column', el, function start__(){
					var config = {
						n_widgets: 1
					}
					var test = this;
					var iter = 0;

					var fnlist = [];
					for(var j = 0; j < config.n_widgets; j++) fnlist.push(do_test);

					async.series(fnlist, function(err){
						if (err) console.log('******** finished ERROR ****.', err);
						else console.log('finished ok.');
						test.finished();
					});

					function do_test(done){
						if(test.running){
							iter++;
							test.status.text('' + iter + ' of ' + config.n_widgets);

							var dashboard = $('.dashboard .ui-tabs-panel:visible');
							var widget = dashboard.find('.dashboardWidget:visible:first');
							dashboard.dashboard('move_widget', widget.attr('id'), 1);
						}
					}
				},
				function stop__(){
				}
			);
		});

		function button(text, clicked){
			return el.p().a(text).click(function(){
				clicked();
			});
		}
	});
});

