define(['jquery', 'bus', '../model', './dashboard_testable'], function($, Bus, Tabs, DashboardTestable){

	var bus = Bus();

	function DashboardSetTestable(o){
		for(var i in o) this[i] = o[i];
		this.test = null;
		//this.set = $('.dashboard'); //too early
		//this.set = $.Deferred(function(df){})
		this.n_dashboards = function(){
			return $('.dashboard ul.ui-tabs-nav > li').length;
		}
		this.change_dashboard = function(name){
			var self = this;
			return $.Deferred(function(df){
				var link = '.ui-tabs-nav a[href="#' + name + '"]';
				self.test.assert($(link).length === 1, 'bad dashboard name: ' + name);
				$(link).click();
				self.test.wait_for(function(){ return $(link).parent().hasClass('ui-state-active'); }, 'tab active').then(function(){
					df.resolve(self.test);
				});
			}).promise();
		}
		this.change_dashboard_by_title = function(title){
			// TODO this is not tested
			this.change_dashboard($('.ui-tabs-nav > li a').filter(function(){ if($(this).text() === 'title') return true }).attr('href').slice(1));
		}
		this.add_dashboard = function(name){
			var self = this, test = self.test;
			return $.Deferred(function(df){
				var dashboard;
				var n = self.n_dashboards();
				test.assert(self.n_dashboards() > 0, 'n_dashboards > 0');

				var a = $.Deferred(function(d){
					self.test._listen_once('dashboard-set-changed')._wait(function(){
						console.log('DashboardSetTestable: test A complete');
						test.assert(self.n_dashboards() === n + 1, 'n_dashboards: ' + self.n_dashboards() + ' != ' + (n + 1));
						d.resolve();
					});
				});
				var b = $.Deferred(function(d){
					$('.dashboard').dashboardSet('addTab', function(_dashboard){
						console.log('DashboardSetTestable.add_dashboard: B done', _dashboard);
						dashboard = $('#' + _dashboard.name).dashboard('instance');

						test.assert(dashboard._active() === true, "active not set:", dashboard.active); //check the active flag is set correctly

						var dashboard_set = dashboard.dashboard_set;
						var dashboards = dashboard_set.dashboards;
						var names = $.map(dashboards, function(v){ return v.options.dashboard.name; });
						test.assert(!!~$.inArray(_dashboard.name, names));

						d.resolve();
					});
				});
				$.when(a.promise(), b.promise()).then(function(){
					var dt = new DashboardTestable(test);
					dt.rename('Test').then(function(){
						df.resolveWith(self.test, [dashboard]);
					})
				})
			}).promise();
		}
		this.remove_dashboard = function(name){
			var self = this, test = this.test;
			self.validate();
			test.set_timeout(10000);
			var set = $('.dashboard').dashboardSet('instance');
			if(typeof name === 'object'){
				var dashboard = name;
				name = dashboard.options.dashboard.name;
			}else if(typeof name === 'string'){
				dashboard = $('#' + name).dashboard('instance');
			}else{
				dashboard = set.get_selected_dashboard2();
				name = dashboard.options.dashboard.name;
			}
			var title = dashboard.options.dashboard.title;
			if(title !== 'Untitled' && !~title.indexOf('Test')) return test.fail('will not remove non test dashboard: ' + title);
			test.assert(!!name, 'dashboard name not specified');
			test.assert(!!~$.inArray(name, $.map(dashboard.dashboard_set.dashboards, function(v){ return v.options.dashboard.name; })), 'cannot remove dashboard not in dashboard list');

			return $.Deferred(function(df){
				Tabs.get_dashboard(name, function(r){
					test.assert(!r.error, 'dashboard not present on server')

					var old_widgets = [];
					dashboard.widgets.all(function(w, c, r){ old_widgets.push(w); })
					function check_old_removed(done){
						var i = -1;
						(function next_widget(){
							var widget = old_widgets[++i];
							if(!widget){
								return done();
							}
							test.wait_for(
								function(){ console.log('    id=', widget.li.attr('id')); return $('#' + widget.li.attr('id')).length === 0; },
								'removed dashboard widget'
							).then(next_widget);
						})()
					}

					var n = self.n_dashboards();
					bus.clear('dashboard-set-changed').listen_once('dashboard-set-changed', function(){
						test.assert(self.n_dashboards() == n - 1, 'n_dashboards=' + self.n_dashboards() + ' expected ' + (n - 1));
						test.wait_for(
							function(){ return !~$.inArray(name, $.map(dashboard.dashboard_set.dashboards, function(v){ return v.options.dashboard.name; })) },
							'removed dashboard still in dashboard list' + name + ' ' + title)
						.then(function(){
							check_old_removed(function(){
								var active = set.get_selected_dashboard2();
								var widgets = []
								active.widgets.all(function(w, c, r){ widgets.push(w); });
								var i = 0;
								(function check_widget(){
									var widget = widgets[i];
									if(!widget){
										return df.resolveWith(test);
									}
									if(!widget.instance || !widget.instance.options.data || !widget.instance.options.data.product){
										i++;
										check_widget();
									}else{
										self.test.wait_for(
											function(){ var img = widget.li.find('img'); return img.length && img.attr('src') },
											function(){ var img = widget.li.find('img'); console.log('check_widget fail: img=', img); console.log('check_widget fail: widget=', widget); return 'check widget: ' + (i+1) + '/' + widgets.length + ': image: ' + img + ' src=' + img.attr('src'); }
										).then(function(){
											i++;
											check_widget();
										})
									}
								})()
							})
						})
					});
					$('.dashboard').dashboardSet('removeTab', name);
				})
			}).promise();
		}
		this.move_widget = function(widget, from, to){
			console.log('----------------------------------');
			console.log('dst.move_widget: widget=', widget);
			console.log('dst.move_widget: widget title=', widget.options.title);
			console.log('dst.move_widget: widget uuid=', widget.uuid);
			console.log('dst.move_widget: dashboard name: ', from.dashboard2.options.dashboard.name, '  --> ', to.dashboard2.options.dashboard.name);
			console.log('dst.move_widget: dashboard title: ', from.dashboard2.options.dashboard.title, '  --> ', to.dashboard2.options.dashboard.title);
			console.log('dst.move_widget: src_count=', from.dashboard2.widgets.count());
			var self = this, test = self.test;
			self.test.assert(!!widget, 'widget');
			var set = $('.dashboard').dashboardSet('instance');
			return $.Deferred(function(df){
				from.check()
					/*
					.then(function(){
						return self.validate_active_dashboard();
					})
					*/
					.then(self.bind(self.validate_active_dashboard))
					.then(function(){
						var n_src_widgets = from.n_widgets();
						var n_dest_widgets = to.dashboard2.widgets.count();
						from.$dashboard.prependTo('#' + to.dashboard.name + ' ul:first > li:first'); // emulate drag and drop
						from.$dashboard.dashboard('move_widget_to_other_tab', widget.options.name, to.dashboard.name, function(){
							test.wait_for(
								function(){ return from.n_widgets() === n_src_widgets - 1; },
								function(){ return 'src widget count: ' + from.n_widgets() + ' expected: ' + (n_src_widgets - 1) }
							).then(function(){
								self.test.assert( set.get_selected_dashboard().name === from.dashboard.name, 'selected-dashboard should be src dashboard');
								self.test.assert( from.$dashboard.data('uiDashboard')._active() === true,    'active should be set on src dashboard:', from.$dashboard.data('uiDashboard').active); //check the active flag is set correctly
								self.test.assert( to.$dashboard.data('uiDashboard')._active() === false,     'active should NOT be set on dest dashboard:', to.$dashboard.data('uiDashboard').active); //check the active flag is set correctly
								self.test.assert( to.dashboard2.widgets.count() === n_dest_widgets + 1, 'wrong widget count on destination: ' + to.dashboard2.widgets.count() + ' expected: ' + (n_dest_widgets + 1));

								var dest_widget = to.dashboard2.widgets.columns[0].rows[0]
								self.test.assert($('#' + widget.options.name).length < 2, 'duplicate widget')
								df.resolveWith(self.test);
							})
						})
				})
			}).promise();
		}

		this.validate = function(){
			var self = this, test = this.test;
			var set = $('.dashboard').dashboardSet('instance');

			// check for duplicate widgets
			var widgets = [];
			$.each(set.options.dashboard_set.dashboards, function(i, dashboard){
				$.each(dashboard.columns, function(j, c){
					$.each(c, function(k, w){
						if(!!~$.inArray(w, widgets)) test.fail('validate: duplicate widget: ' + w);
						widgets.push(w);
					})
				})
			})

			var widgets2 = []; // not a full list, only ones which have been shown
			$.each(set.dashboards, function(i, dashboard){
				dashboard.widgets.all(function(w, c, r){
					w.li && widgets2.push(w.li.attr('id'));
				})
			})

			// check for zombie widgets
			$('.dashboardWidget').each(function(i, v){
				var id = $(v).attr('id');
				if(!~$.inArray(id, widgets2)){
					console.log('  ', !!~$.inArray(id, widgets), v);
					test.fail('validate: zombie widget: ' + id);
				}
			});

			return this;
		}

		this.validate_active_dashboard = function(){
			var self = this, test = self.test;
			return $.Deferred(function(df){
				var set = $('.dashboard').dashboardSet('instance');
				var dashboard = set.get_selected_dashboard2();

				test.wait_for(
					function(){
						var ok = true
						dashboard.widgets.all(function(w, c, r){
							 if($('#' + w.widget.name).length !== 1) ok = false;
						})
						return ok;
					},
					function(){ return 'dom widget count: ' + $('#' + w.widget.name).length + ' (expected 1) widget=' + w.widget.name}

				).then(function(){

					var seen = []
					$('.dashboardWidget').each(function(){
						var a = $(this);
						var name = a.attr('id')
						if(!!~$.inArray(name, seen)){
							$('.dashboardWidget').each(function(i, v){
								if($(this).attr('id') === name){
									var dt = new DashboardTestable(test)
									var widget = dt.get_ui_instance($(v));//$(v).data('ui-chartDashboardWidget')
									if(widget){
										var dashboard = widget.options._dashboard;
										console.log(' * dupe: ', i, widget, dashboard.name, [dashboard.title]);
									}else{
										console.warn('TODO validate_active_dashboard: support other widget types');
									}
								}
							});
						}
						test.assert(!~$.inArray(name, seen), 'validate: duplicate widget: ' + name)
						seen.push(name)
					});

					var v = '.dashboardWidget:visible';
					test.wait_for(
						function(){ return $(v).length === dashboard.widgets.count()},
						function(){ return 'wrong visible widget count on ' + dashboard.options.dashboard.name + ': ' + $(v).length + ' (expected ' + dashboard.widgets.count() + ')'}
					).then(function(){
						$(v).each(function(){
							var a = $(this);
							var name = a.attr('id').substring(1)
							if(a.hasClass('chartDashoardWidget')){
								var w = a.chartDashoardWidget('instance');
							}
						})

						if(!dashboard.widgets.count()){
							test.wait_for('.empty-dashboard-content', 'empty message').then(function(){ df.resolveWith(test) });
						}else{
							df.resolveWith(test);
						}
					})
				})
			}).promise();
		}

		this.bind = function(fn) {
			var self = this;
			return function() {
				return fn.apply(self, arguments);
			};
		}
	}

	return DashboardSetTestable;
});
