define(['config', 'jquery', 'user_info', 'settings', 'basic_chart/tools', 'chart/plugins/parameter_select', 'bus', 'chart/layer', 'form/handlers/catalogue', 'form/handlers/catalogue2', './widget', 'form', 'chart', 'container', 'chart/plugins/title_overlay', 'thumbnail-tools', './chart.css'], function(ECMWF, $, User, Settings, Tools, ParameterSelect, Bus, Layer, ObjectListHandler, ObjectListHandler2, DashboardWidget){

var chart_options = {priority: 'normal', editable: false, show_latlong: false, show_title_overlay: true};

var product_select_handler_eccharts, product_select_handler_catalogue;
var projection_select_handler;

$.widget('ui.chartDashboardWidget', $.ui.dashboardWidget, DashboardWidget.register('chartDashboardWidget', {

	factory : {
		title : 'New chart widget',
		kind  : 'chart', // this is the widget type, not the chart type
		data  : {
			chart_type: 'eccharts', // temporary flag, perhaps only used during widget creation
			product: '2mT_wind30m'
		}
	},

	options: {
		data: { },
		has_help: true,
		has_external: true,
		fixed_size: false,
		availability: true,
		messageBus: Bus() // normally the dashboard bus - used as the parent for the local bus so we can deal with local and dashboard level messages in the same handler.
	},

	_init: function() {
		var self = this, o = this.options;

		o.editable = o.editable && User.is_forecaster();

		// TODO *** if is a new unsaved widget where product is specified by name, there is no way to tell what kind of chart it is
		var is_eccharts = this._is_eccharts()

		if(o.data.product && !is_eccharts){ // allow empty products to be editable
			o.editable = false
			if(__DEV__ && ECMWF.options.enable_experimental) o.editable = true; // for new selector widget
		}
		// disable open-in-eccharts for catalogue products
		if(o.original_path || !is_eccharts){
			o.has_external = false
		}
		if(is_eccharts){
			console.log('%cNEW CHART WIDGET - is eccharts! has_external=', 'background-color:#fec', o.has_external, ' editable=', o.editable, ' original_path=', o.original_path);
		}else{
			console.log('%cNEW CHART WIDGET - NOT eccharts: has_external=', 'background-color:#cef', o.has_external, ' editable=', o.editable, ' original_path=', o.original_path);
		}

		$.ui.dashboardWidget.prototype._init.call(this);

		var bus = self.chart_bus = self.element.messageBus(o.messageBus);
		var save = false;

		self.old_basetime_requested = false;

		if(!o.data.projection){
			o.data.projection = Settings.get('dashboard-chart-projection', 'europe');
			save = true;
		}

		chart_options.editable = User.is_forecaster();

		self._chart = self
			._update_height({width: this.element.width()})
			.content
				.chart($.extend({}, chart_options, {
					product: o.data.product,
					kind: is_eccharts ? 'chart' : 'interactive',
					projection: o.data.projection,
					messageBus: bus,
					availability: o.active && o.availability && (ECMWF.options.update_availability !== false),
					use_outline: false,
					preload_animations: true
				}))
				.thumbnailTools({selector: '.chart-clip'})
				.thumbnailTools('add', [
					['ui-icon-play',               function(){ bus.send('tool-animation-start'); }, 'Animate'],
					['ui-icon-pause',              function(){ bus.send('tool-animation-pause'); }, 'Pause animation'],
					['ui-icon-stop',               function(){ bus.send('tool-animation-stop');  }, 'Stop animation'],
					['ui-icon-carat-1-w',          function(){ bus.send('prev-step');            }, 'Previous time step'],
					['ui-icon-carat-1-e',          function(){ bus.send('next-step');            }, 'Next time step'],
					['ui-icon-seek-prev',          function(){ bus.send('prev-day');             }, 'Previous day'],
					['ui-icon-seek-next',          function(){ bus.send('next-day');             }, 'Next day'],
					['ui-icon-arrowthickstop-1-w', function(){ bus.send('first-step');           }, 'First time step'],
					['ui-icon-arrowthickstop-1-e', function(){ bus.send('last-step');            }, 'Last time step'],
					['ui-icon-arrowthick-1-s',     function(){ self.old_basetime_requested = true;
																			 bus.send('prev-base-time');       }, 'Previous base time'],
					['ui-icon-arrowthick-1-n',     function(){ bus.send('next-base-time');       }, 'Next base time']
				])
				.chart('instance');

		self._product = self._chart.product

		bus
			.listen('chart-content-update', function(_, msg) {
				if(msg.changed.get('all')){
					if(this._false){
						o.data.product = msg.product.get_save_state();
						if(!self.editing){
							self._emit_changed();
						}
					}
				}
			})
			.listen('chart-product-change', function(){
				self._notify_content_changed();
			})
			.listen('chart-product-availability', function(_, avail) {
				//console.log("BEST BASE TIME ", avail.best_base_time, " OLD CYCLE ", avail.old_cycle);
				if (avail.old_cycle && !self.old_basetime_requested && $.getUserPreference('dashboard_autoupdate')) {
					console.log('dashboardChartWidget.on_availability: auto_update enabled. updating...');
					self.bring_up_to_date();
				}
				else {
					self.out_of_date(avail.old_cycle);
				}

				//TODO clicking on disabled buttons is propogated to the chart in MSIE

				//TODO setting button state stops the tooltips from working (mouseover event no longer received)
				//     (it looks like the button is destroyed and recreated? though the tooltips are not destroyed).
				//     Tooltip is re-applied as a temporary workaround.
				/*
				self.content.find('.ui-icon-carat-1-w'         ).parent()
					.modeTooltip('destroy')
					.button(avail.current.previous(avail.valid_time.values) ? 'enable' : 'disable')
					.modeTooltip({content: 'Previous time step'});
				self.content.find('.ui-icon-carat-1-e'         ).parent()
					.modeTooltip('destroy')
					.button(avail.current.next(avail.valid_time.values) ? 'enable' : 'disable')
					.modeTooltip({content: 'Next time step'});
				self.content.find('.ui-icon-arrowthickstop-1-w').parent()
					.modeTooltip('destroy')
					.button(avail.current.previous(avail.valid_time.values) ? 'enable' : 'disable')
					.modeTooltip({content: 'First time step'});
				self.content.find('.ui-icon-arrowthickstop-1-e').parent()
					.modeTooltip('destroy')
					.button(avail.current.next(avail.valid_time.values) ? 'enable' : 'disable')
					.modeTooltip({content: 'Last time step'});
				self.content.find('.ui-icon-arrowthick-1-n'    ).parent()
					.modeTooltip('destroy')
					.button(avail.old_cycle ? 'enable' : 'disable')
					.modeTooltip({content: 'Next base time'});
				self.content.find('.ui-icon-arrowthick-1-s'    ).parent()
					.modeTooltip('destroy')
					.button(avail.current.previousBaseTime(avail.base_time.values) ? 'enable' : 'disable')
					.modeTooltip({content: 'Previous base time'});
				*/
			})
			.listen('start-animation', function() {
				if(o.active) self.content.chart('animator').start();
			})
			.listen('stop-animation', function() {
				if(o.active) self.content.chart('animator').stop();
			});

		self.element.helpProvider({getContent: function(){
			return self._get_help_content();
		}});

		self.content.on('resize', function(e, size) {
			self._chart.resize(size && size.width && {width: size.width, height: size.width * o.height});
			self._update_height(size);
		})

		if(save){
			setTimeout(function() {
				self._emit_changed();
			},500);
		}
	},

	bring_up_to_date: function() {
		this.content.chart('showLatestCycle');
	},

	activate: function(size) {
		var self = this;

		$.ui.dashboardWidget.prototype.activate.call(this);

		self._chart && (ECMWF.options.update_availability === false ? self._chart.getAvailability() : self._chart.checkAvailability(true)).redraw();

		setTimeout(function(){ self.chart_bus && self._update_height(size); }, 300);
	},

	deactivate: function() {
		var self = this, o = this.options;

		o.active = false;

		if(self._chart){
			self._chart.animator && self._chart.chart.animator().stop();
			self._chart.checkAvailability(false);
		}
	},

	_destroy: function() {
		var self = this;

		/* enabling this increases memory usage in MSIE
		self.chart_bus
			.forget('chart-content-update')
			.forget('chart-product-availability')
			.forget('start-animation')
			.forget('stop-animation');
		*/

		this.content.chart('instance') && this.content.chart('destroy');
		self.chart_bus.destroy();
		//FIXME destroyed busses do not get removed.
		self.chart_bus = null;

		/*
		self.element.removeData('chartDashboardWidget');
		var d = self.element.data();
		for(var i in d){
			console.log('* ' + i);
		}
		*/
	},

	edit: function(enable, form) {
		var self = this;
		console.log('chart.edit', (enable !== false));

		$.ui.dashboardWidget.prototype.edit.call(this, enable, form);

		self.content.data('uiChart').animator && self.content.chart('animator').stop;
		self.content.thumbnailTools('enable', enable === false)
	},

	_on_preview_open: function(el, height, width) {
		var self = this, o = this.options;

		var bus = Bus(el);
		el.css({overflow: 'hidden'}); // interactive charts fit to size

		o.messageBus.send('tool-animation-stop');

		var controls = el.div('.chart-controls')
		controls.div('.bar-2');

		self._chart_preview = {
			bus: bus,
			chart: el.div({css: {height: height - 80 - el.find('h3').outerHeight(true), width: width - 32}})
				.chart($.extend({}, chart_options, {
					product: o.data.product,
					kind: this._is_eccharts() ? 'chart' : 'interactive',
					time: self.content.chart('getTime'),
					priority: 'very-high',
					messageBus: bus,
					plugins: [
						{mixin: Tools, options: {parent: '.chart-controls .bar-2'}},
						{mixin: ParameterSelect, options: {controls: controls}},
					]
				}))
		};

		require(['time-navigator'], function() {
			(self._chart_preview.nav = el.div()).timeNavigator({
				messageBus: bus,
				show_bt: false,
				ready: function(){
					self._chart_preview.nav.find('.time-nav').trigger('layout', {width: self._chart_preview.chart.width() - 2});
				}
			});
		});
	},

	_on_preview_close: function(el) {
		var self = this, preview = self._chart_preview;

		preview.chart.data('uiChart') && preview.chart.chart('destroy');
		preview.nav.remove();

		if(preview.bus.listeners.listeners.length){
			console.warn('_on_preview_close: bus still has listeners. applying workaround...');
			preview.bus.forgetAll();
		}

		el.css({overflow: ''});
	},

	_get_form: function() {
		return (true || this._is_eccharts()) // is_eccharts is failing. for now we only support selecting eccharts products
			? this._get_form_eccharts()
			: this._get_form_catalogue()
	},

	_get_form_eccharts: function() {
		var self = this;

		function on_update(updated_item, selection){
			var r = {};
			r[updated_item._item.name] = selection;
			self.update_from_form(r);
		}

		product_select_handler_eccharts = product_select_handler_eccharts || $.newFormItemHandler(ObjectListHandler);
		product_select_handler_eccharts.on_update = on_update;

		projection_select_handler = projection_select_handler || $.newFormItemHandler(ObjectListHandler);
		projection_select_handler.on_update = on_update;

		return [
			{ type: 'catalogue', subtype: 'product', title: 'Product', name: 'product', handler: product_select_handler_eccharts},
			{ type: 'catalogue', subtype: 'projection', title: 'Projection', name: 'projection', handler: projection_select_handler}
		];
	},

	_get_form_catalogue: function() {
		// Only the product can be changed via the edit form.
		// To change projection and other variables, the user can make persistent changes from the preview window.
		var self = this;

		function on_update(updated_item, selection){
			var r = {};
			r[updated_item._item.name] = selection;
			self.update_from_form(r);
		}

		product_select_handler_catalogue = product_select_handler_catalogue || $.newFormItemHandler(ObjectListHandler2);
		product_select_handler_catalogue.on_update = on_update;

		return [
			{type: 'catalogue', subtype: 'product', title: 'Product', name: 'product', handler: product_select_handler_catalogue},
		];
	},

	/*
	 *  This test is not reliable, it is not possible to easily distinguish between eccharts charts and interactive charts
	 *
	 *  For now the temporary official test is to check for the original_path flag which means it is not an eccharts chart
	 *
	 *  note:
	 *     - we cannot use the widget type (as opposed to looking at the chart data) because the type is 'chart' for both eccharts and interactive.
	 *
	 *  TODO
	 *     - why can we not use has_widgets('interactive') ?
	 */
	_is_eccharts: function() {
		var o = this.options;

		if(o.original_path) return false;

		var product = o.data.product;

		if(!product) return true;

		// sometimes the products dont have state! state does not come from get_chart, it comes from the chart widget by calling ProductFactory?
		if(product.state){
			if(product.state.chart_type === 'interactive') return false;
			if(product.state.product && product.state.product.indexOf('uid_') === 0) return true;
			if(product.state.product === 'empty') return true;
		}

		return true;
	},

	open: function(){
		var o = this.options;
		window.location.href = ECMWF.options.site.forecaster + '?widget=' + o.name;
		return this;
	},

	_get_help_content: function() {
		var self = this, o = this.options;

		if(!self.content) return;

		var product = self.content.data('uiChart').product;

		return $.Deferred(function(dfd){
			var layers = [];
			$.each(product.layerList(new Layer().is_visible).reverse(), function(){
				var item = {};
				item.promise = new Layer(this).get_help_content(item, product);
				layers.push(item);
			});

			$.when.apply($, $.pluck(layers, 'promise')).then(function(){
				var markup = [];
				$.each(layers, function(i, item) {
					markup.push("**" + item.title + " layer**:");
					markup.push(item.help);
					markup.push('');
					if(item.legend) {
						markup.push("![legend](" + item.legend + ")");
						markup.push('');
					}
				});
				dfd.resolve({title: o.title, content: markup.join('\n')});
			});
		}).promise();
	},

	update_from_form: function(form){
		var self = this, o = this.options, product;

		if(form){
			if(form.projection){
				console.log('dashboard.chart.update_from_form: projection=', form.projection);
				self.content.chart('setProjection', o.data.projection = form.projection);
				Settings.set('dashboard-chart-projection', o.data.projection);
			}
			if((product = form.product)){
				if((typeof product !== 'string') && product.has_widget('basic')){
					$('.dashboard').dashboardSet('instance').get_selected_dashboard2().transmute_widget(self, product);
				}else{
					console.log('dashboard.chart.update_from_form: product=', product);
					self.content.chart('setProduct', o.data.product = product);
					Settings.set('dashboard-chart-product', o.data.product);
				}
			}
		}
	},

	serialize: function() {
		var s = $.ui.dashboardWidget.prototype.serialize.call(this);
		s.data.product = this.content.chart('getState');
		return s;
	},

	get_state: function(state) {
		var self = this;
		return self.content.chart('getState', true);
	},

	set_state: function(state) {
		var self = this;
		console.log('chart.STATE', state);
		self.content.chart('setState', state);
	},

	get_bus: function() {
		return this.chart_bus;
	},

	share: function () {
		Bus().send('product-send', {type: 'chart', product: this.options.data.product.name});
		return this;
	},

	on_show: function() {
		var self = this;
		self.content.resize();
		return this;
	},

	_update_height: function(size) {
		var o = this.options;

		if(size){
			var width = size.width; //size ? size.width : this.element.width();

			if(width){
				var new_height = o.height * width;
				if(new_height !== this._height){
					this._height = new_height;
					this.content.css({height: new_height});

					// cannot trigger a new content resize in a resize handler, will cause feedback loop
					//self.content.triggerHandler('resize', {size: {height: new_height}});

					this.get_bus().send('height-changed', this.widget_height());
				}
			}
		}
		return this;
	},

	widget_height: function () {
		var self = this, o = self.options;

		var border_height = 2;
		var header_height = 24;

		if(!self._height) console.warn('chart: _height not set');

this._true = true;
if(this._true) return (o.collapsed ? 0 : (self._height || self.content.height())) + border_height + header_height;

		return $.Deferred(function(dfd){
			setTimeout(function(){
				var height = self.content.height() + border_height;
				dfd.resolve(height);
			}, 2000);
		}).promise();
	},

	content_height: function() {
		var self = this;

		return self.content.height();
	}
}));

});
