define(['jquery', './widget', 'bus', 'utils', 'chart/model/parameters', 'basic_chart/parameter_controls/factory', './controls.css'], function($, DashboardWidget, Bus, Utils, Parameters) {

$.widget('ui.controlsDashboardWidget', $.ui.dashboardWidget, DashboardWidget.register('controlsDashboardWidget', {

	factory : {
		kind  : 'controls',
		title : 'Controls',
		data  : {}
	},

	options: {
		editable: false,
		messageBus: Bus()
	},

	_init: function() {
		var self = this, o = this.options, bus = o.messageBus;

		$.ui.dashboardWidget.prototype._init.call(this);

		self._height = 32;

		var param_bar = self.content.div('.chart-controls').div('.chart-parameter-bar')

		self.content.div('.tools')
			.thumbnailTools({selector: '.tools', always_on: true})
			.thumbnailTools('add', [
				['ui-icon-play',               function(){ bus.send('tool-animation-start'); }, 'Animate all charts'],
				['ui-icon-pause',              function(){ bus.send('tool-animation-pause'); }, 'Pause all animation'],
				['ui-icon-stop',               function(){ bus.send('tool-animation-stop');  }, 'Stop all animations'],
				['ui-icon-carat-1-w',          function(){ bus.send('prev-step');            }, 'Advance all charts to their previous time step'],
				['ui-icon-carat-1-e',          function(){ bus.send('next-step');            }, 'Advance all charts to their next time step'],
				['ui-icon-seek-prev',          function(){ bus.send('prev-day');             }, 'Step all products back by 24 hours'],
				['ui-icon-seek-next',          function(){ bus.send('next-day');             }, 'Advance all products by 24 hours'],
				['ui-icon-arrowthickstop-1-w', function(){ bus.send('first-step');           }, 'Set all charts to their first time step'],
				['ui-icon-arrowthickstop-1-e', function(){ bus.send('last-step');            }, 'Set all charts to their last time step'],
				['ui-icon-arrowthick-1-s',     function(){ bus.send('prev-base-time');       }, 'Advance all products to their previous base time'],
				['ui-icon-arrowthick-1-n',     function(){ bus.send('next-base-time');       }, 'Advance all products to their next base time'],
				['ui-icon-arrowthickstop-1-n', function(){ bus.send('bring-up-to-date');     }, 'Bring all products up-to-date']
			])

		bus.listen('dashboard-changed', update_parameters);

		bus.listen_once('tab-activated', function() {
			update_parameters()
		});

		function update_parameters() {
			return console.log('controls: parameter selectors disabled');
			param_bar.empty()

			var dashboard_set = $('.dashboard').dashboardSet('instance')
			var dashboard = dashboard_set.get_selected_dashboard2()

			var products = []
			self.charts = []
			var abort;
			dashboard.widgets.all(function(w, c, r){
				if(w.widget.kind !== 'controls'){
					if(!w.instance) return abort = true, false;
					w.instance._product && products.push(w.instance._product)
					w.instance._product && self.charts.push(w.instance._chart)
				}
			})
			if(abort) return;

			var ready = []
			$.each(self.charts, function(i, chart) {
				var product = chart.product;
				ready.push(new $.Deferred(function(df) {
					//if(product._loaded){ // TODO not passed by reference?
					if(product.data.img){
						df.resolve()
					}else{
						self.charts[i].bus.listen_once(self, 'chart-content-update', function() {
							df.resolve()
						})
					}
				}))
			})

			$.when.apply(self, ready).then(function() {

				var params = {}
				$.each(self.charts, function(i, chart) {
					var product = chart.product;
					$.each(product.variables, function(i, v) {
						if(i !== 'time') params[i] = true;
					})
				})
				console.log('controls: params=', $.map(params, function(v, k){ return k }));

				$.each(self.charts, function(i, chart) {
					chart.bus.forget_me(self)
					chart.bus.listen(self, 'product-swapped', update_parameters, true); // TODO interactive charts
				})

				self.common = {}
				$.each(params, function(pname, c) {
					var found = true
					$.each(self.charts, function(j, chart) {
						var w = chart.product;
						if(!w.variables[pname]) found = false
					})
					if(found) self.common[pname] = true
				})
				console.log('controls: common=', $.map(self.common, function(v, k){ return k }));

				self.content.trigger('resize', {width: self.element.width()})

				$.each(self.common, function(pname, c) {
					var p = self.charts[0].product.variables[pname]
					var labels = []
					self.common[pname] = $.map(p.values, function(val, i) {
						var found = true
						$.each(products, function(j, w) {
							var param = w.variables[pname]
							if(!~$.inArray(val, param.values)) found = false
						})
						if(found){
							p.labels && labels.push(p.labels[i])
							return val
						}
					})

					var v = {
						name: p.name,
						title: p.title,
						type: p.type,
						values: self.common[pname],
						labels: labels,
					}
					var param = new Parameters[(v.type === 'dynamic') ? 'dynamic' : 'string'](v)

					param_bar.div({id: param.name + '-navigator'})
						.basicControlFactory({
							parameter: param,
							instance: products,
							//bus: chart.bus,
							on_select: function(val){
								console.log('catalogue2.basic.control.on_select: ', val);
							}
						});

					self._height = 32 + param_bar.height()
				})
			})
		}

		self.content.on('resize', function(e, size) {
			var n_controls = Utils.n_properties(self.common);
			var is_narrow = size.width < 180;
			if(n_controls){
				is_narrow = size.width / n_controls < 100
			}
			self.element[is_narrow ? 'addClass' : 'removeClass']('narrow')
		})
	},

	_destroy: function() {
		var self = this;
		this.options.messageBus.forget_me(this)
		$.each(self.charts, function(i, chart){ chart.bus.forget_me(self) })
	},

	widget_height: function () {
		var border_height = 2;
		var header_height = 24;

		return this._height + border_height + header_height;
	}

}));


});
