define(['jquery', 'bus', './widget', 'settings', 'multigrams', 'form/handlers/point', 'form/handlers/location', 'markup'], function($, Bus, DashboardWidget, Settings){

	var Widget = {

		options: {
			data: {
				// products: ["10d_epsgram_2t"], location: {lat: 50, lon: 0},
			},
			has_help   : true,
			messageBus : Bus(),
			controls   : [
				['Previous base time', 'ui-icon-arrowthick-1-s', 'prev-base-time'],
				['Next base time',     'ui-icon-arrowthick-1-n', 'next-base-time']
			]
		},

		_init: function () {
			var self = this, o = this.options;

			$.ui.dashboardWidget.prototype._init.call(this);
			self.element.addClass('meteogramDashboardWidget_' + o.data.family);

			if (!o.data.location) {
				o.data.location = Settings.get_pref('default_location');
				self._emit_changed();
			}

			var bus = self.local_bus = self.element.messageBus(o.messageBus);

			self.gram = self.content
				.div({css: {overflow: 'hidden', paddingTop: 10, paddingBottom: 2}})
					.multigrams($.extend({
							width: o.width,
							messageBus: bus,
							show_more: false,
							auto_show_tools: false,
							availability: o.active
						},
						o.data)
					);

			bus
				.listen('multigrams-meteogram-ready', function(_, msg) {
					self._notify_content_changed();
				})
				.listen('multigrams-list-changed', function(_, msg) {
					o.data.products = msg.products;
					self._emit_changed();
				})
				.listen('meteogram-out-of-date', function(_, on) {
					self.out_of_date(on);
				});

			self.content.thumbnailTools({
				selector: '.multigrams'
			});

			$.each(o.controls, function(_, c) {
				self.content.thumbnailTools('add', c[1], function () { bus.send(c[2]); }, c[0])
			});

			self.content.resize(function(e, size) {
				self.gram.triggerHandler('resize', size);
			});

			if (!o.active) self.deactivate();

			self.element.helpProvider({getContent: function(){
				return self.gram.multigrams('getHelp', o.title);
			}});
		},

		edit: function (enable, form) {
			console.log('meteogram.edit');
			$.ui.dashboardWidget.prototype.edit.call(this, enable, form);
			this.gram.multigrams('edit', enable);
		},

		_get_form: function () {
			var self = this, o = this.options;

			function on_update(updated_item, selection) {
				console.log('meteogram.form.on_update', selection);
				self.update_from_form({
					'product': selection
				});
			}

			self.product_select_handler = self.product_select_handler || $.newFormItemHandler(ObjectListHandler);
			self.product_select_handler.on_update = on_update;

			function city_changed(pt) {
				self.update_from_form({location: pt});
				self._emit_changed();
			}

			return [{
				type: 'point',
				title: 'Location',
				name: 'location',
				value: o.data.location,
				city: city_changed
			}];
		},

		get_state: function() {
			var o = this.options;
			return { location : o.data.location, products : o.data.products } ;
		},

		set_state: function(state) {
			var self = this;
			self.gram.multigrams('setProducts', state.products);
			self.gram.multigrams('setLocation', state.location);
			return null;
		},

		_on_preview_open: function (el, height, width) {
			var self = this, o = this.options;

			o.messageBus.send('tool-animation-stop');

			self._preview = {
				gram: el.div({css: {
						height: height - 60,
						width: width - 32,
						marginTop: 8
					}})
					.multigrams($.extend({
							time: self.gram.multigrams('getTime'),
							messageBus: Bus(el),
							show_more: false,
							auto_show_tools: false
						},
						o.data))
			}
		},

		_on_preview_close: function () {
			var self = this;

			console.log('meteogram preview close');
																								console.log('TODO check data: ', self._preview.gram.data());
			self._preview.gram.data('uiMeteogram') && self._preview.gram.meteogram('destroy');
		},

		activate: function () {
			var self = this, o = self.options, multigrams;
			console.log('meteogram.activate');

			$.ui.dashboardWidget.prototype.activate.call(this);

			//self.chartBus.enable();

			if((multigrams = self.gram.data('uiMultigrams'))){
            self.gram.find('.meteogram:first .meteogram-image')
					.filter(function(){ return $(this).attr('src'); })
					.orelse(function() {
						multigrams.redraw();
					});
				multigrams.checkAvailability(true);
			}

			self.on_show();
		},

		deactivate: function () {
			var self = this, o = this.options;

			o.active = false;
			self.gram.data('uiMultigrams') && self.gram.multigrams("checkAvailability", false);
			//self.chartBus.disable();
		},

		update_from_form: function (form) {
			var self = this, o = this.options;

			if (form) {
				o.data = $.extend(o.data, form);
				self.gram.multigrams('setLocation', o.data.location);
			}
		},

		on_show: function () {
			var self = this;

			self.gram.triggerHandler('resize');
			return this;
		},

		bring_up_to_date: function () {
			var self = this;

			self.out_of_date(false); // TODO, move elsewhere
			self.gram.multigrams('showLatestCycle');
		},

		/*
		_update_height: function() {
			var self = this;

			var width = self.element.width();
			var height = self.content.height();
			if(width){
				this.get_bus().send('height-changed', this.widget_height());
			}
			return this;
		},
		*/

		content_height: function() {
			var self = this;

			return self.content.height();
		},

		share: function () {
			var o = this.options;
			Bus().send('product-send', {type: 'meteograms', product: o.data});
			return this;
		},

		dont_serialize : function() {
			return $.merge($.merge([], $.ui.dashboardWidget.prototype.dont_serialize.call(this)), ['controls']);
		}
	};

	$.widget('ui.meteogramDashboardWidget', $.ui.dashboardWidget, Widget);

	var Families = [
		{
			family: '10_days',
			title: '10 days',
			products: ['10d_epsgram_2t'],
		},
		{
			family: '15_days',
			title: '15 days',
			products: ['15d_epsgram_2tmax'],
		},
		{
			family: 'efi',
			title: 'EFI/CDF',
			products: ['efi_2t'],
			controls: [
				['Previous cycle', 'ui-icon-arrowthick-1-s', 'prev-base-time'],
				['Next cycle',     'ui-icon-arrowthick-1-n', 'next-base-time'],
				['Previous day',   'ui-icon-seek-prev',      'prev-day'],
				['Next day',       'ui-icon-seek-next',      'next-day']
			]
		},
		{
			family: 'extended',
			title: 'Extended range',
			products: ['extended_anomaly_clim_2t']
		}
	];

	$.each(Families, function (_, f) {

		var name = 'meteogramDashboardWidget_' + f.family;

		// location=null will give user's preference
		var Widget = {

			factory: {
				title: "New " + f.title + " meteogram widget",
				kind: "meteogram_" + f.family,
				data: {
					location: {lat: 0, lon: 0},
					products: f.products,
					family: f.family
				}
			},

			options: {
				controls: f.controls,
				data: {
					location: null,
					products: f.products,
					family: f.family
				}
			}
		};

		$.widget('ui.' + name, $.ui.meteogramDashboardWidget, DashboardWidget.register(name, Widget));
	});

	//FIXME needs to be moved - having the handler private to the dashboard-widget breaks testability.
	function ObjectListHandler() {

		this.render = function (item) {
			var self = this;
			var div = $('<div/>').a('Select ' + item._item.subtype + ' ...').css({
				position: 'relative',
				top: '0.5em'
			}).attr('product', item.value).click(function () {
				$(this).thumbnailDialogue({
					text: 'Please select one of these ECMWF ' + item._item.title + 's',
					kind: item._item.subtype,
					actions: [{
						name: 'Select',
						on_click: function (selection) {
							$('.filterable-thumbnail-list').dialog('close');
							div.attr({
								product: selection
							});
							if (self.on_update) self.on_update(item, selection);
						}
					}],
					hide_user_items: false,
					modal: true
				});
			});
			return div;
		};

		this.collect = function (item, widget) {
			return widget.attr('product');
		};
	}
});
