/*
 *  HTML Selector replacement widget for dates.
 *
 *  wraps jquery-ui.selectmenu
 *
 *  The height of the popup window is sized to fit both its content, and the document window.
 *  The width of the popup attempts to match its controlling button, so there can be issues if the content is wide.
 *
 *  TODO revise the code for selecting the current item once we upgrade to ui-1.11
 */
define(['jquery', 'jquery-ui/selectmenu', './date_select.css'], function($) {

$.widget('ui.dateSelectmenu', $.ui.selectmenu, {
	_renderMenu: function(ul, items) {
		var self = this;

		ul.addClass('date-select-menu');

		self._prev_day = null;
		$.each(items, function(i, item) {
			self._renderItemData(ul, item);
		});
	},

	_renderItem: function(ul, item) {
		var self = this;

		var date = this.dates[item.index]
		var day = date.format('ddd d mmm');
		var is_new_day = day !== this._prev_day;
		this._prev_day = day;

		var label = item.label.split(', ');

		return ul.li('<span class="day">' + (is_new_day ? label[0] : '') + '</span>' + label[1]).each(function() {
			date === self.current && $(this).addClass('selected');
			date.format('ddd') === 'Sun' && $(this).find('.day').css({color: '#f00'});
		});
	}
});

$.widget('ui.dateSelect', {
	options: {
		mode: 'enhanced',
		decorator: 'shadow',
		title: '',
		position: {my: 'left top', at: 'left bottom', collision: 'none'}, // TODO remove this default
		select: $.noop,
		on_open: $.noop,
		on_close: $.noop
	},

	_init: function() {
		var self = this, o = this.options;

		this.current_date = null;
		this.element.addClass('date-select');

		this.setDates();
	},

	destroy: function(){
		this.element.removeClass('date-select');
	},

	setDates: function(dates, current){
		var self = this, o = self.options;
		dates = dates || [];

		if(dates.length && this._lookup_date_index(current, dates) < 0){ console.warn('setDates: bad arg: current - not found in list.'); this.current_date = dates[0];}

		this._menu && (this._menu.destroy(), this.element.find('select').remove());

		this._menu = this.element
			.formSelect({
				label: o.title,
				options: $.map(dates, function(v) {
					return self._date_to_string(v);
				})
			})
			.val(self._date_to_string(current))
			.dateSelectmenu({
				position: o.position,
				select: function(event, ui) {
					console.log('select.select: ui.item=', ui.item);
					console.log('select.select: date=', self._menu.dates[ui.item.index]);
					o.select(self._menu.dates[ui.item.index]);
				},
				open: function() {
					o.on_open();
				},
				close: function() {
					o.on_close();
				}
			})
			.data('ui-dateSelectmenu');

		this._menu.dates = dates;
		this._menu.current = current;
	},

	setDate: function(date){
		if(!this._menu.current || (date.getTime() !== this._menu.current.getTime())){
			this._menu.current = date;
			this._menu.refresh();
			this.element.find('.ui-selectmenu-text').text(this._date_to_string(this._menu.current));
		}
		return this;
	},

	_date_to_string: function(date){
		if(!date) return;
		return date.format('ddd d mmm, HH') + 'Z';
	},

	_lookup_date_index: function(date, dates){
		for(var i in dates){
			if(!(date - dates[i])) return i;
		}
		return -1;
	},

	_enable: function(){
	},

	_disable: function(){
	}
});

});

