/*jslint white: false, nomen: false, onevar: false, forin: false, browser: true, devel: true, immed: false */
/*global jQuery window ECMWF */

/*
 *  Widget to wrap an element in a table showing a border with a drop-shadow.
 *
 *  features:
 *    -bitmap-based styling for versatility, but still allows arbitrary content size.
 *    -browser-based automatic resizing following content change.
 *
 *  The widget works by wrapping the content in a table withe the outer rows/columns being used for styling.
 *  For several reasons[1], reparenting this.element is problematic, so a child element (with class 'content')
 *  is required in the wrapped element. This is inconvenient, but as far as is known, not a serious limitation.
 *
 *  [1] jquery doesnt allow a widget to change 'this.element', and we cannot make use of the parent element as
 *  the element may not be added to the page DOM (ie it has no parent).
 *
 *  MSIE 8 still has problems with transparent pngs (!), notably when using css opacity.
 */
define(['jquery', 'jquery-ui/widget'], function($) {

var html = '<table class="glow-shadow" border="0" cellspacing="0" cellpadding="0" style="">' +
	'<tbody><tr>' +
		'<td id="topleft" class="corner" style=""></td>'  +
		'<td class="top"></td>' +
		'<td id="topright" class="corner"></td>' +
	'</tr>' +

	'<tr>' +
		'<td width="19" class="left">&nbsp;</td>' +
		'<td><div class="contents">' + '</div></td>' +
		'<td class="right">&nbsp;</td>' +
	'</tr>' +

	'<tr>' +
		'<td class="corner" id="bottomleft"></td>' +

		'<td class="bottom">' +
			//'<img width="30" height="29" alt="popup tail" src="' + /*ecmwf.urls.img_path*/'/wrep/pics/'  + '../contrib/bubbleinfo/images/bubble-tail2.png"/>' +
		'</td>' +
		'<td id="bottomright" class="corner"></td>' +
	'</tr>' +
	'</tbody></table>';

$.widget("ui.shadow_box", $.extend({}, {

	_init: function(){
		var content = this.element.find('.content');
		if(!content.length){ console.warn('shadow_box decorator: element must contain an element with class=content'); return; }

		var box = $(html)
			.attr({id:this.options.id})
			.css('display', 'block')
			//.css('opacity', 0);
			.appendTo(this.element);

		var new_content_location = box.find('.contents');

		content.appendTo(new_content_location).end();

		//make slight adjustments to the size and position:
		var position_type = this.element.css('position');
		if(!position_type || (position_type == 'absolute')){ //webkit returns empty string even when absolute. because div is hidden?
			var left = this.element.css('left');
			if(!left || (left == 'auto')) left = 0;
			//console.log('shadow_box: absolute. left=' + left + ' element=' + this.element);
			this.element.css('left', parseInt(left, 10) - 8);
			this.element.css('right', -8);
		}else{
			left = this.element.css('margin-left');
			//console.log('shadow_box: not absolute. left=' + left + ' type=' + typeof(left));
			left = (!left || (left === 'auto')) ? -8 : (parseInt(left, 10) - 7);
			//console.log('shadow_box: not absolute. left=' + left);
			this.element.css('margin-left', left);
		}
	},

	contents: function(str){
		this.element.find('.contents').html(str);
	}
}));

$.extend($.ui.shadow_box, {
	defaults: {
	}
});

});
