/*
 * Create and append new dom elements.
 */
define(['config', 'jquery', 'utils', 'ajax_queue', 'settings'], function(ECMWF, $, Utils, Ajaxqueue, Settings){

function _elem(type, args, par){

	var options = (args && typeof args !== 'string' ) ? args : {};

	var class_name = ECMWF.remove(options, 'className');
	class_name && (options['class'] = class_name);

	var el = $('<' + type + '>', typeof args === 'string' ? undefined : options).appendTo($(par));
	if(typeof args === 'string'){
		if(args[0] === '.')
			el.addClass(args.slice(1));
		else if(args[0] === '#')
			el.attr({id: args.slice(1)});
		//for security reasons, allowing html in elements is restricted.
		else if($.inArray(type, ['h3', 'h4']) > -1)
			el.text(args);
		else
			el.html(args);
	}else{
		if(Array.isArray(args)){
			$.each(args, function(i, child){
				child.appendTo(el);
			});
		}
	}
	return el;
}

$.fn.div = function(args){

	return _elem('div', args, this);
};

$.fn.section = function(args){

	return _elem('section', args, this);
};

$.fn.ul = function(args){

	return _elem('ul', args, this);
};

$.fn.li = function(args){

	return _elem('li', args, this);
};

$.fn.p = function(args){
	return _elem('p', args, this);
};

$.fn.a = function(args){
	return _elem('a', args, this);
};

$.fn.span = function(args){
	return _elem('span', args, this);
};

$.fn.br = function(args){
	return _elem('br', args, this);
};

$.fn.hr = function(args){

	//var options = $.extend({}, args || {});

	return $('<hr/>', args).appendTo($(this));
};

$.fn.img = function(args){
	args = args || {};

	var json_url = ECMWF.remove(args, 'json_url');

	var img = $('<img/>', $.extend({}, args)).appendTo($(this));

	if(json_url){
		Ajaxqueue({
			url: json_url,
			img: img
		});
	}

	return img;
};

$.fn.h2 = function(args){
	return _elem('h2', args, this);
};

$.fn.h3 = function(args){

	var options = (args && typeof args !== 'string' ) ? args : {};

	var h3 = $('<h3>', $.extend({}, options || {})).appendTo($(this));
	if(typeof args === 'string' || typeof args === 'number') h3.html(args);
	return h3;
};

$.fn.h4 = function(args){
	return _elem('h4', args, this);
};

$.fn.h5 = function(args){
	return _elem('h5', args, this);
};

$.fn.table = function(_args){
	var args = _args || {};
	var headers = args.headers;
	delete args.headers;

	var table = $('<table>', $.extend({}, args)).appendTo($(this));

	if(headers){
		var tr = $('<thead>').appendTo(table).tr();
		for(var i in headers) tr.th(headers[i]);
	}
	return table;
}

$.fn.tbody = function(args){
	return _elem('tbody', args, this);
}

$.fn.tr = function(args){

	return _elem('tr', args, this);
};

$.fn.th = function(args){

	return _elem('th', args, this);
};

$.fn.td = function(args){

	return _elem('td', args, this);
};

$.fn.canvas = function(args){

	return _elem('canvas', args, this);
};

$.fn.object = function(args){

	return _elem('object', args, this);
};

$.fn.iframe = function(args){

	return _elem('iframe', args, this);
};

$.fn.formElement = function(args){

	var options = $.extend({}, args || {});

	return $('<form/>', options).appendTo($(this));
};

$.fn.input = function(args){

	return _elem('input', $.extend({'type': 'text'}, args || {}), this);
};

var radio_group = 0;
$.fn.radio = function(args){
	// create a complete radio group.
	//
	// args:
	// - buttons      - array (mandatory).
	// - labels       - array (optional). If not present, label will be same as value.
	// - group        - string (optional). Use the given group name.
	// - user_setting - string (optional). name of setting to get and set.
	// - checked      - integer (optional). index of element to select.

	var options = $.extend({}, args || {});
	var el = (this);

	var to_select = -1;

	if(options.user_setting){
		var current_setting = Settings.get(options.user_setting);
		$(options.buttons).each(function(i){
			if(options.buttons[i] === current_setting) to_select = i;
		});
	}

	if(typeof args.checked !== 'undefined'){
		to_select = args.checked;
	}

	if(to_select < 0) to_select = 0;

	var group = (options.group || ('radio-' + radio_group));
	$(options.buttons).each(function(i){
		var id = group + '-' + radio_group + '-' + Math.floor(Math.random() * 100000000);
		var o = {type: 'radio', id: id, name: group, value: this};
		if(to_select > -1 && i === to_select) o = $.extend(o, {checked: true}); // msie7 requires checked to be set at creation for hidden elements.
		el.input(o);
		options.labels && el.append('<label for="' + id + '">' + (options.labels[i] || this) + '</label>');
	})

	radio_group++;

	if(options.user_setting){
		el.click(function(e){
			Settings.set(options.user_setting, $(this).find('input:radio:checked').val());
			e.stopPropagation();
		});
	}

	return this;
};

$.fn.formSelect = function(args){
	// if creating a new select element, a child element is created.
	// if updating, the existing element is used.
	// args.options is an optional array. Each array member can either be a string, or array(value, title)

	var options = $.extend({}, args || {});
	var el;

	var _items = Array.isArray(options.options)
		? Array.isArray(options.options[0])
			? options.options
			: $.map(options.options, function(a){ return [[a, a]]; })
		: null;

	function add_items(){
		for(var i in options.options){
			el.append('<option value="' + _items[i][0] + '">' + _items[i][1] + '</option>');
		}
		return el;
	}

	if(options.replace){
		el = this.empty();
		return add_items();
	}

	var label = ECMWF.remove(options, 'label');
	label && this.append('<label for="' + options.id + '">' + label + '</label>');

	el = $('<select class="form-select">');
	return add_items().appendTo($(this));
};

var checkbox_id = 0;
$.fn.checkbox = function(args){

	var options = $.extend({
		'type'  : 'checkbox',
		checked : false,
		id      : 'checkbox-' + checkbox_id++
	}, (args || {}));

	var label = ECMWF.remove(options, 'label');
	var user_setting = ECMWF.remove(options, 'user_setting');
	var changed = ECMWF.remove(options, 'changed');

	user_setting && (options.checked = Settings.get(user_setting, false));
	var checked = ECMWF.remove(options, 'checked');

	var checkbox = _elem('input', options, this).prop('checked', checked);
	label && this.append('<label for="' + options.id + '">' + label + '</label>');

	if(user_setting){
		var setting = Settings.get(user_setting, false);
		checkbox.click(function(){
			setting = !setting;
			Settings.set(user_setting, setting);
			changed && changed(setting);
		});
	}

	return checkbox;
};

$.fn.textArea = function(args){

	return _elem('textarea', args, this);
}

$.fn.label = function(args){
	return _elem('label', args, this);
}

var Dom =  {
	div: function(o) {
		return $('<div>').div(o);
	},
	p: function(o) {
		return $('<div>').p(o);
	},
	span: function(o) {
		return $('<div>').p(o);
	},
	h2: function(o) {
		return $('<div>').h2(o);
	},
	h3: function(o) {
		return $('<div>').h3(o);
	},
	h4: function(o) {
		return $('<div>').h4(o);
	},
	br: function(o) {
		return $('<div>').br(o);
	},
	hr: function(o) {
		return $('<div>').hr(o);
	},
	ul: function(o) {
		return $('<div>').ul(o);
	},
	li: function(o) {
		return $('<div>').li(o);
	},
	textArea: function(o) {
		return $('<div>').textArea(o);
	}
};

return Dom;

});

