// based on ellipsis plugin http://devongovett.wordpress.com/2009/04/06/text-overflow-ellipsis-for-firefox-via-jquery/ + comments + custom mods
//
// -unlike many ellipsis plugins, this one can work on multiple-line blocks.
// -the actual ellipsis operation is not carried out until a doEllipsis event is received.

define(['config', 'jquery'], function(ECMWF, $) {

	$.fn.ellipsis = function(args) {

		var o = $.extend({
			container:      null, //constrain the text to fit inside this parent element.
			moreText:       null, //show a link that expands the element to show the complete text.
			lessText:       null,
			enableUpdating: true
		}, args);

		return $(this).each(function() {
			var el = $(this).addClass('ellipsis');
			if (!(el.css('overflow') === 'hidden' || (o.container && o.container.css('overflow') === 'hidden'))) {
				console.warn('ellipsis: element ignored - no overflow:hidden');
				return;
			}

			var originalText = el.html();
			var oldW;
			var oldH;
			bind();

			function bind(){
				el.on('doEllipsis', function() {
					if (el.width() != oldW || (o.lines && el.height != oldH)) {
						do_ellipsis();
					}
				});
			}

			function do_ellipsis(){
				el.html(originalText);

				var availWidth = el.width();
				var availHeight = el.height();
				var container;
				if ((container = o.container)) {
					availHeight = container.height();
					container.children().each(function(){
						if(this != el[0]){
							availHeight -= $(this).outerHeight(true);
						}
					}); 
				}

				var MoreLessTag = '';
				if (o.moreText) {
					o.enableUpdating = true;
					MoreLessTag = ' <a class="MoreLessTag" href="#" >' + o.moreText + "</a>";
				}

				// warning! css properties are not cloned, so do not set properties on the element that will affect the height
				var toClone = container ? container[0] : el[0];
				var t = $(toClone.cloneNode(true))
					.hide()
					.css({
						'position': 'absolute',
						'overflow': 'visible',
						'max-width': 'none',
						'max-height': 'none',
						lineHeight: $(toClone).css('lineHeight') // will only work for line-height set on the container, not on the element.
					});

				if (o.lines) t.css({height: 'auto'}).width(availWidth - 7);
				else t.css({width: 'auto'});
				el.after(t);

				//var fullHeight = t.height();

				var avail = (o.lines) ? availHeight : availWidth;
				if(o.max) avail = o.max;
				var test = (o.lines) ? t.height() : t.width();
				var foundMin = false, foundMax = false;
				if (test > avail) {
					//Binary search style trimming of the temp element to find its optimal size
					var min = 0;
					var max = originalText.length;
					while (min <= max) {
						var trimLocation = (min + max) / 2;
						var text = originalText.substr(0, trimLocation);
						t.html(text + "&hellip;" + MoreLessTag);

						test = (o.lines) ? t.height() : t.width();
						if (test > avail) {
							if (foundMax) foundMin = true;

							max = trimLocation - 1;
							if (min > max) {
								//If we would be ending decrement the min and regenerate the text so we don't end with a
								//slightly larger text than there is space for
								trimLocation = (max + max - 2) / 2;
								text = originalText.substr(0, trimLocation);
								t.html(text + "&hellip;" + MoreLessTag);
								break;
							}
						}
						else if (test < avail) {
							min = trimLocation + 1;
						}
						else {
							if (foundMin && foundMax && ((max - min) / max < 0.2)) break;
							foundMax = true;
							min = trimLocation + 1;
						}
					}
				}

				if(o.container){
					if(trimLocation){
						if(originalText.length - trimLocation < 3) el.html(originalText);
						else el.html(originalText.substr(0, trimLocation) + "&hellip;" + MoreLessTag);

						if(ECMWF.browser.msie) bind();
					}
				}
				else el.html(t.html());
				t.remove();

				if (o.moreText) {
					jQuery('.MoreLessTag', this).click(function(event) {
						event.preventDefault();
						el.html(originalText);
						/*
						embeddedPlayerManager.getPlayer().hidePlayer();
						el.animate({ height: fullHeight }, 'normal', null, function() {
							embeddedPlayerManager.manager.updatePositions();
							embeddedPlayerManager.getPlayer().unHidePlayer();
						});
						el.append(' <a class="MoreLessTag" href="#" >' + o.lessText + '</a>');
						$('.MoreLessTag', el).click(function(event) {
							event.preventDefault();
							resetDescription(availHeight, originalText);
						});
						*/
					});
				}
				else {
					var replaceTags = new RegExp(/\s*<\/?[^>]+>\s*/gi);
					var escaped = originalText.replace(replaceTags, ' ').replace(/\s*&nbsp;\s*/g, ' ')
					el.prop({alt: escaped, title: escaped});
				}

				oldW = el.width();
				oldH = el.height();
			}
		});
	};
});


