/*
 *  A widget displaying a list of selectable facets
 *  -----------------------------------------------
 *
 *  The facets are now passed as an argument (orginally were taken from the 'tags' property of each member of the given products collection).
 *  The facet values are now required to be arrays of strings instead of strings.
 *
 *  If the facets list changes, just call facets() again on the same element
 */
define(['jquery', 'utils', 'observable', 'virtual-dom/h', './facets.css'], function($, Utils, Observable, h) {

var Facets = function(options) {
	this.options = $.extend({
		products: null, // the product list is needed to provide the facet count.
		facets: null,   // ordered list of facet objects
		renderer: undefined,
		selection: {},  // the initial facet selection
		on_change: $.noop
	}, options);

	var self = this, o = self.options;

	self._active = {}; // flags to indicate facets that have selections.
	self._cache_view = {};
	self._initial_render_done = false;
	self._total = null;

	this.change = $.extend(new Observable(), {
		have: {
			timeout: $.Deferred(function(df) {
				setTimeout(df.resolve, 0);
			}),
			update_queued: false
		},
		req: function() {
			if(!this.have.update_queued){
				this.have.update_queued = true;

				this.have.timeout.done(function() {
					self.change.emit();
				});
			}
		}
	});

	self.change.subscribe(function() {
		self._on_filter_change();
		this.have.update_queued = false;
	});

	$.each(o.selection, function(f, vals) {
		$.each(vals, function(i, v) {
			o.on_change(f, v, true);
		});
	});

	this.change.req()
}

Facets.prototype = {
	clear: function() {
		this._active = {};
		for(var i in this._tree){
			var facet = this._tree[i]
			for(var j in facet.vals){
				if(facet.vals[j][1]){
					facet.vals[j][1] = false
				}
			}
		}
		this.change.req()
	},

	render: function(children) {
		var self = this, o = self.options;

		self._cache || self._analyze();
		self._tree || self._create_tree();

		var n_visible_outer = 0;

		return h('.facets', [h('ul', (function() {

			var children = [];
			$.each(self._tree, function(ii, facet) {

				var c = {
					children: [
						h('h3', {
							onclick: function(e) {
								console.log('click');
								var h3 = $(e.target);
								h3.next().slideToggle();
								h3.find('.ui-icon-triangle-1-s').removeClass('ui-icon-triangle-1-s').addClass('ui-icon-triangle-1-e')
									.orelse(function() {
										h3.find('.ui-icon-triangle-1-e').removeClass('ui-icon-triangle-1-e').addClass('ui-icon-triangle-1-s');
									});
							}
						},
						[
							h('span.ui-icon.ui-icon-triangle-1-s'),
							facet.name
						])
					]
				};
				var values = [];
				var show_all = self._cache_view[facet.name] && self._cache_view[facet.name].show_all;

				var n = 0, n_visible = 0;
				$.each(facet.vals, function(j, val) {
					var name = val[0];
					var is_selected = val[1];

					var count = (false && self._active[facet.name]) ? self._count_no_facets(facet.name, name) : self._count(facet.name, name);

					if(show_all || /*self._active[facet.name]*/is_selected || (self._total[facet.name][name].count && count)){
						var cl = '';
						var height = '1000px';
						n_visible++;
					}else{
						var cl = '.disabled';
						var height = '0';
					}

					// TODO use css animation keyframes to replace slideup and remove overflow:hidden
					if(true){
							var id = 'checkbox-fct-' + (iii++); // the 2 is to avoid classes with checkboxes created using the Element library
							var totals = self._count(facet.name, name) + ((self._active[facet.name] && (self._count(facet.name, val[0]) !== self._count_no_facets(facet.name, val[0]))) ? '/' + self._total[facet.name][val[0]].count : '');
							values.push(h('li.styled' + cl, {style: {maxHeight: height}}, [
								h('input#' + id, {
									type: 'checkbox',
									checked: is_selected && 'checked',
									onclick: function(e) {
										var checkbox = $(e.target);
										var li = checkbox.parent();

										var is_checked = checkbox.is(':checked');
										var tree = $.findWhere(self._tree, {name: facet.name});

										if(li.find('li :checked').length){
											self._active[name] || (self._active[name] = true);
											tree.has_selection = true;
										}else{
											delete self._active[name];
											tree.has_selection = li.find(':checked').length;
										}

										var vals = self.lookup_facet_vals(facet.name, name);
										vals[1] = is_checked;

										o.on_change(facet.name, name, is_checked, true);
									}
								}),
								h('label', {attributes: {'for': id}}, [
									h('span', [val[0] + ' (' + totals +')'])
								])
							]));
					}else{
							/* TODO reusable component not working for updates
							values.push(new Checkbox(
								{text: val[0]},
								self._count(facet.name, val[0]) + ((self._active[facet.name] && (self._count(facet.name, val[0]) !== self._count_no_facets(facet.name, val[0]))) ? '/' + self._total[facet.name][val[0]].count : ''),
								cl,
								{height: height, overflow: 'hidden'},
								is_selected
							));
							*/
					}
					n++;
				});
				if(facet.has_selection || (n_visible > 1)) ++n_visible_outer;

				(n_visible < n)
					&& values.push(h('a', {
						href: '#see-all',
						'onclick': function(e){
							self._cache_view[facet.name] || (self._cache_view[facet.name] = {});
							self._cache_view[facet.name].show_all = true;

							self.change.req();
							e.preventDefault();
						}},
						'show ' + (n - n_visible) + ' more'));

				c.children.push(h('ul', {}, values));

				var child = h('li.' + ((facet.has_selection || (n_visible > 1)) ? '' : 'disabled'), {}, c.children);
				children.push(child);
			});

			return n_visible_outer
				? children
				: h('p.disabled', 'No facets');
		})())]);
	},

	/*
	 *   The tree currently contains selection information - it might be better
	 *   to move it elsewhere so that the tree never changes for a given package.
	 */
	_create_tree: function() {
		var self = this, o = self.options;

		if(__DEV__ && this._tree) console.warn('facets.create_tree: expected empty cache')
		this._tree = []

		self._count_totals();

		$.each(o.facets, function(_, v) {
			var name = v.name;
			var values = v.values;

			var node = {name: name, vals: []};

			o.selection[name] && (node.has_selection = self._active[name] = true);

			$.each(values, function(j, v) {
				var selected = o.selection[name] && !!~$.inArray(v, o.selection[name]);
				var val = [v];
				selected && val.push(true);
				node.vals.push(val);
			});

			self._tree.push(node);
		});
	},

	on_filter_change: function() {
		var self = this;
		self._cache = null;

		self.change.have.update_queued || setTimeout(function() {
			self.change.req();
		}, 200);
	},

	// dont call directly. call change.req()
	_on_filter_change: function() {
		this.options.renderer.render();
	},

	_analyze: function() {
		var self = this, o = self.options;
		if(__DEV__ && this._cache) console.warn('facets.analyze: expected empty cache')
		this._cache = {};

		$.each(o.products._products, function() {

			if((this.filter_pass !== false) && this.data.tags){
				$.each(this.data.tags, function(tag, v){
					for(var i in v){
						self._cache[tag] || (self._cache[tag] = {});
						self._cache[tag][v[i]] || (self._cache[tag][v[i]] = {count: 0, count_no_facets: 0});
						self._cache[tag][v[i]].count++;
					}
				});
			}

			if((this.filter_pass_no_facets !== false) && this.data.tags){
				$.each(this.data.tags, function(tag, v){
					for(var i in v){
						self._cache[tag] || (self._cache[tag] = {});
						self._cache[tag][v[i]] || (self._cache[tag][v[i]] = {count: 0, count_no_facets: 0});
						self._cache[tag][v[i]].count_no_facets++;
					}
				});
			}
		});
		if(__DEV__ && !Utils.n_properties(self._cache)) console.warn('facets._analyze: empty');
	},

	lookup_facet_vals: function (name, b) {
		var tree = $.findWhere(this._tree, {name: name});
		for(var i in tree.vals){
			if(tree.vals[i][0] === b) return tree.vals[i];
		}
	},

	_count_totals: function() {
		// count tag usage with no filtering applied.
		var self = this, o = self.options;
		this._total = {};

		$.each(o.products._products, function(){
			if(this.data.tags){

				$.each(this.data.tags, function(tag, v){
					for(var i in v){
						self._total[tag] || (self._total[tag] = {});
						self._total[tag][v[i]] || (self._total[tag][v[i]] = {count: 0});
						self._total[tag][v[i]].count++;
					}
				});
			}
		});
	},

	_count: function(facet, value) {
		var o = this.options;

		if(true !== false){
			return this._cache[facet] && this._cache[facet][value] && this._cache[facet][value].count || 0;
		}else{
			var c = 0;
			$.each(o.products._products, function(){
				if((this.filter_pass !== false) && this.data.tags && this.data.tags[facet] && !!~$.inArray(value, this.data.tags[facet])) c++;
			});
			return c;
		}
	},

	/*
	_count_active: function(facet, value) {
		return this._active[facet] && this._active[facet][value] && this._active[facet][value].count || 0;
	},
	*/

	_count_no_facets: function(facet, value) {
		return this._cache[facet] && this._cache[facet][value] && this._cache[facet][value].count_no_facets || 0;
	}
};

var iii = 0; // TODO use static property instead
/*
var Checkbox = function(options, count, cl, style, is_selected) {
	this.o = options;
	this.text = options.text;
	this.count = count;
	this.cl = cl;
	this.style = style;
	this.is_selected = is_selected;
	this.instance_num = iii++;
};
$.extend(Checkbox.prototype, {
	type: 'Widget',
	//instance_num: 0,
	init: function() {
		//console.log('init', this.instance_num);
		if(this.o.text === 'Operational') console.log('init: style=', this.style);
		var id = 'checkbox-2-' + (this.instance_num++);
		return createElement(h('li.styled' + this.cl, {style: this.style}, [
			h('input#' + id, {type: 'checkbox', checked: this.is_selected && 'checked'}),
			h('label', {attributes: {'for': id}, style: {fontSize: '14px'}}, [this.text + ' (', h('span', '' + this.count), ')'])
		]));
	},
	update: function(previous, previous_el) {
		// when does this get called - can we reuse the same Checkbox instead of recreating?
		console.log('checkbox.update:', previous.text, this.text, previous_el);
	}
});
*/

return Facets

});
