/*
 * Filterable List of Catalogue items
 *
 */
define(['config', 'jquery', 'ajax_queue', 'bus', 'utils', 'jquery-ui/widget', 'input-filter', 'thumbnail', 'ellipsis'], function(ECMWF, $, Ajaxqueue, Bus, Utils) {

var tag = '_dialog_thumbnail_tag';
var cancel_id = 0; // cancelByTag requests must have different tags as they remain in effect. Is this a bug?
var max_retry_mins = 10;

$.widget('ui.filterableImageList', {

	widgetEventPrefix: 'filterable',

	options: {
		kind: 'product',
		package: 'eccharts',
		orientation: 'vertical',
		retry_min_later: 1,
		merge_user_catalogue : false,
		actions: [],
		filter: null,
		on_got_list: $.noop,
		on_update_item: function(item){},
		on_show_item: function(item){},
		on_scroll: function(item){}
	},

	_create: function() {
		var self = this;
		var o = this.options;

		self.regex = null,
		self._input = {
			previous : null,
			wait     : 0
		}
		self._visible_items = {};
		self._hiding = {};

		this.container = $('<div>', {'class': 'filterable-thumbnail-list'});
		this.list_container = this.container.div('.ul-outer');
		this.ul = this.list_container
			.ul()
				.scroll(function(){
					setTimeout(function(){ on_scroll(); }, 100);
				});
		this.element.data('filterableThumbnailList', {ul: this.ul, filters: []});

		if($.isFunction(o.filter)){
			self._add_filter(o.filter);
		}

		var on_scroll = $.throttle(500, function(){
			self._update_visibility();
			o.on_scroll();
		});
		setTimeout(function(){ on_scroll(); }, 2000);

		self._start_queue();
	},

	_start_queue: function(){
		var self = this;

		this.element.queue__({on_timeout: function(idx){
			self.ul.find('li:eq(' + idx + ')').find('div.text > p').trigger('doEllipsis');
		}});

		return this;
	},

	get_thumbnails: function(args) {
		var self = this, o = this.options;

		args = $.extend({success: $.noop, dblclick: $.noop}, args);
		var data = this.element.data('filterableThumbnailList');

		Ajaxqueue({
			url      : '/backend/catalogue-items/',
			json     : {
				type           : o.kind,
				package        : o.package,
				user_items     : o.merge_user_catalogue,
				standard_items : true,
				allow_experimental: ECMWF.request.allow_experimental,
				access         : ECMWF.request.access,
				groups         : true,
				keywords       : true,
				family         : true,
				descriptions   : true
			},
			priority : args.priority || 'very-high',
			logErrors: 'Catalogue list (' + o.kind + ')',
			success: function (list) {
				if(typeof list !== 'object' || list.length < 1){
					Bus().send('log-warning', {title: 'Failed to get ' + o.kind + 's'});
					args.error && args.error();
					return;
				}
				data.list = self.list = list = $(list);
				o.on_got_list();

				self.dom_lookup_by_name = {};
				self.dom_lookup_by_title = {};

				var holding = self.ul.clone(true).show().delay(0);

				// quickly add placeholder lines all in one go
				for(var i=0;i<list.length;i++){
					var item = list[i];
					var li = item.li = holding.li({name: item.name});
					if(typeof self.dom_lookup_by_title[item.title] !== 'undefined') console.warn('catalogue error! duplicate title: ' + item.title + '. Thumbnail list will not operate correctly.');
					self.dom_lookup_by_name[item.name] = li;
					self.dom_lookup_by_title[item.title] = li;
				}

				self.list_container.empty().append(holding);
				self.ul = holding;

				list.each(function(i){
					var item = list[i];
					var li = item.li;

					holding.delay(0).queue(function(next){
						li.div()
							.thumbnail({
								kind: o.kind,
								title: this.title,
								description: this.description,
								click: function() {
									self.ul.find('li.selected').removeClass('selected').find('.text p').trigger('doEllipsis');
									li.addClass('selected');
									li.find('.text p').triggerHandler('doEllipsis');
								},
								dblclick: function() {
									return args.dblclick(item);
								}
							});

						var a = li.div({className: 'text', css: {overflow: 'hidden'}});
						a.h4(item.title);

						(item.title == item.description) || a
							.p()
								.html(ECMWF.add_links(item.description)) //TODO decide whether to enforce markup language only.
								.markup()
								.delay(100).queue(function(next){
									$(this).ellipsis({lines: 2, container: li.find('.text')});
									next();
								});

						if(o.actions.length){
							var actions = li.div('.actions');
							$(o.actions).each(function(){
								var action = this;
								$('<a href="#' + this.name + '" class="text-link">' + this.name + '</a>').appendTo(actions).click(function(){
									action.on_click(item.name);
									return false;
								});
								$('<span>&nbsp;&nbsp;&nbsp;</span>').appendTo(actions);
							});
						}

						li.br();
						next();
					});
				});

				holding.queue(function(next) {
					self._filter_list();

					args.success(list);

					self._load_thumbnails(self._get_visible_items('reload'));
					next();
				})
			}, //end success

			error: function () {
				if(args.error) args.error();

				setTimeout(function(){
					args.priority = 'very-low';
					self.get_thumbnails(args);
				}, 1000 * 60 * o.retry_min_later);

				o.retry_min_later = Math.min(o.retry_min_later + 0.5, max_retry_mins);
			}
		});
	},

	_add_filter_input: function(element, callback) {
		var self = this;

		element
			.inputFilter({on_filter: function(text){
				self.regex = new RegExp(Utils.escape_regex(text), 'i');
				self._filter_list();
				if($.isFunction(callback)) callback();
				self._update_descriptions();
			}});
	},

	_add_filter: function(fn) {
		var data = this.element.data('filterableThumbnailList');

		if($.isFunction(fn)) data.filters.push(fn);
	},

	 /*
	  * return true or false
	  */
	_filter: function(item) {
		var data = this.element.data('filterableThumbnailList');
		var self = this;

		for(var i in data.filters){
			if(!data.filters[i].apply(self, [item])) return false;
		}
		return true;
	},

	_filter_list: function() {
		var self = this;
		if(this.destroyed) return self;
		this._visible_items = {} //items that are currently shown and are inside the current viewport.

		var fast = true;
		if(fast){
			var stats = {n_shown: 0, n_hidden: 0};
			var found_scrollregion = false;
			var found_scrollregion_end = false;
			self.element.data('filterableThumbnailList').list.each(function() {
				var item = this;
				if(self._filter(this)){
					if(item.match === false){ //showing is very slow, even if already shown.
						self._show_item(item);
						stats.n_shown++;
					}
					item.match = true;
					// checking the visibility directly on the dom is slow, however as the list is changing the cached values are invalid.
					if(!found_scrollregion_end && self._item_is_visible(item)){
						found_scrollregion = true;
						if(!self._visible_items[item.title]){
							self.options.on_show_item(item);
						}
						self._visible_items[item.title] = item;
						self._load_thumbnail(item);
					}
					else if(found_scrollregion) found_scrollregion_end = true;
				} else {
					if(item.match !== false){
						self._hide_item(item);
						stats.n_hidden++;
					}
					item.match = false;
					if(self._visible_items[item.title]){
						self._visible_items[item.title] = null;
					}
				}
			});
			//console.log('filter stats: n_shown=' + stats.n_shown + ' n_hidden=' + stats.n_hidden);
		}else{
			self.element.data('filterableThumbnailList').list.each(function() {
				self._filter_item(this);
			});
		}

		return self;
	},

	_filter_item: function(item) {
		if(this._filter(item)){
			if(item.match !== true){
				this._show_item(item);
			}
			item.match = true;
			if(this._item_is_visible(item)){
				if(!this._visible_items[item.title]){
					this.options.on_show_item(item);
				}
				this._visible_items[item.title] = item;
				this._load_thumbnail(item);
			}
		} else {
			if(item.match !== false){
				this._hide_item(item);
			}
			item.match = false;
			if(this._visible_items[item.title]){
				this._visible_items[item.title] = null;
			}
		}
		return this;
	},

	_hide_item: function(item) {
		var self = this;

		function hiding_complete(){
			//only now can we update based on visibility. note that with _show_item(), updates can be done earlier.
			self._load_thumbnails(self._get_visible_items('reload'));
			self.element.triggerHandler('hideAnimationFinished');
		}

		self._hiding[item.name] = (typeof self._hiding[item.name] === 'undefined') ? 1 : self._hiding[item.name] + 1;

		item.li
			.animate(
				{opacity: 0}, {
					duration: 200,
					complete: function(){
						item.li.animate({height: 0}, {duration: 200, complete: function(){
							item.li.hide();

							if(!--self._hiding[item.name]) delete self._hiding[item.name];
							if($.isEmptyObject(self._hiding)) hiding_complete();
						}});
					}
				}
			);
	},

	_show_item: function(item) {
		var self = this;
		if(true)
			item.li
				.css({opacity: 0, height: 0, display: 'list-item'})
				.animate(
					{height: 67}, {
						duration: 200,
						complete: function(){
							item.li.animate({opacity: 1}, {duration: 200, complete: function(){
								self.ul.find('li.first').removeClass('first');
								self.ul.find('li:visible:first').addClass('first');
							}});
						}
					}
				);
		else
			item.li.show().css({height:67, opacity: 1});
	},

	_item_is_visible: function(item) {
		if(item.match === false) return false;

		if(this.options.orientation === 'vertical'){
			var li = item.li;

			var ul_top = 0;
			var ul_bot = parseInt(this.ul.outerHeight(), 10);

			var li_top = li.position().top - this.ul.scrollTop();
			var li_bot = li_top + li.height();
			var visible = (li_bot > ul_top) && (li_top < ul_bot);
			return visible;
		}
		else console.warn('TODO horizontal lists not yet supported.');
	},

	_load_thumbnail: function (item) {
		var self = this, o = this.options;

		if(item.done !== true){

			var img = item.li.find('img');

			Ajaxqueue({

				url        : '/backend/' + o.kind + '-thumbnail/' + item.name + '/',
				tag        : self._get_cancel_tag(),
				priority   : 'low',
				cache      : 30,
				logErrors  : 'Get ' + o.kind + ' thumbail (' +  item.title + ')',
				success: function (json) {
					if(!json) return;

					img.css({ height: 'auto', width: 'auto'}).attr('src', json.url);
					item.done = true;
				},
				error : function() {
					img.css({height: 'auto', width: 'auto'}).attr('src', Utils.blank_gif);
				}
			});
		}
	},

	_load_thumbnails : function(list) {
		var self = this;
		list.each(function(){ self._load_thumbnail(this); });
	},

	_get_visible_items: function(reload) {
		//return a jquery object containing each item within the current scroll port.

		var self = this, f;

		if(reload){
			function find_first(){
				//do a binary search of the list to find the first visible line.
				if(!self.list) return null;
				var items = self.list.map(function(){if(this.match !== false) return this;});
				if(!items.length) return null;

				var scrolltop = self.ul.scrollTop();
				var min = 0;
				var max = items.length - 1;
				var i = max;
				var top = 0;
				do{
					if(top < -91){ // move forward
						min = i;
					}else{         // move back
						max = i;
					}
					var next = Math.floor((max + min) / 2);
					if(next === i) break;
					i = next;

					top = Math.floor(items[i].li.position().top - scrolltop);
				} while(top < -90 || top > 0);

				return self._find_item_by_name(items[i].name);
			}

			var new_visible_items = {};
			if((f = find_first())){
				var found_scrollregion = false;
				for(var i=f;i<this.list.length;i++){
					var item = this.list[i];
					if(self._item_is_visible(item)){
						new_visible_items[item.title] = item;
						found_scrollregion = true;
					}
					else if(found_scrollregion && item.match){
						break;
					}
				}
			}
			self._visible_items = new_visible_items;
		}

		function hash_to_jquery(hash){
			var ret = [];
			for(var i in hash){
				ret.push(hash[i]);
			}
			return $(ret);
		}

		return hash_to_jquery(self._visible_items);
	},

	_cancel_requests : function() {
		Ajaxqueue.cancelByTag(this._get_cancel_tag());
		cancel_id++;

		return this;
	},

	_get_cancel_tag : function() {
		return this.options.kind + tag + '_' + cancel_id;
	},

	_find_item_by_name : function(name) {
		for(var i in this.list){
			if(this.list[i].name === name) return i;
		}
	},

	_find_item_by_title : function(title) {
		for(var i in this.list){
			if(this.list[i].title === title) return i;
		}
		return -1;
	},

	_find_item_by_li : function(li) {
		var name = li.attr('name');
		for(var i in this.list){
			if(this.list[i].name === name) return this.list[i];
		}
	},

	__resize : function() {
		this._update_visibility();
		this._update_descriptions();
	},

	_update_visibility : function() {
		var self = this, o = this.options;

		//TODO put this somewhere better.
		//it is needed only once following unhiding of a previously used dialog.
		self._start_queue();

		self
			._cancel_requests()
			._get_visible_items({reload: true})
				.each(function(){
					self._load_thumbnail(this);
					o.on_update_item(this);
					self.element.queue__('add', this.li.index());
				});

		return this;
	},

	_update_descriptions : function() {
		var self = this;

		self._get_visible_items()
			.each(function(){
				self.element.queue__('add', this.li.index());
			});
	},

	destroy : function() {
		this.element.removeData('filterable')
		this.element.queue__('destroy');
		this.destroyed = true;
		return this;
	}

});


/*
 *  Provides support for a single delayed callback which will be called once for each added item.
 *  Requests to add an item already queued are ignored.
 *
 *  usage: initialise the element to supply the callback, then call successively with the 'add' argument for each item to be queued.
 *
 *  It differs from jquery.ba-throttle-debounce in that it is a queue on multiple elements instead of global callbacks.
 */
$.fn.queue__ = function(args, callback_data) {

	if(args === 'destroy'){
		clearTimeout(this.data('queueTimer'));
		this.removeData('queueOptions').removeData('queuedItems').removeData('queueTimer');
		return this;
	}

	if(args === 'add'){
		return this.each(function() {
			var el = $(this);
			var o = el.data('queueOptions');

			var q = el.data('queuedItems');
			var items = q.one;
			if($.inArray(callback_data, items) >= 0) return; //TODO doesnt work for jquery objects
			items.push(callback_data);
			el.data('queuedItems', q);

			function set_timeout(){
				return setTimeout(function(){
					if(!items.length) return;
					el.data('queueTimer', 0);
					var item = items.shift();
					o.on_timeout(item);

					if(items.length) set_timeout();
				}, o.interval);
			}

			function add_item(_item){
				var timer = el.data('queueTimer');

				if(!timer) timer = set_timeout();
				el.data('queueTimer', timer);
			}
			add_item(callback_data);
		});
	}

	var o = $.extend({
		interval: 1000,
		queue: 'default',
		on_timeout: function(){}
	}, args);

	return this.each(function() {
		var el = $(this);

		if(el.data('queueOptions')) return;

		el
			.data('queueOptions', o)
			.data('queuedItems', {one: []})
			.data('queueTimer', 0);
	});
};

});
