define(['jquery', 'bus', 'settings', 'chart/layer', './big_title.css'], function($, Bus, Settings, Layer){

	var count = 0;

	$.widget('ui.bigTitle', {

		options: {
			messageBus: Bus(),
			autoPosition: false
		},

		_init: function(){
			var self = this, o = this.options;

			count++;

			self.name = 'big_title_' + count;

			self.factor = Settings.get('big_title_size', 1);

			self.element.addClass('big-title');

			var buttons = self.element.div('.hidden').div('.buttons');

			buttons.span('.ui-icon ui-icon-zoomin')
				.click(function() {
					self.wrapper.css('font-size', function(_, old) {
						self.factor *= 1.2;
						setTimeout(function() { Settings.set('big_title_size', self.factor); }, 10);
						return parseFloat(old)*1.2;
					});
				})
				.css({top: 0, left: 0});

			buttons.span('.ui-icon ui-icon-zoomout')
				.click(function() {
					self.wrapper.css('font-size', function(_, old) {
						self.factor /= 1.2;
						setTimeout(function() { Settings.set('big_title_size', self.factor); }, 10);
						return parseFloat(old) / 1.2;
					});
				})
				.css({top: 0, left: 16});

			self.wrapper = self.element.div('.wrapper');
			self.content = self.wrapper.div('.content');
			self.layers  = self.wrapper.div('.layers');

			self.size = self.content.css('font-size');

			o.messageBus
				.listen('chart-time-select', function(event, msg){
					self.time = msg;
					self._update();
				})
				.listen('chart-content-update', function(event, msg){
					self.time = msg.time || self.time;
					self._layers(msg);
					self._update();
				})
				.listen('layers-visibility-change', function(event, msg) {
					self._layers(msg);
				})
				.listen('chart-animation-start', function() {
					self.animating = true;
				})
				.listen('chart-animation-stop', function() {
					self.element.css({minWidth: ''});
					self.animating = false;
				});

			if(self.factor != 1){
				self.wrapper.css('font-size', function(_, old) {
					return parseFloat(old) * self.factor;
				});
			}

			self._update();
		},

		_reset: function(txt){
			var self = this, o = this.options;
			self.content.text('------------');
		},

		_layers: function(msg) {
			var self = this;

			var ul = self.layers.empty().ul();

			$.each(msg.product.layerList(new Layer().is_visible), function(j, layer) {
				layer.has_data && $('<li>').prependTo(ul).html(new Layer(layer).get_title());
			});

			self._update();
		},

		_update: function(){
			var self = this;

			if (!self.time) { // || !(self.element.is(":visible"))) {
				self._reset();
				self.element.resize();
				return;
			}

			self.content.text(self.time.valid_time.format('dddd d mmmm yyyy HH:MM') + ' UTC');

			self.animating && setTimeout(function(){
				self.element.css({minWidth: self.element.width()});
			}, 50);
		},

		destroy: function(){
			this.element.empty().removeClass('big-title');
		}
	});
});
