/*
 *  ECWMF Layer collection
 *  ----------------------
 *
 *  A generic collection object for chart layers.
 *
 *  Please note that this conforms to the Product Collection Interface (see documentation). Do not modify the API.
 *
 *  changes:
 *    ajax request parameters are different
 *    'product_adaptor' method added
 */
define(['config', 'jquery', 'ajax_queue', 'storage', 'bus', 'mixins', 'collection', 'collection/item', 'utils', 'throttle-debounce'], function(ECMWF, $, Ajaxqueue, localStorage, Bus, Mixins, Collection, ChartListItem){

	/*
	 * Facets are formatted as below:
	 *
	 * product: {
	 *     tags: {
	 *         facet_name1: value1,
	 *         facet_name2: [value2, value3]
	 *     }
	 * }
	 *
	 */

	ECMWF.options.timeout && $.extend(Ajaxqueue.options, {
		timeout: ECMWF.options.timeout * 1000
	});

	function Layers(){

		this.product_type = 'eccharts' // used by vgrid
		this.query = {}
		this.thumbnail_priority = 'low'

		this.list_init()

		/*
		 *  Workaround for non-standard group format in old Eccharts lists (see WREP 2392).
		 *  TODO move the adaptor to django by adding a new API endpoint.
		 */
		this.product_adaptor = function(products) {
			$.each(products, function(i, v) {
				if(v.group){
					products[i].tags = {
						'Product group': v.group.slice(5)
					}
				}
			})
			return products;
		}

		this.products = function() {
			var self = this;

			// TODO change so that it does not return a new promise each time
			return $.Deferred(function(df) {

				function have_products(result) {
					result = self.product_adaptor(result)

					if(window._ECMWF && _ECMWF.all_products){
						$.each(_ECMWF.all_products, function(i, v){
							var p = $.findWhere(result, {name: v.name})
							if(p){
								p.thumbnail = v.thumbnail
							}
						})
					}

					__DEV__ && Bus().send('event-start', ['start verification']);
					self._products = $.map(result/*.records*/, function(item){
						if(self.validate([
								['not empty', function(){ return item; }],
								['has name', function(){ return item.name; }],
								['has title', function(){ return item.title; }],
							])){
							var product = new ChartListItem(item);
							if((!__DEV__) || (product.cache.score.apply(product) > -1)) return product;
						}
					});
					if(self._products.length){
						var rejected = result.query_count - self._products.length
						console.log('n_products=' + self._products.length + (rejected ? ' (' + (rejected) + ' rejected)' : ''))
					}else{
						console.log('no products found for:', query || self.query)
					}

					//self.packages.then(function() {
						df.resolve(self._products)
					//});
				}

				var user_only = false

				Ajaxqueue({
					url: '/backend/catalogue-items/',
					json: {
						type: self.kind,
						/*
						user_items: user_only,
						standard_items: !user_only,
						allow_experimental: ECMWF.request.allow_experimental,
						access: ECMWF.request.access,
						*/
//						package        : o.package,
//						user_items     : o.merge_user_catalogue,
						standard_items : true,
//						allow_experimental: ECMWF.request.allow_experimental,
//						access         : ECMWF.request.access,
						groups         : true,
						keywords       : true,
						family         : true,
						descriptions   : true
					},
					cache: 30,
					tag: self.name,
					complete: have_products
				})
			}).promise()
		}

		this._store = $.debounce(2000, function() {
			localStorage.products = JSON.stringify(this.__cache);
		})
	}

	Layers.prototype = $.extend(new Collection(), {
		kind: 'layer',

		/* not used ?
		item_by_name: function(name) {
			for(var i in this.products){
				if(this.products[i].data.name === name) return this.products[i];
			}
		}
		*/
	})

	var layers
	return layers = new Layers();
});
