/*
 *  ************ TEMPORARY FILE **************
 *  ************ copied from grid/products -- once we know what needs to be done, diff against original and decide how to proceed and what refactoring needs to be done
 *  (common properties and methods can be moved to the prototype (Collection))
 *
 *  Differences:
 *   - different ajax call find-product/catalogue-items
 *        catalogue-items has some additional request parameters, and response is not inside the 'results' key
 *   - there is a 'packages' property but this doesn't make much sense for the projections type
 *   - validation is different for different types
 *  ...apart from the above, there are actually very few differences.
 */
/*
 *  ECWMF Products
 *  --------------
 *
 *  A generic collection object for chart products.
 *
 *  Please note that this conforms to the Product Collection Interface (see documentation). Do not modify the API.
 *
 */
define(['config', 'jquery', 'ajax_queue', 'storage', 'bus', 'mixins', 'collection', 'collection/item', 'utils', 'throttle-debounce'], function(ECMWF, $, Ajaxqueue, localStorage, Bus, Mixins, Collection, ChartListItem){

	ECMWF.options.timeout && $.extend(Ajaxqueue.options, {
		timeout: ECMWF.options.timeout * 1000
	});

	function Products(){
		this.kind = 'product'
		this.product_type = 'eccharts' // used by vgrid
		this.query = {}
		this.thumbnail_priority = 'low'

		this.list_init()

		this.products = function() {
			var self = this;

			// TODO change so that it does not return a new promise each time
			return $.Deferred(function(df) {

				function have_products(result) {
					result = self.product_adaptor(result)

					/*
					if(window._ECMWF && _ECMWF.all_products){
						$.each(_ECMWF.all_products, function(i, v){
							var p = $.findWhere(result, {name: v.name})
							if(p){
								p.thumbnail = v.thumbnail
							}
						})
					}
					*/

					__DEV__ && Bus().send('event-start', ['start verification']);
					self._products = $.map(result/*.records*/, function(item){
						if(self.validate([
								['not empty', function(){ return item; }],
								['has name', function(){ return item.name; }],
								['has title', function(){ return item.title; }],
								['has no empty facets', function(){
									if(item.tags){
										for(var tags in item.tags){
											if(Array.isArray(item.tags[tags]) && !item.tags[tags].length){
												console.warn('product.validate: product error: ' + item.name + ': tag is empty array (' + tags + ')');
												delete item.tags[tags];
											}
										}
									}
									return true;
								}],
								['path has no trailing slash', function(){ if(Array.isArray(item.path))
									for(var i in item.path){
										var p = item.path[i];
										if((item.path[i] = item.path[i].replace(/\/$/, '')) !== p)
										console.warn('path has trailing slash: ', p, item.name);
									}
									return true;
								}],
								['has keywords', function(){ return item.keywords || (item.keywords = ' '); }] // temporary. should be server-side.
							])){
							var product = new ChartListItem(item);
							if((!__DEV__) || (product.cache.score.apply(product) > -1)) return product;
						}
					});
					if(self._products.length){
						var rejected = result.query_count - self._products.length
						console.log('n_products=' + self._products.length + (rejected ? ' (' + (rejected) + ' rejected)' : ''))
					}else{
						console.log('no products found for:', query || self.query)
					}

					self.packages.then(function() {
						df.resolve(self._products)
					});
				}

				/*
				if(window._ECMWF && _ECMWF.products){
					var public_packages = ['s2']
					if(!!~$.inArray(self.query.package, public_packages)){
						if(_ECMWF.all_products){
							$.each(_ECMWF.all_products, function(i, v){
								var p = $.findWhere(_ECMWF.products, {name: v.name})
								if(p){
									p.thumbnail = v.thumbnail
								}
							})
						}
						return have_products({records: []});
					}
				}
				*/

				var user_only = false

				Ajaxqueue({
					url: '/backend/catalogue-items/',
					json: {
						type: self.kind,
						user_items: user_only,
						standard_items: !user_only,
						allow_experimental: ECMWF.request.allow_experimental,
						access: ECMWF.request.access,
						groups: true,
						keywords: true,
						descriptions: true
					},
					cache: 30,
					tag: self.name,
					complete: have_products
				})
			}).promise()
		}

		this.packages = $.Deferred(function(df){
			console.log('products.packages');
			var self = this;

			Ajaxqueue({
				url: '/plots/get-packages/',
				priority: 'very-high',
				complete: function(result){
					if(result.packages){
						products._packages = result.packages;
						products.hierarchy.update(result.categories);
						df.resolve(products._packages);
					}
				}
			});
		}).promise()

		// because of debounce, cannot be in prototype
		this._store = $.debounce(2000, function() {
			localStorage[this.kind + 's'] = JSON.stringify(this.__cache);
		})
	};

	Products.prototype = new Collection()

	var products;
	return {products: products = new Products()};
});
