define(['jquery', 'unit', 'select-image'], function($, Unit) {

var _Form = function() {
    var self = this;

    var handlers = {};

    $.fn.form = function (options) {

        if(options === 'destroy'){
            return this.find('.form').remove();
        }

        options = $.extend({
            title: null,
            items: [],
            values: {},
            ok: ignore,
            cancel: ignore,
            show_ok: true,
            show_cancel: true,
            ok_label: 'OK',
            cancel_label: 'Cancel'
        },
        options || {});

        var items     = options.items;
        var ok_cb     = options.ok;
        var cancel_cb = options.cancel;
        var values    = options.values;

        var form_items = [];

        function FormItem(item) {

            this._item = item;

            var _handler = item.handler || (handlers[item.type] || handlers["text"]);

            this.value    = function() { return ((item.type === 'float' && typeof values[item.name] !== 'undefined') || values[item.name]) ? values[item.name] : item.value; }
            this.menu     = function() { return item.menu;     }
            this.multiple = function() { return item.multiple; }
            this.change   = function() { return item.change; }   // a fn called at render time to bind the change handler
            this.name     = function() { return item.name; }

            this.collect = function() {
                return _handler.collect(this, this.widget);
            }

            this.check = function(value) {
                return _handler.check(this, value);
            }

            this.render = function(owner) {
                this.widget = _handler.render(this);
                this.widget.appendTo(owner);
                if($.isFunction(this._item.widget)) {
                    // Notify widget creation
                    this._item.widget(this.widget);
                }
            }

            this.render_help = function(owner) {
                var h = _handler.help(this);
                if(typeof h === "string") {
                    this.help_widget = owner.html(h);
                }
                else {
                    this.help_widget = h;
                    this.help_widget.appendTo(owner);
                }
            }

            this.get_title = function () {
                var s = item.title;
                while (typeof(s) === "function") { s = s(); }

                if (typeof(s) === "string") { return s; }

                return item.name;
            };

            this.get_values = function () {
                return item.values;
            }

            this.get_defaults = function () {
                var s = this.value;
                if(s === undefined || s === null) { s = item["default"];  }

                while (typeof(s) === "function") { s = s(); }

                if(s === undefined || s === null) { s = [];  }

                if (typeof(s) === "object") { return s; }

                return [s];
            };

            this.get_help = function () {
                var s = item.help;
                while (typeof(s) === "function") { s = s(); }
                return s ? s : "";
            };

            this.get_unit = function () {
                var s = item.unit;
                while (typeof(s) === "function") { s = s(); }
                return Unit.get(s ? s : "");
            };

            this.get_precision = function () {
                var s = item.precision;
                while (typeof(s) === "function") { s = s(); }
                return (s === null || s === undefined) ? 2 : s;
            };

            this.get_range = function () {
                var s = item.range;
                while (typeof(s) === "function") { s = s(); }
                return s;
            };

            this.get_transform = function() {
                var s = item.transform;
                while (typeof(s) === "function") { s = s(); }

                var def = { ui2value: identity, value2ui: identity };
                var unit = this.get_unit();

                if(unit) {
                    def = {
                        ui2value: function(value) { return unit.fromDisplay(value); },
                        value2ui: function(value) { return unit.toDisplay(value); }
                    };
                }

                return s ? s : def;
            };

            this.get_label = function (name, i) {
                var s = item.labels, n = name;
                while (typeof(s) === "function") s = s(name);

                if (typeof(s) === "string") return s;

                if (Array.isArray(s)) return item.labels[i];

                if (typeof(s) === "object") return s[name];

                if (name === undefined || name === null)
                {
                    name = n;
                    if (name === undefined || name === null)
                    {
                        name = "?";
                    }
                }

                return name;
            };

            this.format = function(value) {
                var p = this.get_precision();

                if(typeof value === "string") {
                    value = parseFloat(value);
                }

                var t = 1;
                while (p < 0) {
                    t /= 10;
                    p++;
                }
                while (p > 0) {
                    t *= 10;
                    p--;
                }

                return Math.floor(value * t + 0.5) / t;
            };

            this.error = function(msg) {
                this.error_text.html(msg);
            }

            this.get = function(what) {
                return item[what];
            }


            return this;
        }

        $.each(items, function(i, item) {
            if(item) form_items.push(new FormItem(item));
        });

        function submit(){
            var c = collect();
            if (c.ok) {
                ok_cb(c.result);
                return c.result;
            }
        }

        function collect() {
            var result = {};
            var ok = true;

            $.each(form_items, function(i, item) {
                var val = item.collect();
                result[item.name()] = val;
                item.error("");
                if (!item.check(val)) { ok = false; }
            });

            return {result: result, ok: ok}
        }

        return this.each(function () {

            var elem = $(this);

            if(elem.closest('form').length){
                elem.closest('form')
                    .off('submit')
                    .on('submit', function() {
                        submit();
                        return false;
                    });
            }else{
                elem.on('submit', function() {
                    submit();
                });
            }

            var table = $("<table/>").addClass('form');

            if (options.title) {
                var tr = $("<tr/>").appendTo(table);
                var td = $("<td colspan='3' class='form-title'/>").appendTo(tr);
                td.html(options.title);
            }
            if (options.help) {
                tr = $("<tr/>").appendTo(table);
                td = $("<td colspan='3' class='form-help'/>").appendTo(tr);
                td.html(options.help);
            }

            $.each(form_items, function(i, item) {
                var tr = $("<tr class='form-row'/>").appendTo(table);
                var unit = item.get_unit();
                var title = item.get_title();
                if(unit) {
                    title += " (" + unit.symbolHTML() + ")";
                }

                $("<td class='form-item-title'/>").appendTo(tr).html(title + ":");
                var td = $("<td class='form-item-value'/>").appendTo(tr);
                /*var widget = */item.render(td);

                item.error_text = $("<span class='form-error'/>").appendTo(td);
                ////item.unit_text  = $("<span class='form-unit'/>").appendTo(td);
                //item.help_text  = $("<span class='form-help'/>").appendTo(td);
                item.render_help($("<td class='form-help'/>").appendTo(tr));
            });

            tr = $("<tr/>").appendTo(table);
            td = $("<td colspan='3' class='form-buttons'/>").appendTo(tr);

            var name = ''; //FIXME

            if (options.show_cancel) {
                $("<input type='button' id='" + name + "_cancel' value='" + options.cancel_label + "'>").
                    appendTo(td).
                    click(function(){
                        cancel_cb();
                    });
            }

            if (options.show_ok) {
                $("<input type='button' id='" + name + "_ok' value='" + options.ok_label + "'>").
                    appendTo(td).
                    click(function(){
                        submit();
                    });
            }

            table.appendTo(elem);
        });
    };

    // register a custom handler class
    $.addFormHandler = function (name, handler, _prototype) {
        //prototype arg is an optional string that can be used to specify the handler type to inherit from.
        handler.prototype = _prototype ? (new self[_prototype + 'Handler']()) : new Handler();
        handlers[name] = new handler();
    }

    // create a custom handler instance
    $.newFormItemHandler = function (handler) {
        handler.prototype = new Handler();
        return new handler();
    }

    /*
    var register = function (handler) {
        handlers[handler.type] = handler;
    };
    */

    function Handler() {
        this.help = function (item) {
            return item.get_help();
        };
        this.check = function () {
            return true;
        };
        return true;
    }

    this.StringHandler = function() {
        this.prototype = new Handler();

        this.render = function(item) {
            var value  = item.get_defaults();
            var val    = item.get_transform().value2ui(value[0]);
            return this.html(item, val);
        };

        this.collect = function (item) {
            var t = item.get_transform().ui2value;
            return t(this.parse($.trim(item.widget.val())));
        };

        // To Override
        this.parse = identity;
        this.align = "left";

        this.html = function (item, value) {
            return $("<input/>", {
                    type: "text",
                    'class': "form-text",
                    value: value
                })
                .css({
                    textAlign: this.align
                });
        };
    }
    this.StringHandler.prototype = new Handler();

    handlers["string"] = new this.StringHandler();

    function TextHandler() {

        this.html = function (item, value) {
            var rows = item.get("rows") || 5;
            var cols = item.get("cols") || 20;
            var widget = $("<textarea/>")
                .attr({ rows: rows, cols: cols, "class": "form-textarea" })
                .html(value);

            if($.fn.onScreenKeyboard) {
                setTimeout(function() {
                    widget.onScreenKeyboard({});
                },10);
            }
            return widget;
        };

        return this;

    }
    TextHandler.prototype = new this.StringHandler();
    handlers["text"] = new TextHandler();

    function NumberHandler() {

        this.align = "right";

        this.check = function (item, value) {
            if (isNaN(value)) {
                item.error('Invalid number');
                return false;
            }

            var range = item.get_range();
            if (range) {
                if (value < range[0] || value > range[1]) {
                    // TODO: Round numbers
                    /*var t = */item.get_transform().value2ui;
                    item.error('Number out of range');
                    //, should be between '
                    // + format(item,t(range[0]))  + ' and ' + format(item,t(range[1])));
                    return false;
                }
            }

            return true;
        };

        this.help = function (item) {

            var help = item.get_help();

            if (help) return help;

            var range = item.get_range();

            if (range) {
                var t = item.get_transform().value2ui;
                return 'Enter a number between <span class="form-highlight">' + item.format(t(range[0])) + '</span> and <span class="form-highlight">' + item.format(t(range[1])) + '</span>.';
            }

            return "";
        };

        this.html = function (item, value) {

            var input = $("<input/>")
                .attr({
                    type: "text",
                    'class': "form-text",
                    value: item.format(value)
                })
                .css({
                    textAlign: this.align
                });

            if($.fn.onScreenKeyboard) {
                setTimeout(function() {
                    input.onScreenKeyboard({
layout : "num"
                        });
                },10);
            }

            return input;
        };

    }
    NumberHandler.prototype = new this.StringHandler();

    function IntegerHandler() {
        //TODO: Use base 10
        this.parse = parseInt;
    }
    IntegerHandler.prototype = new NumberHandler();
    handlers["integer"] = new IntegerHandler();

    function FloatHandler() {
        this.parse = function (text) {
            // Many peaple use decimal comma
            return parseFloat(text.replace(",", "."));
        };
    }
    FloatHandler.prototype = new NumberHandler();
    handlers["float"] = new FloatHandler();


    function ChoiceHandler() {

        function MenuMaker(multiple) {

            this.widget = function () {
                var w = $("<select/>", {"class": "form-select"});
                if(multiple) w.attr("multiple", "multiple");
                return w;
            };

            this.item = function (value, label, checked) {
                var w = $("<option/>");
                w.attr("value", value)
                    .html(label)
                    ;

                if(checked) {
                    w.prop("selected", "selected");
                }

                return w;
            };

        }

        function ButtonMaker(multiple) {

            var name  = make_id();

            this.widget = function () {
                return $("<span/>");
            };

            this.item = function (value, label, checked) {
                var id    = make_id();
                var span  = $("<span/>");
                var input = $("<input/>")
                    .attr({
                        name : name,
                        id   : id,
                        type : multiple ? 'checkbox' : 'radio',
                        value: value
                    })
                    .appendTo(span);

                if(checked) {
                    input.prop("checked", "checked");
                }

                // jQuery bug, label does not accept the html()
                $("<label>" + label + "</label>").attr("for", id).appendTo(span);

                return span;
            };

            this.change = function (widget, cb) {
                widget.find("input").change(function (e) {
                    cb(e.target.value);
                });
            };

        }

        this.render = function (item) {

            var checked  = {};
            var defaults = item.get_defaults();

            $.each(defaults, function(_,def) { checked[def] = true; });

            var maker  = item.menu() ? new MenuMaker(item.multiple()) : new ButtonMaker(item.multiple());
            var widget = maker.widget(item);

            $.each(item.get_values(), function(i, value) {
                var label = item.get_label(value, i);
                maker.item(value, label, checked[value]).appendTo(widget);
            });

            if (item.change()) { maker.change(widget, item.change()); }

            return widget;
        };

        this.collect = function (item, widget) {
            var l = [];
            $.each(widget.find(":selected, :checked"), function () { l.push($(this).val()); });
            return item.multiple() ? l : l[0];
        };
    }

    ChoiceHandler.prototype = new Handler();
    handlers["choice"] = new ChoiceHandler();

    function ThumbnailHandler() {

        this.render = function (item) {
            var checked  = {};
            var defaults = item.get_defaults();

            $.each(defaults, function(_,def) { checked[def] = true; });

            var div = $("<div/>").addClass("form-thumbnail-item-wrapper");
            var img = $("<img/>").appendTo(div);
            $("<br/>").appendTo(div);

            var urls = {};

            var select = $("<select/>").addClass("form-thumbnail-item").appendTo(div);
            $.each(item.get_values(), function(i, value) {
                var label = item.get_label(value);
                var opt = $("<option/>",{
                        value : value,
                        text  : label.title,
                        title : label.url
                    }).appendTo(select);
                if(checked[value]) {
                    opt.prop("selected", "selected");
                    img.attr("src", label.url);
                    if(label.fetch_image) {
                        label.fetch_image(img, function(url) {
                            urls[value] = label.url = url;
                        });
                    }
                }

                urls[value] = label.url;
            });

            select.change(function() {
                //console.log("CHANGE " , select.val());
                img.attr("src", urls[select.val()]);
            });

           //setTimeout(function() { select.selectImage(); }, 1);
           select.selectImage(item._item);

           return div;

        };

        this.collect = function (item, widget) {
            var l = [];
            // TODO: change to widget.val();
            $.each(widget.find(":checked"),  function () { l.push(this.value); });
            $.each(widget.find(":selected"), function () { l.push(this.value); });
            return item.multiple() ? l : l[0];
        };
    }

    ThumbnailHandler.prototype = new Handler();
    handlers["thumbnail"] = new ThumbnailHandler();

    function BooleanHandler() {

        this.render = function (item) {
            var value = item.get_defaults();
            var input = $("<input type='checkbox'/>");
            if(value[0]) {
                input.prop("checked", "checked");
            }
            return input;
        };

        this.collect = function (item, widget) {
            return widget[0].checked;
        };
    }

    BooleanHandler.prototype = new Handler();
    handlers["boolean"] = new BooleanHandler();

    var ignore = $.noop;

    function identity (x) {
        return x;
    }

    var cnt = 0;

    function make_id() {
        cnt++;
        return "_form-random-id-" + cnt;
    }
}

return new _Form();

});
