define(['jquery', 'bus', 'form', 'cities'], function($, Bus) {

function PointHandler() {

	// TODO: use $.formatLatitude ...

	function format_lat(item, lat) {
		var N = 'N';
		if (typeof(lat) === 'string') { lat = parseFloat(lat);}
		if (lat < 0) {
			lat = -lat;
			N = 'S';
		}
		return item.format(lat) + ' ' + N;
	}

	function format_lon(item, lon) {
		var E = 'E';
		if (typeof(lon) === 'string') { lon = parseFloat(lon); }
		if (lon < 0) {
			lon = -lon;
			E = "W";
		}
		return item.format(lon) + ' ' + E;
	}

	function parse_ll(text) {
		text = text.replace(",", ".");
		var val = parseFloat(text);
		if (text.indexOf("S") !== -1 || text.indexOf("s") !== -1 || text.indexOf("W") !== -1 || text.indexOf("w") !== -1) { val = -val; }
		return val;
	}

	this.render = function (item) {
		var self = this;
		var value = item.get_defaults();

		//console.warn(value);

		function notify() {
			if(typeof item._item.city === 'function') 
				item._item.city({ title : title.val(), lat : parse_ll(lat.val()), lon : parse_ll(lon.val()) });
		}

		var span = $('<span class="latlon">');

		span.label('Latitude');
		var lat = span.input({value: format_lat(item, value.lat), 'class': 'lat form-text'})
			.css({textAlign: 'right'})
			.change(notify);

		span.label('Longitude');
		var lon = span.input({'class': 'lon form-text', value: format_lon(item, value.lon)})
			.css({textAlign: 'right'})
			.change(notify);

		var title = span.input({'class': 'title', value: value.title});

		if($.fn.onScreenKeyboard) {
			setTimeout(function() {
				lat.onScreenKeyboard({});
				lon.onScreenKeyboard({});
			}, 10);
		}

		item._item.city = item._item.city || true;

		// TODO: Don't access private _item
		if(item._item.city && $.fn.cityAutocomplete && $.formatGeoLocation) {

			if(true) {
				title.cityAutocomplete({
					name: item.name,
					select: function(event, ui){
						var pt = ui.item;
						lat.val(format_lat(item, pt.lat));
						lon.val(format_lon(item, pt.lon));
						delete self.save;
						if(typeof item._item.city === 'function') item._item.city(pt);
					},
					open: function(event, ui) { console.log('PointHandler OPEN', ui);  self.save = { lat : lat.val(), lon : lon.val() }; },
					close: function(event, ui) {
						console.log('PointHandler CLOSE save=', self.save);

						if(self.save) {
							lat.val(self.save.lat);
							lon.val(self.save.lon);
							notify();
							delete self.save;
						}
					},
					focus: function(event, ui) {
						var pt = ui.item;
						lat.val(format_lat(item, pt.lat));
						lon.val(format_lon(item, pt.lon));
						if(typeof item._item.city === 'function') item._item.city(pt);
					}
				});

			} else {
				//TODO: Check if this creates a memory leak
				var bus = Bus(span);
				$('<div/>').appendTo(span).cities({messageBus: bus});
				bus.listen('latlon-location-select',function(_, msg) {
					lat.val(format_lat(item,msg.lat));
					lon.val(format_lon(item,msg.lon));
					title.val(msg.title);
					if(typeof item._item.city === 'function') item._item.city(msg.title);
				});
			}
		}

		return span;
	};

	this.collect = function (item, widget) {
		var val =  {
			lat: parse_ll(widget.find('.lat').val()),
			lon: parse_ll(widget.find('.lon').val())
		};

		if(item._item.city) {
			val.title = widget.find('.title').val();
			if(val.title === placeholder)
			val.title = undefined;
		}

		return val;
	};

	this.check = function (item, value) {

		var id = '';
		if (isNaN(value.lat)) {
			$('#' + id + '_error').html('Invalid number (latitude)');
			return false;
		}

		if (isNaN(value.lon)) {
			$('#' + id + '_error').html('Invalid number (longitude)');
			return false;
		}

		var range = item.get_range() || {
			lat: [-90, 90],
			lon: [-180, 360]
		};

		range.lat = range.lat || [-90, 90];
		if (value.lat < range.lat[0] || value.lat > range.lat[1]) {
			item.error('Latitude out of range');
			//, should be between '
			//  + format(item,range.lat[0])  + ' and ' + format(item,range.lat[1]));
			return false;
		}

		range.lon = range.lon || [-180, 360];
		if (value.lon < range.lon[0] || value.lon > range.lon[1]) {
			item.error('Longitude out of range');
			//, should be between '
			//  + format(item,range.lon[0])  + ' and ' + format(item,range.lon[1]));
			return false;
		}

		return true;
	};

	this.help = function (item) {

		var help = item.get_help();
		if (help) { return help; }

		var range = item.get_range();
		if (!range) { return item.get_help(); }

		var list = [];

		if (range.lat) { list.push('a latitude between <span class="form-highlight">' + format_lat(item, range.lat[0]) + '</span> and <span class="form-highlight">' + format_lat(item, range.lat[1]) + '</span>');}

		if (range.lon) { list.push('a longitude between <span class="form-highlight">' + format_lon(item, range.lon[0]) + '</span> and <span class="form-highlight">' + format_lon(item, range.lon[1]) + '</span>'); }

		return 'Enter ' + list.join(', and ') + '.';
	};
}

$.addFormHandler('point', PointHandler);

var placeholder = 'Enter a city name';

});

