/*
 *  Widget to provide status information about current filters applied to a content list.
 */
define(['jquery', 'bus'], function($, Bus){

$.widget('ui.filterStatus', {

	options: {
		remove: true,                     // specifies whether clicking on a filter button either removes or disables the filter.
		filters: [],
		on_disable_change: $.noop,
		on_remove: $.noop,
		bus: Bus()
	},

	_init: function() {
		this.update();

		this.options.products.subscribe(this.update)
	},

	update: function(){
		var self = this, o = this.options;

		this.element.empty().addClass('filter-status');

		var filters = $.map(o.filters, function(f) {
			return (f.status && f.status()) ? f : null;
		});

		filters.length || this.element.span('No filters applied').addClass('disabled');

		$.each(filters, function(i, f) {
			var button = self.element.div(f.status())
				.button({icons: {secondary: 'ui-icon-closethick'}})
				.modeTooltip({
					content: function() {
						return '<b>' + f.name + '</b>: <em>' + f.status() + '</em>';
					}
				})
				.each(function(){
					f.disabled && $(this).addClass('disabled');
				})
				.click(function() {
					button.modeTooltip('close');

					if(o.remove){
						o.on_remove(f.name);
					}else{
						if(f.disabled){
							o.on_disable_change(f.name, f.disabled = !f.disabled);
						}else{
							o.on_disable_change(f.name, f.disabled = true);
						}
					}

					//o.bus.send('filters-invalidated');
					return false;
				});
		});
	},

	_lookup_filter_by_name: function(name){
		var self = this, o = self.options;

		for(var i in o.filters){
			var f = o.filters[i];
			if(f.name === name) return f;
		}
	},

	destroy: function(){
		this.element.removeClass('filter-status');
	}
});

});
