define(['jquery', 'vgrid/products'], function($, products){

	function GridTestable(test, selector){
		var self = this;
		this.test = test;

		this.ready = $.Deferred(function(df){
			test.wait_for(selector, 'GridTestable: ' + selector).then(function(){
				self.el = $(selector);
				df.resolve();
			});
		})
	}

	GridTestable.prototype = {
		has_status: function() {
			return !!this.el.find('.foot .status > div:eq(0)').text();
		},
		status_count: function() {
			return parseInt(this.el.find('.foot .status > div:eq(0)').text(), 10);
		},
		has_thumbnails: function() {
			return this.el.find('.grid-items .thumbnail-wrapper:visible').length > 0;
		},
		has_scrollbar: function() {
			return this.el.find('.ps-scrollbar-y-rail:visible').length === 1;
		},
		not_has_scrollbar: function() {
			return !this.has_scrollbar();
		},
		first_product: function() {
			// a variation of $.findWhere what assumes that the properties are getter functions instead of plain keys
			function findWhere(list, properties){
				return $.map(list, function(l){
					if($.map(properties, function(v, i){
						if(l[i]() === v) return true;
					}).length) return l;
				})[0];
			}

			var name = this.el.find('.grid-items > div:visible:first').find('.thumbnail-img').attr('name');
			return findWhere(products.items, {name: name});
		},
		visible_items: function() {
			return this.el.find('.grid-items > div:visible');
		},
		n_visible_items: function() {
			return this.el.find('.grid-items div.thumbnail:visible').length;
		},
		bind: function(fn, args){
			var self = this;
			return function(){ return fn.apply(self, arguments); }
		}
	};

	return GridTestable;
});
