
/*
 *   The Details View maintains a set of dom elements and handles all aspects of their presentation.
 *
 *   features:
 *   - filterable - elements can be hidden and shown.
 *   - large datasets - there cannot be an element per data item, though it must appear so.
 *
 */
define(['jquery', '../assigned_item', './detail_item', './view_base'], function($, AssignedItem, ItemView){

$.widget('ui.details', $.ui.viewBase, {

	_create: function() {
		var self = this;
		var obj = $.ui.viewBase.prototype._create.apply(this, arguments);

		this.ItemView = ItemView;

		this.item_height = 34;
		this.padding = {x: 10, y: 10};
		this.n_columns = 1;

		this.element
			.addClass('details')
			.parent().scrollTop(0);

		//  FIXME
		setTimeout(function() {
			self._set_scrollbar_size();
		}, 3000);
		setTimeout(function() {
			self._set_scrollbar_size();
		}, 6000);

		return obj;
	},

	_destroy: function() {
		this.element.empty().removeClass('details');
		this._super();
	},

	_on_select: function(a, el) {
		console.log('details._on_select', a);
		this.options.click(a);
	},

	scroll: function(top) {
		var self = this, o = this.options, list = o.products;

		//var container_bottom = -top + self.element.parent().height();

		var first_visible_result = Math.floor((-top) / (self.item_height + self.padding.y)) * self.n_columns;
		var last_visible_result = first_visible_result + self.cache.n_cells - 1;
		//console.log('details.scroll: first_visible_result=' + first_visible_result + ' last_visible_result=' + last_visible_result);
		var to_reassign = [];
		for(var f=first_visible_result;f<first_visible_result+self.cache.n_cells;f++){
			if(!(f < list.filtered.length)) break;

			var m = list.filtered[f];
			if(!self._m_is_assigned(m)){
				to_reassign.push(m);
			}
		}

		var available = [];
		self._each_allocated(function(item, m, d){
			var ai = self.assigned[d];
			if(!ai) return console.warn('item not assigned: d=' + d);
			if(ai.filtered_idx < first_visible_result || ai.filtered_idx > last_visible_result || item.filter_pass !== true){
				//console.log('  available: ' + d + ': ' + item.name + ' row=' + (parseInt(ai.dom.css('top')) / (self.item_height + self.padding.y)) + ' filtered_idx=' + ai.filtered_idx + ' before=' + (ai.filtered_idx < first_visible_result) + ' after=' + (ai.model_idx > last_visible_result));
				available.push(d);
			}
		});
		if(to_reassign.length > available.length) console.warn('details.scroll: not enough available elements: need=' + to_reassign.length + ' available=' + available.length);
		for(var i in to_reassign){
			if(i < available.length){
				self._assign(available[i], to_reassign[i]);
			}
		}

		//console.log('details.scroll: to_reassign=' + to_reassign.length + ' available=' + available.length);
		if(false && to_reassign.length){
			setTimeout(function(){
				for(var d in self.assigned){
					var ai = self.assigned[d];
					console.log('  ' + d + ': ' + ai.model_idx + ': x=' + ai.position.left + ' y=' + ai.position.top + ' ' + ai.dom.find('img').length);
				}
			}, 1000);
		}

		$.ui.viewBase.prototype.scroll.apply(this, arguments);
	},

	resize: function() {
		var self = this, o = self.options;

		if(!self.element.is(':visible') && !o.grid.options.size_fits_content) return;
		self._old_n_visible_items = self.cache.n_cells;
		var old_n_columns = self.n_columns;

		//how many items can we fit?
		var container_height = o.grid.options.size_fits_content
			? self.cache.max_height
			: self.element.parent().height();
		var n_items = self.n_columns * Math.ceil(container_height / self.item_height);
		self.cache.n_cells = n_items + self.n_columns * settings.overscan;

		if((self.n_columns !== self._old_n_columns) || (self.cache.n_cells !== self._old_n_visible_items)){
			self.__post_resize_assignment();

			o.products.have.filtering.done(function(){
				self.cache.n_items = Math.min(n_items, o.products.filtered.length);
				self._assign_fresh();
				self._set_scrollbar_size();
			});
		}
	},

	_new_element: function(d) {
		return {css: {top: d * this.item_height}};
	},

	_assign: function(dom_idx, model_idx) {
		var self = this, o = this.options;

		if(model_idx >= o.products.items.length) return;

		if(!self.assigned[dom_idx]){ console.warn('_assign'); self.assigned[dom_idx] = new AssignedItem('detailItem', self.item_options); }

		$.ui.viewBase.prototype._assign.apply(this, arguments);

		var ai = self.assigned[dom_idx];

		var row = Math.floor(self._get_filter_idx(model_idx) / self.n_columns);
		//console.log('assign: m=' + model_idx + ' row=' + row + ' column=' + column + ' y=' + (row * (self.item_height + self.padding.y)));

		ai.dom.css(ai.position = {
			top: row * (self.item_height + self.padding.y)
		});
	}
});

var settings = {
	overscan: 2   // assign extra rows to allow for scrolling. currently only helps when scrolling down, not up.
}

});
