/*
 *  This experimental version of the groups2 view uses virtual-dom to simplify updates.
 *
 *  It appears to be fully working but is not optimised. Is there any benefit
 *  in not redrawing the whole grid each time a thumbnail is retrieved?
 *
 *  TODO
 *    - it appears that _all_ products are being allocated
 */
define(['jquery', 'ajax_queue', 'utils', './thumb_item', 'virtual-dom/h', 'virtual-dom/diff', 'virtual-dom/patch', 'virtual-dom/create-element', 'vdom-virtualize', '../products', 'chart-app/router', './view_base'], function($, Ajaxqueue, Utils, ItemView, h, diff, patch, createElement, virtualise, Products, page){

var show_title = true;
var img_data = {}

$.widget('ui.groups', $.ui.viewBase, {

	_create: function() {
		var self = this, o = self.options;
		var obj = $.ui.viewBase.prototype._create.apply(this, arguments);

		this.ItemView = ItemView;

		this.item_height = 80 + (show_title ? 38 : 0);
		this.item_width = 127;
		this.padding = {x: 8, y: 16};
		this.n_columns = 0;
		this.group_label_height = 34;

		show_title && (this.item_options.show_title = true);

		this.element.addClass('thumbs');

		var root = self.element.find('.vdom-root')
		if(!root.length){
			self.dom_tree = h('div.vdom-root');
			self.element.append(self.dom_tree_root = createElement(self.dom_tree));
		}else{
			self.dom_tree = virtualise(self.dom_tree_root = root[0]) // get a virtualNode object
		}

		this._queue_redraw = $.throttle(100, function() {
			self._redraw();
		});

		self.element.find('.vdom-root').on('click', function(ev) {
			console.log('groups.click');
			var target = $(ev.target);

			var item = Products.product_by_name(target.attr('name') || target.find('img.thumbnail-img').attr('name'));
			self.item_options.on_click(item);
		});

		// use thumbnail data from the product list if available
		$.each(o.products.items, function(i, v){
			if(v.data.thumbnail) img_data[v.name()] = v.data.thumbnail
		});

		return obj;
	},

	_destroy: function() {
		console.log('thumbs.destroy');
		this.element.empty().removeClass('thumbs');
		this.options.products.unsubscribe(this);
		this._super();
	},

	_assign_fresh: function() {
		$.ui.viewBase.prototype._assign_fresh.apply(this, arguments);

		this.groups_valid = false;
		this._redraw();
	},

	// base methods we _may_ wish to prevent
	__post_resize_assignment: function() {
	},
	add_remove_items: function(add, remove) {
	},
	_unassign: function(dom_idx){
	},
	_start_progress: function() {
	},
	_assign_fresh: function() {
		// dont know if these are relevant with vdom
		this._on_n_viewable_items_changed();
		this.have.assignment.resolve();
	},
	_assign: function(dom_idx, model_idx) {
	},

	_invalidate_positions: function() {
		this._positions_valid = false;
	},

	_calc_positions: function() {
		var self = this;

		this.groups_valid || this._update_groups();

		var height = self.cache.height
		var scrolltop = self.element.parent().scrollTop();
		var overscan = settings.overscan * self.item_height;

		var label_height = (self.groups.length > 1) ? self.group_label_height : 0;

		var found = false;
		var y = -(self.item_height + self.padding.y);
		self._results_by_group_all(
			function(group){
				y += self.item_height + self.padding.y;
				group.y = y;
			},
			function(item, i, ig, gr){
				y = gr.y + label_height + Math.floor((ig / self.n_columns)) * self.item_height;
				gr.height = label_height + Math.ceil(((ig + 1) / self.n_columns)) * self.item_height + self.padding.y;

				var visible =
					(y > scrolltop - overscan) &&
					(y < scrolltop + height + overscan);

				if(!found && visible){
					found = true;
					self._first = i;
				}

				item[self.eventNamespace] = item[self.eventNamespace] || {};
				$.extend(item[self.eventNamespace], {
					ig: ig,
					y: y,
					visible: visible
				});
			}
		);
		this._positions_valid = true;
	},

	_redraw: function() {
		var self = this, o = self.options;
		//console.log('%cgroups.redraw', 'background-color:#ff9')

		Products.cancel_thumbnails();

		this.groups_valid || this._update_groups();
		this._positions_valid || self._calc_positions();

		labels = [];
		items = [];
		var group_label_height = (self.groups.length > 1) ? self.group_label_height : 0;

		var y = -(self.item_height + self.padding.y);
		self._results_by_group(
			function(group){
				if(self.groups.length > 1){
					labels.push(h('h3', {
						style: {top: (group.y - 8) + 'px'},
						onclick: function(e){
							page.add_state();
							facets = o.products.lookup_filter_by_name('facets')
							facets.set(['Product group', group.name], true)
							e.stopPropagation()
						}
					},
					[group.name]));
				}
			},
			function(item, m, d, ig, gr){
				var local = item[self.eventNamespace];

				var column = ig % self.n_columns;

				var url = img_data[item.name()] || item.cache.thumbnail.call(item); // cannot use the normal thumbnail method here because it is asynchronous

				items.push(h('div.item.thumbnail-wrapper' + (local.selected ? '.selected' : ''), {style: {top: local.y + 'px', left: column * (self.item_width + self.padding.x) + 'px'}}, [
					h('div.thumbnail', [
						h('img.thumbnail-img', {
							alt  : item.title_(),
							title: $('<div>').html(item.data.description).text() || item.title_(),
							name : item.name(),
							src  : url || Utils.blank_gif,
							style: url
								? { visibility: 'visible', opacity: 1.0 }
								: { visibility: 'hidden', opacity: 0.0 },
							onclick: function() {
							}
						})
					]),
					h('div' + (show_title ? '.title' : '.hidden') + (item.has_widget('clickable') ? '.clickable' : ''), item.title_())
				]));

				!url && item.thumbnail && item.thumbnail({tag: item.name() + self.eventNamespace}, function(url, image, error){
					if(!error){
						if(image){
							// we may have multiple callbacks from the same request, so img_data may already be set
							img_data[item.name()] = image;
						}
						self._queue_redraw();
					}
				});
			}
		);

		items.push(h('div.group-labels', labels));

		var new_tree = h('div.vdom-root', items);
		self.dom_tree_root = patch(self.dom_tree_root, diff(self.dom_tree, new_tree));
		self.dom_tree = new_tree;

		self.element.removeClass('floated') // remove class used for backend renders
	},

	select: function(item) {
		var self = this, products = this.options.products;

		products.have.filtering.done(function() {
			if(!item[self.eventNamespace]) item[self.eventNamespace] = {}; // do this earlier?
			var local = item[self.eventNamespace];
			self._each_allocated2(function(item, m, d){
				delete item[self.eventNamespace].selected;
			});
			local.selected = true;
			self._queue_redraw();
		});
	},

	scroll: function(top) {
		var self = this, o = this.options;

		self._invalidate_positions();
		self._queue_redraw();

		this.have.assignment.done(function() {
			$.ui.viewBase.prototype.scroll.apply(self, arguments);
		});
	},

	_update_groups: function() {
		var self = this, group, list = this.options.products;

		self.groups = [];
		for(var i=0;i<list.filtered.length;i++){
			var item = list.items[list.filtered[i]];
			var g = ((item.data.tags || [])['Product group'] || [])[0] || 'Other';
			if(g !== group){
				self.groups.push({name: group = g, i: i});
			}
		}
		//console.log('* _update_groups: groups=', self.groups.length, self.groups);

		this.groups_valid = true;
		self._positions_valid = false;
		return self.groups.length > 1;
	},

	_each_allocated2: function(fn) {
		var self = this, o = self.options, list = o.products;

		self._positions_valid || self._calc_positions();

		for(var i=self._first;i<list.filtered.length;i++){
			var m = list.filtered[i];
			var item = list.items[list.filtered[i]];
			var local = item[self.eventNamespace];
			//if(!local.visible) console.log('each_allocated: range ', self._first, i, ' (' + (i - self._first) + ')');
			if(!local.visible) break;

			fn(item, m, i);
		}
	},

	// only results in viewport
	_results_by_group: function(fn1, fn2) {
		var self = this;

		self._positions_valid || self._calc_positions();

// TODO g may be incorrect if scrolling
		var g = -1, gr = {};
		this._each_allocated2(function(item, m, d){
			var local = item[self.eventNamespace];
			var ig = local.ig;
			var group_name = ((item.data.tags && item.data.tags['Product group'] || [])[0]) || 'Other';
			if(g < 0 || (group_name && group_name !== gr.name)){
				g++;
				gr = $.findWhere(self.groups, {name: group_name});
				fn1(gr);
			}
			fn2(item, m, d, ig++, gr, g);
		});
	},

	_results_by_group_all: function(fn1, fn2, done) {
		var self = this, list = this.options.products;

		var g = -1, ig = 0, gr = {};
		for(var i=0;i<list.filtered.length;i++){
			var item = list.items[list.filtered[i]];
			var group_name = ((item.data.tags && item.data.tags['Product group'] || [])[0]) || 'Other';
			if(!i || (group_name !== gr.name)){
				g++;
				gr = $.findWhere(self.groups, {name: group_name});
				fn1(gr);
				ig = 0;
			}
			fn2(item, i, ig++, gr, g);
		}
		done && done(gr);
	},

	// should not be called directly: call grid._update({sizing: true}) instead
	resize: function() {
		var self = this, o = self.options;

		if(!self.element.is(':visible') && !o.grid.options.size_fits_content) return;
		self._old_n_visible_items = self.cache.n_cells;
		var old_n_columns = self.n_columns;

		var container_size = {width: self.element.parent().width(), height: window.is_node ? 600 : self.element.parent().height()};
		self.cache.height = container_size.height;

		self.n_columns = Math.floor(container_size.width / (self.item_width + self.padding.x));

		if(!self.n_columns) self.n_columns = 5, console.warn('failed to calculate number of columns');

		//how many items can we fit?
		self.cache.n_cells = self.n_columns * (Math.round(container_size.height / (self.item_height + self.padding.y)) + settings.overscan);

		if((self.n_columns !== old_n_columns) || (self.cache.n_cells !== self._old_n_visible_items)){
			//console.log('thumbs._resize: need re-layout: height=' + container_size.height + ' n_visible=' + self.cache.n_cells);
			self.__post_resize_assignment();
			o.products.have.filtering.done(function(){
				self._assign_fresh();
				self._set_scrollbar_size();
			});
		}
	},

	hide_show_items: function(args) {
		this.cache.max_items = null;
		this.cache.natural_height = null;
		this._positions_valid = false;
		this.groups_valid = false;

		$.ui.viewBase.prototype.hide_show_items.apply(this, arguments);
	},

	// return maximum number of items that can be shown when scrolled to top.
	max_items: function() {
		var self = this;
		var max = 0;

		if(this.cache.max_items) return this.cache.max_items;

		var y = 0;
		var gi = 0;
		this.options.products.each_by_facet('Product group',
			function(group){
				y += self.group_label_height;
				gi = 0;
			},
			function(item, i, ig){
				if(ig % self.n_columns === 0){
					y += self.item_height + self.padding.y;
				}
				if(y < self.cache.max_height){
					max = i + 1;
				}else{
					// TODO list.each_by_facet should respect return value of false
					return false;
				}
				gi++;
			}
		);

		return this.cache.max_items = max;
	},

	// return the height of all the filtered items
	natural_height: function() {
		var self = this;

		if(this.cache.natural_height) return this.cache.natural_height;

		this.groups_valid || this._update_groups();
		self._positions_valid || self._calc_positions();

		var h = 0;
		$.each(this.groups, function(i, gr) {
			if(gr.height) h += gr.height;
		});

		return this.cache.natural_height = h;
	},

	_new_element: function(d) {
		return {};
	}
});

var settings = {
	overscan: 2   // assign extra rows to allow for scrolling. currently only helps when scrolling down, not up.
}

});
