/*
 *  Thumbnail view with grouping.

 *  Contrary to the purpose of the Grid widget, this view Does NOT use dynamic allocation of items.
 *  Note also that there is a dependency on a particular filter (path filter)
 *
 *  This groups by package, and not by Product Group
 */

define(['utils', 'jquery', 'filterable-list', '../filters/path', './thumb_item', './view_base'], function(Utils, $, list, path_filter, ItemView){

$.widget('ui.groups', $.ui.viewBase, {

	_create: function() {
		var self = this;
		var obj = $.ui.viewBase.prototype._create.apply(this, arguments);

		this.ItemView = ItemView;

		this.element.addClass('thumbs groups');

		setTimeout(function() {
			self.element.is(':empty') && self._update();
		}, 1000);

		return obj;
	},

	_destroy: function() {
		console.log('thumbs.destroy');
		this.element.empty().removeClass('thumbs groups');
		this._super();
	},

	_update: function() {
		var self = this, o = this.options;

		self.element.empty();

		var path = path_filter._selected;
		var section = '';
		$.each(list.filtered, function(i, v) {
			var item = list.items[v];
			var _path = (item.data.path ? item.data.path[0].split('/') : []).slice(path.length);
			var _section = _path.length ? _path[0] : path[path.length - 1]
			_section === section || self.element.h2(Utils.ucfirst(_section.replace(/-/g, ' ')));
			
			self.element.div().thumbItem($.extend({}, self.item_options, {item: item}));

			section = _section;
		})
	},

	_on_n_viewable_items_changed: function() {
		this._update();
		this._super();
	},

	scroll: function(top) {
	},

	resize: function() {
	},

	_assign: function(dom_idx, model_idx) {
	},

	_set_scrollbar_size: $.debounce(400, function() {
	})
});

});
