/*
 *  This experimental version of the thumbs view adds grouping but unlike
 *  the other groups view, preserves the dynamic allocation.
 *  The intention is to merge this back into thumbs.js once it is working.
 *  It would need to be optimised before merging back - updates are currently
 *  not as efficent because the layout is more dynamic as the filtering changes.
 *  See also the version that uses virtual-dom.
 */
define(['jquery', 'filterable-list', '../assigned_item', './thumb_item', './view_base'], function($, list, AssignedItem, ItemView){
var show_title = true;

$.widget('ui.groups', $.ui.viewBase, {

	_create: function() {
		var self = this, o = self.options;
		var obj = $.ui.viewBase.prototype._create.apply(this, arguments);

		this.ItemView = ItemView;

		this.item_height = 64 + (show_title ? 38 : 0);
		this.item_width = 127;
		this.padding = {x: 8, y: 18}; // spacing outside the item border
		this.n_columns = 0;
		this.group_label_height = 34;

		show_title && (this.item_options.show_title = true);

		this.element.addClass('thumbs');

		list.subscribe(self, function() { // todo unsubscribe
			self.cache.max_items = null;
		});

		return obj;
	},

	_destroy: function() {
		console.log('thumbs.destroy');
		this.element.empty().removeClass('thumbs');
		this._super();
	},

	_assign_fresh: function() {
		var self = this;
		var group_label_height = 0;

		$.ui.viewBase.prototype._assign_fresh.apply(this, arguments);

		$('.group-labels').remove();
		if(this._update_groups()){
			var labels = self.element.div('.group-labels');
			group_label_height = self.group_label_height;
		}

		// calculate positions
		var y = -(self.item_height + self.padding.y);
		self._results_by_group(
			function(group){
				y += self.item_height + self.padding.y;
				group.y = y;
				labels && labels.h3(group.name).css({top: y});
				y += group_label_height + self.padding.y;
				console.log(' ^ new group', group.i, [group.name], 'gr.y=', group.y);
			},
			function(item, m, d, ig, gr){
				y = gr.y + (self.groups.length > 1  ? group_label_height : 0) + Math.floor((ig / self.n_columns)) * (self.item_height + self.padding.y);
				$.extend(self.assigned[d], {ig: ig, gr: gr, y: y});
				console.log('    g=', gr.i + '.' + ig, ' y=', y, ' gr.y=', gr.y);
				y += self.padding.y;

				// TODO not always neccesary?
				// TODO if m is already assigned to a different d, the existing assignment should be preserved.
				self._assign(d, m);
			}
		);
	},

	_layout: function() {
	},

	scroll: function(top) {
		var self = this, o = this.options;

		this.have.assignment.done(function() {
			//var container_bottom = -top + self.element.parent().height();

			var visible_results = {
				first: Math.floor((-top) / (self.item_height + self.padding.y)) * self.n_columns
			}
			visible_results.last = visible_results.first + self.cache.n_cells - 1
			var to_reassign = [];
			for(var f=visible_results.first;f<visible_results.first+self.cache.n_cells;f++){
				if(!(f < list.filtered.length)) break;

				var m = list.filtered[f];
				if(!self._m_is_assigned(m)){
					to_reassign.push(m);
				}
			}
			var available = [];
			self._each_allocated(function(item, m, d){
				var ai = self.assigned[d];

				if((typeof ai.model_idx === 'undefined') || (ai.model_idx === null)){ console.warn('scroll: TODO error iterating over self.assigned. d=', d, ' ai=', ai, ' n_assigned=', self.assigned.length); return; }

				if(ai.filtered_idx < visible_results.first || ai.filtered_idx > visible_results.last || item.filter_pass !== true){
					//var item = self.items[ai.model_idx];
					//console.log('  available: ' + d + ': ' + item.name + ' row=' + (parseInt(ai.dom.css('top')) / (self.item_height + self.padding.y)) + ' filtered_idx=' + ai.filtered_idx + ' before=' + (ai.filtered_idx < visible_results.first) + ' after=' + (ai.model_idx > visible_results.last));
					available.push(d);
				}
			});
			if(to_reassign.length > available.length) console.warn('not enough available elements');
			for(var i in to_reassign){
				if(i < available.length){
					self._assign(available[i], to_reassign[i]);
				}
			}

			//console.log('thumbs.scroll: ' + parseInt(top, 10) + ' visible: ' + visible_results.first + ' --> ' + visible_results.last + ' to_reassign=' + to_reassign.length + ' available=' + available.length);
			if(false && to_reassign.length){
				setTimeout(function(){
					for(var d in self.assigned){
						var ai = self.assigned[d];
						console.log('  ' + d + ': ' + ai.model_idx + ': x=' + ai.position.left + ' y=' + ai.position.top + ' ' + ai.dom.find('img').length);
					}
				}, 1000);
			}

			$.ui.viewBase.prototype.scroll.apply(self, arguments);
		});
	},

	_update_groups: function() {
		var self = this, group;

		self.groups = [];
		for(var i=0;i<list.filtered.length;i++){
			var item = self.items[list.filtered[i]];
			var g = ((item.data.tags || [])['Product group'] || [])[0] || 'Other';
			if(g !== group){
				self.groups.push({name: group = g, i: i});
			}
		}
		//console.log('* _update_groups: groups=', self.groups.length, self.groups);

		return self.groups.length > 1;
	},

	_results_by_group: function(fn1, fn2) {
		var self = this;
		var g = -1, ig = 0, gr = {};
		var other = {name: 'Other'};
		this._each_allocated(function(item, m, d){
			var group_name = (item.data.tags && item.data.tags['Product group'] || [])[0];
			if(!d || (group_name && group_name !== gr.name)){
				g++;
				gr = group_name ? $.findWhere(self.groups, {name: group_name}) : (other.i = g, other);
				fn1(gr);
				ig = 0;
			}
			fn2(item, m, d, ig++, gr, g);
		});
	},

	// should not be called directly: call grid._update({sizing: true}) instead
	resize: function() {
		var self = this, o = self.options;

		//self._update_groups();
		this.cache.natural_height = null;
		this.cache.max_items = null;

		if(!self.element.is(':visible') && !o.grid.options.size_fits_content) return;
		self._old_n_visible_items = self.cache.n_cells;
		var old_n_columns = self.n_columns;

		var container_size = {width: self.element.parent().width(), height: self.element.parent().height()};

		self.n_columns = Math.floor(container_size.width / (self.item_width + self.padding.x));

		if(!self.n_columns) self.n_columns = 5, console.warn('failed to calculate number of columns');

		//how many items can we fit?
		self.cache.n_cells = self.n_columns * (Math.round(container_size.height / (self.item_height + self.padding.y)) + settings.overscan);

		if((self.n_columns !== old_n_columns) || (self.cache.n_cells !== self._old_n_visible_items)){
			//console.log('thumbs._resize: need re-layout: height=' + container_size.height + ' n_visible=' + self.cache.n_cells);
			self.__post_resize_assignment();
			list.have.filtering.done(function(){
				self._assign_fresh();
				self._set_scrollbar_size();
			});
		}
	},

	_new_element: function(d) {
		return {};
	},

	_assign: function(dom_idx, model_idx) {
		var self = this, o = this.options;

		if(model_idx >= list.items.length) return;

		var ai = self.assigned[dom_idx];
		if(!ai){ console.warn('thumbs._assign'); ai = self.assigned[dom_idx] = new AssignedItem('thumbItem', self.item_options); }

		$.ui.viewBase.prototype._assign.apply(this, arguments);

		if(!ai.gr){
			// TODO do not call assign until the group has been set
			//console.log(' :-( ai=', ai);
			return;
		}

		var column = ai.ig % self.n_columns;
		//console.log('     * g=', ai.gr.i + '.' + ai.ig, ' row=', row, ' top=', ai.gr.y, ai.gr.y + self.group_label_height + row * (self.item_height + self.padding.y));
		ai.dom.css(ai.position = {
			top: ai.y,
			left: column * (self.item_width + self.padding.x),
			width: self.item_width,
			height: self.item_height
		});
	},

	// return maximum number of items that can be shown when scrolled to top.
	max_items: function() {
		var self = this;
		var max = 0;

		if(this.cache.max_items) return this.cache.max_items;

		var y = 0;
		var gi = 0;
		list.each_by_facet('Product group',
			function(group){
				if(self.groups.length > 1) y += self.group_label_height;
				gi = 0;
			},
			function(item, i, ig, gname, g){
				var ai = self.assigned[i];
				if(ig % self.n_columns === self.n_columns - 1){
					y+= self.item_height + self.padding.y;
				}
															console.error('max never updated');
				/*
															// XXX this will not work if not all items are assigned
				if(ai.y < self.cache.max_height){
					max = i + 1;
				}
				*/
				gi++;
			}
		);

		return this.cache.max_items = max;
	},

	natural_height: function() {
		// TODO can this be cached?
		var self = this;

		if(this.cache.natural_height) return this.cache.natural_height;

		var y = 0;
		var gi = 0;
		list.each_by_facet('Product group',
			function(group){
				y += self.group_label_height + Math.ceil(gi / self.n_columns) * (self.item_height + self.padding.y);
				gi = 0;
			},
			function(item, i, ig){
				gi++;
			}
		);
		y += Math.ceil(gi / self.n_columns) * (self.item_height + self.padding.y);
		console.log('groups2.natural_height: done: y=', y);

		this.cache.natural_height = y;
		return y;
	}
});

var settings = {
	overscan: 1   // assign extra rows to allow for scrolling. currently only helps when scrolling down, not up.
}

});
