define(['jquery', 'ajax_queue', 'utils', './item_base'], function($, Ajaxqueue, Utils){

$.widget('ui.thumbItem', $.ui.viewItemBase, {

	options: {
		item: null,   // a copy - modifications will not be seen by caller. Is allowed to be null
		kind: 'product',
		grid: null,
		view: null,      // used to get read/write access the model.
		progress: true,
		on_select: $.noop
	},

	_create: function() {
		var self = this, o = this.options;

		self.request_counter = 0;

		$.fn.thumbnailTools && self.element
			.thumbnailTools()
			.thumbnailTools('add', 'ui-icon-trash', function(){
				self.bus.send(o.kind + '-thumbnail-delete', o.item);
			})
			.thumbnailTools('add', 'ui-icon-pencil', function(){
				self.bus.send(o.kind + '-thumbnail-edit', o.item);
				self.element.thumbnailTools('hide');
			})
			.thumbnailTools('add', 'ui-icon-extlink', function(){
				self.bus.send(o.kind + '-thumbnail-open', o.item);
				self.element.thumbnailTools('hide');
			});
	},

	_init: function() {
		var self = this, o = this.options;

		if(!o.item){
			if(self.existing){
				self.__cancel();
				self.existing = null;
			}
			self.element.addClass('empty').empty();
			return;
		}

		this._super();

		this.element.removeClass('empty').addClass('thumbnail-wrapper');
		self.element.find('.thumbnail').removeClass('pending');

		this.__cancel = (function(id) {
			return function(){ self._cancel(id); };
		})(this.eventNamespace);

		o.progress && self._cancel_progress();

		(self.img = self.element
			.find('img')
			.orelse(function(){
				var img = self.element.div('.item thumbnail')
					.img()
						.addClass('thumbnail-img');
				o.show_title && (self.title = self.element
					.div('.title'));
				return img;
			}))
			.each(function(){
				if(self.existing !== o.item.name()){
					$(this).attr({
						alt  : o.item.title_(),
						title: $('<div>').html(o.item.data.description).text() || o.item.title_(),
						name : o.item.name(),
						src  : Utils.blank_gif
					})
					.css({visibility: 'hidden', opacity: 0});

					self.title && self.title
						.text(o.item ? o.item.title_() : '')
						[o.item.has_widget('clickable') ? 'addClass' : 'removeClass']('clickable');

					var request = ++self.request_counter;

					o.progress && self._start_progress();

					self.thumbnail_loading = true;
					o.item.thumbnail && o.item.thumbnail({tag: o.item.name() + self.eventNamespace}, function(url, image, error){

						if(request !== self.request_counter) return;

						self.thumbnail_loading = false;
						o.progress && self._cancel_progress();

						if(!error){
							if(request === self.request_counter){
								self.img/*.css({height: 'auto', width: 'auto'})*/
									.attr('src', image || url)
									.css({visibility: 'visible', opacity: 1.0});
							}
						}else{
							console.warn('thumbItem: ', error);
							self.img
								.css({visibility: 'visible', opacity: 1});
						}
						self.element.find('.thumbnail').removeClass('pending');

						self.__cancel = $.noop;
					});

					self.existing = o.item.name();
				}
				else console.warn('thumbItem.init: unchanged');
			});
	},

	_cancel: function(id) {
		Ajaxqueue.cancelByTag(this.existing + id);
	},

	_start_progress: function() {
		var self = this;
		var p = self._progress = self._progress || {
			bar: $([])
		};
		var request = self.request_counter;

		self._progress.timer = setTimeout(function() {
			p.timer = null;
			if(self.request_counter !== request) return;

			var time = 4;
			(function(){
				var r = request;

				self.element.find('.thumbnail').addClass('pending');

				p.interval = setInterval(function() {
					if(time < 90 && self.thumbnail_loading){
						p.bar = self.element.find('.thumb-progress-bar').orelse(function(){ return self.element.div('.thumb-progress-bar smooth-width'); });
						p.bar.css({width: (Math.min(100, (100 * Math.log(time/2) / 4)) - 10) + '%'});
					}else{
						self._cancel_progress();
					}
					time++;
				}, 1000);
			})();
		}, 4000);

	},

	_cancel_progress: function() {
		var p = this._progress;

		if(p){
			p.timer = clearTimeout(p.timer);
			p.interval = clearInterval(p.interval);
			p.bar.remove(); p.bar = $([]);
		}
	},

	destroy: function(){
		this.element.off().removeClass('thumbnail-wrapper');
	},

	_set_thumbnail: function(){
	}
});

return {
	widgetName: 'thumbItem',
	className: 'thumbnail-wrapper'
}

});
