/*
 *   Abstract base class for managing event handlers.
 */
define(['jquery', 'bus', 'jquery-ui/widget'], function($, Bus) {

$.widget('ui.handler', {

	options: {
		messageBus: Bus()
	},

	_message_handler_init: function(){
		var self = this;

		self._handlers = {};
		self._live_handlers = {};
	},

	_message_handler_destroy: function(){
		this._remove_handlers();
		this._die_all();
		$.Widget.prototype.destroy.call(this);
	},

	_add_handler: function(message_name, fn){
		var self = this, o = this.options;

		//if(self._handlers[message_name]) o.messageBus.forget(message_name); //no, allow multiple listeners.

		o.messageBus.listen(message_name, fn);
		self._handlers[message_name] = true;
		return self;
	},

	_remove_handlers: function(){
		var self = this, o = this.options;

		for(var i in self._handlers){
			o.messageBus.forget(i, self._handlers[i]);
		}
		self._handlers = {};
		return self;
	},

	_add_live: function(selector, event_name, fn){
		var self = this;

		$(document).on(event_name, selector, fn);
		self._live_handlers[selector] = {selector: selector, event_name: event_name};
		return self;
	},

	_die_all: function(){
		var self = this;

		for(var i in self._live_handlers){
			$(document).off(self._live_handlers[i].event_name, self._live_handlers[i].selector);
		}
		self._live_handlers = {};
		return self;
	}
});

});
