/**
 *  Help panel
 *
 *  (C) Copyright ECWMF http://www.ecmwf.int
 *
 *  This software is licensed under the terms of the Apache Licence Version 2.0
 *  which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 *  In applying this licence, ECMWF does not waive the privileges and immunities
 *  granted to it by virtue of its status as an intergovernmental organisation nor
 *  does it submit to any jurisdiction.
 *
 */
define(['config', 'jquery', './help_panel.css'], function(ECMWF, $){

var no_content = {title: '', content: ''};

$.fn.helpProvider = function(args){

	var o = $.extend({
		getContent: function(){ return no_content; }
	}, args || {});

	if(args === 'getContent'){
		var data = $(this).data('helpProvider');
		return data ? data.getContent() : null;
	}

	return $(this).each(function(){
		$(this).data('helpProvider', o);
	});
}

return function(args) {

	var o = $.extend({
		el: null,         //dom element that we are providing help for.
		id: null
	}, args || {});

	require(['light_dialogue'], function() {
		var help;

		if(!o.el || !o.el.data('helpProvider')) {
			return console.warn('helpPanel: element is not a HelpProvider', o.el);
		}

		function ensure_panel() {
			var id = o.id || o.el.attr('id') || o.el.parent().parent().attr('id')
			$('body > .help-panel[name="' + id + '"]')
				.orelse(function(){
					var button = o.el.find('.ui-icon-help:first')
					return $('body')
						.div({className: 'help-panel', name: id})
							.html('<div class="content">Loading help ...</div>')
							.lightDialogue({
								title: '<span>Dashboard Widget Help</span>',
								name: 'dashboard_help',
								loader: true,
								position: button.offset(),
								zIndex: 2000
							})
							//.on('remove', function(){  console.log('>>>>> dialogue was removed');})
							.find('.content').each(function(){
								help = $(this);
							});
				})
		}

		function update_help() {
			ensure_panel();

			if(help){
				help.trigger('loading');

				$.when(o.el.helpProvider('getContent')).then(function(out){
					out || (out = no_content);
					help
						.markup(out.content || 'no help available')
						.siblings('.title').find('> span').text(out.title || 'No Help')
						.trigger('loading', 'false');
				});
			}
		}

		o.el.on('contentChanged', update_help); //FIXME needs to be unbound when active widget changes

		/*
					var help_top = help ? ($(help).position().top + $(help).offset().top) : 0;
					if(help) console.log('help_top: pos.top=' + $(help).position().top, help); else console.log('no help');
					var st = $(window).scrollTop();
					var new_position = (help_top < st) ? {top: st + 20} : null;

					//set position.left
					if((help && new_position) || !help){
						new_position || (new_position = {});
						var button_left = button.position().left + button.offset().left;
						new_position.left = button_left + 48;
						if(new_position.left + 250 > $(window).width()){
							console.log('too far right. moving to left...');
							var button_left = button.position().left + button.offset().left;
							new_position.left = button_left - 250;
						}
					}
		*/

		ensure_panel();

		o.el.on('click', '.ui-icon-help', function(){
			update_help();
			return false;
		});

		update_help();
	})
	return this;
}

});
