define(['config', 'jquery', 'bus', 'ajax_queue', 'chart/layer', 'form', './layer_list.css'], function(ECMWF, $, Bus, Ajaxqueue, Layer) {

	var count = 0;

	$.widget('ui.layerList', {

		options: {
			messageBus: Bus()
		},

		_init: function() {
			var self = this, o = this.options;

			self.element.addClass('layer-list');
			self.layers = [];
			self.layers_map = {};
			self.id = ++count;

			o.messageBus
				.listen('chart-content-update', function(event, msg) {
					self._update(msg.product, msg.time);
				})
				.listen('open-layer-thumbnail-dialogue', function(event, msg) {
					self._open_thumbnail_dialogue();
				});
		},

		destroy: function() {
			this.element.removeClass('layer-list');

			this.sortable && this.sortable.sortable('destroy');

			this.element.empty();

			return this;
		},

		_update: function(product, time) {
			var self = this;

			var layers = product.layerList();
			var same = false;

			if (layers.length === self.layers.length) {
				same = true;
				for (var i = 0; i < layers.length; i++) {
					if (layers[i].id !== self.layers[i].id) {
						same = false;
						break;
					}
				}
			}

			self[same ? '_refresh' : '_rebuild'](product, layers, time);

			self.layers = layers;
		},

		_refresh: function(product, layers, time) {
			var self = this;

			$.each(layers, function(j, layer) {
				var meta = self.layers_map[layer.id];
				meta.input.attr('checked', layer.visible);

				meta.title.markup(new Layer(layer).get_title());

				Ajaxqueue({
					cancelled_by: product.owner,
					url: ECMWF.options.ajax.backend.layer_icon,
					priority: 'low',
					cache: 60 * 60,
					json: {
						product: product.as_json(null, time),
						layer: layer.id
					},
					img: meta.img
				});
			});

			self._update_form(layers);
		},

		_rebuild: function(product, layers, time) {
			var self = this, o = this.options;

			self.element.parents('.menu-pane').css({width: ''});
			self.layers_map = {};

			self.sortable && self.sortable.sortable('destroy');

			self.element.empty().div()
				.div('.add-layer')
				.button({
					label: 'Add Layers ...',
					icons: {primary: 'ui-icon-circle-plus'}
				})
				.click(function () {
					self._open_thumbnail_dialogue();
					return false;
				})

			var table = $('<table>');
			var tbody = table.tbody();

			var handles = $();

			$.each(layers, function(j, layer) {

				var i = layers.length - j - 1;
				layer = layers[i];
				var layer_id = layer.id;

				var meta = self.layers_map[layer_id] = {};

				var tr = tbody.tr('.ui-state-default layer-list-row').attr('valign', 'top').data('layer_id', layer_id);

				handles = handles.add(tr.td().span('.ui-icon ui-icon-arrowthick-2-n-s'));

				var input = tr.td()
					.input({
						type: 'checkbox',
						checked: layer.visible
					})
					.change(function() {
						o.messageBus.send('layer-visibility-change', {
							layer_id: layer_id,
							visibility: input.is(':checked')
						});
						self._update_form(layers);
					});

				meta.input = input;

				// Hardcoding the width prevents unexpected resizing of the layer list
				var img = tr.td().img({width: 48, height: 24});

				meta.img = img;
				handles = handles.add(img);

				product.layer_icon(layer.id, img, time);
				meta.title = tr.td('.title').markup(new Layer(layer).get_title());

				var vars = layer.variables || [];
				var styles = layer.styles || [];

				function make_form() {
					var items = [];
					$.each(vars, function (i, val) {
						items.push(val);
					});

					var names = [];
					var labels = {};
					$.each(styles, function(j, style) {
						names.push(style.name);

						labels[style.name] = {
							url: '//' + ECMWF.options.backend_host + '/backend/get-style-thumbnail/' + style.name + '/',
							title: style.title
						};
					});

					if (names.length > 1) {
						items.push({
							name: 'style',
							title: 'Style',
							type: 'thumbnail',
							value: layer.style,
							values: names,
							labels: labels,
						});
					}

					var form = {
						items: items,
						values: layer.values || {},
						show_cancel: false,
						ok_label: 'Apply',
						ok: function(result) {
							o.messageBus.send('layer-options-change', {
								layer_id: layer_id,
								options: result
							});

						}
					};

					return form;
				}

				tr.td().each(function() {
					var td = $(this);

					if (vars.length || styles.length > 1) {

						td.a({title: 'Edit'})
							.button({
								text: false,
								icons: {
									primary: 'small-icon-pencil small-icon'
								}
							})
							.click(function() {
								meta.form_row.toggle();
								self.element.delay(0).resize();
							})
							.find('.ui-icon')
								.removeClass('ui-icon')

					} else {
						td.span();
					}
				});

				meta.trash = tr.td().a({title: 'Delete'})
					.button({
						icons: {
							primary: 'small-icon-trash small-icon'
						}
					})
					.click(function() {
						if(!$(this).button('option', 'disabled')){
							tr
								.addClass('disabled')
								.find('.ui-button').button('option', 'disabled', true);

							o.messageBus.send('layer-delete', { layer_id: layer_id });
						}
					})

				if (vars.length || styles.length > 1) {
					meta.form_row = $('<tr>').appendTo(tbody).addClass('layer-list-form-row');
					meta.form = $('<td>').addClass('layer-list-form').attr('colspan', 6).appendTo(meta.form_row).form(make_form());
					meta.form_row.hide();
				}
			});

			self.sortable = tbody.sortable({
				handle: handles,
				axis: 'y',
				items: '> .layer-list-row',
				//containment: self.element.parent(), //disabled due to jquery-ui 1.8rc1 bug.
				start: function (e, ui) {
					tbody.find('> .layer-list-form-row').hide();

					var n_visible = $.map(layers, function(l){ return l.visible; }).length;
					self.element.css({width: Math.max(self.element.width(), ui.item.width() + 4), height: self.element.find('> div').height() + n_visible * 28 + 13})
				},
				stop: function () {
					self.element.css({width: '', height: ''})
				},
				update: function (event, ui) {
					var order = [];
					tbody.find('> .layer-list-row').each(function () {
						var layer = $(this).data('layer_id');
						order.push(layer);
					});

					order = order.reverse();

					o.messageBus.send('layer-order-change', { order: order });
				}
			});

			table.appendTo(self.element);

			self._update_form(layers);
		},

		_update_form: function(layers) {
			var self = this;

			(function disallow_hiding_of_last_layer(){
				var visible = {n: 0, layer: null};

				$.each(layers, function (j, layer) {
					if (self.layers_map[layer.id].input.is(':checked')) { //we have to use the dom here because the local model is out of sync during transient conditions.
						visible.n++;
						visible.layer = layer;
					}
				});

				if(visible.n === 1){
					var meta = self.layers_map[visible.layer.id];
					meta.input.attr('disabled', 'disabled');
					meta.trash.button('disable');
				}else{
					$.each(layers, function (j, layer) {
						var meta = self.layers_map[layer.id];
						meta.input.removeAttr('disabled');
						meta.trash.button('enable');
					});
				}
			})();
		},

		_open_thumbnail_dialogue: function() {
			/*
			var o = this.options;

			this.element.thumbnailDialogue({
				kind: 'layer',
				text: 'Select one or more Layers to add to the current chart.',
				actions: [{
					name: 'Add to map',
					on_click: function (item) {
						o.messageBus.send('layer-dialogue-selection', {
							names: [item]
						});
					}
				}]
			});
			*/
			require(['thumbnail-dialogue/2'], function(LayerSelector){
				self.dialogue = self.dialogue
					? self.dialogue.open()
					: new LayerSelector({kind: 'layer', hide_user_items: false})
			});
		}
	});
});
