define(['jquery', 'bus', 'jquery-ui/button', 'jquery-ui/draggable', './light_dialogue.css'], function($, Bus) {

	var zIndex = 50;

	var n = 0;

	$.fn.lightDialogue = function(options) {

		options = $.extend({
			name: null,
			title: null,
			show_hide_menu: null,
			messageBus: Bus(),
			dismiss: null,
			visible: false,
			zIndex: null,
			position: null,
		},
		options || {});

		return this.each(function() {
			var elem = $(this);

			var position = {
				top: (options.position && options.position.top) ? options.position.top : 100 + n,
				left: (options.position && options.position.left) ? options.position.left : 100 + n
			}
			var visible = options.visible;

			function check_position(done) {
				var orig = $.extend({}, position);

				var min_height = 100;
				var min_width = 100;

				if (position.top > w.height() - min_height) {
					position.top = w.height() - min_height;
					position.top = Math.max(position.top, 0);
				}

				position.left = Math.min(position.left, Math.max(w.width() - min_width, 0))

				done();

				if (position.top !== orig.top || position.left !== orig.left) {
					notify();
				}
			}

			function check_position2() {
				elem.css($.extend(position, {
					left: Math.max(position.left, -(elem.width() - 32)),
					top: Math.max(position.top, -(elem.height() - 32))
				})).attr({positioner: 'light-dialogue'});
			}

			Bus().listen('window-size', function on_window_resize(){
				check_position(function() {
					elem.css(position);
				});
			});

			check_position(function() {
				elem.addClass('light-dialogue drop-shadow-small')
					.css($.extend({},
						position,
						{ zIndex: options.zIndex || zIndex }
					)).attr({positioner: 'light-dialogue'});
			});

			setTimeout(check_position2, 1000);

			n += 20;

			function notify() {
				if (options.name) {

					options.messageBus.send("dialogue-state-changed", {
						name: options.name,
						visible: visible,
						zIndex: elem.css("zIndex"),
						position: position
					});
				}
			}

			if (options.show_hide_menu) {
				elem.hide();

				options.messageBus.listen(options.show_hide_menu, function(_, msg) {
					if ((visible = msg.value)) {
						elem.show().trigger("show");
						move_to_top();
						notify();
					}
					else {
						elem.fadeOut('fast', function() {
							elem.hide().trigger("hide");
							notify();
						});
					}
				});
			}

			var title_bar;
			if (options.title) {
				title_bar = elem.div('.title ui-widget-header').prependTo(elem)
					.html(options.title)
					.click(function () {
						move_to_top();
					});
				var title_close = title_bar.div('.close').button({icons: {primary: 'ui-icon-closethick'}});

				if (!options.dismiss) {
					title_close.click(function () {
						elem.remove();
						return false;
					});
				}

				var loader;
				elem.on('loading', function(_, loading){
					if(loader){
						if(loading !== false && loading !== 'false') loader.show(); else loader.hide();
					}else{
						loader = title_bar.div('.loader loading');
					}
				});
			}

			if (options.dismiss) {
				(options.title
					? title_close
					: elem.div('.dismiss').a({href: 'close', className: 'text-link'}).text('Dismiss')
				)
				.click(function () {
					options.messageBus.send(options.dismiss.message, options.dismiss.value);
					return false;
				})
			}

			elem.draggable({
				stack: '.light-dialogue',
				zIndex: zIndex + 50,

				drag: function(event){
					if($(this).hasClass('ui-draggable-disabled')){ // ui.draggable does not disable dragging when disabled.
						return false;
					}
				},

				stop: function (event, ui) {
					position = elem.position();
					notify();
				}
			});

			function move_to_top() {
				var group = $.makeArray($('.light-dialogue')).sort(function (a, b) {
					return (parseInt($(a).css('zIndex'), 10) || 0) - (parseInt($(b).css('zIndex'), 10) || 0);
				});

				if (!group.length) {
					return;
				}

				$(group).each(function(i) {
					this.style.zIndex = zIndex + i;
				});

				elem.css({zIndex: zIndex + group.length});
			}

			if (options.visible) {
				setTimeout(function () {
					if (options.show_hide_menu) {
						options.messageBus.send("do-" + options.show_hide_menu, { value: true });
					}
					else {
						elem.show();
						elem.trigger("show");
					}
				},
				500);
			}
		});
	};

	var _window = {};
	var w = $(window);
	$(window).resize($.debounce(500, function () {
		var __window = {width: w.width(), height: w.height()};
		if((__window.width === _window.width) && (__window.height === _window.height)) return;
		Bus().send('window-size', __window);
		_window = __window;
	}));

});
