/** @license
 *  ECMWF markup
 *
 *  (C) Copyright ECMWF http://www.ecmwf.int
 *
 *  This software is licensed under the terms of the Apache Licence Version 2.0
 *  which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 *  In applying this licence, ECMWF does not waive the privileges and immunities
 *  granted to it by virtue of its status as an intergovernmental organisation nor
 *  does it submit to any jurisdiction.
 *
 */
define(['config', 'jquery', 'showdown'], function(ECMWF, $, Showdown) {
	var converter = new Showdown.Converter();

	var ref = 1000000;

	function make_link(links, name, url, title) {
		ref++;

		l = [" [", ref, "]: ", url ];
		if(title) {
			l.push(" (");
			l.push(title);
			l.push(")");
		}
		links.push(l.join(""));

		return "[" + name + "][" + ref + "]";
	}

	function image(links, name, url, title) {
		return "!" + make_link(links, name, url, title);
	}

	/*
	ECMWF.send_message = function(message) {
		Bus().send(message,{});
	}
	*/

    var extensions = {

        thumbnail : function(layer, url, title) {
            ref++;
            return "!" + make_link(this, layer, url, title);
        },
        action : function(label, message)
        {
            return  make_link(this, label, "javascript:ECMWF.send_message('" + message + "');")
        },

        thumbnail : function(what, name) {
            var url = '/backend/get-' + what + '-thumbnail/' + name + '/';

            return image(this, name, url, name);
        },

        meteogram_thumbnail : function(what, meteogram) {
            return meteogram;
        }
	};

	var Markup = {
		toHTML: function(text) {

			var links = ["\n"];

			text = text.replace(/@(\w+)\(([^)]*)\)/gm, function(match, name, args) {
				if(!extensions[name]){
					return match;
				}

				args = $.map(args.split(","), function(a) { return $.trim(a); });

				if($.isFunction(extensions[name])) {
					return extensions[name].apply(links,args);
				}

				return extensions[name];
			});

			links.push("\n");

			text = text + links.join("\n");

			return converter.makeHtml(text);
		}
	}

	$.fn.markup = function(args) {
		var o = (typeof args === "string") ? {text: args} : (args || {});
		return $(this).each(function() {
			$(this).addClass("markup").html(Markup.toHTML(o.text || $(this).html()));
		});
	}

	return Markup;
});
