/*
 *  In order to determine an elements dimensions, it must be displayed.
 *  Invisible elements can be measured by attaching them to the body
 *  with opacity 0.
 *
 *  Warning: will not work unless element is box-sizing:content-box
 */
define(['jquery'], function($) {
	function realize(elem, cb) {

		var hidden = elem.parents().filter(function(i, e) {
			return $(e).css('display') === 'none';
		});
		__DEV__ && !hidden.length && console.warn('realize: hidden element not found');

		var save = [];

		var attached = !!elem.parents('body').length;
		var root;

		if (!attached) {
			root = $(elem.parents().get(-1) || elem);
			root.appendTo($('body'));
		}

		hidden.css('opacity', function(i, old) {
			save.push(old);
			return 0;
		}).show();

		var result = cb(elem);

		hidden.hide().css('opacity', function(i, old) {
			return save.pop();
		});

		if (!attached) {
			root.detach();
		}

		return result;
	}

	$.fn.hiddenWidth = function(options) {
		return this.width() ||
			realize(this, function(e) {
				return e.width();
			});
	}

	$.fn.hiddenHeight = function(options) {
		return this.height() ||
			realize(this, function(e) {
				return e.height();
			});
	}

	$.fn.dimensions = function(options) {
		return realize(this, function(e) {
			return {
				width: e.width(),
				height: e.height(),
				innerWidth: e.innerWidth(),
				innerHeight: e.innerHeight(),
				outerWidth: e.outerWidth(),
				outerHeight: e.outerHeight(),
				outerWidthWithMargins: e.outerWidth(true),
				outerHeightWithMargins: e.outerHeight(true)
			}
		});
	}
});
