define(['config', 'jquery', 'bus', './dimensions', './menu_pane.css', 'measure', 'core/browser'], function(ECMWF, $, Bus) {

var count = 0;

/*
 *  A pop-up list of items assigned to a top-level menu item.
 *  Element should be a li of the top-level menu item.
 */
$.fn.menuList = function(options) {
	options = $.extend({
		messageBus: Bus()
	}, options || {});

	return this.each(function() {
		var elem = $(this);
		var menu = elem.attr('name');

		if($.fn.disableSelection) {
			elem.disableSelection();
		}

		elem.find('li').each(function() {
			var item  = $(this);
			var klass = item.attr('class');
			var name  = item.attr('name');
			var bus   = item.attr('bus');
			var html  = item.html();
			var id    = [menu, name, ++count].join('-');
			var msg;

			switch(klass) {

				case 'menu-radio':
					var group = name.replace(/-.*/, '');
					msg = ['menu', menu, group, 'change'].join('-');
					var radio_options = {group: group, buttons: [id], labels: [html]}
					if(item.is(':checked')) radio_options.checked = 0;
					item.empty()
						.addClass('enabled')
						.radio(radio_options)
						.change(function() {
							elem.trigger('select');
							options.messageBus.send(msg, { value : item.find(':checked').val() })
						})
						.bindBusMessage(options.messageBus, bus || 'do-' + msg, 'click');
					break;

				case 'menu-check':
					item.empty().addClass('enabled');
					msg = ['menu', menu, name, 'change'].join('-');
					$("<input>", {type: 'checkbox', name: name, id: id, value: name})
						.prop({checked: item.is(':checked')})
						.appendTo(item)
						.change(function(event,info) {

							if(info && info.value !== undefined) {
								$(this).prop('checked', info.value);
							}

							elem.trigger('select');
							options.messageBus.send(msg, { value : $(this).is(':checked') })
						})
						.bindBusMessage(options.messageBus, bus || 'do-' + msg , 'change');
						$('<label>').attr('for', id).html(html).appendTo(item);
					break;

				case 'menu-separator':
					item.empty();
					$('<hr>').appendTo(item);
					break;

				default:
					msg = ['menu', menu, name, 'select'].join('-');
					item
						.addClass('enabled')
						.click( function(event) {
							if(event.shiftKey || event.altKey) {
								elem.trigger('select');
								options.messageBus.send(msg + '-alt');
							}
							else {
								elem.trigger('select');
								options.messageBus.send(msg);
							}
							return false;
						})
						.bindBusMessage(options.messageBus, bus || 'do-' + msg, 'click');
					break;
			}
		});
	});
};

/*
 *  A menuPane is a panel shows content for a drop-down menu.
 */
$.fn.menuPane = function(options) {
	options = $.extend({
		group: null,
		owner: null,
		tipSize: 11, // Cannot be changed
		leftMargin: 16,
		rightMargin: 32, // Take the shadow into acccount
		align: 'left',
		noWidth: false,
		hover: false,
		resize: true,
		direction: 'down'
	},
	options || {});

	if(!$.fn.hoverIntent) {
		options.hover = false;
	}

	return this.each(function() {
		var elem = $(this).addClass('menu-pane drop-shadow z-6');

		var tip = elem.div('.z6')
		if(options.direction === 'down'){
			tip.addClass('menu-pane-tip-up');
		} else {
			tip.addClass('menu-pane-tip-down')
			.append('<div class="menu-pane-tip-down-inner">');
		}

		if(!options.width)
			options.width = elem.hiddenWidth();

		if(options.resize) {
			elem.resize(function() {
				setSizeAndPosition();
			});
		}

		// part of jquery-ui
		if($.fn.disableSelection) {
			$(options.owner).disableSelection();
		}

		function setSizeAndPosition() {
			var owner = options.owner;
			if(!owner || !owner.length) return;

			var offset = owner.offset();
			if(isNaN(offset.top)) return;
			var window_width = $(window).width() - options.rightMargin;

			if(options.resize) {
				options.width = elem.hiddenWidth();
			}

			var offset_parent = (function(elem) {
				while((elem = elem.parent())){
					if(!elem.length || !elem[0].ownerDocument || !!~$.inArray(elem.css('position'), ['relative', 'absolute'])){
						break;
					}
				}
				return elem;
			})(elem);

			var l = offset.left - offset_parent.offset().left;

			if(l + options.width > window_width) {
				l = window_width - options.width;
			}

			if(owner.width() < 2 * options.tipSize + 10){
				l -= (2 * options.tipSize - owner.width() + 10);
			}

			if(l < options.leftMargin) {
				l = options.leftMargin;
			}

			elem.css({
				width: options.noWidth || options.width,
				top: Math.floor(
					(options.direction === 'down')
						? offset.top + options.owner.height() + options.tipSize - offset_parent.offset().top + 3
						: offset.top - elem.height() - options.tipSize
				),
				left: options.align !== 'right' ? l : undefined
			});

			var tip_left = Math.floor(offset.left - offset_parent.offset().left + parseInt(owner.css('padding-left'), 10) + owner.width()/2 - options.tipSize - l - parseInt($('body').css('padding-left'), 10));
			var width = elem.hiddenWidth() - 8;
			tip_left = Math.min(tip_left, width - options.tipSize);

			tip.css(options.align === 'right' ? {left: 'auto', right: 15} : {left: tip_left});
		}

		setSizeAndPosition();

		$(window).resize(setSizeAndPosition);

		function show() {
			$(options.group || elem).hide();
			setSizeAndPosition();
			if(elem.measure().width < 20){ elem.hide(); return; }
			elem.show();

			// position this menu in front of all other menus
			var zindex = 0;
			var menus = $('.menu-pane:visible');
			menus.each(function(){ zindex = Math.max(zindex, $(this).css('z-index'))});
			zindex && elem.css('z-index', zindex);
			menus.not(elem).each(function(){
				$(this).css('z-index', --zindex);
			});
		}

		function hide() {
			elem.hide();
		}

		elem.on('close', hide);

		$(options.owner).click(function() {

			if(elem.is(":visible")) {
				hide();
			} else {
				show();
			}
			return false;
		});

		if(options.hover) {
			var timer;
			function delayedHide() {
				if(timer) clearTimeout(timer);
				timer = setTimeout(hide, 200);
			}

			$(options.owner).hoverIntent({
				over:     show,
				interval: 10,
				out:      delayedHide
			});

			elem.hoverIntent({
				over:     function() { timer = clearTimeout(timer); show(); },
				out:      delayedHide,
				interval: 10,
				timeout:  0
			});
		}
	});
};
});
