/** @license
 *  ECMWF Messages
 *
 *  Messages listens on the given message bus and displays a message to the user when it receives a log-* message.
 *
 *  (C) Copyright ECMWF http://www.ecmwf.int
 *
 *  This software is licensed under the terms of the Apache Licence Version 2.0
 *  which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 *  In applying this licence, ECMWF does not waive the privileges and immunities
 *  granted to it by virtue of its status as an intergovernmental organisation nor
 *  does it submit to any jurisdiction.
 *
 */
define(['config', 'jquery', 'bus', 'jquery-ui/widget', './messages.css'], function(ECMWF, $, Bus) {

	var tracker;
	var count = 0;
	var classes = 'log-info log-warning log-error log-message';

	$.widget('ui.messages', {

		options: {
			enable_popup: false,
			message_class: '',
			messageBus: Bus()
		},

		_init: function () {
			var self = this, o = this.options;

			count++;

			self.name = 'messages_' + count;

			////var clickable = $('<div class="clickable"/>').appendTo(self.element).click(function(){ remove_message(); return false; });
			var message = $('<div>').appendTo(self.element)
				.attr({'class': 'message'})
				.html('Welcome');

			self.element.addClass('messages' + (o.enable_popup ? ' messages-type-popup' : ''));
			/*
				.modeTooltip({
					content: function(){ return {text: self.element.data('message')}; },
					position: { my: 'left center', at: 'left top', offset: '-16 -22' }
				});
			*/

			if (o.enable_popup) {
				o.messageBus.clear('log-info').clear('log-message').clear('log-warning').clear('log-error');

				var popup = $('body')
					.div({className: 'messages'}) // TODO this div is not needed?
						.hide()
						.div({className: 'content drop-shadow-small ' + o.message_class})
						.menuPane({
							resize: true,
							owner: self.element,
							direction: 'down'
						});
			}

			o.messageBus.listen('log-info', function (event, msg) {
				set_message(msg, 'info');
				tracker = ECMWF.last_tracker;
			}).listen('log-message', function (event, msg) {
				set_message(msg, 'message');
				tracker = ECMWF.last_tracker;
			}).listen('log-warning', function (event, msg) {
				set_message(msg, 'warning');
				tracker = ECMWF.last_tracker;
			}).listen('log-error', function (event, msg) {
				set_message(msg, 'error');
				tracker = ECMWF.last_tracker;
			}).listen('log-clear', function (event, msg) {
				o.enable_popup && popup.parent().hide();
			});

			var removal_timeout = 0;
			function set_message(msg, msg_type) {
				self.element.removeClass(classes).addClass('log-' + msg_type).data('message', (msg.message != msg.title) ? f(msg.message) : '');

				if (o.enable_popup)
					popup.removeClass(classes).addClass('log-' + msg_type)
						.html(
							'<a href="#close" class="close">&times;</a>' +
							'<h4>' + msg.title + '</h4><p>' + f(msg.message) + '</p>')
						.click(function(e) {
							$(e.target).hasClass('close') && popup.parent().hide();
							return false;
						})
						.each(function(){ self.element.triggerHandler('click'); })
						.parent().show();
				else
					message.html(ECMWF.truncate(msg.title, 150));

				if (removal_timeout) {
					clearTimeout(removal_timeout);
					removal_timeout = 0;
				}
				removal_timeout = setTimeout(remove_message, msg.timeout || (15 * 60 * 1000));

				function f(msg) {
					return (typeof msg === 'object') ? JSON.to_string(msg) : msg;
				}
			}

			function remove_message() {
				message.empty();
				if (o.enable_popup) popup.trigger('close');
			}

			self.element.click(function () {
				message.html(
					// TODO: Move URL into options.js
					$('<a>').attr('href', 'http://wrep-monitor.ecmwf.int/statmon/display/track/?id=' + tracker).text(tracker)
				)
			});
		},

		_reset: function (txt) {
		},

		_update: function () {
		},

		destroy: function () {
			var self = this;

			self.element.removeClass('ui-widget ui-widget-content ui-corner-all messages ' + classes);

			// Cleanup CSS and event handling
			// self.element.removeClass("....").unbind("....")
			// Destroy created DOM elements
			self.element.empty();

			return this;
		}

	});

	$.ui.messages.eventPrefix = 'messages';

	$.ui.messages.defaults = $.extend({}, {
		messageBus: Bus()
	});

});
