define(['config', 'jquery', 'ajax_queue', 'unit', 'bus', 'time', './meteogram.css'], function(ECMWF, $, Ajaxqueue, Unit, Bus, Time) {

	var count = 0;

	$.widget('ui.meteogram', $.ui.mouse, {

		options: {
			location: null,
			time: null,
			messageBus: Bus(),
			priority: 'very-high',
			availability: false,
			product: '10d_epsgram_2t',
			width: null,
			height: 120,
			heightAdjustment: false,
			ready: function () {},
			// Called when the image is loaded
			newAvailability: function() {}
		},

		_init: function () {
			var self = this, o = this.options;

			count++;

			self.name = 'meteogram_' + count;
			self.graphs = {};
			self._requested_size = {};
			self.time = o.time;

			// Call super class
			this._mouseInit();

			this.element.addClass('meteogram').css({position: 'relative'});

			this.clip = this.element.div('.meteogram-clip')
				.css({position: 'absolute'})
				.mousedown(function(event) {
					event.preventDefault();
				});

			this.img = $(new Image())
				.addClass('meteogram-image')
				.css({position: 'absolute'})
				.appendTo(this.clip);

			self.options.messageBus.listen('latlon-location-select', function(event, location) {
				o.location = location;
				if (self.element.is(':visible')) {
					self._redraw();
				}
			});

			setTimeout(function() {
				self._redraw();
			}, 10);

			this.element.resize(function(e, size) {
				o.width = size.width || o.width;
				self.resize();
			});
			this.element.on('show', function() {
				self.resize();
			});

			self._throttled_resize = $.throttle(2000, function() {
				var viewport = self._viewport();
				if (viewport.width && (viewport.width !== self._requested_size.width)) {
					self._redraw();
					self._requested_size = viewport;
				}
			});
			self.resize = self._throttled_resize;

			self.checkAvailability(o.availability);
		},

		setLocation: function(loc) {
			var self = this, o = this.options;

			o.location = loc;
			self._redraw();
		},

		setData: function(data) {
			var self = this;
			self.options = $.extend(self.options, data);
			self._redraw();
		},

		_redraw: function() {
			var self = this, o = this.options;
			var viewport = self._viewport();

			if (!self.element.is(':visible')) {
				return;
			}

			var json = $.extend({
					product: o.product,
					//base_time  : self.time.base_time,
					viewport: viewport,
					height_adjustment: o.heightAdjustment
				},
				viewport,
				o.location
			);

			//depending on the meteogram type, sometimes magics respects the requested aspect ratio, sometimes not.
			json.viewport.height || (json.viewport.height = o.height);

			// TODO:
			function date2cycle(date) {
				return date.format('yyyymmddHHMM');
			}

			if (self.time) {

				json.base_time = date2cycle(self.time.base_time);
				json.step = self.time.step;
			}

			if (json.viewport.height && json.viewport.width) {
				Ajaxqueue({
					url: '/backend/meteogram/',
					json: json,
					status: ECMWF.ajax_loader(self.clip),
					priority: o.priority,
					logErrors: 'Meteogram',
					img: self.img,
					name: self.name,
					tag: self.name,
					success: function (json) {
						self.metadata = json.magics ? json.magics.metadata : {};
						self.availability = $.map(json.availability, function (x) {
							return new Time(x);
						});

						self.availability = self.availability.sort(function (a, b) {
							return a.base_time - b.base_time;
						});

						self.time = new Time(json.base_time, json.step, json.valid_time);

						o.messageBus.send('meteogram:' + self.time.key);

						self.element
							.stop()
							.animate({ height: self.img.height() }, o.ready);

						self._check_out_of_date();
						o.newAvailability(self.time, self.availability);

						if (json.title && o.title) {

							self.metadata.title = json.title;

							var ajusted = '';
							if (self.metadata && self.metadata.temperature_correction) {
								ajusted = ', adjusted to ' + self.metadata.height + 'm height';
							}

							setTimeout(function() {
								// TODO: something cleaner...
								var bt = $.dateToTitle(self.time.base_time);
								var vt = $.dateToTitle(self.time.valid_time);
								if (bt !== vt) {
									o.title.html(json.magics && json.magics.metadata && !!~$.inArray(json.magics.metadata.db, ['efi', 'cdf'])
										? json.title + '<br>Valid for: ' + vt + ajusted
										: json.title + '<br>Base date: ' + bt + ' Valid for: ' + vt + ajusted
									);
								}
								else {
									o.title.html(json.title + '<br>Base date: ' + bt + ajusted);
								}
							},
							10);
						}

						/*
						//we assume that the returned image size is actually the one requested.
						if(ECMWF.browser.msie) self.img.css(viewport);
						*/
					}
				});
			}
			//else console.log('METEOGRAM redraw: no size. not requesting image. viewport=' + json.viewport.width + 'x' + json.viewport.height);
		},

		showLatestCycle: function() {
			var self = this;
			self.time = null;
			self._redraw();
		},

		_update_cycle: function(n) {
			var self = this;
			if (self.time && self.availability) {
				var idx = self.time.index(self.availability);
				idx += n;
				if (idx < 0) idx = 0;
				if (idx > self.availability.length - 1) idx = self.availability.length - 1;
				self.time = self.availability[idx];
				self._redraw();
			}
		},

		setHeightAdjustment: function(on) {
			var self = this,
			o = this.options;
			console.warn('setHeightAdjustment', on);
			o.heightAdjustment = on;
			self._redraw();
		},

		setCycle: function(time) {
			var self = this,
			o = this.options;
			if (time) {
				self.time = time;
				self._redraw();
			}
		},

		previousStep: function() {
			var self = this, o = this.options;
			self.setCycle(self.time.previous(self.availability));
		},

		nextStep: function() {
			var self = this, o = this.options;
			self.setCycle(self.time.next(self.availability));
		},

		previousDay: function() {
			var self = this, o = this.options;

			if (self.time) {
				var prev_day = self.time.valid_time.getTime() - 1000 * 60 * 60 * 24;
				for(var prev = self.time; prev; prev = prev.previous(self.availability)){
					if(prev.valid_time.getTime() <= prev_day){
						self.setCycle(prev);
						break;
					}
				}
			}
		},

		nextDay: function() {
			var self = this, o = this.options;

			if (self.time) {
				var next_day = self.time.valid_time.getTime() + 1000 * 60 * 60 * 24;
				for(var next = self.time; next; next = next.next(self.availability)){
					if(next.valid_time.getTime() >= next_day){
						self.setCycle(next);
						break;
					}
				}
			}
		},

		previousCycle: function() {
			var self = this,
			o = this.options;
			console.log("previousCycle");
			self.setCycle(self.time.previousBaseTime(self.availability, true));
		},

		nextCycle: function() {
			var self = this,
			o = this.options;
			console.log("nextCycle");
			self.setCycle(self.time.nextBaseTime(self.availability, true));
		},

		_check_out_of_date: function() {
			var self = this,
			o = this.options;
			if (self.time && self.availability) {
				var latest_basetime = self.availability[self.availability.length - 1].base_time;
				var selected_basetime = self.time.base_time;
				o.messageBus.send("meteogram-out-of-date", selected_basetime - latest_basetime !== 0);
			}
		},

		checkAvailability: function(on) {
			var self = this,
			o = this.options;
			//        console.log("checkAvailability ", on);
			if (on) {

				if (!self.timer) {

					self.timer = setInterval(function () {

						if (!self.element.is(":visible")) return;

						Ajaxqueue({
							url: '/backend/meteogram-availability/',
							json: {
								product: o.product
							},
							priority: 'low',
							logErrors: 'Meteogram availability',
							success: function(json) {
								if (json) {
									var avail = $.map(json.availability, function(x) {
										return new Time(x);
									});
									self.availability = avail;

									self.availability = self.availability.sort(function(a, b) {
										return a.base_time - b.base_time;
									});

									o.newAvailability(self.time, self.availability);
									self._check_out_of_date();
								}
							}
						});

					},
					60 * 1000);
				}

			}
			else {
				if (self.timer) clearInterval(self.timer);
				self.timer = null;
			}

		},

		getHelp: function () {
			var self = this,
			o = this.options;
			var list = [];

			if (!self.metadata) {
				return "Cannot get help information.";
			}

			if (!self.metadata.distance) {
				return "Cannot display help information. Missing Distance data.";
			}

			var m = Unit.get('m');
			var km = Unit.get(self.metadata.distance.distance_unit || 'km');

			var gridpoint_type = (self.metadata.land_sea_mask < 0.5) ? "sea" : "land";

			list.push("### " + self.metadata.title );
			list.push("Location requested: **" + $.formatGeoLocation(self.metadata.requested_location) + "**, ");
			list.push("nearest EPS grid point: **" + $.formatGeoLocation(self.metadata.location) + '**.');
			list.push('');
			list.push("Height of requested location (based on the GTOPO30 dataset): **" + m.asHTML(self.metadata.height) + "**, ");
			list.push("height of EPS orography at nearest grid point: **" + m.asHTML(self.metadata.eps_height) + "**, ");
			list.push("height of deterministic orography interpolated to EPS resolution at nearest grid point: **" + m.asHTML(self.metadata.deterministic_height) + '**.');
			if (self.metadata.temperature_correction) {
				list.push('');
				if(self.metadata.temperature_adjustment) {
					list.push('Temperature parameter is adjusted to the requested location by **' + $.formatCelcius(self.metadata.temperature_adjustment.deterministic_adjustement, 1) + '**' + ' in deterministic forecast curve and ' + '**' + $.formatCelcius(self.metadata.temperature_adjustment.eps_adjustement, 1) + '** in EPS forecast curves.');
				}
				else
				{
					list.push('Temperature parameter is adjusted to the requested location');
				}
				list.push('');
			}

			list.push('');
			list.push('');
			//TODO: Fix me later
			if (self.metadata.db === "10_days_wave") {
				list.push("The nearest EPS grid point is located at **" + km.asHTML(self.metadata.distance.distance) + " " + self.metadata.distance.direction + "** of the selected location.");
			}
			else {
				list.push("The nearest EPS grid point is a **" + gridpoint_type + " point**, ");
				list.push("located at **" + km.asHTML(self.metadata.distance.distance) + " " + self.metadata.distance.direction + "** of the selected location.");
			}

			/*

            list.push('TODO:');
            list.push('');
            list.push('* Land/point or sea point (DONE)');
            list.push('* Distance to point (DONE)');
            list.push('* Model resolutions (Monday the 28th)');
            list.push('* Explain red/blue curves(Monday the 28th)');
            list.push('');
            */

			/* $.each(self.metadata, function(k,v) { list.push("* " + k + " = " + v); }); */

			return list.join("\n");
		},

		destroy: function () {
			var self = this,
			o = this.options;

			if (self.timer) clearInterval(self.timer);

			this.element.removeClass('meteogram');
			this.clip.remove();

			// Cleanup event handling
			// self.element.off("....")
			// Destroy created DOM elements
			// self.element.empty();
			// Call super class
			this._mouseDestroy();

			return this;
		},

		_viewport: function () {
			return {
				top: 0,
				left: 0,
				width: this.options.width || this.element.width(),
				height: this.element.height()
			};
		}
	});
});
