/*
 *  These mixins provide additional generic functionality to a jquery-ui widget.
 *  They can be used individually or all together
 *
 *  Current methods:
 *     - throttle
 *     - bind
 *     - storage
 */
define(['storage'], function(localStorage){

	/*
	 *  Provides localStorage for a widget.
	 *  The widget should instantiate one item per variable that needs to be stored.
	 */
	var data = {};

	function _Storable(name, context) {
				if(!context.widgetName) console.warn('Storable: widgetName not set');
				this.db = context.widgetName;
				this.name = name;
				data[this.db] = null;
			}

	var Storable = (
		localStorage
			? _Storable
			: function() {
				this.get = $.noop;
				this.set = $.noop;
			}
		)

	_Storable.prototype = {
		get: function(){
			try {
				data[this.db] || (data[this.db] =  JSON.parse(localStorage[this.db] || '{}'))
				return data[this.db][this.name] || {};
			} catch(e) {
				return {};
			}
		},

		set: function(obj){
			var self = this;

			data[this.db][this.name] = obj
			setTimeout(function(){
				localStorage.setItem(self.db, JSON.stringify(data[self.db] || {}));
			}, 0);
		}
	}

	return {
		/*
		 *  Delay execution of a fn and ensure it is only called once within a time window.
		 *  It is different to throttle-debounce which executes the fn as soon as possible.
		 */
		debounce: function throttle(fn, t) {
			t = t || 1000;
			var throttle_timer;

			return function() {
				var self = this;
				throttle_timer && clearTimeout(throttle_timer);
				throttle_timer = setTimeout(function(){
					throttle_timer = null;
					fn.apply(self);
				}, t);
			};
		},

		bind: function(fn) {
			var self = this;
			return function() {
				return fn.apply(self, arguments);
			}
		},

		storage: function(name) {
			this._storables = this._storables || {};
			return this._storables[name] || new Storable(name, this);
		}
	};
});
