define(['config', 'jquery', 'ajax_queue', 'help-panel', 'settings', 'forecaster/meteograms', 'jquery-ui/sortable', './multigrams.css', 'meteogram', 'handler'], function(ECMWF, $, Ajaxqueue, HelpPanel, Settings, Meteograms) {

	var count = 0;
	var dialogue;

	$.widget('ui.multigrams', $.ui.handler, {
        options: {
            autoPosition: false,
            // This does not seem to work ...
            //products : ["10d_epsgram_temp", "10d_epsgram_tp"],
            products: null,
            product: null,
            height : 120,
            width  : null,
            time   : null,
            show_more: true,
            show_load: false,
            auto_show_tools: true,
            availability : false,
            family: null,
            location: null
        },

        _init: function(){
            var self = this, o = this.options;

            self._message_handler_init();
            self.height_adjustment = Settings.get_pref('height_adjustment', true);

            self._edit = false;

            if(!o.products) {
                o.products = o.product ? [o.product] : ['10d_epsgram_2t', '10d_epsgram_tp'];
            }

            if(!o.location){
                o.location = Settings.get_pref('default_location');
            }

            if(!Array.isArray(o.products)) {
                o.products = [o.products];
            }

            count++;

            self.times = {};
            self.availabilities = {};
            self.name = 'multigrams_' + count;

            self.element.addClass('multigrams');

            self.loader = self.element.div('.loader');
            self.load   = self.element.div('.more').each(function(){ o.show_load || $(this).hide() });
            self.more   = self.element.div('.more').each(function(){ o.show_more || $(this).hide() });
            self.loc    = self.element.div('.location');

            self.content = self.element.div('.content');
            self.ul = self.content.ul();
            self.link = self.more
                .a({href: '#add_more', className: 'text-link'}).css({'float': 'right'}).text('More ...').parent()
                .div().icon2({css_class: 'small-icon small-icon-circle-plus'}).parent()
                .click(function() {
							require(['thumbnail-dialogue/2'], function(Dialogue){
								Meteograms.lookup_filter_by_name('family').set(o.family);

								if(!dialogue){
									dialogue = new Dialogue({kind: 'meteogram', hide_user_items: false, bus: o.messageBus})
								}else{
									dialogue.bus = o.messageBus;
									dialogue.open()
								}
							});
							return false;
						});

            self.load
                .a({href: '#load_meteograms', className: 'text-link'}).css({'float': 'right'}).text('Load ...').parent()
                .div().icon2({css_class: 'small-icon small-icon-circle-plus'}).parent()
                .click(function() {
                    console.log('load click');
                    ECMWF.dialogue({
                        title: 'Meteogram Load',
                        content: 'Loading Dashboard meteograms...',
                        ok: function (choices) {
                        }
                    })
                    .each(function(){
                        var content = $(this).find('.content').addClass('meteogram-loader-content');
                        var url = '/backend/get-dashboard-multigrams/';
                        Ajaxqueue({
                            url      : url,
                            json     : {dummy: 'true'},
                            priority : 'very-high',
                            success  : function(result) {
                                content.empty();
                                if(result && Array.isArray(result.result)){
                                    content.h3('Please select a meteogram to load');
                                    $.each(result.result, function(){
                                        var product = this;
                                        //content.div().a(this.title).attr('href', '#');
                                        content.div().a({href: '#', 'class': 'text-link'}).text(product.title).click(function(){
                                            console.log('loading: ', product);
                                            content.parent().dialog('close');
                                            self.clearProducts();
                                            if(product.data && product.data.products){
                                                $.each(product.data.products, function(){
                                                    add_product(this + '');
                                                    o.products.push(this + '');
                                                });
                                                self._on_list_changed();
                                            }
                                            return false;
                                        });
                                    });
                                }
                                else content.h3('No meteograms found');
                            },
                            logErrors: url,
                            error    : function(result) {
                                console.warn(url + ': ', result);
                                if(result.error) for(var i in result.error) console.log(result.error[i]);
                            }
                        });
                    });
                    return false;
                });

				self.element.helpProvider({getContent: function(){ return self.getHelp('Meteogram Help'); }});

            function add_product(product) {
                var li = self.ul.li();
                var title = li.h5('.meteogram-title').span('-');

                var mg = li.div()
                    .meteogram({
                        messageBus : o.messageBus,
                        product    : product,
                        time       : o.time,
                        location   : o.location,
                        width      : o.width,
                        height     : o.height,
                        title      : title,
                        heightAdjustment : self.height_adjustment,
                        newAvailability : function(time, avail) {
                            self.times[product]          = time;
                            self.availabilities[product] = avail;
                            self._check_availabilities();
                        },
                        ready      : function() {
                            self.adjustHeight();
                            o.messageBus.send('multigrams-meteogram-ready');
                        }
                    })
                    .css({width: o.width || self.element.width(), height: o.height})
                    .thumbnailTools({selector: '.meteogram-clip', enable: o.auto_show_tools || self._edit, always_on: !o.auto_show_tools})
                    .thumbnailTools('add', [
                        ['ui-icon-trash', function(){
                            mg.meteogram('destroy');
                            mg.parent().remove();
                            o.products = $.grep(o.products, function(p) { return p !== product; });
                            self._on_list_changed();
                            delete self.times[product];
                            delete self.availabilities[product];
                            self._check_availabilities();
                        }],
                        ['ui-icon-help', function(){
                            console.log('multigram: help tool clicked');
                            self._help = self._help || HelpPanel({el: self.element});
                            self.element.triggerHandler('contentChanged'); // because for some reason click is not being handled in the HelpPanel
                        }]
                    ]);
                    //.thumbnailTools('add', 'ui-icon-pencil', function(){ })

                o.title_size = title.height(); // !!!!! too early --- not being used?

                self.element.on('show', function(){ mg.triggerHandler('show'); });

                title.click(function(){ return false; }).mousedown(function(event){ event.stopPropagation(); });

                self.ul.sortable({
                    handle: 'h5',
                    helper: function(event, ui){ return $('<div>', {css: {width: ui.width() * 0.85, height: ui.find('.meteogram-clip').height() - 30}}); },
                    placeholder: 'ui-sortable-placeholder',
                    axis: 'y',
                    start: function(_, ui){
                        if(self.element.data('uiDraggable')) self.element.draggable('option', 'disabled', true);

                        var meteogram = ui.item.find('> .meteogram');
                        meteogram.data('uiThumbnailTools') && meteogram.thumbnailTools('enable', false);

                        ui.placeholder.css({width: self.element.width() * 0.85, marginLeft: self.element.width() * 0.075});
                        ui.helper.css({marginLeft: self.element.width() * 0.075});
                    },
                    stop: function(_, ui){
                        var products = [];
                        self.ul.find('.meteogram').each(function(){
                            products.push($(this).data('uiMeteogram').options.product);
                        });
                        o.products = products;
                        self._on_list_changed();
                        self.element.data('uiDraggable') && self.element.draggable('option', 'disabled', false);

                        var meteogram = ui.item.find('> .meteogram');
                        meteogram.data('uiThumbnailTools') && meteogram.thumbnailTools('enable', true);
                    }
                });
            }

				function add_from_dialogue(event, msg) {
                $.each(msg.names, function(_, product) {
                    add_product(product);
                    o.products.push(product);
                });
                self._on_list_changed();
				}

            self._add_handler('meteogram-dialogue-selection', add_from_dialogue);

            self._update_location(o.location);

				o.messageBus
					.listen('meteogram-thumbnail-click', function(event, msg){
						// double click in Add dialogue
						add_from_dialogue(event, {names: [msg.name]})
					})
					.listen('latlon-location-select', function(event, msg) {
						self._update_location(msg);
						self.adjustHeight();
					})
					.listen('prev-step', function(event, msg) {
						self.previousStep();
					})
					.listen('next-step', function(event, msg) {
						self.nextStep();
					})
					.listen('next-day', function(event, msg) {
						self.nextDay();
					})
					.listen('prev-day', function(event, msg) {
						self.previousDay();
					})
					.listen('prev-base-time', function(event, msg) {
						self.previousCycle();
					})
					.listen('next-base-time', function(event, msg) {
						self.nextCycle();
					})
					.listen('preferences-changed', function (_, msg) {
						if(msg.height_adjustment !== self.height_adjustment) {
							self.height_adjustment = msg.height_adjustment;
							self.ul.find('.meteogram').meteogram('setHeightAdjustment', self.height_adjustment);
							self.adjustHeight();
						}
					});

            $.each(o.products, function(_, product) {
                add_product(product);
            })

            setTimeout(function(){ // adjustHeight prevents the lightDialogue from hiding it.
                self.adjustHeight();
            }, 300);

            self.resize = $.throttle(100, function(e, size){
                var size = {width: (size && size.width) || self.element.width(), height: self.element.height()};
                if(!size.width) return;

                if(!self.size || size.width !== self.size.width || size.height !== self.size.height){
                    self.element.find('.meteogram').css({width: size.width}).triggerHandler('resize', size);
                    self.adjustHeight();
                    self.size = size;
                }
            })

            this.element.on('resize', self.resize);

            self._on_list_changed = $.throttle(50, function(){
                o.messageBus.send('multigrams-list-changed', {products: o.products});
                self.element.triggerHandler('contentChanged');
            });

            self.checkAvailability(o.availability);
        },

        edit : function(enable) {
            console.log('multigrams.edit');
            if((this._edit = (enable !== false))){
                this.more.show();
                this.loc.hide();
            }else{
                this.more.hide();
                this.loc.show();
            }
            this.element.find(".meteogram").thumbnailTools('enable', enable);
            this.adjustHeight();
        },

        adjustHeight : function() {
            //update the multigram height to fit the size of the child meteograms.

            var self = this;

            if(!self.element.is(':visible')) return;

            function adjust(){
                /*
                var offset = self.content.offset().top - self.content.parent().offset().top;
                var height = self.ul.height() + offset;

                self.element.stop().animate({height: height});
                */
                self.element.css({height: ''});
            }
            adjust();
        },

        setLocation : function(location) {
            var self = this, o = this.options;

            self._update_location(o.location = location);
            self.ul.find('.meteogram').meteogram('setLocation', o.location);
        },

        setProducts : function(products) {
            console.log("SET PRODUCTS", products);
            /*
            var o = this.options;
            o.products = products;
            $.each(o.products, function(_, product) {
                add_product(product);
            });
            */

            //self._update_location(o.location);
        },

        removeProduct : function(product) {
            var self = this, o = this.options;
            console.log(' ', typeof product, product);
            switch(typeof product){
                case 'number':
                    self.ul.find('> li:eq(' + product + ')').remove().orelse(function(){ console.log('          not found'); });
                    o.products.splice(product, 1);
                    break;
                default:
                    break;
            }
            self._on_list_changed();
        },

        clearProducts : function() {
            var self = this, o = this.options, i = o.products.length - 1;
            console.log('clearProducts: n=', o.products.length);
				var reversed = o.products.reverse();
            $.each(reversed, function(){
                console.log('   product=' + this);
                self.removeProduct(i--);
            });
            o.products = [];
        },

        showLatestCycle : function() {
            var self = this;
            self.element.find('.meteogram').each(function() {
                $(this).meteogram("showLatestCycle");
            });
        },

        getProducts : function() {
            var o = this.options;
            return o.products;
        },

        getTime : function() {
            var self = this;

            //assume that availability is the same for each meteogram
            for(var i in self.times){
                return self.times[i];
				}
        },

        getHelp : function(title) {
            // eventually the help will come from cms content. It is static per meteogram.
            var self = this;

            return $.Deferred(function(dfd){

                var list = [];

                self.element.find('.meteogram').each(function(i) {
                    if(i) list.push("\n---\n");
                    list.push($(this).meteogram("getHelp"));
                });

                dfd.resolve({title: title, content: list.join("\n") });

            }).promise();
        },

        redraw: function() {
            this.ul.find('.meteogram').triggerHandler('show');
            return this;
        },

        destroy: function(){
            var self = this;

            self.element.removeClass("multigrams");

            // Cleanup CSS and event handling
            // self.element.removeClass("....").off("....")
            // Destroy created DOM elements
            self.element.empty();

            self._message_handler_destroy();

            return this;
        },

        _update_location: function(loc) {
            var self = this;

            var title = "Location: " + $.formatGeoLocation(loc);
            if(loc.title) title += ", " + loc.title;
            self.loc.html(title);

            self.element.triggerHandler('contentChanged');
        },

        _on_change: function(){
            var self = this, o = this.options;

            self.ul.find('.meteogram').meteogram('setLocation', o.location);
            self._update_location(o.location);
        },

        _check_availabilities : function(){
            /*
            var self = this;
            console.log(self.times);
            console.log(self.availabilities);
            */
        },


        nextCycle : function(){
            var self = this;
            self.ul.find('.meteogram').meteogram('nextCycle');
        },

        previousCycle : function(){
            var self = this;
            self.ul.find('.meteogram').meteogram('previousCycle');
        },

        nextStep : function(){
            var self = this;
            self.ul.find('.meteogram').meteogram('nextStep');
        },

        previousStep : function(){
            var self = this;
            self.ul.find('.meteogram').meteogram('previousStep');
        },

        nextDay : function(){
            var self = this;
            self.ul.find('.meteogram').meteogram('nextDay');
        },

        previousDay : function(){
            var self = this;
            self.ul.find('.meteogram').meteogram('previousDay');
        },

        checkAvailability : function(on) {
            var self = this;
            self.ul.find('.meteogram').meteogram('checkAvailability', on);
        }
    });
});
