/*
 *  ECMWF Notify
 */
define(['jquery', 'markup', 'pnotify/pnotify.callbacks', 'pnotify', './notify.css', 'pnotify/pnotify.css'], function($, Markup, _, PNotify) {

var stories = {};

var OPTIONS = {
	'service-alert' :        { type: 'error' },          // e.g. emergency
	'service-notification' : { },                        // planned system session
	'service-allclear' :     { style: 'all-clear' },     // green
	'service-news' :         { style: 'service-news' },  // e.g. new layer is now available
	'ok' :                   { hide: true },
	'error' :                { type: 'error', hide: true }
};

var default_options = {type: 'notice', history: false, hide: false/*, styling: 'jqueryui'*/};

PNotify.prototype.options.styling = 'jqueryui';

$.notify = function __notify(args, id) {

	if((args === 'close') && id){
		var element = find_by_id(id);
		element && element.pnotify_remove();
		return this;
	}

	if($('body > .ui-pnotify').length > 10) return this;

	var o = $.extend({
		id: null,
		kind: null, // should be one of the OPTIONS above
		title: '',
		message: '',
		created: null,
		story: null,
		delivered: $.noop
	}, args);

	var klass = (o.kind && OPTIONS[o.kind]) || {};

	var autoclose;
	if(!klass.hide && ((autoclose = $.getUserPreference('notification_autoclose'), '0') !== '0')){
		klass.hide = true;
		klass.delay = autoclose * 60 * 1000;
	}

	if(o.story){
		var story;
		if((story = stories[o.story])){

			function reset_timer(){
				clearTimeout(story.timer);

				story.timer = story.options.hide
					? setTimeout(function(){
							story.queue_remove();
						}, story.delay)
					: null;
			}

			if(o.id){
				var pnotify_options = $.extend({}, default_options, klass);
				if(pnotify_options.type){
					if(story.options.type !== pnotify_options.type){
						//ui-state-highlight is used to change icon colour however it has unwanted effects on the text styles.
						//though this usage of highlight _is_ demonstated on the jquery theme page ...
						story.container.toggleClass('ui-state-highlight').toggleClass('ui-state-error');
						story.options.type = pnotify_options.type;
						story.options.hide = pnotify_options.hide;
					}
				}

				story.container.removeClass('all-clear service-news');
				pnotify_options.style && story.container.addClass(pnotify_options.style);
			}

			story.title_container.text(o.title);
			story.text_container.html(o.message);
			story.message_id = o.id;
			reset_timer();
			return;
		}
	}

	new PNotify($.extend({}, default_options, klass, {
		message_id: o.id,
		title: o.title,
		text: Markup.toHTML(o.message),
		after_init: function(pn){
			o.story && (stories[o.story] = pn);
		},
		before_open: function(pn){
			// override the standard close button which is only shown during mouseover.
			// TODO this was added for pnotify v1. check it is still needed
			/*
			pn.closer.remove();
			pn.container
				.div({
					className: 'ui-pnotify-closer',
					css: {cursor: 'pointer'},
					click: function(){
						pn.pnotify_remove();
						pn.closer.hide();
					}
				})
				.prependTo(pn.container)
				.span('.ui-icon ui-icon-circle-close');

			if(pn.opts.style){
				pn.container.addClass(pn.opts.style);
			}
			*/
		},
		after_open: function(pn){
		},
		after_close: function(pn){
			o.delivered();
			o.story && delete stories[o.story];
		}
	}));

	return this;
};

function find_by_id(id) {
	var element;
	var pnotify = $('body').data('pnotify');
	$.each(pnotify, function(){
		if(this.opts.message_id == id) element = this;
	});
	return element;
}

});
