/*
 *  Loosely based on rxjs
 *
 *  TODO add priority property as in signals.js
 */
define(['jquery'], function($) {

	/*
	 * args:
	 *     state (bool): if set, subscribers will receive the current value via the subscribed fn.
	 */
	function Observable(args) {
		this.options = args || {};
		this._listeners = [];
	}

	Observable.prototype = {

		/*
		 *  Subscribe has an optional subscriber argument that enables easier unsubscription.
		 *  Subscriber can be any kind of reference as long as it can be matched in unsubscribe().
		 *
		 *  If the 'state' option is set, the callback will be imediately called with the previous value.
		 */
		subscribe: function(fn) {
			var subscriber;
			if(arguments.length == 2){
				subscriber = arguments[0];
				fn = arguments[1];

				/* subscribers can and do subscribe multiple times.
				if(this.is_subscribed(subscriber)){
					console.warn('subscribe: already subscribed', this.name || this, subscriber.eventNamespace || subscriber);
				}
				*/
			}
			this._listeners.push([fn, subscriber]);

			this.options.state && this.value && fn.call(this, this.value);
			return this;
		},

		/*
		 *  Unsubscribing can be done by either fn or object.
		 */
		unsubscribe: function(fn) {
			var self = this;
			if(typeof fn === 'function'){
				$.each(self._listeners, function(i, v){ var f = v[0]; if(f === fn) delete self._listeners[i]; });
			}else{
				// unsubscribe by subscriber object.
				$.each(self._listeners, function(i, v) {
					var subscriber = v[1];
					if(subscriber === fn){
						self._listeners.splice(i, 1);
						return false; // FIXME need to cater for subscribers that have multiple subscriptions.
					}
				});
			}
			return this;
		},

		unsubscribe_all: function(fn) {
			this._listeners = [];
			return this;
		},

		is_subscribed: function(obj) {
			for(var i in this._listeners){
				if(this._listeners[i][1] === obj) return true;
			}
			return false;
		},

		emit: function(val) {
			var self = this;
			$.each(this._listeners, function(i, v) {
				var fn = v[0];
				fn.call(self, val);
			});
		},

		set: function(value) {
			if(!this.is_current(value)){
				this.prev = this.value;
				this.value = value;
				this.emit(this.value);
			}
			return this;
		},

		unset: function() {
			if(this.value){
				this.prev = this.value;
				this.value = undefined;
			}
			return this;
		},

		is_current: function(val) {
			return val === this.value;
		},

		// note that filter is static in rxjs
		filter: function(fn) {
			var upstream = this;
			var observable = new Observable();
			upstream.subscribe(function filter_listener(value){
				observable.value = value;
				if(fn.call(this, value)) observable.emit.call(observable, value);
			});
			return observable;
		},

		map: function(fn) {
			var upstream = this;
			var observable = new Observable();
			upstream.subscribe(function filter_listener(value){
				observable.emit.call(observable, fn.call(this, value));
			});
			return observable;
		},

		// note that currently (possibly incorrectly) this does not return a new observable
		// TODO would be better to unsubscribe
		once: function(fn) {
			var upstream = this;
			var count = 0;
			this.subscribe(function once_listener(value){
				count++ || fn.call(upstream, value);
			});
			return this;
		}

		/*
		copy: function() {
			var copy = $.extend(true, {}, this);
			copy._listeners = [];
			return copy;
		}
		*/
	}

	return Observable;
});
