/*
 *  'page level' chart type handler for epsgram charts
 *
 *   It is activated when a product is loaded that has widgets=epsgram
 *   See also the epsgram (web) package handler
 */
define(['jquery', 'time', '../chart_handler', 'virtual-dom/h', 'chart-app/router'], function($, Time, CH, h, router) {
	$('.column-1 .filters').remove();

	var block = $('.chart-column').div().div('.metadata-content')
		.parent()
		.hide()
		.addClass('chart-metadata')
		.insertBefore('.product-text')

	function Handler(app) {
		var self = this;
		this.app = app;

		CH.product_change.subscribe(function() {
			var is_wmo = CH.current.instance.has_widget('wmo_locations');
			if(is_wmo !== self.is_wmo){
				self.is_wmo = is_wmo;
				self.app.vdom.l.render();
			}
		});
	}

	Handler.prototype = {
		widgets: ['epsgram'],

		layout: function() {
			setTimeout(function() {
				var page_width = $('#main').width();
				var col1_width = $('.column-1').outerWidth(true);
				var col3_width = $('#basic-chart > img').outerWidth(true);

				block[page_width - col1_width - col3_width < 280 ? 'addClass' : 'removeClass']('inline').show()
			}, 0);
		},

		col1: function() {
			var self = this;

			var sections = [{
				name: 'history',
				title: 'Recently viewed',
				render: function(){
					if(CH.current.instance && CH.current.instance.has_widget('epsgram')){
						var storage = self.is_wmo ? 'ensgram_history_wmo' : 'ensgram_history';
						epsgram_history = JSON.parse(localStorage[storage] || '[]');
						return [h('ul.chart-history', $.map(epsgram_history, function(item){
							if(!item.values) item = item.product; // old format

							var name = item.values.station_name || item.values.meteogram_location;
							item = item.values;

							var location = (name || $.formatLatitude(item.lat, 1) + ' ' + $.formatLongitude(item.lon, 1)).replace('_', ' ');
							var time = new Time(item.time);
							var date = time.base_time.format('dd mmm HH:MM');
							var type = item.epsgram.replace('classical_', '');

							return h('li', {
								title: location + ' - ' + type + (date ? ' - ' + time.base_time.toString() : ''),
								onclick: function(e) {
									CH.current.instance.set_parameters(item);
								}}, [
									h('div', [h('div', [location])]),
									h('div', [h('div', [date])]),
									h('div.epsgram-type', [h('div', [type])])
								]
							);
						}))]
					}
					return [h('div', 'history')];
				}
			}]

			this.app.have_chart.done(function() {
				CH.current.bus.listen('location-changed', function() {
					self.app.vdom.l.render()
				})
				var c = {};
				CH.current.bus.listen('wmo-country', function(_, country){
					self.app.vdom.l.render();
					c = country;

					self.app.vdom.l.on.render.once(function() {
						var block = $('.country-cities-content');
						block.css({backgroundColor: '#ffc'}).delayed(1000, function() {
							block.css({backgroundColor: ''});
						});
					})
				});

				self.app.vdom.l.sections.splice(1, 0, {
					name: 'cities',
					render: function() {
						var country = JSON.parse(localStorage.ensgram_wmo_country || '{}');
						return self.is_wmo
							? h('div.country-cities-content', [
								h('h2', country.title),
								h('ul.menu.country-cities', $.map(country.result, function(v){ return h('li', {
									onclick: function() {
										router.set_parameter({meteogram_location: v[6]});
									}
								}, v[0]) }))
							])
							: []
					}
				})
			})

			return sections;
		},

		remove: function() {
			// TODO we are cleaning up from the chart plugin. These items should be created here and not in the chart plugin
			// (they cannot be in the chart plugin because they relate also to the listing page where there is no chart)
			console.log('epsgram.remove: package=', this.app._package);

			var sections = this.app.vdom.l.sections
			for(var i in sections){
				if(sections[i].name === 'history'){
					sections.splice(i, 1)
					break;
				}
			}
		}
	}

	return Handler
});
