/*
 *  Author: Cihan
 */
define(['config', 'jquery', 'ajax_queue', 'unit', 'dialogue2013', 'form/handlers/location', 'jquery-ui/draggable'], function(ECMWF, $, ajaxqueue, Unit, Dialogue) {

/* TODO
	+ Modal window drag and move (semi tranparent whilst dragging)
	- Find a suitable place to display metadata
	+ Bring Parameter bar
    - PDF download
	+ Disable click on non-geographical area (Issue for global view)
	+ Bring lat-lon display on left-bottom of the chart
	+ Loader when loading meteograms
	- Time navigator is buggy and base time selection does not work.
	+ Moving from one product to another does not work
	+ Pass base time and step of the background chart to epsgram
	+ PDf download
	+ EFI CDF to be added
	+ Location/lat/lon entry on modal window
	+ Background chart title/legend
	+ High resolution coastlines for background chart

*/
$.widget('ui.meteogramOverlay', {
	options: {
		time: {},
		position: {},
		product: null
	},

	recently_viewed     : [],
	current_selection   : null,
	d                   : null,

	_init: function(){
		var self = this, o = self.options;

		self.d.modal('show').draggable({
			drag: function( event, ui ) {
				self.d.css({opacity: 0.5})
			},
			stop: function( event, ui ) {
				self.d.css({opacity: 1})
			}
		});

		self.selected_epsgram = self.selected_epsgram || 'classical_10d';
		if (self.check_coordinates(o.position)) {
			self.get_meteogram(self.prepare_data(self.selected_epsgram, o.position));
		}
	},

	_create: function(){
		var self = this, o = self.options;

		var dlg = new Dialogue({
			id: 'location-overlay',
			title: 'Meteograms'
		}).getElements();

		self.d = $('#location-overlay')
			.each(function(){
				if($('#location-overlay').hasClass('in')) $('#location-overlay').modal('hide');
			})
			.modal('show').draggable({
				drag: function( event, ui ) {
					self.d.css({opacity: 0.3});
				},
				stop: function( event, ui ) {
					self.d.css({opacity: 1});
				}
			});

			var left= $('<div>', {id: 'left-overlay'});
			var right= $('<div>', {id: 'right-overlay'});

			// form
			left.div().form({
				items: [
					{'type': 'location', name: 'location', title: '', value: {lat: 50, lon: -1}}
				],
				show_cancel: false,
				ok: function(form_data) {
					o.position = form_data.location;

					if (self.check_coordinates(o.position)) {
						self.get_meteogram(self.prepare_data(self.selected_epsgram, o.position));
					}
				}
			});

		var interval = setInterval(function(){
			var autocomplete = left.find('.city-autocomplete')
			if(autocomplete.length){
				$('<label>Search:</label>').insertBefore(autocomplete);
				clearInterval(interval);
			}
		}, 50);

		dlg.body.each(function() {
			var epsgrams = $('<div>', {id: 'epsgram-selection'});
			for (var i in metgrams_ordered) {
				var key = metgrams_ordered[i];
				var a = metgrams[key];

				epsgrams.a({id: key, href: a.title}).text(a.title).click(function() {
					self.selected_epsgram = $(this).attr('id');
					if (self.check_coordinates(o.position)) {
						self.get_meteogram(self.prepare_data(self.selected_epsgram, o.position));
					}
					return false;
				});

				epsgrams.div();
			}

			left.append(epsgrams);

			left.append($('<h3>Download</h3>'));
			var download = left.div();
			download.a({id: 'download-pdf', href: 'download-pdf'}).text('PDF').click(function() {
				self.current_selection && self.get_meteogram($.extend(true, {}, self.current_selection, {
						//FIXME: Basic and interactive charts expect format on different places!
						product: {
							format: 'pdf'
						}
					}), false);
				return false;
			});

			left.div({id: 'metadata'}).text('Show grid point info');
			left.h3('Recently viewed');
			left.div({id: 'recently_viewed'});
			dlg.body.append(left, right);
		});
	},

	check_coordinates: function(position) {
		var self = this;

		if (position.lon > 180)
			position.lon = position.lon - 360;

		if (position.lat > 90 || position.lat < -90) {
			$('#right-overlay').text('A meteogram can not be produced for given coordinates. Please click on the map');
			return false;
		}
		if (position.lon > 180 || position.lon < -180) {
			$('#right-overlay').text('Check coordinates: Lon: ' + position.lon + ' Lat: ' + position.lat);
			return false;
		}
		self.update_form(position);

		return true;
	},

	update_form: function(position) {
		$('.form-location .lat').val((position.lat).toFixed(2));
		$('.form-location .lon').val((position.lon).toFixed(2));

		var city = position.city || '';
		if (city === '') {
			$('.form-location .altitude').val('');
			$('.city-autocomplete').val('');
		}
		else {
			$('.form-location .altitude').val(position.altitude);
			$('.city-autocomplete').val(position.city);
		}
	},

	prepare_data: function(selected_epsgram, position) {
		var self = this, o = self.options;

		var product_name = 'classical_meteogram';

		var altitude = position.altitude || '';
		var station_name = position.city || '';
		var expver = "0001";
		if (o.product.variables) {
			var btime = o.product.variables.time.value;
			o.time = [btime.base_time.format('yyyymmddHH'), btime.step, btime.valid_time.format('yyyymmddHH')];
			if (o.product.variables.expver)
				expver = o.product.variables.expver.value || expver;
		}
		__DEV__ && console.log("Expver selected: ", expver);

		var data = {
			product : {
				product: product_name,
				values: {
					epsgram : selected_epsgram,
					lat     : position.lat,
					lon     : position.lon,
					expver  : expver,
					//format  : 'png',
					time    : o.time,
					validate: 'base_time',
					station_name: station_name,
					altitude: altitude
				},
				viewport: {
					height: 800,
					width: 600
					//thumbnail: true
				}
			}
		}

		if (selected_epsgram === 'classical_efi_cdf') {
			data.product.product = 'classical_efi_cdf';
			data.product.values.validate = 'efi_cdf';
			delete data.product.values.epsgram;
		}

		return data;
	},

	get_meteogram: function(data, add_to_history) {
		var self = this;

		if(__DEV__ && isNaN(data.product.values.lat)) throw new Error('lat isNaN')

		add_to_history = (add_to_history === undefined) ? true : add_to_history;

		if (data.product.format === 'pdf')
			data.product.viewport.thumbnail= false;

		$('#right-overlay').empty();

		ajaxqueue({
			url: '/plots/get-map/',
			json: data,
			timeout: 20 * 1000,
			complete: function(result) {
				result = result.result;
				if (result && result.url) {
					if (data.product.format === 'pdf')
						location.href = result.url; // TODO: Change to download
					else {
						$('#right-overlay').img({src: result.url});

						self.current_selection = data;

						if (result.magics && 'metadata' in result.magics)
							self.update_metadata(result.magics.metadata);
						if (add_to_history)
							self.update_recently_viewed(data);
					}
				}
			},
			status: ECMWF.ajax_loader($('#right-overlay'))
		});
	},

	update_recently_viewed: function(data) {
		var self = this;
		var el = data;
		for (var i in self.recently_viewed) {
			var sel = self.recently_viewed[i];
			var check = (
				sel.product.values.epsgram === el.product.values.epsgram &&
				sel.product.values.lat === el.product.values.lat &&
				sel.product.values.lon === el.product.values.lon &&
				sel.product.values.time[0] === el.product.values.time[0]
			)

			if (self.selected_epsgram === 'classical_efi_cdf') {
				// IF both selections are on the same day, do not create a new entry
				check = check && (sel.product.values.time[2].slice(6,8) === el.product.values.time[2].slice(6,8));
			}
			if (check)
				return;
		}

		self.recently_viewed.unshift(data);

		var a = $('<a>', {title: "Base time: " + el.product.values.time[0] + " "+ el.product.values.station_name })
			.text(metgrams[self.selected_epsgram].short_title + '(' + (el.product.values.lat).toFixed(2) + '/' + (el.product.values.lon).toFixed(2) + ')')

			.click(function() {
				//var selection = el.product.values.epsgram;
				self.selected_epsgram = el.product.values.epsgram;
				self.options.position.lat = el.product.values.lat;
				self.options.position.lon = el.product.values.lon;
				self.options.position.altitude = el.product.values.altitude;
				self.options.position.city = el.product.values.station_name;

				self.update_form(self.options.position);
				self.get_meteogram(el, false);
			});

		var rv = $('#recently_viewed');
		rv.prepend(a);
		rv.prepend($('<div>'));
		if (self.recently_viewed.length > 15) {
			self.recently_viewed.pop();
			rv.find('a:last').remove();
		}
	},

	update_metadata: function(data) {
		var self = this;
		var ready = self.get_epsgram_help(data);
		var metadata = $('#metadata');
		$('.metadata-help').remove();
		var help = $('<div>', {'class': 'metadata-help'})
			.hide();
		help.append(ready);
		metadata.append(help);
		metadata.click(function() {
			help.toggle();
		});
	},

	// FIXME duplicated in basic_chart/epsgram.js
	get_epsgram_help: function (data) {
		var self = this;
		var list = [];

		if (!data) {
			return 'Cannot get help information.';
		}
		var metadata = data;

		if (!metadata.distance) {
			return 'Cannot display help information. Missing Distance data.';
		}

		var m = Unit.get('m');
		var km = Unit.get(metadata.distance.distance_unit || 'km');

		var gridpoint_type = (metadata.land_sea_mask < 0.5) ? "sea" : "land";
		metadata.title && list.push("### " + metadata.title );
		list.push("Location requested: <b>" + $.formatGeoLocation(metadata.requested_location) + "</b>.<br>");
		list.push("Nearest ENS grid point: <b>" + $.formatGeoLocation(metadata.location) + '</b>.<br>');
		list.push('');

		if (metadata.height !== "" &&  metadata.height !== -9999 && metadata.db !== "10_days_wave") {
			console.log("metadata.height:<" , metadata.height, ">");
			list.push("Height of requested location (based on the GTOPO30 dataset): <b>" + m.asHTML(metadata.height) + "</b>, ");
			if (metadata.db != "plumes") {
				list.push("height of ENS orography at nearest grid point: <b>" + m.asHTML(metadata.eps_height) + "</b>, ");
				list.push("height of high resolution orography interpolated to ENS resolution at nearest grid point: <b>" +
				m.asHTML(metadata.deterministic_height) + '</b>.');
			}
			if (metadata.temperature_correction) {
				list.push('');
				if(metadata.temperature_adjustment) {
					list.push('Temperature parameter is adjusted to the requested location by **' + $.formatCelcius(metadata.temperature_adjustment.deterministic_adjustement, 1) + '**' + ' in high resolution forecast curve and ' + '**' + $.formatCelcius(metadata.temperature_adjustment.eps_adjustement, 1) + '** in ENS forecast curves.');
				}
				else
				{
					list.push('Temperature parameter is adjusted to the requested location');
				}
				list.push('');
			}
		}
		list.push('');
		list.push('');
		//TODO: Fix me later
		if (metadata.db === "10_days_wave") {
			list.push("The nearest ENS grid point is located at **" + km.asHTML(metadata.distance.distance) + " " + metadata.distance.direction + "** of the selected location.");
		}
		else {
			list.push("The nearest ENS grid point is a <b>" + gridpoint_type + " point</b>, ");
			list.push("located at <b>" + km.asHTML(metadata.distance.distance) + " " + metadata.distance.direction + "</b> of the selected location.");
		}

		return list.join("\n");
	}
});

var metgrams = {
	classical_10d : {
		title: "10-day meteogram",
		short_title: "10-day"
	},
	classical_15d: {
		title: "15-day meteogram",
		short_title: "15-day"
	},
	classical_15d_with_climate: {
		title: "15-day with climate",
		short_title: "15-day clim"
	},
	classical_plume : {
		title: "Plume",
		short_title: "Plume"
	},
	classical_wave: {
		title: "10-day wave",
		short_title: "Wave"
	},
	classical_efi_cdf: {
		title: "EFI-CDF",
		short_title: "EFI-CDF"
	}
};

var metgrams_ordered = ['classical_10d', 'classical_15d', 'classical_15d_with_climate', 'classical_plume', 'classical_wave', 'classical_efi_cdf'];

});
