
define(['jquery', 'utils', 'grid/products', 'grid/facet_filter', 'grid/package_filter', 'product_browser/fixed_filter', '../chart_handler', '../router', 'mixins', 'page', 'page/query', 'grid', 'facets', 'perfect_scrollbar', 'tooltip', 'tooltip/tipsy', 'throttle_debounce'], function($, products, facet_filter, package_filter, p_filter, chart_handlers, controller, mixins, page, qs) {

	var center = $('#main, #top');

	var handler;

	return {
		config: {
			panel: {
				resizable: true, closeable: true
			},
			col2: {
				panel: {
					resizable: true, closeable: false, title: 'Results'
				}
			}
		},

		col1: function(o) {
			var w = this.widgets;
			var self = this;

			this.loader = new (function _loader(){
				this.checks = { package_select: new $.Deferred() };
				this.init = function() {
					this.deferred = $.when
						.apply(this, $.map(this.checks, function(i){ return i; }))
						.done(function(){ self.do_layout(); });
					return this;
				}
			})().init();

			this.have_layout = new $.Deferred(function(df) {
			});

			if(!this._package){
				this.widgets.package_select = {
					init: function(packages) {
						$.extend(w.package_select, {widgetName: 'packageSelect', storage: mixins.storage});
						var pkg = w.package_select.storage('package');

						(w.package_select.select = o.el
							.div('.group packages row'))
							.div('.cell')
							.formSelect({
								id: 'package-select',
								options: $.map((packages.unshift({name: 'all', title: 'All'}), packages), function(v){ return [[v.name, v.title]]; }),
								label: 'Packages'
							})
							.addClass('package-select')
							.change(function(ev){
								var p = $(this).val();
								p === 'all' && (p = null);
								package_filter.set([p], p);
								p === pkg.get() || pkg.set(p);
							})
							.each(function(){
								var p = pkg.get() || 'all';
								(p === 'all') || $(this).val(p).change();
							})
							.parent().parent()
							.prependTo(o.el)

						self.loader.checks.package_select.resolve();
					},

					on_filter_change: function() {
						var pkg = package_filter._selected || 'all';
						pkg === w.package_select.select.val() || w.package_select.select.val(pkg);
					}
				}
			}
		},

		col2: function(o) {
			var w = this.widgets;
			var col1 = this.widgets.col1;

			setTimeout(function(){ o.el.show(); }, 5000); // TODO

			p_filter._selected = this._package;

			list.set_collection(products);

			this.widgets.grid = o.el
				.find('.panel-content')
				.grid({
					filters: [facet_filter, this._package ? p_filter : package_filter],
					text_filter_parent: col1.div('.row').div('.cell'),
					on_click: this._bind(this.on_product_select),
					on_model_change: function(items){

						w.package_select && products.packages.then(w.package_select.init);

						w.facets = col1.div('.row').div('.cell').div().facets({
							products: products,
							on_change: function(a, b, val) {
								facet_filter.set([a, b], val);
							}
						})
						.perfectScrollbar({suppressScrollX: true, wheelSpeed: 10});
					},

					on_filter_change: function() {
						if(!facet_filter.status()) w.facets.facets('clear');

						w.facets
							.facets('on_filter_change')
							.delayed(300, function() { // wait for animations to finish
								var scrollable = w.facets.find('> ul');
								var bottom = -w.facets.scrollTop() + scrollable.height();

								if(bottom < w.facets.height()){
									w.facets.scrollTop(Math.max(0, scrollable.height() - w.facets.height()  + 10));
								}

								w.facets.perfectScrollbar('update');
							})

						w.package_select && w.package_select.on_filter_change();
					}
				})
				.each(function() {
					o.el.find('.foot').prependTo($(this));
				});
		},

		col3: function(o) {
			var self = this;

			//App.prototype.col3.apply(this, arguments);

			o.el.panel({fullscreenable: function(fullscreen, size){
				require(['related_charts'], function() {
					o.el.related({});
				});

				setTimeout(function(){
					if(chart_handlers.basic.scrollable){
						var height = size.height;
						o.el.find('> *:visible').not('.scrollable, .product-name, .next-chart, .prev-chart').each(function() {
							height -= $(this).outerHeight(true);
						});
						chart_handlers.basic.scrollable
							.css({height: height})
							.perfectScrollbar('update');
					}
				}, fullscreen ? 150 : 0);
			}});

			o.el.div({className: 'next-chart'}).click(function() {
				self.widgets.grid.grid('next');
			});
			o.el.div({className: 'prev-chart'}).click(function() {
				self.widgets.grid.grid('prev');
			});
		},

		on_product_select: function(product) {
			var self = this;
			chart_handlers.state.product = product;

			this.widgets.col3.panel('set_title', product.title_ ? product.title_() : product.title());

			var is_interactive = !!~$.inArray('interactive', product.data.widgets);
			var chart_type = is_interactive ? 'interactive' : 'basic';

			//var request_id = basic.request_id = basic.request_id ? ++basic.request_id : 1;
			this.have_layout.done(function() {

				handler = chart_handlers[chart_type];
				handler.host = self;
				if(handler !== chart_handlers.current){
					var width = Math.floor(self.widgets.col3.width() / 128) * 128;

					chart_handlers.current && chart_handlers.current.cleanup();
					handler.build(product, chart_handlers.current && chart_handlers.current.state, {el: self.widgets.col3, size: {/*width: 600, */height: self.widgets.col3.height()}}, function() {
						chart_handlers.widgets.chart.on('click', 'img', function() {
							self.widgets.col3.find('.button-fullscreen > div').click();
						});

						chart_handlers.widgets.chart.perfectScrollbar({wheelSpeed: 10});
						chart_handlers.basic.scrollable.perfectScrollbar({wheelSpeed: 10});
					});
					chart_handlers.current = handler;
				}else{
					handler.update(product);
				}
			});
		},

		do_layout: function() {
			// TODO should only set outer width if is already set

			var self = this;
			var col1 = this.widgets.col1;
			var col2 = this.widgets.col2;
			var col3 = this.widgets.col3;

			(function layout_column1() {
				var height = $('.column-1').height();
				col1.find('> div').not(':last').each(function() {
					height -= $(this).outerHeight();
				});
				self.widgets.facets && self.widgets.facets
					.css({height: height})
					.perfectScrollbar('update');
			})();

			//$('#main').trigger('layout', {width: $(window).width()});

			setTimeout(function(){
				// set height of column-3 and then the basic-chart

				var height = col1.height();
				col2.each(function(){
					height -= $(this).outerHeight();
				});
				col3
					.css({height: height - parseInt(col3.css('marginBottom'), 10) + 2});

				if(chart_handlers.basic.scrollable){
					height = col3.height();
					col3.find('> *:visible').not('.scrollable, .product-name, .next-chart, .prev-chart, .button-fullscreen').each(function() {
						height -= $(this).outerHeight(true);
					});
					chart_handlers.basic.scrollable
						.css({height: height})
						.perfectScrollbar('update');
				}

				$('#time-navigator').triggerHandler('layout', {width: col3.find('h2').width()});

				self.have_layout.resolve();
			//}, 0);
			}, 3000);
		},

		on_url_change: function(ctx) {
			var self = this, product;

			if(ctx.params && (!chart_handlers.state.product || (ctx.params.product && chart_handlers.state.product && ctx.params.product !== chart_handlers.state.product.name()))){
				console.log('>>> on_url_change: product has changed!', chart_handlers.state.product && chart_handlers.state.product.name(), ' --> ', [ctx.params.product]);
				if(ctx.params.product){
					setTimeout(function() { // FIXME
						if(chart_handlers.basic.instance) console.warn('on_url_change: unexpected instance');
						if((product = products.product_by_name(ctx.params.product))){
							chart_handlers.state.params = ctx.query;
							self.on_product_select(product, ctx.query);

							self.widgets.grid.grid('select', ctx.params.product);
						}
					}, 1000);
				}
			}

			if(ctx.params && ctx.params.product && chart_handlers.state.product && ctx.params.product !== chart_handlers.state.product.name()){
				console.log('>>> on_url_change: product has changed!', chart_handlers.state.product.name(), ' --> ', [ctx.params.product]);
			}

			chart_handlers.basic.instance && chart_handlers.basic.instance.set_parameters(ctx.query);

			if(chart_handlers.basic.instance){

				// update for product first.
				controller.path = '/browser/' + (this._package || 'all' ) + '/' + chart_handlers.basic.instance.name() + '?' + qs.stringify(chart_handlers.state.params).replace(/%2C/g, ',');

				chart_handlers.basic.instance.have.availability.done(function() {
					$.extend(chart_handlers.state.params, Utils.map(chart_handlers.basic.instance.variables, function(v){ return v.to_string(); }));
					controller.path = '/browser/' + (this._package || 'all' ) + '/' + chart_handlers.basic.instance.name() + '?' + qs.stringify(chart_handlers.state.params).replace(/%2C/g, ',');
				});
			}
		},

		_bind: function(fn) {
			var self = this;
			return function() {
				fn.apply(self, arguments);
			};
		}
	}
});
