/*
 *   Chart app variation with 3 columns: left filters, centre chart, right results
 */
define(
		['config', 'jquery', 'bus', 'chart_app/application', 'grid/products', 'product_browser/packages', 'grid/facet_filter', /*'grid/package_filter', */'grid/path_filter', 'product_browser/chart_handler', '../router', 'mixins', 'ecmwf-page/query', 'grid/filterable_list', 'storage', 'grid', 'facets', 'jquery-ui/button', 'mode-tooltip', 'mode-tooltip/tipsy', 'throttle-debounce', 'messages', 'perfect_scrollbar', './view.css', '../product_browser.css', './results.css'],
		function(ECMWF, $, Bus, ChartApplication, products, packages, facet_filter, /*package_filter, */path_filter, CH, page, mixins, qs, list, localStorage) {

	var overlay = false; // config determining whether thumbnails are an overlay or main landing page content.
	var config = {
		show_view_buttons: false,
		show_sidenav: true,
		crop: false // large charts can be either cropped or scaled down.
	}

	var chart_handlers = CH;

	var App = function(){};
	App.prototype = ChartApplication;

	return $.extend(true, new App(), {
		config: {
			col2: {
				panel: {
					closeable: overlay, title: 'Results'
				}
			},
			enable_hierarchy: false,
			use_column_width: false
		},

		col1: function(o) {
			var w = this.widgets;
			var self = this;

			// set this to have the product list sorted (will be used for grouping)
			//list.sort_by = 'title';

			App.prototype.col1.apply(this, arguments);

			$('<div>', {'class': 'breadcrumbs'}).prependTo('#content-heading');

			this.have_url.done(function() {
				path_filter.set([page.params.product_path]);
				self.update_category_links();
			});

			products.query = this._package
				? {package: this._package}
				: {};

			overlay && o.el.div('.show-results submit').text('Show product results').button().click(function() {
				self.widgets.col2.panel('show');
			});

			this.config.enable_hierarchy && (
				$.extend(self.widgets, {
					category_h2: o.el.h2('Categories'),
					path_links: o.el.ul('.path-links').text('Path links')
				}),

				path_filter.events.bind('changed', function(path){
					console.log('-> path_filter changed', path ? path[0][0] : '');
					page.goto_list_page(path ? path[0][0].join('/') : (self._package || ''));
					self.update_category_links();
					self.scroll_to_col2();
				})
			);

			o.el.find('h2:contains(Filters)').length || o.el.h2('Filters');

			// navigation
			products.packages.then(function(packages) {
				var p = $.findWhere(packages, {name: self._package});
				if(p && p.links && p.links[0]){
					$('.column-1 .menu-name-main-menu li:first')
						.addClass('active')
						.ul('.menu')
							.li('.menu__item is-leaf first leaf')
								.a({href: p.links[0].link, className: 'menu__link'}).text(p.links[0].title);
				}
			});

			facet_filter.events.bind('changed', function() {
				if(self._package && facet_filter._active){
					localStorage[self._package] = JSON.stringify({facet_selection: $.map(facet_filter._active, function(v, i) {
							return i + ',' + v.join(',');
						}).join(';')
					});
				}
			});

			config.show_sidenav && require(['sidenav'], function() {
				// TODO try easeOutQuint
				$('#sidebar').tipsy().simplerSidebar({
					selectors: {
						trigger: '#toggle-sidebar'
					},
					events: {
						callbacks: {
							animation: {
								open: function() {
									$('#sidebar-wrapper-1 li').length ||
										products.packages.then(function(packages) {
											$.each(packages, function(i, v) {
												$('#sidebar-wrapper-1').li('.menu-item').a({href: '#' + v.name}).click(function(){ page.set_dir(v.name); }).text(v.title);
											});
										});
								}
							}
						}
					}
				});
				$('#sidebar-2').simplerSidebar({
					selectors: {
						trigger: '#toggle-sidebar-2',
						quitter: '.close-sidebar'
					},
					events: {
						callbacks: {
							animation: {
								open: function() {
									$('#sidebar-wrapper-2').hasClass('catalogue-grid') ||
											$('#sidebar-wrapper-2').css({height: 640}).grid({
												view_mode: 'details',
												on_click: function(product) {
													console.log('sidenav.click:', product);
													$('#sidebar-2 .close-sidebar').click();
													self._on_thumbnail_click(product);
												}
											});
								}
							}
						}
					}
				});
				$('#sidebar-3').simplerSidebar({
					selectors: {
						trigger: '#toggle-sidebar-3'
					},
					events: {
						callbacks: {
							animation: {
								open: function() {
									$('#sidebar-wrapper-3').hasClass('catalogue-grid') || (function() {
										})();
								}
							}
						}
					}
				});
			});
		},

		col2: function(o) {
			var self = this;
			var w = this.widgets;

			o.el.addClass(overlay ? 'overlay' : 'no-overlay');

			o.el.parent().div('.column col2-control').prependTo(o.el.parent())
				.h2('Results').parent()
				.a().div().text('show product results').click(function() {
					w.col2.panel('show');
					return false;
				});

			(this.login_box = o.el.div('.login-message')).messages({messageBus: self.login_box.messageBus(), enable_popup: true, message_class: 'col-2-message'});

			list.selection.always = false;
			list.set_collection(products);

			var filters = [facet_filter];
			this.config.enable_hierarchy && filters.push(path_filter);

			w.grid = o.el
				.find('.panel-content')
				.grid({
					filters: filters,
					text_filter_parent: w.col1.find('.text-filter-row .cell').orelse(function(){ return w.col1.div('.row text-filter-row').hide().div('.cell'); }),
					on_click: self._bind(self._on_thumbnail_click),
					on_model_change: function(items){

						w.package_select && products.packages.then(w.package_select.init);

						w.facets = w.col1.find('.facets').orelse(function() { return w.col1.div('.row').div('.cell').div(); }).facets({
							products: products,
							selection: (function() {
								var r = {};

								var sel_package = products.query.package;
								var facet = localStorage[sel_package]
									? JSON.parse(localStorage[sel_package]).facet_selection
									: null;
								if (page._path.q){
									var args = qs.parse(page._path.q.slice(1));
									if (args.facets){
										facet = args.facets;
									}
								}
								// example url query: ?facets=Product group,Basic EFI;Component,Atmosphere,Geography
								if(facet){
									var facets = facet.split(';');
									$.each(facets, function(i, v) {
										var f = v.split(',');
										if(f.length > 1){
											r[f.shift()] = $.map(f, function(v, i) { return $.trim(v); });
										}
									});
								}
								return r;
							})(),
							on_change: function(a, b, val) {
								facet_filter.set([a, b], val);
							}
						});
					},
					on_filter_change: function() {
						if(!facet_filter.status()) w.facets.facets('clear');

						w.facets.facets('on_filter_change');

						w.package_select && w.package_select.on_filter_change();

						// filter changes have no impact on history state unless the view is changed from product to listing.
						this.n_filter_runs && self.col2.css.left && page.add_state();

						self.on_filter_change();

						var n_products = $.where(products._products, {filter_pass: true}).length;

						if(overlay){
							setTimeout(function() {
								w.col2.panel('show');
								self.widgets.grid.grid('resize'); // TODO panel should send event
							}, 1000);
						}else{
							this.n_filter_runs && self.col2.css.left && page.goto_list_page();
							self.do_layout();

							setTimeout(function(){ self.do_layout(); }, 1000);
						}
					},
					on_view_change: function() {
						self.do_layout();
					},
					on_view_ready: function() {
						config.show_view_buttons && self.widgets.grid.find('.head').show(); // buttons hidden until width stabilizes.
						if(list.items.length > 20) w.col1.find('.text-filter-row').show();
						self.do_layout();
					}
				})
				.each(function() {
					var grid = $(this);
					grid.find('.foot').prependTo(grid);
				});

			CH.product_change.subscribe(function() {
			});

			/*
			list.selection.subscribe(function()	{
				// -as much as we would like, this cannot be the main handler because we do not get clicks on an already-selected product.
			});
			*/

			if(!overlay){
				o.el.show();

				this.col2.css = {};
				this.col3.css = {};
			}
		},

		col3: function(o) {
			var self = this;
			var panel = this.col3;
			panel.widgets = {results: {}};

			App.prototype.col3.apply(this, arguments);

			if(true){
				o.el.show();
				this.col3.size = {width: 400, height: 400};
			}

			this.size_override_targets = $('.top-bar #main').orelse(function(){ return $('#centred, #top, #footer > div'); }); // www vs apps

			Bus()
				.listen('chart-added', function() {
					CH.widgets.chart.messageBus().listen('chart-product-availability', function(_, msg) {
						if(msg.valid_time.values.length > 1){
							self.do_layout();
						}
					});
				})
				// TODO there is a better way than forwarding global to local bus?
				.listen('prev-step', function() {
					(CH.current === CH.interactive) && CH.current.bus.send('prev-step');
				})
				.listen('next-step', function() {
					(CH.current === CH.interactive) && CH.current.bus.send('next-step');
				/*
				})
				.listen('tool-animation-toggle', function() {
					(CH.current === CH.interactive) && CH.current.bus.send('tool-animation-toggle');
				*/
				});

			/* **** text is underneath chart - how to stop it moving down on hover?  **** */
			$.extend(this.widgets, {
				col3_content: o.el.find('.panel-content').addClass('enlargeable'),
				legend:       o.el.div('#chart-legends').hide(),
				content:      o.el.div('.product-text text-content')
			});

			this.have_chart.done(function() {
				var results = panel.widgets.results;

				chart_handlers.widgets.controls.rhs
					.div('.back-button keep')
						.button({label: 'Back', icons: {primary: 'ui-icon-arrowthick-1-w'}, text: false})
						.click(function() {
							page.add_state().goto_list_page(page._path.dir);
						});

				(results.button = chart_handlers.widgets.controls.rhs
					.div('.results-button keep')
						.button({label: 'Product results', icons: {secondary: 'ui-icon-triangle-1-s'}}))
						.click(function() {
							self.widgets.col3.find('.ui-menu').not(results.menu || null).hide();

							results.menu = results.menu
								? results.menu.toggle()
								: results.button.ul('.results-menu')
									.menu({
										select: function(event, ui){
											console.warn('NEVER GET HERE');
											console.log('selection: ', ui.item.text(), ui.item.attr('data-value'));
											o.on_click(event, ui, o.name);
											self.select(ui.item);

											setTimeout(function() {
												var data = ui.item.parent().data('uiMenu');
												data && (data.mouseHandled = false); // jquery ui menu workaround
											}, 5);
										}
									})
									.each(function() {
										$(this).div().grid({
											collection: products,
											filters: [facet_filter/*, path_filter*/],
											view_mode: 'details',
											text_filter_parent: $(),
											container: false,
											size_fits_content: true,
											enable_status: false,
											on_click: self._bind(self._on_thumbnail_click)
										})
										.find('.scrollable').perfectScrollbar({suppressScrollX: true, wheelSpeed: ECMWF.wheelspeed});

										$(this).find('.outer').div('.mask'); // prevents us having to set the detail item width
									});
							return false;
						});

				$('body').off('click', close).click(close);

				// magnify charts too big for column
				// TODO do nothing if chart already fits in the column
				// TODO close the image on click and ESC
				// TODO improve styling of the overlay
				$('#basic-chart img').click(function() {
					$('body').div('.chart-overlay').div().img({src: $(this).attr('src')});
				});

				function close() {
					results.menu && results.menu.hide();
				}
			});

			/*
			chart_handlers.on.size_change = function(size) {
				overlay || self.do_layout();
			}
			*/

			/*
			this.widgets.next = o.el.div('.button-next').click(function() {
				self.widgets.grid.grid('next');
			});
			*/
		},

		on_filter_change: function() {
		},

		_on_thumbnail_click: function(item) {
			if((!page._path.page) || item !== CH.state.list_product){
				page.add_state();

				if(item === CH.state.list_product){
					// the selected product is the same as the previous product
					page.set_product(item.name(), item.title_());
				}else{
					// In this case, the page status is updated via the chart_handler.
					// This is wrong but fixing it requires the chart controls to be decoupled from the chart widget.

					var chart;
					if((chart = CH.widgets.chart && CH.widgets.chart.data('ui-basicChart'))){
						chart.grey();
					}
					this.on_product_select(item);
				}

				overlay && this.options.el.find('.ui-icon-close').click();
			}
		},

		scroll_to_col2: function() {
			/*
			if(!overlay){
				this.col2.css.left = 0;
				this.col3.css.left = this.col2.css.width;

				this.widgets.col2.css(this.col2.css);
				this.widgets.col3.css(this.col3.css);
			}
			*/
		},

		scroll_to_col3: function() {
			if(!overlay){
				this.col3.css.left = 0;
				this.widgets.col3.css(this.col3.css);
			}
		},

		q_size: function() {
			var width = Math.max(512, Math.min(1536, Math.floor(this.widgets.col3.width() / ECMWF.chart.q_width) * ECMWF.chart.q_width));
			return {
				width: width,
				height: width * 0.667
			}
		},

		on_product_select: function(product, params) {
			var self = this;

			App.prototype.on_product_select.apply(this, arguments);

			this.scroll_to_col3();

			if(CH.state.list_product === product) return;
			CH.set_state({list_product: product});

			this.widgets.col3.panel('set_title', product.title_ ? product.title_() : product.title());

			var is_interactive = !!~$.inArray('interactive', product.data.widgets);
			var chart_type = is_interactive ? 'interactive' : 'basic';

			var handler = chart_handlers[chart_type];
			handler.host = this;
			if(handler !== chart_handlers.current){
				var size = self.q_size();

				// 'current' is now set earlier to ensure that it is always set before listeners run.
				// (different require implements have varying amounts of asynchronicity)
				var old_handler = chart_handlers.current;
				chart_handlers.current = handler;

				old_handler && old_handler.cleanup();
				handler.build(product, params || (old_handler && old_handler.state), {el: this.widgets.col3_content, size: size}, function() {
					CH.widgets.chart.messageBus().listen('size-changed', function(_, size) {
						self._on_resize(size);
					})
					.listen('chart-content-update', function(_, msg) {
						overlay || self.do_layout();
					})
					.listen('chart-legend-loaded', function() {
						self.do_layout();
					});

					$('body').off('layout.viewer').on('layout.viewer', function() {
						if(is_interactive){
							var size = self.q_size();
							if(size.width !== CH.widgets.chart.width()){
								CH.widgets.chart.css(size);
								CH.widgets.chart.resize();
							}
							// update for height. TODO this negates the point of the check above.
							CH.widgets.chart.css(size);
							CH.widgets.chart.resize();
							overlay || self.do_layout();
						}
					});

					$.each(['legend', 'content', 'next'], function(i, v) {
						self.widgets[v] && self.widgets[v].appendTo(self.widgets.col3);
					});
				});
			}else{
				handler.update(product);
			}
		},

		do_layout: $.debounce(100, function() {
			var self = this;

			(this.widgets.time_navigator && this.widgets.time_navigator.length) || (this.widgets.time_navigator = $('#time-navigator, .time-nav'));
			this.widgets.time_navigator.triggerHandler('layout', {width: this.widgets.time_navigator.width()});

			if(!overlay){
				var col1_width = this._col1_width();
				var col2_width = this._col1_width();
				var left_width = col1_width + (col1_width ? 50 : 0);

				var fix = 20;
				var normal_width = this.config.use_column_width
					? $(window).width() - this._col1_width() - col2_width - 2 * 52 - 60 - fix
					//? $(window).width() - this._col1_width() - 2 * 52 - 60
					: 0; // ignore the column width, just use the chart width
				var is_interactive = !!~$.inArray('interactive', CH.state.list_product && CH.state.list_product.data.widgets);
				var q_size = is_interactive ? self.q_size() : {};
				self.col3.size.width = q_size.width || Math.max(normal_width, (CH.widgets.chart && CH.widgets.chart.find('img').width()) || 800) + 10;
				self.col3.size.width = q_size.width || normal_width;

				var max_width = parseInt(this.widgets.grid.find('.grid-items').css('max-width'), 10);

				self.col3.size.height = (chart_handlers.widgets.chart ? chart_handlers.widgets.chart.height() : 0) + $('#chart-legends').height() + $('.product-text').height() + 300;
				self.widgets.col3.css(self.col3.size);
				self.widgets.grid
					.triggerHandler('layout');

				(function enlarge_on_hover() {
					// TODO ensure image is loaded
					if(CH.widgets.chart){
						var scrollable = self.widgets.col3.find('.scrollable');
						var img_width = CH.widgets.chart.find('img').width();
						var col_width = self.widgets.col3.width();
						scrollable[img_width > col_width ? 'addClass' : 'removeClass']('enlargable');
					}

											var scrollable = self.widgets.col3.find('.scrollable > div');
					function leave() {
						if(config.crop){
							self.widgets.col3.removeClass('overflow').addClass('no-overflow');
						}else{
							scrollable.removeClass('no-scale').addClass('scale');
							var width = self.widgets.col3.width();
																console.log('    width=', width);
							CH.widgets.chart && CH.widgets.chart.find('img').css({maxWidth: width});
						}
					}

					leave();

					// TODO dont bind each time layout is done
					self.widgets.col3.off('mouseenter mouseleave').on({
						mouseenter: function() {
							if(config.crop){
								self.widgets.col3.removeClass('no-overflow').addClass('overflow');
							}else{
								scrollable.removeClass('scale').addClass('no-scale');
								CH.widgets.chart && CH.widgets.chart.find('img').css({maxWidth: ''});
											self.widgets.col3.css({overflow: 'visible'});
							}
						},
						mouseleave: leave
						/*
						mouseleave: function() {
							if(config.crop){
								self.widgets.col3.removeClass('overflow').addClass('no-overflow');
							}else{
								self.widgets.col3.removeClass('no-scale').addClass('scale');
							}
						}
						*/
					}, '.basic');
				})();
			}
		}),

		on_url_change: function(ctx) {
			var self = this;

			App.prototype.on_url_change.apply(this, arguments);

			if(self.is_landing_page){
				list.have.items.then(function(items) {
					self.h1(products.hierarchy.lookup(page._path.dir).title);

					// ensure there is always a chart
					if(!CH.current){
						list.have.filtering.done(function() {
							console.log('instantiating chart... list=', list.items[list.filtered[0]]);
							self._on_thumbnail_click(list.items[list.filtered[0]]);
						});
					}
				});
			}else{
				self.have_chart.done(function() {
					self.h1(CH.state.list_product.title_());
				});
			}
		},

		update_category_links: function() {
			var self = this;

			this.config.enable_hierarchy && products.products().then(function(items) {
				var children = products.hierarchy.children(page.params.product_path);
				self.widgets.path_links.empty();
				for(var c in children){
					var child = children[c];
					self.widgets.path_links.li().a({href: child.name + '/'}).text(child.title).click(function() {
						console.log('category.click: child=', $(this).attr('href'));
						path_filter.set_child($(this).attr('href').replace('/', ''));
						return false;
					});
				}
				self.widgets.category_h2.text(ECMWF.ucfirst(page.params.product_path[page.params.product_path.length - 1].replace(/-/g, ' ')));

				self.widgets.path_links.add(self.widgets.category_h2)[(!children) || $.isEmptyObject(children) ? 'hide' : 'show']();
			});
		}
	});
});
