// TODO user_info can be removed when menu is moved
define(['config', 'jquery', 'bus', 'user_info', 'ajax_queue', 'chart-app/application', 'vgrid/products', 'basic_chart/model/product', '../fixed_filter', '../chart_handler', 'chart-app/router', 'page/query', 'mixins', 'app-menu', 'tooltip', 'tooltip/tipsy', 'throttle-debounce', './tcyclone.css', '../product_browser.css', 'perfect-scrollbar'], function(ECMWF, $, Bus, User, Ajaxqueue, ChartApplication, products, BasicInstance, p_filter, chart_handlers, controller, qs, mixins, AppMenu) {

	var handler;
	var CH = chart_handlers;
	var region_filter, year_filter;
	var selection  = {
		product: 'tc_strike_probability',
		unique_id: null,
		year: null,
		region: null
	};
	var App = function(){};
	App.prototype = ChartApplication.default;

	// this is intended to be refactored after we know what the menu contents are. It is shared with other pages
	function make_app_menu() {
		User.info.then(function(user) {
			var items = [
				// TODO links are better expressed as being relative to the application root
				{ title: 'View all',                      href: '/en/forecasts/charts/catalogue/' },
				{
					title: 'ENS Meteograms',
					items: [
						{ title: 'ENS meteograms',          href: '/en/forecasts/charts/web/classical_meteogram' },
						{ title: 'WMO ENS meteograms',      href: '/en/forecasts/charts/web/classical_meteogram_wmo' }
					]
				},
				{
					title: 'Cyclones',
					items: [
						{ title: 'Tropical cyclones',       href: '/en/forecasts/charts/tcyclone/' },
						{ title: 'Extra tropical cyclones', href: '/en/forecasts/charts/extra-tropical-cyclones' }
					]
				},
				{
					title: 'More charts',
					items: [
						{ title: 'Monitoring',              href: '/en/forecasts/charts/monitoring-observing-system/' },
						{ title: 'Ocean Reanalysis',        href: '/en/research/climate-reanalysis/ocean-reanalysis' },
						{ title: 'EUROSIP',                 href: '/en/forecasts/charts/seasonal/' }
					]
				}
			]

			var menu = new AppMenu({
				el: $('#app-menu .row'),
				items: items,
				help: [{
					name: 'help',
					title: 'Help',
					items: [
						{title: 'FAQ', href: '/search/faqs?f[0]=im_field_faq_category%3A1082'},
					]//,
					//enabled: User.is_forecaster()
				}]
			})
			menu.render();
		})
	}

	return $.extend(true, new App(), {
		config: {
			col2: {
			}
		},

		col1: function(o) {
			var self = this;
			this._package = 'tcyclone';

			App.prototype.col1.apply(this, arguments);

			make_app_menu()

			products.set_list();

			products.have.items.then(function(prods) {
				products.emit();

				if (!products._products.length) {
					self.h1('ECMWF tropical cyclones');
					return;
				}

				for (var i in prods) {
					if (prods[i].data.name === 'tc_overview') {
						prods.splice(i, 1);
					}
				}

				o.el.section('.links').ul('.menu').li('.menu__item is-leaf leaf').a({href: 'overview'}).text('Overview map').click(function() {
					self.is_landing_page = true; // TODO should be done by parent class
					controller.set_path(undefined, null);
					return false;
				});

				var latest = self.widgets.latest = o.el.section('#latest-cyclones');
				latest.h3('Latest cyclones');

				var filters = o.el.section('.column-tcyclone-filters');
				var results = o.el.section('.column-tcyclone-results');

				filters.h3('Filters');

				year_filter = filters.div();
				year_filter.label('Year');

				region_filter = filters.div('.last');
				region_filter.label('WMO region');

				results.h3('Cyclone products');
				var p = results.ul('#product-list');
				$.each(products._products, function(i, v) {
					p.li()
						.a({href: v.name()})
						.text(v.title_())
						.each(function(){ v.name() === selection.product && $(self).addClass('active'); })
						.click(function() {
							console.log('product click');
							controller.add_state();
							self.show_product(v);
							selection.product = v.name();
							return false;
						});
				});

				results.h3('Cyclone results');

				self.widgets.cyclones = results.ul({id: 'cyclone-list', className: 'last'});

				Ajaxqueue({
					url: '/plots/latest-cyclones/',
					data: JSON.stringify({}),
					complete: function(result) {
						if(!result || !result.latest_cyclones){
							Bus().send('log-error', 'Failed to retrieve cyclones');
							return;
						}

						selection.unique_id = selection.unique_id || result.latest_cyclones[0].unique_id;

						var list = latest.ul('.last');
						for (var i in result.latest_cyclones) {
							list.li().append(self.add_cyclone_link(result.latest_cyclones[i]));
						}

						require(['jquery-ui/selectmenu'], function() {
							self.year_selector(result.base_dates);
							self.region_selector(result.regions);
						});

						for (i in result.cyclones) {
							self.widgets.cyclones.li().append(self.add_cyclone_link(result.cyclones[i]));
						}

						self.highlight_selected_link(selection.product, $('#product-list'));
						self.highlight_selected_link(selection.unique_id, self.widgets.cyclones);
					}
				});
			});
		},

		col2: function(o) {
			p_filter._selected = this._package;
		},

		col3: function(o) {
			App.prototype.col3.apply(this, arguments);

			this.col3.css = {};
			this.col3.size = {width: 400};

			this.widgets.content = o.el.div('.product-text');
		},

		scroll_to_col2: function() {
		},

		highlight_selected_link: function(id, list) {
			list.find('.active').removeClass('active');
			list.find('[href=' + id + ']').addClass('active');
		},

		cyclone_overview: function() {
			var self = this;

			Ajaxqueue({
				url: '/plots/cyclone-overview/',
				data: JSON.stringify({}),
				complete: function(result) {
					if(!result || !result.result){
						Bus().send('log-error', 'Failed to retrieve cyclones');
						return;
					}
					//Clear overview map
					$('.overview-map').empty();

					//FIXME: Cleanup the chart properly
					//       This is causing problems because a delayed plot request completion sometimes overwrites the overview
					if (typeof chart_handlers !== 'undefined') {
						$('.scrollable').hide();
						$('.chart-controls').hide();
						self.widgets.content.hide();
					}
					var overview = self.widgets.overview = $('<div>', {'class': 'overview-map'});
					var image = overview.div('#wmomap-wrapper').img({src: result.result.url, id: 'wmomap'});
					overview.div('#wmomap-text').html(result.meta.text);
					$('.column-2').append(overview);

					var selected_cyc = '';
					self.is_on_cyclone = false;

					image.on('mousemove', function(e){
						var imgPos = [
							image.offset().left,
							image.offset().top,
							image.offset().left + image.outerWidth(),
							image.offset().top + image.outerHeight()
						];
						var cyclones = result.meta.cyclones;
						var x = e.pageX-imgPos[0];
						var y = e.pageY-imgPos[1];
						for (var i in cyclones) {
							var x1 = cyclones[i].x1;
							var x2 = cyclones[i].x2;
							var y1 = cyclones[i].y1;
							var y2 = cyclones[i].y2;
							$('#coords').html(x + ':' + y);
							if ((x>x1 && x<x2) && (y>y1 && y<y2)) {
								selected_cyc = cyclones[i].unique_id;
								$(this).css('cursor', 'pointer');
								self.is_on_cyclone = true;
								$('<p class="cyclone-tooltip">').text(cyclones[i].name).appendTo('body');
								$('.cyclone-tooltip').css({top: e.pageY - 10, left: e.pageX + 10});
								return;
							}
							else {
								self.is_on_cyclone = false;
								$(this).css('cursor', 'default');
								$('.cyclone-tooltip').remove();
							}
						}
					});

					image.on('click', function(e) {
						console.log('overview.click: is on cyclone?', self.is_on_cyclone, 'selected:', selected_cyc);
						$('.cyclone-tooltip').remove();

						if(self.is_on_cyclone){
							self._scroll_into_view();
							controller.add_state().set_path(undefined, selection.product, $.extend({}, CH.state.params, {unique_id: selected_cyc}));
						}
					});
				},
				error: function(a, b) {
					console.log('cyclone overview data retrieval failed');
				}
			});
		},

		add_cyclone_link: function(cyc) {
			var self = this;

			var link_text = cyc.title + ' (' + cyc.tc_id + ') - Region ' + cyc.region;
			return $('<a>', {id: cyc.unique_id, href: cyc.unique_id}).text(link_text).click(function(e) {
				controller.add_state();

				self.highlight_selected_link(cyc.unique_id, self.widgets.cyclones.add(self.widgets.latest));

				self._scroll_into_view();

				controller.set_path(undefined, selection.product, $.extend({}, CH.state.params, {unique_id: cyc.unique_id}));

				return false;
			});
		},

		show_product: function(product) {
			controller.set_product(product.name(), product.title_());
		},

		show_product_by_name: function(product_name) {
			var product = products.product_by_name(product_name);
			product && this.show_product(product);
		},

		// TODO product title must come from the product
		_product_title: function(name, unique_id) {

			var product = products.product_by_name(name);
			if(product){
				var title = product.title_() + ' for ' + unique_id.split('_')[1];
				$.extend(chart_handlers.basic.instance, {title_ : function(){ return title; }});
				return title;
			}
		},

		_scroll_into_view: function() {
			var page = $('html, body');
			var chart_top = $('.chart-column').offset().top - 30;
			page.scrollTop() > chart_top + 60 && page.animate({scrollTop: chart_top}, 400);
		},

		year_selector: function(base_dates) {
			var self = this;
			var menu = year_filter
				.formSelect({options: base_dates})
				.selectmenu({})
				.selectmenu('instance');

			menu.menu.on('click', 'li', function(event) {
				var li = $(this);
				var item = {index: li.index(), label: li.text()};
				menu._select(item, event);

				var year = item.label;
				var update_list = ['region'];
				self.get_cyclone_list(selection.product, year, selection.region, update_list);
			});
		},

		region_selector: function(regions) {
			var self = this;
			var selected = region_filter.find('> select option:selected').val();
			region_filter.find('> select, > span').remove();

			var menu = region_filter
				.formSelect({options: $.map(regions, function(v){ return [[v.id, v.title]] })})
				.val(selected || 'All')
				.selectmenu({})
				.selectmenu('instance');

			// TODO after upgrade to jquery-ui 1.11 should use proper select callback
			menu.menu.on('click', 'li', function(event) {
				var li = $(this);
				var item = {index: li.index(), label: li.text()};
				menu._select(item, event);

				var region = $.findWhere(regions, {title: li.text()});
				var update_list = [];
				self.get_cyclone_list(selection.product, selection.year, region.id, update_list);
			});
		},

		get_cyclone_list: function(product, year, region, update_list) {
			var self = this;

			Ajaxqueue({
				url: '/plots/cyclone-list/',
				data: JSON.stringify({
					product: product,
					params: {
						date     : year,
						region   : region
					}
				}),
				success: function(result) {
					if(!!~$.inArray('region', update_list)){
						self.region_selector(result.regions);
					}

					self.widgets.cyclones.empty();
					for (var i in result.cyclones) {
						self.widgets.cyclones.li().append(self.add_cyclone_link(result.cyclones[i]));
					}
					selection.region = region;
					selection.year = year;
					self.highlight_selected_link(selection.product, $('#product_list'));
					self.highlight_selected_link(selection.unique_id,self.widgets.cyclones);
				},
				error: function(a, b) {
					console.log('cyclone data retrieval failed');
				}
			});
		},

		on_filter_change: function() {
		},

		on_product_select: function(product, params) {
			var self = this;

			App.prototype.on_product_select.apply(this, arguments);

			chart_handlers.state.product = product;

			handler = chart_handlers.basic;
			handler.host = this;

			// Clear overview map
			if (this.widgets.overview) {
				this.widgets.overview = this.widgets.overview.remove() && null;
				$('.scrollable').show();
				$('.chart-controls').show();
				this.widgets.content.empty().show();
			}

			if(handler !== chart_handlers.current){
				chart_handlers.current && chart_handlers.current.cleanup();
				handler.build(product, params || (chart_handlers.current && chart_handlers.current.state), {el: this.widgets.col3}, function() {

					CH.widgets.chart.messageBus().listen('size-changed', function(_, size) {
						self._on_resize(size);
					})

					$.each(['content'], function(i, v) {
						self.widgets[v] && self.widgets[v].appendTo(self.widgets.col3);
					});
				});
				chart_handlers.current = handler;
			}else{
				handler.update(product, params);
			}

			self.highlight_selected_link(product.name(), $('#product-list'));
		},

		// do not call directly, call _on_resize
		_do_layout: function() {
		},

		on_url_change: function(ctx) {
			var self = this;

			App.prototype.on_url_change.apply(this, arguments);

			selection.product = ctx.params.product || selection.product;
			selection.unique_id = ctx.query.unique_id || selection.unique_id;

			products.products().then(function(prods) {

				self.h1(self.is_landing_page
					? 'ECMWF tropical cyclones'
					: self._product_title(selection.product, selection.unique_id)
				);

				// FIXME page title should be set automatically by the page controller in set_path().
				setTimeout(function() {
					controller.set_title(self.is_landing_page ? 'Tropical cyclones' : selection.unique_id.split('_')[1] + ' ' + products.product_by_name(selection.product).title_());
				}, 0);
			});

			if (self.is_landing_page) {
				self.h1('ECMWF tropical cyclones');
				self.cyclone_overview();

				this.widgets.col2.show();
				this.widgets.col3.hide();
			}else{
				this.widgets.col2.hide();
				this.widgets.col3.show();
			}

			products.have.items.then(function(prods) {
				self.highlight_selected_link(selection.unique_id, self.widgets.cyclones.add(self.widgets.latest));
			});
		},

		set_page_title: function() {
			console.log('active chart: ', chart_handlers.state);
			return chart_handlers.state.product.title_();
		}
	});
});
