define(['jquery', 'mixins', 'observable', 'virtual-dom/h', 'virtual-dom/diff', 'virtual-dom/patch'], function($, mixins, Observable, h, diff, patch, virtualise) {

/*
 *  Render Tree
 *
 *  Ready:
 *     Users can set the optional 'ready' property to be a promise which will cause
 *     rendering to be delayed until the promise is resolved
 *
 *  TODO
 *    - perhaps remove the public render method in favour of watching state?
 *    - normally the render/view fn would have the model passed as first arg
 *      and 'handler' callback as 2nd arg. These help to reduce global state
 */
function RenderTree(args){
	var self = this

	$.extend(this, args)

	function set_render() {
		self.render = bunch(function() {
			self.render_now()
		}, 50, 300)
	}

	if(this.ready){
		// delay rendering until all promises resolved

		this.render = $.noop

		setTimeout(function(){
			$.when.apply(self, self.ready).then(function() {
				set_render()
				// TODO should probably use render_now here
				//      render_now can be made internal if all users who require imediate rendering use promises.
				self.render()
			})
		}, 0)
	}else{
		set_render()
	}

	this.render_now = function() {
		var self = this

		var new_tree = self.main.render()
		self.root = patch(self.root, diff(self.tree, new_tree));
		self.tree = new_tree;

		self.on.render.emit();
	}

	this.on = {
		render: new Observable
	}
}

RenderTree.prototype = $.extend({
}, mixins)

/*
 *   The bunch function bunches together expensive low-priority calls.
 *   It assumes that when a call is made, another one is likely to be made
 *   shortly after so the call is delayed until no more calls are received.
 *
 *   It is not proven that it gives much benefit over $.throttle, but one
 *   definite advantage is that $.throttle always executes the first call
 *   immediately and this is sometimes a problem
                                                                              // TODO $.debounce does not run imediately - check
 *   (it means that the precise order of callback fns is sometimes critical)
 *
 *   TODO
 *    - perhaps merge with mixins.throttle (is the same but has tmax added)
 *    - should it prioritise the first call (for each window) ?
 */
function bunch(fn, t, tmax) {
	t = t || 100;
	var throttle_timer;
	var max_timer;

	return function() {
		var self = this;

		throttle_timer && clearTimeout(throttle_timer);
		throttle_timer = setTimeout(function() {
			throttle_timer = null;
			max_timer = clearTimeout(max_timer);
			fn.apply(self);
		}, t);

		max_timer = max_timer || setTimeout(function() {
			throttle_timer = clearTimeout(throttle_timer);
			max_timer = null;
			fn.apply(self);
		}, tmax)
	};
}

return RenderTree

})
