/*
 *  Handling of debug data returned from ECMWF backend services.
 */
define(['jquery', 'ajax_queue'], function($, Ajaxqueue) {
	Ajaxqueue.add_callback(function(req, result) {

		// FIXME remove meta (basic charts backend uses incompatible format)
		if(!(result.metadata || result.meta)) return;

		var tree = {
			task: 'browser',
			kids: [result.metadata],
			timers: {},
			stop: (new Date()).getTime() / 1000.0,
			start: req.start
		};

		// temporary workaround for backend inconsistencies.
		if(!result.metadata){
			console.warn('bad metadata');
			tree.kids = [result.meta];
		}

		var total = 0;
		var list = ['queued', 'ajax', 'image'];
		for (var i in list) {
			var n = list[i];
			if (req.timers[n] !== undefined) {
				tree.timers[n] = req.timers[n];
				total += req.timers[n];
			}
			tree.total = total;
			req.timers.total = total;
		}
		tree.timers.elapsed = total;

		if (req.call_tree) {
			try {
				req.call_tree(tree, result.tracker);
			} catch (e) {
				req.exception = e;
				console.warn(e);
			}
		}

		if (req.call_times && !req.from_cache) {
			var timers = {};

			function collect(obj, path) {
				path.push(obj.task);

				$.each(obj.timers, function(k, v) {
					path.push(k);
					timers[path.join('.')] = v;
					path.pop();
				});

				$.each(obj.kids, function(_, kid) {
					collect(kid, path);
				});
				path.pop();
			}

			collect(tree, []);
			req.call_times(timers, result.tracker);
		}
	});
});
