/*
 *   Next-gen replacement for thumbnail-dialogue/index.js
 *
 *   To re-use the dialogue after closing, users must retain a reference and call the open() method
 *
 *   TODO
 *    - scrollbar size not updated after add/remove user_products filter
 *    - group headings not being shown
 *    - when opening dialogue a 2nd time, need to re-run the filter
 */
define(['jquery', 'bus', 'utils', 'settings', 'mixins', 'observable', 'vgrid', 'forecaster/products', 'forecaster/projections', 'forecaster/layers', 'forecaster/meteograms', 'vgrid/filter', 'vgrid/filters/text', 'dialogue', 'virtual-dom/h', 'virtual-dom/diff', 'virtual-dom/patch', 'vdom-virtualize', 'render-tree'], function($, Bus, Utils, Settings, mixins, Observable, VGrid, Collections, projections, layers, meteograms, Filter, TextFilter, Dialogue, h, diff, patch, virtualise, RenderTree) {

var setting = 'thumbnail_dialogue_hide_items_you_already_have'

var UserProducts = function(o){
	var self = this
	this.collection = o.collection
	this.products = []

	this.filter = new Filter({
		name: 'user-products',
		fn: function(item){
			// We determine if an ECMWF product is in the user product list by comparing title.
			// We cannot use name, because the user products have the user name prepended
			var title = item.title_()
			for(var i in self.products.data){
				if(self.products.data[i].title === title) return false;
			}
			return true;
		},
		deactivate: function() {
			Settings.set(setting, false)
			self.collection.remove_filter(this.name);
		},
		status: function(){ return self.products.data.length + ' user products'; },
		text: null
	})
}

function ThumbnailDialogue (options) {
	var grid, user_products, self = this;

	var o = $.extend({
		kind: 'product',
		hide_user_items: true,
		bus: Bus()
	}, options);

	this.bus = o.bus;

	var el = $('body').div()

	var inner_render = function(){ return h('div') }

	var products = (o.kind === 'product') ? Collections.products
		: (o.kind === 'projection' ? projections
			: (o.kind === 'layer' ? layers : meteograms));

	$.Deferred(function(df){
		if(o.hide_user_items){
			user_products = new UserProducts({collection: products})
			require(['user_catalogue'], function(UserCatalogueStore) {
				user_products.products = new UserCatalogueStore()
					.create(o)
					.get_catalogue(df.resolve)
			})
		} else df.resolve();
	}).then(function(){
		self.vdom = new RenderTree({
			main: {
				render: function(){
					var body = h('.modal-body', [
						h('p', ['Select from these ECMWF ' + Utils.ucfirst(o.kind) + 's to add to your personal list, or double click to view']),
						inner_render(),
						user_products
							? h('.check-options', [
								h('input', {type: 'checkbox', checked: Settings.get(setting) && 'checked', onclick: function(e){
									var checked = e.target.checked
									if(checked)
										products.add_filters([user_products.filter]);
									else
										products.remove_filter(user_products.filter.name);
									Settings.set(setting, checked);
								}}),
								h('label', ['Hide ' + Utils.ucfirst(o.kind) + 's you already have'])
							])
						: null
					])
					return self.dialogue.render(body)
				}
			},
			root: el[0],
			tree: virtualise(self.dom_tree_root = el[0])
		})

		var height = $(window).height() - 80;
		var width = Math.min(750, $(window).width() - 80)

		// get the grid height for a given total dialogue box height
		function grid_height(dialogue_height) {
			return Math.round(
				dialogue_height
				- el.find('> .modal-header').outerHeight()
				- el.find('> .modal-body').outerHeight() + (el.find('.catalogue-grid').length ? el.find('.catalogue-grid').outerHeight() : 0)
				- 16 // TODO
			);
		}

		var id = o.kind + '-select'

		self.dialogue = new Dialogue({
			renderer: self.vdom,
			id: id,
			title: Utils.ucfirst(o.kind) + ' select',
			size: {width: width, height: height},
			position: {top: 40, left: $(window).width() / 2 - width / 2},
			on_resize: function(size) {
				grid.resize({width: size.width - 50/* TODO */, height: grid_height(size.height)})
				self.vdom.render()
			}
		})

		$('#' + id).on('render', self.vdom.bind(self.vdom.render))

		products.subscribe(self.vdom.bind(self.vdom.render))
		products.set_list()

		products.on.items.value = 1, products.on.items.options.state = true; // TODO observable does not have a value?
		products.on.items.subscribe(self, function() {
			products.filter(); // TODO should be done inside the list
		});

		// TODO this duplicates fn in grid.js - should be done in the list, not in the grid
		$.map(products.filters, function(f){
			$.isFunction(f.fn) && f.events && f.events.bind('changed', function() {
				products.filter();
				self.vdom.render();
			});
		});

		self.vdom.on.render.once(function(){
			grid = new VGrid({
				renderer: self.vdom,
				products: products,
				width: el.find('.modal-body').width(),
				height: grid_height(height),
				on_click: function(selection){
					console.log('ThumbnailDialogue2: closing... selection=', selection);
					self.bus.send(o.kind + '-dialogue-selection', {names: [selection.name()]});
				},
				on_dblclick: function(selection){
					console.log('ThumbnailDialogue2: dblclick: selection=', selection);
					self.dialogue.close()
					self.bus.send(o.kind + '-thumbnail-click', {name: selection.name()});
				}
			})

			products.have.filtering.then(function() {
				inner_render = grid.bind(grid.render);
			})
		})

		self.vdom.render()
	})

	products.add_filters([new TextFilter()]);
	o.hide_user_items && Settings.get(setting) && products.add_filters([user_products.filter]);
}

ThumbnailDialogue.prototype = {
	open: function() {
		this.dialogue.open()
		return this
	}
}

return ThumbnailDialogue
})
