/*jslint white: false, nomen: false, onevar: false, forin: false, browser: true, devel: true, immed: false */
/*global ECMWF: true, console: true, jQuery: true */

/*
 *  A dialogue box showing a list of Products. Multiple products can be selected. A message is sent upon submit.
 *
 *  TODO: we do not currently know which products the user has, and whether they are modified.
 *  A "user has it property" may possibly be added to the ajax product list.
 *  Or maybe do 2 requests to server - one for all products, one for products the user doesn't have.
 */
define(['config', 'jquery', 'bus', 'settings', 'jquery-ui/widget', 'jquery-ui/resizable', 'jquery-ui/dialog', 'filterable_thumbnail_list', './thumbnail_dialogue.css'], function(ECMWF, $, Bus, Settings) {

//var catalogue_caching_disabled = true;

$.widget('ui.thumbnailDialogue', $.ui.filterableImageList, {

	options: {
		kind: 'product',
		text: null,
		hide_user_items: false,
		modal: false,
		messageBus: Bus()
	},

	_init: function() {
		var self = this, o = this.options, div;

		if((div = this.element.data('_thumbnailDialogue'))){
			div.dialog('open');
			self
				._update_visibility()
				._update_descriptions();
			return;
		}

		function on_get_hidelist(_list){

			var list = [];
			for(var i in _list){
				var item = _list[i];
				var p = self._find_item_by_title(item.title);
				if(p > -1) list.push(self.list[p]);
				/*
				else{
					console.log('on_get_hidelist: not found in ECMWF catalogue: ' + item.title + ' (' + item.name + ')');
				}
				*/
			}

			self.hidelist = $(list);

			self.hidelist.each(function(){
				self._filter_item(this);
			});

			setTimeout(function(){ //wait for animations to finish
				self
					._update_visibility()
					._update_descriptions();
			}, 1000);
		}

		o.user_item_added = function(item){
			if(self.userlist){
				self.userlist.get_catalogue(function(list){
					var found = false;
					$(list).each(function(){
						if(this.title === item.title) found = true
					});
					if(!found) console.warn('on_user_item_added: ' + o.kind + ' not found: "' + item.title + '".');
					on_get_hidelist(list);
				});
			}
		}

		o.user_item_removed = function(item_title){
			if(self.userlist){
				self.userlist.get_catalogue(function(list){
					on_get_hidelist(list);
				});

				var p = self._find_item_by_title(item_title);
				if(p > -1){
					var item = self.list[p];
					self
						._filter_item(item)
						._on_list_visibility_change()
						._update_descriptions();
				}
				else console.log('on_user_item_removed: not found in ECMWF catalogue: "' + item_title + '"');
			}
		}

		o.on_got_list = function(){
			header.find('> div:first').html('Showing ' + ECMWF.ucfirst(o.kind) + 's...');
		}

require(['user_catalogue'], function(UserCatalogueStore) {
		self.userlist = new UserCatalogueStore().create(o);
});

		var header = self.container.div('.header').prependTo(self.container);
		var footer = self.container.div('.footer');
		self.hidelist = [];

		self._add_filter(function(item){
			return (!self.regex || item.title.match(self.regex) || item.keywords.match(self.regex));
		});

		self._add_filter(function(item){
			if(Settings.get('thumbnail_dialogue_hide_items_you_already_have', true)){
				for(var i=0;i<self.hidelist.length;i++){
					if(self.hidelist[i].title === item.title) return false;
				}
			}
			return true;
		});

		show_loading_header();

		var on_resize = $.throttle(500, function(){
			self.__resize();

			if(self.container.hasClass('containr')){
				self.container.trigger('layout', {width: -1, height: self.container.height()});
				self._on_list_visibility_change();
			}
		});

		div = self.container
			.dialog({
				title: 'ECMWF ' + ECMWF.ucfirst(o.kind) + 's',
				autoOpen: false,
				dialogClass: 'catalogue-dialogue' + (o.modal ? '' : ' z-7') + ' ' + o.kind + '-dialogue',
				width: 480,
				height: 540,
				modal: o.modal,
				open: function(){
					setTimeout(on_resize, 1000);
				}
			})
			.dialog('open');
		var dialogue = $('.' + o.kind + '-dialogue');

		dialogue
			.on('resize', function(e){
				if(e.originalEvent && (!ECMWF.browser.msie || e.button)){
					on_resize();
				}
			});

		this.element.on('hideAnimationFinished', function(){
			self._apply_group_labels();
		});

		this.element.data('_thumbnailDialogue', div);

		self.container.container({
			layout : 'vertical',
			expandable : self.list_container,
			root : self.container
		});

		var thumbnail_args = {
			success: function(list){
				header.empty();
				footer.empty();

				self._apply_group_labels();

				if(o.text) header.p().html(o.text);

				self._add_filter_input(
					header.div(),
					function(){
						self._on_list_visibility_change();
					}
				);

				//hide items that are already used.
				//note: the current method of determining what is 'used' is somewhat provisional.
				if(o.hide_user_items){
					self.userlist.get_catalogue(on_get_hidelist);

					footer
						.checkbox({
							label: 'Hide ' + ECMWF.ucfirst(o.kind) + 's you already have',
							user_setting: 'thumbnail_dialogue_hide_items_you_already_have',
							changed: function(checked){
								self.userlist.get_catalogue(on_get_hidelist);
							}
						});
				}

				o.on_update_item = function(item){
				}

				o.on_show_item = function(item){
				}

				o.on_scroll = function(){
					/*
					for(var i in self._visible_items){
						var li = self._visible_items[i].li;
					}
					*/
				}
			},

			error: function () {
				header.empty().p('.failed').html('Getting ' + ECMWF.ucfirst(o.kind) + 's failed. ')
					.a({href: '#retry', 'class': 'text-link'}).text('retry').click(function(){
						show_loading_header();
						self.get_thumbnails(thumbnail_args);
						return false;
					});

			},

			dblclick: function(item){
				var action;
				if((action = o.actions[0])){
					div.dialog('close');
					action.on_click(item.name);
					return false;
				}
			}
		};

		self.get_thumbnails(thumbnail_args);

		function show_loading_header(){
			header.empty().div('.loader-status-active').html('Getting ' + ECMWF.ucfirst(o.kind) + 's...');
		}
	},

	destroy: function(){
		//these dialogues should not be destroyed. They are designed to be hidden and then later re-used.
		console.log('ThumbnailDialogue.destroy: widget is designed to be hidden and re-used. ignoring destroy.');

		//this.element.removeData('filterableThumbnailList');
	},

	// update the view for the whole list
	_on_list_visibility_change: function() {
		var self = this;

		return self
			//._cancel_requests() // now done by the list - check whether we still need this here.
			._apply_group_labels();
	},

	_apply_group_labels: function() {
		var self = this;

		var prev_group = '';
		self.list.each(function(){
			var li = this.li;
			if(this.match !== false){
				var group = this.group;
				if(group){
					group = group.substring(group.indexOf(' ') + 1);
					if(group !== prev_group){
						if(!li.find('h3').length)
							li
								.addClass(group.length < 46 ? 'group-heading' : 'group-heading-long')
								.h3({css: {position: 'absolute', width: '100%'}}).html(group).prependTo(li);
					} else {
						li
							.removeClass('group_heading group-heading-long')
							.find('h3').remove();
					}

					prev_group = group;
				}
			}
		});
		return this;
	}
});

});

