/*
 *   Eccharts Forecaster widget for displaying thumbnails for user lists of products and projections.
 */
define(['config', 'jquery', 'ajax_queue', 'bus', 'settings', 'utils', 'markup', 'jquery-ui/widget', 'thumbnail-tools', 'jquery-tools/scrollable.navigator', './thumbnail_list.css'], function(ECMWF, $, Ajaxqueue, Bus, Settings, Utils, Markup) {

    var count = 0;

    $.widget('ui.thumbnailList', {
        options: {
            kind: 'product',
            package: 'eccharts',
            content_url: null,
            visible_items: 5,
            circular: true, // Does not work yet
            orientation: 'horizontal',
            margin: 5,
            item_width: 128,
            item_height: 128 / 2 + 46,

            button_size: 30,
            button_margin: 8,

            circle_size: 8,
            circle_margin: 3,

            priority: 'low',
            show_names: true,
            click: $.noop,
            messageBus: Bus(),
            retry_min_later: 5, // 5 minute
            swap: false,
            user_only: true,

            show_add_button: null
        },

        _init: function() {
            var self = this,
                o = this.options;
            count++;

            self.name = 'thumbnail_list_' + count;

            self.element.addClass('thumbnail-list');
            self._build();

            o.messageBus.listen(o.kind + '-dialogue-selection', function(_, msg) {
                var url = '/backend/add-to-user-space/';
                msg.type = o.kind;

                Ajaxqueue({
                    url: url,
                    priority: 'very-high',
                    json: msg,
                    success: function(result) {
                        $.each(result, function(_, item) {
                            o.messageBus.send('new-' + o.kind + '-added-to-catalogue', item)
                        })
                    }
                });
            });

            o.messageBus.listen('new-' + o.kind + '-added-to-catalogue', function(_, item) {

                if (o.user_only) {
                    self.items.unshift(item);
                    self._add_item(item, true);
                    self._load_thumbnails(item.item);
                    self.api.addItem();
                    self._update_navi();
                }

                $.logInfo(ECMWF.ucfirst(o.kind) + ' added.');
            });

            o.messageBus.listen(o.kind + '-removed-from-catalogue', function(_, msg) {
                var items = [];
                $.each(self.items, function(_, item) {
                    if (item.name === msg.name) {
                        item.item.remove();
                        $.logInfo(ECMWF.ucfirst(o.kind) + ' removed.');
                    } else {
                        items.push(item);
                    }
                });
                self.items = items;

                self._update_navi();
                self._load_thumbnails(self.api.getVisibleItems());

                self.element.find('.scrollable').trigger('removed', msg);
            });

            o.messageBus.listen(o.kind + '-modified-info', function(_, msg) {
                $.each(self.items, function(_, item) {
                    console.log(item.name, msg.name);
                    if (item.name === msg.name) {
                        item.title = msg.title;
                        item.description = msg.description;
                        item.keywords = msg.keywords;
                        if (item.label) item.label.text(msg.title);
                        item.img.attr(self._get_tooltip_attributes(item));
                    }
                });
            });
        },

        _build: function() {
            var self = this, o = this.options, api, container;

            var width = o.visible_items * (o.item_width + 2 * o.margin) + 2 * o.button_margin + 2 * o.button_size;

            self.element.css({
                width: width,
                height: o.item_height + 20
            });

            var thumbs = [];

            if (o.show_add_button && o.user_only)
                self.element
                    .div({className: 'more', css: 'margin-right:10px'})
                    .click(function() {
                        o.show_add_button();
                        return false;
                    })
                    .a({className: 'text-link', href: '#add_more_' + o.kind + 's'})
                    .text('More ' + ECMWF.ucfirst(o.kind) + 's ...').parent()
                    .div()
                    .icon2({css_class: 'small-icon small-icon-circle-plus'});

            var filter = self.element.span('.thumbnail-list-filter');

            var input = filter.input({placeholder: 'search'}/*, ').attr('type', 'text'*/).keyup(function() {
                var txt = $.trim(input.val());
                var re = new RegExp(txt, "i");
                if (api) {
                    api.begin();

                    $.each(thumbs, function(i, item) {
                        if (item.title.match(re) || item.keywords.match(re)) {
                            item.item.show();
                        } else {
                            item.item.hide();
                        }
                    });

                    self._update_navi();
                    self._load_thumbnails(api.getVisibleItems());
                }
            });

            if (o.kind === 'product') self.element.div().addClass('radio').radio({
                buttons: ['use_map_projection', 'use_product_projection'],
                labels: ['Use map projection', 'Use product projection'],
                user_setting: 'use_product_projection'
            });

            if (o.swap) {
                var s = $("<span/>").addClass("swapper").appendTo(filter);
                var x = o.user_only ? "ECMWF" : "your";
                $("<a>").appendTo(s).attr("href", "#").addClass("text-link").click(function() {
                    self.swap();
                }).text("View " + x + " " + ECMWF.ucfirst(o.kind) + "s Catalogue");
            }

            Ajaxqueue({
                url: '/backend/catalogue-items/',
                json: {
                    type: o.kind,
                    package: o.package,
                    user_items: o.user_only,
                    standard_items: !o.user_only,
                    allow_experimental: ECMWF.request.allow_experimental,
                    access: ECMWF.request.access,
                    keywords: true,
                    descriptions: true
                },
                priority: "normal",
                //status   : ECMWF.ajax_loader(elem),
                cache: 30,
                // 30 seconds, will be overriden by backend
                tag: self.name,

                success: function(list) {
                    //console.log("list-->", list);
                    thumbs = self.items = list;

                    self.navi = self.element
                        .div({className: 'navi-wrapper'})
                        .div({className: 'navi'});
                    self._update_navi();

                    self.left = $("<a class='prevPage browse left'/>").appendTo(self.element).click(function() {
                        api.move(-(o.visible_items - 1));
                    });
                    var scroll = self.scroll = $("<div class='scrollable'/>").css({
                        width: o.visible_items * (o.item_width + 2 * o.margin),
                        height: o.item_height,
                        position: "relative",
                        overflow: "hidden"
                    }).appendTo(self.element);

                    var div = $("<div class='items'/>").css({
                        width: "20000em",
                        position: "absolute",
                        clear: "both"
                    }).appendTo(scroll);

                    self.div = container = div;

                    $.each(list, function(i, item) {
                        self._add_item(item)
                    });

                    self.right = $("<a class='nextPage browse right'/>").appendTo(self.element).click(function() {
                        api.move(o.visible_items - 1);
                    });

                    scroll.scrollable({
                        clickable: false,
                        keyboard: false,
                        size: o.visible_items - 1
                    }).navigator();
                    api = scroll.data('scrollable');
                    $.extend(api, {
                        getItems: function() {
                            return div.find('> .thumbnail');
                        },
                        getVisibleItems: function() {
                            return api.getItems().slice(api.getIndex(), api.getIndex() + o.visible_items);
                        }
                    })
                    //.mousewheel({api:true})
                    self._load_thumbnails(api.getVisibleItems());

                    // Set the item filter after, so we can created invisible thumbnail_list
                    api.getConf().item = ":visible";

                    api.onAddItem(function(item) {
                        self._load_thumbnails($(item));
                    });
                    api.onSeek(function() {
                        self._load_thumbnails(api.getVisibleItems());
                    });

                    self.api = api;
                }
            });
        },

        swap: function() {
            var self = this, o = this.options;

            o.user_only = !o.user_only;
            Ajaxqueue.cancelByTag(self.name);

            self.element.empty();
            self._build();
            self.element.parent().removeClass("user-only-true");
            self.element.parent().removeClass("user-only-false");
            self.element.parent().addClass("user-only-" + o.user_only);
        },

        _get_tooltip_attributes: function(item) {
            return {
                alt: item.title,
                title: Markup.toHTML(item.description).replace(replaceTags, ' ').replace(/\s*&nbsp;\s*/g, ' ')
            }
        },

        _add_item: function(item, first) {
            var self = this,
                o = this.options;

            var img = $('<img>').attr(
                $.extend({
                    //src: ECMWF.options.files["default_" + o.kind + "_thumbnail"],
                    'class': 'thumbnail-img'
                }, self._get_tooltip_attributes(item)));

            item.img = img;

            if (o.show_names) {
                var d = $("<div>", {
                    'class': 'thumbnail'
                });
                img.appendTo(d);

                d.thumbnailTools();

                if (o.user_only) {
                    d.thumbnailTools('add', [
                        ['ui-icon-trash',
                            function() {
                                o.messageBus.send(o.kind + "-thumbnail-delete", item);
                            }, 'Delete'
                        ],
                        ['ui-icon-pencil',
                            function() {
                                o.messageBus.send(o.kind + "-thumbnail-edit", item);
                                d.thumbnailTools('hide');
                            }, 'Edit'
                        ]
                        /*,
                        ['ui-icon-person', function(){
                                o.messageBus.send('share-item', { type : o.kind, item : item });
                                d.thumbnailTools('hide');
                            }, 'Share']
                        */
                    ]);
                } else {
                    d.thumbnailTools('add', 'ui-icon-circle-plus', function() {
                        o.messageBus.send(o.kind + "-dialogue-selection", {
                            names: [item.name]
                        });
                    });
                }

                item.label = d.span({
                    'class': 'thumbnail-label'
                }).text(item.title);
                d.data("thumbnail", {
                    name: item.name
                });
                item.item = d;
            } else {
                img.addClass('thumbnail');
                img.data('thumbnail', {name: item.name});
                item.item = img;
            }

            if (first) {
                item.item.prependTo(self.div);
            } else {
                item.item.appendTo(self.div);
            }

            item.item.css({'float': 'left', margin: o.margin, width: o.item_width});

            item.img.click(function() {
                self.element.trigger("select", item.name);
                o.click(item.name);
                o.messageBus.send(o.kind + "-thumbnail-click", {
                    name: item.name,
                    use_product_projection: (Settings.get('use_product_projection') === 'use_product_projection')
                });
                setTimeout(function() {
                    self._load_thumbnails(item.item, 'very-low');
                }, 10);
            });
        },

        _update_navi: function() {
            var self = this,
                o = this.options;
            if (!this.items) return;

            // we now let the browser do the positioning
            //var w = Math.floor((this.items.length + o.visible_items - 1) / o.visible_items) * (o.circle_size + 2 * o.circle_margin);
            self.navi.css({
                height: o.circle_size + 2 * o.circle_margin
                //width: w,
                //marginLeft: (o.visible_items * (o.item_width + 2 * o.margin) - w) / 2 + o.button_size + o.button_margin
            });
        },

        _load_thumbnails: function(list, priority) {
            var self = this,
                o = this.options;

            priority = priority || o.priority;

            list.each(function() {
                var el = $(this);
                var data = el.data("thumbnail");
                if (data && !data.done) {

                    var name = [o.kind, 'thumbnail'].join('_');
                    var url = ECMWF.options.ajax.backend[name];

                    var img = el.find('img');

                    Ajaxqueue({
                        url: url.replace('NAME', data.name),
                        priority: priority,
                        cache: 30, // 30 seconds, will be overriden by backend
                        tag: self.name,
                        //name       : ["thumbnail_list", o.kind, data.name, count].join("_"),
                        success: function(json) {
                            if (!json) return;

                            img.css({'height': 'auto', 'width': 'auto'}).attr('src', json.url);
                            data.done = true;
                        },
                        error: function() {
                            img.addClass('failed ui-icon ui-icon-alert').attr('src', Utils.blank_gif);

                            // Post a new request to be handled in 5 mins
                            setTimeout(function() {
                                self._load_thumbnails(el, 'very-low');
                            }, 1000 * 60 * o.retry_min_later);
                        }
                    });
                }
            });
        },

        destroy: function() {
            var self = this;

            self.element.removeClass("thumbnail-list");

            // Cleanup CSS and event handling
            // self.element.removeClass("....").unbind("....")
            // Destroy created DOM elements
            self.element.empty();

            return this;
        },

        setSize: function(size) {
            var self = this,
                o = this.options;

            // TODO: Something more subtle
            var n = Math.floor((size.width - (o.circle_size + 2 * o.circle_margin) * 2 - 14) / (o.item_width + 2 * o.margin));
            if (n !== o.visible_items) {
                o.visible_items = n;
                self._resize();
            }
        },

        _resize: function() {
            var self = this,
                o = this.options;

            if (!self.api || !self.items) {
                //
                setTimeout(function() {
                    self._resize();
                }, 100);
                return;
            }

            var width = o.visible_items * (o.item_width + 2 * o.margin) + 2 * o.button_margin + 2 * o.button_size;

            self.element.css({
                width: width,
                height: o.item_height + 20
            });


            self._update_navi();

            self.scroll.css({
                width: o.visible_items * (o.item_width + 2 * o.margin),
                height: o.item_height
            });

            self.api.getConf().size = o.visible_items;
            self.api.reload();
            self._load_thumbnails(self.api.getVisibleItems());

        }
    });

    //$.ui.thumbnailList.eventPrefix = "thumbnailList";
    var replaceTags = new RegExp(/\s*<\/?[^>]+>\s*/gi);
});
