define(['jquery', './thumbnail_tools.css'], function($) {

$.widget('ui.thumbnailTools', {

	options: {
		selector: '.thumbnail-img',
		enable: true,
		always_on: false,
		on_create: $.noop // called after the buttons are created
	},

	_init: function() {
		var self = this, o = this.options;
		self.enabled = o.enable;
		self.in_tools = false;
		self.in_image = false;
		self._items = []

		var timeout;
		self.tools = this.element.div('.tools')
			.css({position: 'absolute'})
			.mouseenter(function() {
				self.in_tools = true;
			})
			.mouseleave(function() {
				self.in_tools = false;
				if (!self.in_image && !o.always_on)
					timeout = setTimeout(function() {
						self.hide();
					}, 250);
			});

		this.img = this.element.find(o.selector)
			.hoverIntent(
				function mouse_in() {
					self.in_image = true;
					if (self.enabled) {
						self.show();
						if (timeout) {
							clearTimeout(timeout);
							timeout = null;
						}
					}
				},
				function out() {
					self.in_image = false;
					if (!self.in_tools && !o.always_on) {
						timeout = setTimeout(function() {
							self.hide();
						}, 250);
					}
				}
			);

		self[o.always_on && self.enabled ? 'show' : 'hide']();
	},

	destroy: function() {
		this.tools.removeClass('tools');

		if (this.timeout) {
			clearTimeout(this.timeout);
			this.timeout = null;
		}
	},

	_make_items: function() {
		var self = this, o = self.options;

		$.each(this._items, function(i, j) {
			var icon    = j[0];
			var clicked = j[1];
			var help    = j[2];

			var button = self.tools.div()
				.button({icons: {primary: icon}})
				.click(function() {
					self.tools.find('span.' + icon).trigger('mouseleave'); // close tooltip
					clicked();
					return false;
				});

			help && button.find('span.ui-icon')
				.tipsy({
					title: function() { return help; },
					fade: true,
					gravity:  $.fn.tipsy.autoBounds(40, 's'),
					delayIn: 500
				});
		});

		o.on_create();
	},

	add: function(_icon, _clicked, _help) {
		if(this._items.length)
			this._items.push([_icon, _clicked, _help]);
		else
			this._items = Array.isArray(_icon) ? _icon : [[_icon, _clicked, _help]];
		this.options.always_on && this.enabled && this.show();
	},

	show: function() {
		var self = this;
		var pos = this.img.position();

		self.tools.find('*').length || this._make_items()

		this.tools
			.css({ left: pos.left })
			.stop().delay(100)
			.show();

		if (self.timeout) {
			clearTimeout(self.timeout);
			self.timeout = null;
		}

		//disabled for now - TODO needs to be re-enabled, we do get stuck tools, eg when dialogue opens.
		/*
		self.timeout = setTimeout(function(){
			self.tools.hide();
			self.in_tools = false;
		}, 10000);
		*/
	},

	enable: function(enable) {
		var self = this, o = this.options;

		self.enabled = (enable !== false);
		self.enabled && o.always_on && self.show();
		!self.enabled && self.hide();
	},

	always_on: function(enable) {
		var self = this, o = this.options;

		if ((o.always_on = (enable !== false)) && self.enabled) self.show();
	},

	hide: function() {
		//if (!this.in_tools) {
		this.tools.stop().delay(1000).hide();
		//}
	}
});

});
