/*
 *  Time slider widget using HTML Canvas.
 *
 *  Messages: this widget does not send or receive messages. Bus communication is done via callbacks in the host widget.
 */
import ECMWF from 'config'
import $ from 'jquery'
import 'jquery-ui/mouse'
import 'jquery-ui/core'
import 'time'
import 'date-format'
import 'throttle-debounce'
import 'focusable'


$.widget('ui.canvasSlider', $.ui.mouse, {
    dates: null,
    current_date: null,
    canvas: null,
    cursor: null,
    flag_width: 100,

    options: {
        height          : 20,
        border          : 0,
        trail           : false,
        big_handle      : false,
        on_select       : $.noop,
        range_selection : {
            enabled     : false,
            on_select   : $.noop
        }
    },

    _init: function() {
        var self = this, o = this.options;
        this._mouseInit();
        this.element.focusable();

        self.dimensions = {};
        self.stepBar = null;
        self.drag = null;
        self.range = {};

        setTimeout(function ready_for_canvas(){
            if(ECMWF.browser.msie && typeof G_vmlCanvasManager != 'undefined'){
                var el = self.canvas = document.createElement('canvas');
                G_vmlCanvasManager.initElement(el);
                self.$canvas = $(el).appendTo(self.element);
                self.$canvas.css({height: o.height, width: '100%'});
                self.$canvas.addClass('canvas-slider');
            }else{
                self.$canvas = self.element.canvas({'class': 'canvas-slider', css: {overflow: 'hidden', height: o.height}});
                self.canvas = self.$canvas[0];
            }

            self.ctx = self.canvas.getContext('2d');
            self.canvas.width = self._target_width();
            self.canvas.height = o.height;
            self.ctx.lineWidth = '2';
            self.ctx.strokeStyle = 'rgb(255, 255, 255)';
            self.ctx.font = ECMWF.browser.msie ? '9px Arial' : '6pt sans-serif';

            self.stepBar = new StepBar(self);

            self.loadComplete = true;

            if(self.dates){
                self.stepBar.update();
                self._draw();
            }else
                self._draw_welcome();

            self.$canvas
                .click(on_click)
                .mousemove(on_mouse_move)
                .mouseout(on_mouse_out);
        }, 10);

        var resize = $.throttle(300, function() {
            if(self.stepBar){
                self._clear();
                self.stepBar.update();
                self._draw();
            }
        })

        self.element.on('container', function(e, size){
            if(size.width !== self.dimensions.element_width){
                self.dimensions = {
                    element_width: size.width/*,
                    paddingLeft: parseInt(self.element.css('padding-left'), 10),
                    paddingRight: parseInt(self.element.css('padding-right'), 10)
                    */
                }
                resize()
            }
        });

        if (ECMWF.browser.msie) {
            this.element.mousedown(function (e) {
                if (e.ctrlKey || e.shiftKey) {
                    this.onselectstart = function () { return false; };
                    var me = this;
                    window.setTimeout(function () { me.onselectstart = null; }, 0);
                }
            });
        }

        function on_click(event){
            if(self.drag){
                self.drag = null;
                return;
            }

            if(self.dates && !self.options.disabled){
                self._set_cursor(event.pageX - Math.floor(self.$canvas.offset().left));
                var date = self.cursor.date;

                if(o.range_selection.enabled && event.shiftKey){
                    if(date){
                        if(!self.animation_start_date){
                            if(!self.animation_end_date || date < self.animation_end_date){
	                            self.__set_animation_start_date(date);
                            }else if(date == self.animation_end_date){
	                            self.__set_animation_end_date(null);
                            }else{
	                            self.__set_animation_end_date(date);
                            }
                        }else{
                            if(date < self.animation_start_date){
                                //before start.
                                if(self.animation_end_date){
                                    self.__set_animation_start_date(date);
                                }else{
                                    var d = self.animation_start_date;
                                    self.__set_animation_start_date(date);
                                    self.__set_animation_end_date(d);
                                }
                            }else if(date == self.animation_start_date){
                                self.__set_animation_start_date(null);
                            }else if(date < self.animation_end_date){
                                //in between start and end
                                var x  = self._date_to_x(date);
                                var x1 = self._date_to_x(self.animation_start_date);
                                var x2 = self._date_to_x(self.animation_end_date);
                                (x < (x1 + x2) / 2) && self.__set_animation_start_date(date) || self.__set_animation_end_date(date);
                            }else if(date == self.animation_end_date){
                                self.__set_animation_end_date(null);
                            }else{
                                //to the right
                                self.__set_animation_end_date(date);
                            }
                        }
                        self._draw();
                    }
                    return false;
                }else{

                    if(date){
                        if(!self.current_date || self.current_date.getTime() !== date.getTime()){
                            self.current_date = date;
                            self.options.on_select(date);
                        }
                    }
                }
            }
        }

        function on_mouse_move(event){
            if(!self || !self.dates || self.options.disabled || self.drag) return;

            self._set_cursor(event.pageX - Math.floor(self.$canvas.offset().left));
            if(self._get_cursor_pos_snapped() !== self.cursor.x_snapped){
                self._draw();
            }

            var feature = self._pick(event);
            feature && self.element.css({cursor: 'move'}) || self.element.css({cursor: 'default'});
        }

        function on_mouse_out(event){
            if((!self.cursor) && (!self.dates || self.options.disabled)) return;

            self.cursor = null;
            self._draw();
        }

        self.__throttled_draw = $.throttle(200, function(){
            setTimeout(function(){
                self._clear();
                self.stepBar && self.stepBar.draw();
            }, 1);
        });
    },

    _destroy: function(){
        this._mouseDestroy();
        return this;
    },

    setDates: function(dates){
        var self = this;
        if(!Array.isArray(dates.dates)){ console.warn('CanvasSlider.setDates: no dates'); this.element.canvasSlider('disable'); return; }
        if(!dates.dates.length && __DEV__) console.warn('CanvasSlider.setDates: bad arg: dates is empty array');
        if(typeof dates.index !== 'number'){ console.warn('CanvasSlider.setDates: bad arg: dates.index'); return; }
        if(dates.dates.length < 2) this.element.canvasSlider('disable');

        self.dates = dates.dates;
        self.current_date = dates.dates[dates.index];
        self.ref_date = dates.ref ? dates.ref : this.dates[0];
        self.format = dates.format;

        self.min_step = find_min_step();

        if(verify_dates()){

            //update animation range
            if(
                (self.animation_start_date && (self.animation_start_date.getTime() > self.dates[self.dates.length - 1])) ||
                (self.animation_end_date   && (self.animation_end_date.getTime()   < self.dates[0]))
              ){
                console.log('canvasSlider.update: no overlap in animation selection. resetting...');
                self._set_animation_start_date();
                self._set_animation_end_date();
            }else if(self.animation_start_date && (self.animation_start_date.getTime() < self.dates[0])){
                console.log('canvasSlider.update: animation start is before new date range. ignoring...');
            }else if(self.animation_end_date && (self.animation_end_date.getTime() > self.dates[self.dates.length - 1])){
                console.log('canvasSlider.update: animation end is after new date range. ignoring...');
            }

            if(self.stepBar){
                self.stepBar.update();
                self._draw();
            }

            // --------------------- temporary ---------------------

            self.range = (function range(){
                var range = self.dates[self.dates.length - 1].getTime() - self.dates[0].getTime();
                var mode = range > (1000 * 60 * 60 * 24 * 365)
                    ? 'seasonal'
                    : 'hourly';
                return {first: self.dates[0], last: self.dates[self.dates.length - 1], range: range, mode: mode};
            })();

            // ------------------- end temporary -------------------
        }

        function verify_dates(){
            //check dates are sorted.
            for(var i in self.dates){
                if((self.dates[i].getTime() - self.dates[0].getTime()) < -0.1){
                    console.warn('canvasSlider.verify_dates: date out of range! i=' + i, self.dates[i].format('mmmdd HH:MM'), self.dates[0].format('mmmdd HH:MM'));
                    setTimeout(function(){$.logWarning('canvasSlider.verify_dates: date (' + i + ') out of range! ' + self.dates[i].format('mmmdd HH:MM') + ' should be later than ' + self.dates[0].format('mmmdd HH:MM'))}, 1000);
                    return false;
                }
            }
            return true;
        }

        function find_min_step(){
            var min = 1000000000;
            var prev;
            for(var i in self.dates){
                if(prev){
                    min = Math.min(min, self.dates[i].getTime() - prev.getTime());
                }
                prev = self.dates[i];
            }
            return min;
        }
    },

    setVal: function(time){
        if(this._dateIsValid(time) && (time.value !== this.current_date.value)){
            this.current_date = time;
            this._draw();
        }
    },

    _set_animation_start_date: function(date){
        this.animation_start_date = date;
        return this;
    },

    __set_animation_start_date: function(date){
        this._set_animation_start_date(date);
        this.options.range_selection.on_select({start: this.animation_start_date, end: this.animation_end_date});
        $.logMessage('Animation start ' + (date ? 'set' : 'cleared'));
        return this;
    },

    _set_animation_end_date: function(date){
        this.animation_end_date = date;
        return this;
    },

    __set_animation_end_date: function(date){
        this._set_animation_end_date(date);
        this.options.range_selection.on_select({start: this.animation_start_date, end: this.animation_end_date});
        $.logMessage('Animation end ' + (date ? 'set' : 'cleared')); //TODO move to chart.
        return this;
    },

    animating: function(is_animating, loaded){
        var self = this;

        if(is_animating && (loaded = loaded.chart)){
            this.loadComplete = loaded.complete;

            var cycles = {};
            for(var j in loaded.dates){
               cycles[j.substring(13)] = loaded.dates[j];
            }

            $.each(this.dates, function(){ this.loaded = false; });

            this._each_date_in_range(function(time, i){
                self.dates[i].loaded = cycles[date2cycle(time)]; // TODO date.loaded should be set elsewhere. responsibility for the object lies elsewhere.
            });
            console.log('canvasSlider.animating: loadComplete=', this.loadComplete);
        }

        if(!is_animating){
            this.loadComplete = true;
        }

        this._draw();

        function date2cycle(date) {
            return date.format("yyyymmddHHMM");
        }
    },

    _each_date_in_range: function(fn){
        var stop = false;
        for(var i in this.dates){
            var time = this.dates[i];
            !(this.animation_start_date && time < this.animation_start_date)
                && !(this.animation_end_date && time > this.animation_end_date)
                && (stop = (fn(time, i) === false));
            if(stop) break;
        }
    },

    frameLoaded: function(time){
        var self = this;

        function _lookup(time){
            for(var i in self.dates){
                if(time === self.dates[i]) return i;
                if(time.value === self.dates[i].value) return i; // TODO why doesnt the direct test work in www_chart ?
            }
        }

        time = time.valid_time;
        var lookup = _lookup(time);
        if(lookup){
            this.dates[lookup].loaded = true;
            this._draw();
        }else{
            console.warn('CanvasSlider.frameLoaded: invalid time specified:', time, ' dates=', self.dates);
            for(var i in self.dates){
                var t = self.dates[i];
                console.warn('  ', (t.month + 1) + '.' + t.day + ':' + t.hour, t.value, t)
            }
        }
    },

    animationLoadComplete: function(msg){
        console.log('________________ loadComplete');
        this.loadComplete = true;
        this._draw();
    },

    setAnimationSelection: function(range){
        console.log('canvasSlider.setAnimationSelection:', range);
        range && this
            ._set_animation_start_date(range.start)
            ._set_animation_end_date(range.end)
            ._draw();
    },

    _setData: function(key, value, animated){
        $.widget.prototype._setData.apply(this, arguments);

        switch (key) {
            case 'disabled':
                this.cursor = null;
                this._draw();
                break;
            default:
                break;
        }
    },

    _event_to_x: function(event){
        return event.pageX - Math.floor(this.$canvas.offset().left);
    },

    _mouseStart: function(event){
        var feature = this._pick(event);
        feature && (this.drag = {item: feature});
    },

    _mouseStop: function(event){
        console.log('_mouseStop..');
        if(!this.drag) return;
        if(this.drag.animation_start_changed) this.__set_animation_start_date(this.animation_start_date);
        if(this.drag.animation_end_changed)   this.__set_animation_end_date  (this.animation_end_date);
    },

    _mouseDrag: function(event){
        if(!this.drag) return;

        var date = this._find_date_from_x(this._snap(this._event_to_x(event)) + 0.1);
        if(date){
            if(this.drag.item == 'loc1'){
                if(!this.animation_end_date || date < this.animation_end_date){
                    this._set_animation_start_date(date);
                    this.drag.animation_start_changed = true;
                    this._draw();
                }
            }
            if(this.drag.item == 'loc2'){
                if(!this.animation_start_date || date > this.animation_start_date){
                    this._set_animation_end_date(date);
                    this.drag.animation_end_changed = true;
                    this._draw();
                }
            }
        }
    },

    _draw_welcome: function(){
        this.ctx.fillStyle = '#333';
        if(this.ctx.fillText){
            this.ctx.fillText('Waiting for availability data...', 2, 8);
        }
    },

    _draw: function(){
        if(!Array.isArray(this.dates)) return;

        this.__throttled_draw();
    },

    _clear: function(){
        if(this.canvas){
            if(this.canvas.width != this._target_width()){
                this.canvas.width = this._target_width();
                this.stepBar.damage = null;
            }else{
                if(this.stepBar.damage) this.ctx.clearRect(this.stepBar.damage.x1, 0, this.stepBar.damage.x2 - this.stepBar.damage.x1, this.canvas.height);
                else this.canvas.width = this._target_width();
            }
        }
    },

    /*
    _draw_background: function(){
        this.ctx.clearRect(0, 0, this.canvas.width, this.canvas.height);

        var gradient = this.ctx.createLinearGradient(0, 0, 0, this.canvas.height);
        gradient.addColorStop(0, 'rgb(200, 200, 200)');
        gradient.addColorStop(1, 'rgb(100, 100, 100)');
        this.ctx.fillStyle = gradient;
        this.ctx.fillRect(0, 0, this.canvas.width, this.canvas.height);
    },
    */

    _pick: function(event){
        var loc1 = this.animation_start_date ? this._date_to_x(this.animation_start_date) : -1;
        var loc2 = this.animation_end_date   ? this._date_to_x(this.animation_end_date) : -1;
        var x = this._event_to_x(event);
        //console.log('_pick: ' + x + ' loc1=' + loc1 + ' loc2=' + loc2);
        if(loc1 > -1 && Math.abs(x - loc1) < 5) return 'loc1';
        if(loc2 > -1 && Math.abs(x - loc2) < 5) return 'loc2';
    },

    _snap: function(x){
        var time = this._find_date_from_x(x);
        if(time){
            var next_time = this._lookup_next_date(time);
            if(next_time){
                var spacing = this._date_to_x(next_time) - this._date_to_x(time);
                if(x > (this._date_to_x(time) + spacing / 2)) time = next_time;
            }
            return this._date_to_x(time);
        }
        return -1;
    },

    _get_cursor_pos_snapped: function(){
        return this._snap(this.cursor.x);
    },

    _getHours: function(time){
        return Math.round((time.getTime() - this.ref_date.getTime()) * ms2hours);
    },

    /*
    _draw_dotted_line: function(a, b){
        //line styles dont appear to be supported.
        host.ctx.dashPattern = "dotted";
        host.ctx.beginPath();
        host.ctx.moveTo(20, 0);
        host.ctx.lineTo(20, 24);
        host.ctx.stroke();
    },
    */

    _find_date_from_x: function(x){
        if(!this.dates) return;

        for(var i in this.dates){
            var xt = this._date_to_x(this.dates[i]);
            if(xt > x) return this.dates[i - 1];
        }
        return this.dates[this.dates.length - 1];
    },

    _date_to_x: function(date){
        return this.stepBar.date_to_x(date);
    },

    _find_day_start: function(date){
        for(var i in this.dates){
            if(this.dates[i].getTime() - date.getTime() === 0){
                var j = Number(i);
                while(j > 0){
                    var hours = this.dates[j].getHours();
                    if(!hours) break;
                    j--;
                }
                return j;
            }
        }
    },

    _find_day_end: function(date){
        for(var i in this.dates){
            if(this.dates[i].getTime() - date.getTime() === 0){
                for(var j = Number(i); j < this.dates.length; j++){
                    var hours = this.dates[j].getHours();
                    if(!hours) break;
                }
                return Math.min(j, this.dates.length - 1);
            }
        }
    },

    _target_width: function(){
        return this.dimensions.element_width/* - this.dimensions.paddingLeft - this.dimensions.paddingRight*/ || 300;
    },

    _set_cursor: function(x){
        if(!this.cursor) this.cursor = new Cursor();
        var canvas_scale = this.canvas.width / this.$canvas.width();
        this.cursor.x = x * canvas_scale;
        this.cursor.date = this._find_date_from_x(this._get_cursor_pos_snapped() + 0.1);
    },

    _dateIsValid: function(time){
        if(!time) return false;
        return true;
    },

    _lookup_next_date: function(time){
        if(!time) return;

        for(var i in this.dates){
            if((time.getTime() - this.dates[i].getTime()) === 0){
                if(i < this.dates.length - 1){
                    return this.dates[Number(i) + 1];
                }
            }
        }
    },

    _lookup_prev_date: function(time){
        if(!time) return;

        for(var i in this.dates){
            if(this.dates[i].getTime() - time.getTime() === 0){
                if(i > 0){
                    return this.dates[Number(i) - 1];
                }
                break;
            }
        }
        console.log('CanvasSlider._lookup_prev_date: not found');
    }
});


var StepBar = function(host){
    this.top = 18;
    this.border = host.options.border;
    this.topmargin = 0;//21; //negative margin extending above the top of the allocated area.
    this.hborder = 4; //left and right edges are inset to allow drawing at endstops.
    this.xstart = 0;

    function ref(dat) {
        //var d =  new Date(dat.getFullYear(), dat.getMonth(), dat.getDate());
        return dat;
    }

    this.update = function(){
        if(!host || !host.dates || !Array.isArray(host.dates) || !host.dates.length) return;

        this._set_scale();
    };

    this.draw = function(){
        if(!host || !host.dates || !Array.isArray(host.dates) || (host.dates.length < 2)) return;

        //this._show_damage();

        var bottom = host.canvas.height - this.border;
        host.ctx.lineWidth = '1';
        host.ctx.strokeStyle = 'rgb(0, 0, 0)';

        var one_day = this.scale * 1000 * 24 * 60 * 60;

        var dates = host.dates;

        if(!this.scale && host.dates.length) this._set_scale(); //temporary!

        //trail:
        if(host.options.trail && ECMWF.isset(host.current_date)){
            var x = this.date_to_x(host.current_date);
            host.ctx.fillStyle = '#fff';
            host.ctx.globalAlpha = 0.45;
            host.ctx.fillRect(this.hborder, this.topmargin, x, bottom);
            host.ctx.globalAlpha = 1.0;
        }

        var old_day, old_year;
        var old_label_rhs = 0;
        for(var i in dates){
            var time = dates[i];
            x = this.date_to_x(time);
            if(x > 0.0){
                var day = time.format('ddd dd');

                if(this._is_damaged(x)){
                    host.ctx.strokeStyle =
                        ((host.loadComplete || time.loaded) &&
                        (!host.animation_start_date || time > host.animation_start_date) &&
                        (!host.animation_end_date || time < host.animation_end_date))
                            ? 'rgb(80, 80, 80)'
                            : 'rgb(200, 200, 200)';
                    var is_major = (host.range.mode === 'seasonal')
                        ? (time.year !== old_year)
                        : (time.getHours() === 0);
                    this._draw_line({x: x, y: (is_major ? this.topmargin : this.topmargin + this.top - (host.loadComplete ? 0 : 2))}, {x: x, y: host.loadComplete ? bottom : bottom - 4});

                    if(time.loaded && !host.loadComplete){
                        host.ctx.fillStyle = '#77b';
                        var xa = (+i === 0) ? 0 : this.date_to_x(dates[i - 1]);
                        var width = (i == dates.length - 1)
                            ? (x - xa)
                            : ((this.date_to_x(dates[i * 1 + 1]) - xa) / 2);
                        host.ctx.fillRect((x + xa) / 2, host.canvas.height - 2, width, host.canvas.height);
                    }
                }

                //date label:
                //TODO this test still fails where there is a single time per day but it is at noon.
                if((old_day && day !== old_day) || (!old_day && (time.getHours() === 0))){
                    var in_range =
                        (!host.animation_start_date || time >= host.animation_start_date) &&
                        (!host.animation_end_date || time < host.animation_end_date)
                    host.ctx.fillStyle = (time.format('ddd') === 'Sun')
                        ? (in_range ? '#f00' : '#e55')
                        : (in_range ? '#555' : '#aaa');
                    if(host.ctx.fillText){
                        var label = time.label || day;
                        if(host.format) label = time.format(host.format);
                        var w = host.ctx.measureText(label).width;
                        var text_x = Math.floor(x + one_day/2 - w/2);
                        if(
                            (text_x > 0) &&
                            (text_x + w) < host.canvas.width &&
                            (
                                ((host.range.mode === 'seasonal') && is_major && (text_x > old_label_rhs + 4)) ||
                                ((host.range.mode !== 'seasonal') && (text_x > old_label_rhs + 4))
                            )
                        ){
                            host.ctx.fillText(label, text_x, this.topmargin + 8);
                            old_label_rhs = text_x + w;
                        }
                    }
                }
            }

            old_day = day;
            old_year = time.year;
        }

        //show the current time:
        if(ECMWF.isset(host.current_date)){
            x = this.date_to_x(host.current_date);

            host.ctx.strokeStyle = '#090';
            host.ctx.lineWidth   = host.options.big_handle ? Math.max(4, host.min_step * this.scale / 2) : '4';
            this._draw_line({x: x});

            if(!host.cursor && host.options.trail){
                host.ctx.globalAlpha = 0.75;
                //this._draw_flag(host.current_date, x - 2, {border: '#f00', fg: '#b33', bg: '#fee'});
                this._draw_flag(host.current_date, x - 2, {border: '#ccc', bg: '#f9f9f9'});
                host.ctx.globalAlpha = 1.0;
            }
        }

        this._draw_cursor();

        // draw start/end flags
        if(host.animation_start_date){
            this._draw_locator({x: this.date_to_x(host.animation_start_date)});
        }
        if(host.animation_end_date){
            this._draw_locator({x: this.date_to_x(host.animation_end_date)});
        }

        this._draw_cursor_b();
    };

    this._draw_locator = function(a){
        var locator = {
            width: 10,
            height: 8
        }
        host.ctx.fillStyle = '#f79922';
        var y = host.canvas.height - this.border - (host.loadComplete ? 0 : 4) - locator.height;
        host.ctx.beginPath();
        host.ctx.moveTo(a.x, y);
        host.ctx.lineTo(a.x + locator.width/2, y + locator.height);
        host.ctx.lineTo(a.x - locator.width/2, y + locator.height);
        host.ctx.closePath();
        host.ctx.fill();
    }

    this._draw_flag = function(date, x, options){
        var o = $.extend({border: '#333', bg: '#fff', fg: '#333', borderWidth: [1, 1, 1, 1]}, options);
        x || (x = this.date_to_x(date));

        var draw_x = Math.max(0, x);

        var text = host.format
            ? date.format(host.format) // temporary. this is a concise format is not intended for the 'flag'.
            : 'T+' + host._getHours(date) + ' ' + date.format('ddd dd mmm HH') + 'Z';

        var w = host.flag_width;
        try {
            w = host.ctx.measureText(text).width + 9;
        }
        catch(e) {}

        var flag_reversed = false;
        if(x > host.canvas.width - w){
            x -= w + 3;
            draw_x -= w + 3;
            flag_reversed = true;
        }

        host.ctx.fillStyle = o.bg;
        host.ctx.fillRect(draw_x + (flag_reversed ? o.borderWidth[3] : o.borderWidth[3]), o.borderWidth[0], w, 15);

        host.ctx.lineWidth   = '1';
        host.ctx.strokeStyle = o.border;
        host.ctx.strokeRect(draw_x, 0, w + o.borderWidth[1] + o.borderWidth[3], 15 + o.borderWidth[0] + o.borderWidth[2]);

        host.ctx.fillStyle = o.fg;
        if(host.ctx.fillText){
            host.ctx.fillText(text, x + 4, 11 + o.borderWidth[0]);
        }
    }

    this._draw_cursor = function(){
        if(host.cursor && !host.drag){
            var top = 2;
            var height = host.canvas.height - top;

            var x = host._get_cursor_pos_snapped();
            var date = host._find_date_from_x(x + 0.1);

            if(date){
                var draw_x = Math.max(1, x);
                host.ctx.lineWidth = '2';
                host.ctx.strokeStyle = '#00f';
                this._draw_line({x: draw_x}, {x: draw_x, y: top + height});

                host.cursor.x_snapped = x;
                host.cursor.date = date;

                //damage optimisation: (still slightly buggy)
                /*
                var one_day = this.scale * 1000 * 24 * 60 * 60;
                var n_days = Math.floor(w / one_day) + 2;
                var x1d = host._find_day_start(date);
                var x1 = host._date_to_x(host.dates[x1d]);
                var x2 = x1 + n_days * one_day * (flag_reversed ? -1 : 1);
                if(flag_reversed){ var x = x1; x1 = x2; x2 = x; }

                this.damage = {x1: x1 - 3, x2: x2 + 1};
                */
                this.damage = null;
            }
        }
    }

    this._draw_cursor_b = function(){
        if(host.cursor && !host.drag){
            if(host.cursor.date){
                this._draw_flag(host.cursor.date, null, '#333');
            }
        }
    }

    this._draw_line = function(a, b){
        b   || (b = {});
        a.y || (a.y = this.topmargin);
        b.x || (b.x = a.x);
        b.y || (b.y = host.canvas.height - this.border - (host.loadComplete ? 0 : 4));

        host.ctx.beginPath();
        host.ctx.moveTo(a.x, a.y);
        host.ctx.lineTo(b.x, b.y);
        host.ctx.stroke();
        host.ctx.closePath();
    }

    this._show_damage = function(){
        var damage = this.damage;
        if(damage){
            this.ctx.strokeStyle = 'rgb(0, 0, 255)';
            this._draw_line({x: damage.x1 + 1, y: 0}, {x: damage.x1 + 1, y: this.canvas.height});
            this._draw_line({x: damage.x2 - 2, y: 0}, {x: damage.x2 - 2, y: this.canvas.height});
        }
    };

    this._is_damaged = function(x){
        return (!this.damage || (x > this.damage.x1 && x < this.damage.x2 - 1));
    };

    this._set_scale = function(){
        this.scale = (host.canvas.width - 2 * this.hborder) / (host.dates[host.dates.length - 1].getTime() - host.dates[0].getTime());
        //console.log('CanvasSlider._set_scale: width=' + host.canvas.width + ' scale=' + this.scale);
    };

    this.date_to_x = function(date){
        return (date.getTime() - host.dates[0].getTime()) * this.scale + this.hborder;
    };
};

var ms2hours = 1 / (1000 * 60 * 60);

var Cursor = function(){
    this.x = null;
    this.x_snapped = null;
    this.date = null;
};

/*
ECMWF.f_date = function(date) {
	if(!date) return 'undefined';
	return ECMWF.pad(date.month+1, 2, '0') + '.' + date.day + ':' + ECMWF.pad(date.hour, 2, '0');
}
*/

