/*
 *  Widget for displaying call graph information.
 *  It is controlled via the servicelib/metadata module
 */
define(['config', 'jquery', 'element', 'bus', 'jquery-ui/dialog', './monitor.css'], function(ECMWF, $, Dom, Bus) {

function timeInterval(node, interval, nodes) {
	if(node.start && node.stop) {
		if(interval.start === null || node.start < interval.start) { interval.start = node.start; }
		interval.stop = Math.max(interval.stop || 0, node.stop);
		nodes.push(node);
	}
	$.each(node.kids, function(_, kid) {
		timeInterval(kid, interval, nodes);
	});
}

function timeLine(div, nodes, total) {
	var tot_len  = total.stop - total.start;
	var min_width = 2 * 100 / div.width();

	$.each(nodes, function(_,node) {
		if(node.start && node.stop) {
			var p = div.p('.timeline-task');
			$('<span>').addClass('timeline-task-service').text(node.task).appendTo(p);
			if(node.name && node.name != 'unnamed') {
			$('<span>').addClass('timeline-task-name').text(node.name).appendTo(p);
		}

		$.each(['host', 'cache', 'queue', 'load'], function(_, key) {
			if(node[key]) {
				$('<span>').text(', ').appendTo(p);
				$('<span>').addClass('timeline-key').text(key + ':').appendTo(p);

				var s = $('<span>').addClass('timeline-value').text(node[key]).appendTo(p);
				if(key == 'load') {
					node[key] = Math.floor(parseInt(node[key], 10)/10.0 + 0.5);
				}
				s.addClass('timeline-value-' + key + '-' + node[key]);
			}
		});

		var left  = Math.floor((node.start - total.start) / tot_len * 100.0 + 0.5);
		var right = Math.min(100.0 - (left + min_width), Math.floor((total.stop - node.stop) / tot_len * 100.0 + 0.5));

		$.each(node.timers, function(key, value) {
			if(key !== 'elapsed') {
				var e = Math.floor(value * 1000 + 0.5);
				if (e) {
					$('<span>').text(', ').appendTo(p);
					$('<span>').addClass('timeline-timer-key').text(key + ':').appendTo(p);
					$('<span>')
						.addClass('timeline-timer-value')
						.text(e >= 100
							? Math.floor(e/100)/10.0 + 's'
							: e + 'ms')
						.appendTo(p);
				}
			}
		});

		var len = Math.floor((node.stop-node.start)*1000+0.5);
		if(len >= 100) {
			len = Math.floor(len/100)/10.0;
			len = '' + len + 's';
		} else {
			len = '' + len + 'ms';
		}

		$('<p>').text(len).appendTo(div)
			.css({marginLeft: '' + left + '%', marginRight: '' + right + '%'})
			.addClass('timeline-bar timeline-bar-' + node.task);
		}
	});
}

function json_tree(parent, json, options) {
	parent.empty();

	var interval = {start: null, stop: null};
	var nodes = [];
	timeInterval(json, interval, nodes);

	interval.stop = interval.start + Math.ceil(interval.stop - interval.start);

	var div = $('<div>').addClass('timeline-grid');
	var inner = div.div().css({width: Math.min(774, (interval.stop - interval.start) * 200)});
	div.appendTo(parent);

	//nodes.sort(function(a,b) { return b.start < a.start; });
	timeLine(inner, nodes, interval);
}

var call_tree;
var timers;

return {
	call_tree: function(tree) {

		if(!call_tree) {

			$('head').append('<style>  .treeview { font-family: sans-serif; font-size: 10pt; } .treeview-label { font-weight: bold; } .treeview-empty { font-style: italic; } .treeview-miss { color: red; } </style>');

			Bus().listen('open-calltree', function(_, msg) {
				console.info('calltree=', call_tree);
				var div = $('<div id="call-tree" title="Timeline" class="treeview">')
					.dialog({width: 800, resizable: true});
				json_tree(div, call_tree, {collapsed: true});
				div.dialog('open');
			});
		}
		call_tree = tree;
	},

	timers: function(time, tracker) {

		if(!timers) {

			$('head').append('<style>  .timers { font-family: sans-serif; font-size: 10pt; } .timerservice { font-weight: bold; padding-top: 5px; padding-bottom: 5px;} .timername { padding-left: 1em; font-style: italic; } .timervalue { text-align: right; } </style>');

			Bus().listen('show-timers', function(_, msg) {
				console.info('calltree=', call_tree);
				var div = $('<div id="times" title="Timers" class="treeview">')
					.dialog({title: 'Timers', resizable: true });

				var names = [];
				for(var n in timers) {
					names.push(n);
				}
				names = names.sort();

				var table = div.table('.timers').attr({width: '100%'});

				var last;
				for(var i=0; i < names.length; i++) {
					var name = names[i].split('.');
					var tr = table.tr();
					if(name[0] != last) {
						last  = name[0];
						tr.td('.timerservice').attr({colspan: 2}).text(last);
						tr = table.tr();
					}

					tr.td('.timername').text(name[1]);
					tr.td('.timervalue').text(Math.floor(timers[names[i]] * 1000 + 0.5) + ' ms');
				}

				div.dialog('open');
			});
		}
		timers = time;
		ECMWF.last_tracker = tracker;
	}
}
});
