/*
 *  ModeTooltip is a tooltip wrapper that adds support for predefined content formats such as 'info' and 'warning'.
 *
 *  The latest tooltip widget to be supported is tipsy. If this turns out to work well,
 *  suppport for other widgets can be removed.
 */
define(['jquery', 'browser', 'jquery-ui/widget', './tooltip.css'], function($, Browser) {

//var content_modes = ['message', 'help', 'warning', 'elaborate'];

var single;

$.widget('ui.modeTooltip', {

	options: {
		style: 'normal',  //see also content.mode which controls the styling of the content _inside_ the tooltip
		position: { my: 'left center', at: 'right top', offset: '-16 -22' }, // not supported by tipsy
		content: function(){return {text: '', mode: 'message'}}, //can be either a function or an object. If the function returns false, this indicates that the client is managing the content itself and nothing further needs to be done by the tooltip.
		maxWidth: 250,
		widget: 'tipsy'
	},

	_init: function() {
		var self = this;
		var o = this.options;

		this.element.addClass('mode-tooltip');

		if($.fn.singleTooltip){
			var pref_disabled = $.getUserPreference('tooltips_disable');
			if(pref_disabled) o.widget = 'none';
			else o.widget = 'single' //force economy

			single || (single = $('body').singleTooltip());
		}

		switch(o.widget){
		case 'single':
			//$('body').singleTooltip('option', 'element', self.element);
			$('body').singleTooltip('addElement', self.element);
			this._disable = $.noop;
			this._enable = $.noop;
			break;

		case 'tipsy':
			this.element
				[o.widget]({
					title: get_content,
					html: true,
					fade: true,
					gravity:  $.fn.tipsy.autoBounds(40, 's'),
					offset: 7,
					delayIn: 500
				});
			this._disable = function(){ self.element[o.widget]('disable')[o.widget]('hide')};
			this._enable = function(){ self.element[o.widget]('enable')};
			break;

		default:
			this._disable = $.noop;
			this._enable = $.noop;
			break;
		}

		function get_content(el){
			var content = o.content;
			$.isFunction(content)    && (content = content(el));
			$.isPlainObject(content) || (content === false) || (content = { text: content});

			if(content === false){
				return el.find(':first'); //client has handled the content itself.
			}

			if(!content) return;

			var text = content.text || '';
			var mode = content.mode || 'message';
			var icon_class = '';
			if(0 && mode == 'message') icon_class = ' ui-icon-info';
			if(mode == 'warning') icon_class = ' ui-icon-alert';
			var icon = icon_class ? '<div class="ui-icon ' + icon_class + ' float-left"></div>' : '';

			return text ? ('<div class="' + mode + '">' + icon + '<div class="content">' + text + '</div><br/></div>') : '';
		}
	},

	destroy: function(){
		this._disable();
		this.element.removeClass('mode-tooltip');
	},

	close: function(){
		switch(this.options.widget){
			case 'tipsy':
				this.element.trigger('mouseleave');
				break;
		}
	},

	enable: function(){
		this._enable();
	},

	disable: function(){
		this._disable();
	}
});

});
