define(['config', 'jquery', 'ajax_queue'], function(ECMWF, $, ajaxqueue){
	var units;
	var units_by_name = {};

	function load_units(ready) {
		ready = ready || $.noop;

		if (units) {
			return ready(units);
		}

		ajaxqueue({
			url: '/backend/unit-list/',
			priority: 'vip',
			success: function(result) {
				$.each(result, function(_, u) {
					units_by_name[u.name] = new Units.Unit(u);
				});
				units = result;
				ready(units);
			},
			error: function(result) {
				// Ignore for now ....
				units = [];
				ready(units);
			}
		});
	}

	var Units = {

		load: $.Deferred(function(df) {
			load_units(function(){
				df.resolve(units);
			});
		}).promise(),

		get: function(name) {

			if (typeof(name) === 'string' && units_by_name[name]) {
				return units_by_name[name];
			}else{
				if (name) {
					return new Units.Unit(name);
				} else {
					return null;
				}
			}
		}
	}

	// TODO: Use Numer(value).toFixed(p) instead
	function round(value, p) {
		if (p === undefined) {
			p = 2;
		}

		var t = 1;
		while (p < 0) {
			t /= 10;
			p++;
		}
		while (p > 0) {
			t *= 10;
			p--;
		}

		return Math.floor(value * t + 0.5) / t;
	}

	function degree(x, X, plain) {
		return plain ? (round(x) + ' ' + X) : '<span class="latlon">' + round(x) + '</span>&deg;' + X;
	}

	$.formatCelcius = function(celcius, decimal) {
        var val = round(celcius, decimal);
        return val + ' &deg;C';
    };

    $.formatLatitude = function(lat, plain) {
        var N = 'N';

        if (typeof(lat) === 'string') {
            lat = parseFloat(lat);
        }

        if (lat < 0) {
            N = 'S';
            lat = -lat;
        }

        return degree(lat, N, plain);
    };

	$.formatLongitude = function(lon, plain) {
		var E = 'E';

		if (typeof(lon) === 'string') {
			lon = parseFloat(lon);
		}

		while (lon < 0) {
			lon += 360;
		}

		while (lon > 360) {
			lon -= 360;
		}

		while (lon > 180) {
			lon -= 360;
		}

		if (lon < 0) {
			E = 'W';
			lon = -lon;
		}

		return degree(lon, E, plain);
	};

	$.formatGeoLocation = function(lat, lon) {
		if (lon === undefined) {
			var pt = Array.isArray(lat) ? { lat: lat[0], lon: lat[1] } : lat;
			lat = pt.latitude === undefined ? pt.lat : pt.latitude;
			lon = pt.longitude === undefined ? pt.lon : pt.longitude;
		}
		return $.formatLatitude(lat) + '&nbsp;' + $.formatLongitude(lon);
	};

	$.formatGeoLocationPlain = function(lat, lon) {
		if (lon === undefined) {
			var pt = Array.isArray(lat) ? { lat: lat[0], lon: lat[1] } : lat;
			lat = pt.latitude === undefined ? pt.lat : pt.latitude;
			lon = pt.longitude === undefined ? pt.lon : pt.longitude;
		}
		return $.formatLatitude(lat, true) + ' ' + $.formatLongitude(lon, true);
	};

	$.formatFloat = function(x, prec) {
		return round(x, prec);
	};

	var directions = ['N', 'N-N-E', 'N-E', 'E-N-E', 'E', 'E-S-E', 'S-E', 'S-S-E', 'S', 'S-S-W', 'S-W', 'W-S-W', 'W', 'W-N-W', 'N-W', 'N-N-W'];

	Units.Unit = function Unit(unit) {

		if (typeof(unit) === 'string') {
			unit = { 'name' : unit };
		}

		this.offset = 0;
		this.factor = 1;

		if (unit) {
			for (var n in unit) {
				this[n] = unit[n];
			}
		}

		this.transform = function(value) {
			return value === null ? null : (value * this.factor + this.offset);
		};

		this.toDisplay = this.transform;
		this.fromDisplay = function(value) {
			return (value - this.offset) / this.factor;
		};

		this.wind_direction = function(value) {
			while (value < 0) value += 360;
			while (value >= 360) value -= 360;

			var i = Math.floor(value / 360.0 * directions.length + 0.5);
			return directions[i];
		};

		this.wave_direction = function(value) {
			return this.wind_direction(value);
		};

		this.symbolHTML = function() {
			var symbol = this.html_symbol || this.name.replace(/\*\*(-?\d+)/g, '<sup>$1</sup>');
			return symbol;
		};

		this.asHTML = function(value) {

			if (value === null) {
				// TODO: Do something better for the missing value
				return '-';
			}

			if (this.html_method) {
				return this[this.html_method](value);
			}

			var symbol = this.symbolHTML();

			return round(this.transform(value), this.precision) + ' ' + symbol;
		};
	}

	return Units;
});



