/*jslint white: false, nomen: false, onevar: false, forin: false, browser: true, devel: true, immed: false */
/*global ECMWF: true, console: true, jQuery: true */

/*
 * Maintains an up-to-date list of user catalogue items without continuously querying the server by monitoring user-catalogue messages.
 * The ajaxqueue cache is not used as it becomes stale.
 */

define(['config', 'jquery', 'bus'], function(ECMWF, $, Bus){

return function UserCatalogueStore(){

	this.options = {
		kind: 'product',
		bus: Bus(),
		user_item_added: $.noop,
		user_item_removed: $.noop
	}

	this.create = function(args) {
		this.options = $.extend(this.options, args);

		var cat = this, o = this.options;

		o.bus
			.listen('new-' + o.kind + '-added-to-catalogue', function(_, item) {
				add(item);
				o.user_item_added(item);
			})
			.listen(o.kind + '-removed-from-catalogue', function(_, item) {
				remove(item);
				o.user_item_removed(item.title);
			});

		function add(item){
			if(cat.data){
				remove(item);
				cat.data.push(item);
			}
		}

		function remove(li){
			var data = cat.data;
			if(data){
				//var len = data.length;
				for(var i=0;i<data.length;i++){
					if(data[i].title === li.title){
						data.splice(i, 1);
						return;
					}
				}
			}else{
				if(cat.pending){
					console.warn('UserCatalogueStore.remove: TODO cannot remove - no store. pending=' + cat.pending);
				//}else{
					//if we haven't yet requested the list there is nothing to do.
				}
			}
		}

		/*
		function print_userlist(){
			var data = self.element.data('userCatalogueStore');

			console.groupCollapsed('userlist (' + data.length + ')');
			for(var i=0;i<data.length;i++){
				console.log(data[i].title);
			}
			console.groupEnd();
		}
		*/
		return this;
	}

	this.get_catalogue = function(callback) {
		var cat = this, o = this.options;

		if(ECMWF.n_properties(this.data)){
			callback(this.data);
			return this;
		}

		cat.pending = true;
		ECMWF.get_user_catalogue({
			kind: o.kind,
			priority: 'high',
			status: ECMWF.ajax_loader($('.' + o.kind + '-dialogue ul')),
			success: function(list){
				cat.data = list;
				cat.pending = false;
				callback(list);
			}
		});

		return this;
	}
}

});
