
define(['config', 'jquery', './utils.css'], function(ECMWF, $) {

var space = function(n) {
	var s = "";
	while(n > 0) { s += " "; n--; }
	return s;
}

var pretty = function(obj, depth)
{
	if(depth/3 > 5)  { return "..."; }

	if(typeof(obj) == 'object'){
		var list = []
		var s = space(depth+3);
		if (obj instanceof Array) {
			var bra = '['; var ket = ']';
			for (var n in obj) {
				var value = pretty(obj[n],depth+3);
				list.push(s + value);
			}
		} else {
			bra = '{'; ket = '}';
			for (n in obj) {
				value = pretty(obj[n],depth+3);
				list.push( s + '"' + n + '"=' + value);
			}
		}

		return bra + (list.length > 0?"\n":"") + list.join(",\n") + (list.length > 0?"\n" + space(depth) :"") + ket;
	}

	if (typeof obj == 'string')
		return '"' + obj + '"';

	return "" + obj;
}


/*
ECMWF.json_to_html = function(obj){

	function do_child(child){
		depth++;
		depth_used = Math.max(depth_used, depth);
		var n = 0;
		for(var propName in child){
			if(depth < max_recursion){
				if(typeof child[propName] === 'object'){
					if($.isPlainObject(child[propName])){
						add_line(propName);
						do_child(child[propName]);
					}
					else add_line(propName + ' ***');
				}
				else add_line(propName);
			}else{
				add_line(propName);
				add_line('[max recursion!! ' + depth + ']');
			}
			n++;
		}
		if(!n) add_line('[no properties]');
		depth--;

		function add_line(s){
			output.push('<div style="margin-left:' + (depth * 10) + 'px">' + s + '</div>');
		}
	}

	var depth = 0;
	var max_recursion = 10;
	var depth_used = 0;
	var output = [];

	do_child(obj);

	return output.join('') + 'depth=' + depth_used;
}

ECMWF.print_element = function(el)
{
	//useful when firebug not available.

	if(typeof(el) == 'undefined') return 'undefined';
	if(el === null) return 'null';

	var o = '';
	if(el.length && ECMWF.isset(el.attr)){
		o += ' $tag=' + el[0].tagName + ' id=\'' + el.attr('id') + '\' class=\'' + el.attr('class') + '\' parent_id=\'' + el.parent().attr('id') + '\' parent_class=\'' + el.parent().attr('class') + '\'';
	}else if(typeof el == 'object' && ECMWF.isset(el.scrollWidth)){
		o += ' html-element: ' + el.tagName + ' class="' + el.className + '" id=' + el.id + '. nodeType=' + el.nodeType + '.';
	}else if(ECMWF.isset(el.nodeName) && el.nodeName == '#document'){
		o += ' #document.';
	}else if(el.parent && el.parent.constructor.toString() == '[object Window]'){
		o += ' window';
	}else if(typeof el == 'object'){
		o += ' object: contructor=' + el.constructor + '. '+ Object.keys(el).join(', ');
	}else if(typeof el == 'function'){
		o += ' type=function'
	}else if(typeof el == 'number'){
		o += ' type=number (' + el + ')'
	}else{
		o += ' dom element? len=' + el.length + ' el=' + el;
	}
	return o;
}

ECMWF.fn_name = function(fn)
{
	var name = /\W*function\s+([\w\$]+)\(/.exec(fn);
	return name ? name[1] : 'anonymous';
}
*/

ECMWF.pad = function(str, len, pad)
{
	if(typeof str === 'undefined') str = '';
	if(typeof str != 'string'){ str = str.toString(); }
	if(typeof len == 'undefined'){ len = 0; }
	if(typeof pad == 'undefined'){ pad = ' '; }

	if(len + 1 >= str.length){
		str = Array(len + 1 - str.length).join(pad) + str;
	}
	return str;
}

ECMWF.truncate = function(str, len){
	if(str && str.length > len){
		var trunc = str.substring(0, len);
		return trunc.replace(/\w+$/, '') + '&hellip;';
	}
	else return str;
}

ECMWF.isset = function(a){
	if(typeof a == 'undefined') return false;
	if(typeof a == 'string' && !a) return false;
	if(typeof a == 'number' && isNaN(a)) return false;
	if(a === null) return false;
	return (a != 'undefined');
}

ECMWF.is_empty = function(obj){
	var i;
	for(i in obj) return false;
	return true;
}

ECMWF.format_date = function(time) {
	if(!ECMWF.isset(time)) return '';
	time = '' + time;

	var d = time.substring(0, 8);
	var t = time.substring(8, 12);
	return d + ' ' + t + ' UTC';
}

/* increases the size of an object by mb megabytes to debug memory problems */
ECMWF.inflate = function (obj, mb) {
	mb = mb || 1;
	obj.padding = new Array(1000000 * mb + 1).join('_');
}

ECMWF.add_links = function (input) {
	return input
		.replace(/(ftp|http|https|file):\/\/[\S]+(\b|$)/gim, '<a href="$&" class="my_link" target="_blank">$&</a>')
		.replace(/([^\/])(www[\S]+(\b|$))/gim, '$1<a href="http://$2" class="my_link" target="_blank">$2</a>');
}

ECMWF.remove = function (obj, name) {
	var val = obj[name];
	delete obj[name];
	return val;
}


$.fn.shadowFix = function(options){
	//enables a shadow underneath titlebar without shadow being visible on other sides when using border-radius.
	return this.each(function(){
		$(this).parent().addClass('titlebar-shadow');
	});
};

ECMWF.timer = function(milliseconds)
{
	this.milliseconds = milliseconds;
	this.id = null;

	this.start = function(done){
		var self = this;
		__DEV__ && this.id && console.warn('timer.start: already started');
		this.id = setTimeout(function(){
			self.id = null;
			done();
		}, this.milliseconds);
	}

	this.clear = function(done){
		clearTimeout(this.id);
		this.id = null;
	}
}


ECMWF.xhr_to_error_string = function(xhr)
{
	if(!xhr.error) return 'Unknown error';

	return xhr.error.length
		? xhr.error[0]
		: xhr.error;
}


var Utils = {
	blank_gif: 'data:image/gif;base64,R0lGODlhAQABAAAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==',

	/*
	 *  Return true if page configuration indicates use of a different host for ajax requests.
	 */
	same_origin: function() {
		var loc = window.location.protocol + '//' + window.location.hostname + (window.location.port ? (':' + window.location.port) : '');
		var same_origin = (!ECMWF.options.backend_host) || (loc === ECMWF.options.backend_host);
		same_origin || console.log('same_origin=', same_origin,' backend=', loc + ' ', ECMWF.options.backend_host);

		return same_origin;
	},

	n_properties: function(obj) {
		var i = 0, propName;
		for(propName in obj) i++;
		return i;
	},

	ucfirst: function(str) {
		str += '';
		return str.charAt(0).toUpperCase() + str.slice(1);
	},

	prettyJSON: function(obj, indent) {
		return pretty(obj, 0);
	},

	escape_regex: function(s) {
		return s.replace(/[.*+?^${}()|[\]\/\\]/g, '\\$0');
	},

	camel_to_dashes: function(str){
		return str.replace( /([a-z])([A-Z])/g, '$1-$2' ).toLowerCase();
	},

	/*
	 *  Execute a list of non-async functions in series and return a list of return values
	 */
	run: function(list, args) {
		var self = this;
		return $.map(list, function(fn) {
			return fn.apply(self, args);
		});
	}
};
$.extend(true, ECMWF, Utils);


$.fn.orelse = function(fn){

	// execute the fn only if no elements are selected.
	// -can be used to create an element if it doesn't already exist.
	// -fn can change the chain by optionally returning a value.
	// -if fn is not a function, it is returned directly (can be used to provide an alternative dom selector if the original one is empty).

	var ret;
	return $(this).length
		? $(this)
		: typeof fn !== 'function'
			? fn
			: ((ret = fn())
				? ret
				: $(this));
};

$.fn.ensure = function(sel){
	// if the named child div exists, return it, otherwise create it first

	var self = this;
	return $(this).find('> ' + sel).orelse(function(){ return $(self).div(sel); });
};

$.fn.delayed = function(delay, fn){

	//wrapper for $.fn.queue() that delays execution, and continues execution of the queue.

	return this.delay(delay).queue(function(next){
		fn.apply(this);
		next();
	});
};

$.fn.clear = function(){

	// adds a br to clear floats.

	return $(this).br('.br-clear');
};

$.fn.pack = function(items){

	// add the given items as children of the node.

	var self = $(this);
	$(arguments).each(function(i, o){ o.appendTo(self); });
};


$.pluck = function(obj, key){

	// given an array of objects, returns an array of values for the given property name

	return $.map(obj, function(value){ return value[key]; });
}

$.where = function(list, properties){

	// filter the list such that only items that match all the given property values are returned.

	return $.map(list, function(l){
		if($.map(properties, function(v, i){
			if(l[i] === v) return true;
		}).length) return l;
	});
}

$.findWhere = function(list, properties){

	// as $.where above but only returns the first item

	return $.where(list, properties)[0];
}

/*
 *   jquery deprecated fn
 */
$.isWindow = function (obj) {
	return obj != null && obj === obj.window;
}

/*
 *  ECMWF.map provides the ability to apply mappings to simple objects
 *  similar to $.map but returns an object instead of an array.
 *  It preserves the original keys so cannot be used to convert an array to an object.
 */
Utils.map = function(object, f, ctx){
	ctx = ctx || this;
	var result = {};
	$.each(Object.keys(object), function(i, v) {
		var val = f.call(ctx, object[v], v, object);
		(typeof val !== 'undefined' && val !== null) && (result[v] = val);
	});
	return result;
}
if(!Object.keys){
	Utils.map = function(object, fn, ctx) {
		var result = {};
		$.each(object, function(i, v) {
			if(typeof v !== 'undefined' && object.hasOwnProperty(i)){
				var val = fn(v);
				(typeof val !== 'undefined' && val !== null) && (result[i] = val);
			}
		});
		return result;
	}
}

/*
 *  Return an array of query parameters for the current page.
 */
ECMWF.query = function(){
    var query_string = {};
    var query = window.location.search.substring(1);
    var vars = query.split('&');
    for (var i=0;i<vars.length;i++) {
        var pair = vars[i].split('=');
        // If first entry with this name
        if (typeof query_string[pair[0]] === 'undefined') {
            pair[0] && (query_string[pair[0]] = pair[1]);
            // If second entry with this name
        } else if (typeof query_string[pair[0]] === 'string') {
            var arr = [ query_string[pair[0]], pair[1] ];
            query_string[pair[0]] = arr;
            // If third or later entry with this name
        } else {
            query_string[pair[0]].push(pair[1]);
        }
    }
    return query_string;
}

ECMWF.get_host = function() {
	return ECMWF.fix_host.to_backend(location.protocol + '//' + location.hostname + (location.port ? ':' + location.port : ''));
}

ECMWF.fix_host = {
	to_backend: function(url) {
		return url.replace(/www|eccharts/gi, 'apps').replace(/stage|eccharts-test/gi, 'apps-test');
	},
	to_frontend: function(url) {
		return url.replace(/apps-test|eccharts-test/gi, 'stage').replace(/apps|eccharts/gi, 'www');
	}
};

return Utils;
});


