define(['jquery', '../filter'], function($, Filter){

	function FacetFilter() {
		this.name = 'facets'

		this.fn = function(item){
			if($.isEmptyObject(this._active)) return true;
			var facets = item.data.tags;
			if(!facets) return false;

			for(var facet in this._active){
				if(!facets[facet]) return false;
				if(!$.map(this._active[facet], function(val){ if(!!~$.inArray(val, facets[facet])) return val; }).length) return false;
			}
			return true;
		}

		this.set = function(prop, active) {
			var f = prop[0];
			var fval = prop[1];
			console.log('filters.facets.set: ', f, fval, active);

			if(active){
				this._active[f] = this._active[f] || [];
				!!~$.inArray(fval, this._active[f]) || this._active[f].push(fval);
			}else{
				this._active[f] = $.map(this._active[f], function(v){ if(v !== fval) return v; });
				if(!this._active[f].length) delete this._active[f];
			}
			this.events.send('changed', [prop, active]);
		}

		this.deactivate = function() {
			console.log('filters.facets.deactivate');
			this._active = {};
			this.events.send('changed', []);
		}

		this.status = function() {
			return $.map(this._active, function(v, i) {
				return i + ': ' + v.join(', ');
			}).join(' / ');
		}

		// example url query: ?facets=Product group,Basic EFI;Component,Atmosphere,Geography
		this.to_url = function() {
			return {
				facets: $.map(this._active, function(v, i) {
					return i + ',' + v.join(',');
				}).join(';')
			}
		}

		this._active = {}
	}

	FacetFilter.prototype = new Filter()

	return FacetFilter;
});
