/*
 *  Filter based on product.path
 */
define(['jquery', 'vgrid/filter', 'chart-app/router', 'vgrid/products'], function($, Filter, controller, products) {

	function PathFilter() {
		this.name = 'path'

		this.fn = function(item) {
			if(!item.data.path || !this._selected) return false;
			for(var p in item.data.path){
				var path = item.data.path[p].split('/'); // TODO must be done in advance
				if(path.length < this._selected.length) return false;
				var found = true;
				for(var i in this._selected){
					if(i > path.length - 1) break;
					if(path[i] !== this._selected[i]) found = false;
				}
				if(found) return true;
			}
			return false;
		},

		this.set = function(prop) {
			if(!Array.isArray(prop[0])) return console.warn('path_filter.set: array expected', prop[0]);

			if(!this._selected || (prop[0].length !== this._selected.length) || (prop[0][prop[0].length - 1] !== this._selected[this._selected.length - 1])){
				this._selected = prop[0];
				this.events.send('changed', [prop, true]);
			}
		}

		this.set_child = function(child) {
			var path = this._selected.slice(0);
			if(!products.hierarchy.path_is_valid(path)) return console.warn('invalid path');
			path.push(child);
			this.set([path]);
		}

		this.deactivate = function() {
			this._selected.pop(); // go up one level.
			this.events.send('changed', [[this._selected], true]);
		}

		this.status = function() {
			if(!this._selected) return 'FIXME';
			return this._selected.join('/');
		},

		this._selected = null
	}

	PathFilter.prototype = new Filter()

	return PathFilter;
});
