define(['jquery', 'mixins', 'virtual-dom/h', 'vdom-virtualize', 'observable', './input_filter', './scrollbar', './filter_status', './index.css'], function($, mixins, h, virtualise, Observable, InputFilter, Scrollbar, FilterStatus){

	var VGrid = function(options){
		this.options = $.extend({
			view_mode: 'groups',
			renderer: undefined,
			products: undefined,
			width: undefined,
			height: undefined, // eg '430' - total height (header, footer, grid)
			enable_search: true,
			enable_status: true,
			on_click: $.noop,
			on_dblclick: $.noop,
			on_middle_click: $.noop
		}, options);

		var self = this, o = this.options;

		// because VGrid uses dynamic loading, use of RenderTree.ready is required
		if(!o.renderer.ready) o.renderer.ready = []
		var ready = new $.Deferred()
		o.renderer.ready.push(ready)

		var head_height = o.enable_search ? 23 + 10 + 16 : 0
		var foot_height = o.enable_status ? 63 + 10 + 16 : 0
		this.height = new Observable().set((o.height || 0) - head_height - foot_height);
		this.content_height = new Observable();

		this.widgets = {
			input_filter: o.enable_search && new InputFilter({
				on_filter: function(text) {
					o.products.lookup_filter_by_name('text').enable().on_change(null, text);
				}
			}),
			filter_status: o.enable_status && new FilterStatus.default({
				filters: o.products.bind(o.products.get_filters),
				on_disable_change: function(filter_name, disabled){
					console.log('on_disable_change: ' + filter_name);
					var f = o.products.lookup_filter_by_name(filter_name);
					f.disabled = disabled;
					o.products.filter();
				},
				on_remove: function(filter_name){
					var filter = o.products.lookup_filter_by_name(filter_name);
					console.log('grid.on_remove: ' + filter_name, filter);

					filter.deactivate();
					switch(filter.name){
						case 'facets':
							o.products.filter();
							break;
						case 'text':
							$('.thumbnail-list-filter input').val('');
							break;
						default:
							break;
					}
				}
			}),
			scrollbar: new Scrollbar({renderer: o.renderer, scrollable: $(o.renderer.root), height: this.height, content_height: this.content_height})
		}

		$.map(o.products.filters, this.bind(self.add_filter));

		function view(View) {
			self.view = new View({
				renderer: o.renderer,
				products: o.products,
				width: o.width,
				height: o.height,
				on_needs_render: function() {
					o.products.have.filtering.then(o.renderer.bind(o.renderer.render))
				},
				click: o.on_click,
				dblclick: o.on_dblclick,
				middle_click: o.on_middle_click,
				select: function(a){
					console.log('vgrid.select');
					o.products.selection.set(o.products.filtered[a.filtered_idx]);
				},
				scrollbar: self.widgets.scrollbar
			})

			o.products.have.filtering.then(function() {
				self.resize();
				o.renderer.render();

				o.products.selection.subscribe(function(a) {
					if(typeof a === 'number'){
						self.view.scroll_to(a)
						o.renderer.render();
					}
				})
			});

			// an attempt was made to move this to views/base but because views/simple does inherit from base, this did not work
			o.products.subscribe(self, self._on_filter = function() {
				// TODO let the view do this internally - what api to use to notify change?
				self.view.groups_valid = false;
				self.view._positions_valid = false;
				self.view.cache && (self.view.cache.natural_height = false);

				// scroll to top
				// we can perhaps do better than this - the idea is to ensure the scrollbar is fully updated after a filter change.
				if(o.scrollbar){
					$(o.renderer.root).find('.scrollable').scrollTop(0);
					o.scrollbar.state.top = 0;
				}

				o.renderer.render();
			})
			ready.resolve()
		}

		switch(o.view_mode){
			case 'groups':
				require(['./views/groups'], view); break;
			case 'simple':
				require(['./views/simple'], view); break;
			case 'details':
				require(['./views/details'], view); break;
		}
	}

	VGrid.prototype = $.extend({
		render: function() {
			var self = this, o = this.options, w = this.widgets;

			__DEV__ && (o.products.have.filtering.state() !== 'resolved') && console.warn('no filtering');

			/* currently we do not render before products and filtering are available so do not have to provide an empty fallback
			if(o.products.have.filtering.state() !== 'resolved'){
				return virtualise($(o.renderer.root).find('.catalogue-grid')[0])
			}
			*/

			// TODO dont want to do this each time?
			if(o.height){
				this.content_height.set(this.view.natural_height());
			}

			this._sizes = this._sizes || {}
			this._sizes.foot || setTimeout(function(){
				// TODO update grid height based on measurement of head and foot
				self._sizes.foot = $(o.renderer.root).find('.foot').outerHeight();
				//this.height.set(self.options.height - self._sizes.foot - self.options.element.find('.head').height());
			}, 0);

			return h('.catalogue-grid', {style: {height: o.height + 'px'}}, [
				h('.head', [o.enable_search ? w.input_filter.render() : undefined]),
				h('.foot', h('.status', [h('div', [o.products.filtered.length + ' matching items']), o.enable_status ? w.filter_status.render() : undefined])),
				h('.outer.ps-container.ps-active-y', o.height ? {style: {height: this.height.value + 'px'}} : {}, [
					h('.scrollable', {style: o.height ? {height: '100%'} : {}}, [
						this.view.render()
					]),
					w.scrollbar.render()
				])
			])
		},

		add_filter: function(f) {
			var self = this, o = this.options;

			// TODO the list should bind to the filter change events, not the grid
			if($.isFunction(f.fn)){
				f.events && f.events.bind('changed', function(){
					o.products.filter();
				});
			}
		},

		resize: function(size) {
			// Size is optional, if missing is unchanged
			// The height includes the header and footer

			// TODO scrollbar is not being updated
			if(size){
				this.view.options.width = size.width;
				this.view.options.height = this.options.height = size.height;
									var o = this.options;
									var head_height = o.enable_search ? 23 + 10 + 16 : 0
									var foot_height = o.enable_status ? 63 + 10 + 16 : 0
				this.height.set(size.height - head_height - foot_height);
			}
			this.view.resize();
		}
	}, mixins)

	return VGrid;
})
