/*
 *  InputFilter provides a text box that notifies the consumer when meaningful changes have been made.
 */
define(['jquery', 'virtual-dom/h', 'input-filter/input_filter.css'], function($, h) {

var InputFilter = function(options){
	this.options = $.extend({
		placeholder: 'Search',
		on_filter: function(text){}
	}, options);

	this._input = {
		wait: 0,
		text: ''
	}
	this.previous = null;
}

InputFilter.prototype = {

	render: function() {
		var self = this, o = self.options;

		return h('.thumbnail-list-filter', [
			h('.search-icon'),
			h('input', {
				type: 'text',
				placeholder: this.options.placeholder,
				onclick: function(e){
					console.log('input.click');
					e.stopPropagation()
				},
				onkeyup: function(e){
					console.log('input.keyup: val=', e.target.value);
					self._input.text = $.trim(e.target.value);

					var apply_filter = $.throttle(1000, function(){
						o.on_filter(self._input.text);
					});

					function delay(){
						setTimeout(function(){
							if(self._input.text || (self._input.wait > 4)){
								self._input.wait = 0;
								if(self._input.text.toLowerCase() != self.previous){
									self.previous = self._input.text.toLowerCase();
									apply_filter();
								}
							}
							else{
								self._input.wait++;
								delay();
							}
						}, 400);
					}
					delay();
				}
			})
		])
	},

	text: function() {
		return this._input.text;
	}
}

return InputFilter;

});
