define(['config', 'jquery', 'virtual-dom/h'], function(ECMWF, $, h){

var settings = {
	wheelPropagation: false
}

var Scrollbar = function(options) {
	this.options = $.extend({
		renderer: undefined,
		scrollable: null,
		height: null,        // Observable
		content_height: null // Observable
	}, options);

	var self = this, o = this.options;

	this.state = {
		inner_height: 0, // handle size
		top: 0
	}

	var getEventClassName = (function() {
		var incrementingId = 0;
		return function() {
			var id = incrementingId;
			incrementingId += 1;
			return '.perfect-scrollbar-' + id;
		};
	}())

	this.eventClassName = getEventClassName();

	o.content_height.subscribe(function(){
		self.state.inner_height = Math.max(16, o.height.value * o.height.value / o.content_height.value);

		var max_scrolltop = o.content_height.value - o.height.value;
	})
}

Scrollbar.prototype = {
	render: function() {
		var o = this.options;

		if(this.state.inner_height >= this.options.height.value) return this.bind_done = false, []

		this.bind_done || this.bind_mouse()
		this.bind2_done || this.bind_mousewheel()
		this.bind3_done || this.bind_touch()

		//console.log('scrollbar.render: scrolltop=', this.state.top, ' height=', o.height.value + '/' + o.content_height.value + ' top=', this._inner_top(), ' inner=', this.state.inner_height);

		return h(
			'.ps-scrollbar-y-rail',
			{style: {
				top: 0,
				bottom: 0,
				backgroundColor: '#eee',
				opacity: 0.6
			}},
			[
				h('.ps-scrollbar-y', {style: {top: this._inner_top() + 'px', height: this.state.inner_height + 'px'}})
			]
		)
	},

	update_bar_size_and_position: function() {
		var scrollable = this.options.scrollable.find('.scrollable');
		scrollable.parent().find('.ps-scrollbar-y').css({top: this._inner_top(), height: this.state.inner_height})
	},

	_inner_top: function() {
		var o = this.options;
		if(!o.content_height.value) return 0;

		var pct = this.state.top / (o.content_height.value - o.height.value);
		return (o.height.value - this.state.inner_height) * pct;
	},

	bind_mouse: function() {
		var self = this, o = this.options, y0, current_page_y;
		var scrollable = o.scrollable.find('.scrollable');

		if((this.rail = scrollable.parent().find('.ps-scrollbar-y-rail')).length){
			this.handle = this.rail.find('.ps-scrollbar-y');

			this.handle.on('mousedown' + this.eventClassName, function(e) {
				y0 = self.handle.position().top;
				current_page_y = e.pageY;
				self.rail.addClass('in-scrolling');
				e.stopPropagation();
				e.preventDefault();
			});

			this.rail.on('mouseup', function(e){
				if(e.target === self.rail[0]){
					var dy = self.state.inner_height * (e.pageY - scrollable.offset().top < self._inner_top() ? -1 : +1)
					dy && scrollto(self._inner_top() + dy)
					return false
				}
			})

			// TODO bind in mousedown handler instead?
			$(document).on('mousemove' + this.eventClassName, function(e) {
				if (self.rail.hasClass('in-scrolling')) {
					var dy = e.pageY - current_page_y
					scrollto(y0 + dy)
					return false
				}
			});

			$(document).on('mouseup' + this.eventClassName, function(e) {
				if (self.rail.hasClass('in-scrolling')) {
					self.rail.removeClass('in-scrolling');
				}
			});

			this.bind_done = true

			function scrollto(pos) {
				var y = Math.max(0, Math.min(o.height.value - self.state.inner_height, pos))
				var val = Math.round(y * (o.content_height.value - o.height.value) / (o.height.value - self.state.inner_height))

				self.state.top = val;
				scrollable.scrollTop(self.state.top);
				self.state.top = scrollable.scrollTop()

				//self.update_bar_size_and_position();
				self.handle.css({top: y})
				scrollable.parent().parent().trigger('vscroll');
			}
		}
	},

	scroll_to: function(pos) {
		// Note that if the scrollable is not visible, then at some pointer after it is
		// made visible, the browser will restore the previous scrollTop and emit a scroll
		// event.
		// It is very important that this.state.top is in sync with scrollTop otherwise the
		// contents will not be visible.
		var self = this, o = this.options

		this.state.top = Math.min(pos, o.content_height.value - o.height.value);
		o.scrollable.trigger('vscroll');

		// TODO how to avoid triggering vscroll twice?
		o.renderer.on.render.subscribe(self.options, function() {
			var t;
			var scrollable = o.scrollable.find('.scrollable');
			__DEV__ && !scrollable.is(':visible') && console.warn('scrollable is not visible');

			function on_visible() {
				if(t) t = clearTimeout(t)
				scrollable.scrollTop(self.state.top);
				o.scrollable.trigger('vscroll');
			}


			if(scrollable.is(':visible')){
				//scrollable.scrollTop(self.state.top);
				//o.scrollable.trigger('vscroll');
				on_visible()
			}else{
				scrollable.one('scroll', on_visible)

				// there will be no scroll event if the browser has no scrollTop to restore
				if(self.state.top)
					t = setTimeout(on_visible, 1000)
			}

			setTimeout(function() {
				o.renderer.on.render.unsubscribe(self.options);
			}, 0)
		})
	},

	bind_mousewheel: function() {
		var self = this;
		var shouldPrevent = false;
		var scrollable = this.options.scrollable.find('.scrollable');
		if(!scrollable.length) return;
		this.bind2_done = true;

		var getDeltaFromEvent = function (e) {
			var deltaY = -1 * e.originalEvent.deltaY;

			if (typeof deltaY === 'undefined') {
				// OS X Safari
				deltaY = e.originalEvent.wheelDeltaY / 6;
			}

			return deltaY;
		};

		var mousewheelHandler = function (e) {
			var delta = getDeltaFromEvent(e)
			var old_top = scrollable.scrollTop();

			shouldPrevent = false;
			scrollable.scrollTop(old_top - (delta * ECMWF.wheelspeed));
			self.state.top = scrollable.scrollTop()
			if(self.state.top !== old_top){
				//shouldPrevent = true;

				self.update_bar_size_and_position(); // update bar position imediately

				scrollable.parent().parent().trigger('vscroll');
			}

			shouldPrevent = (shouldPrevent || self.shouldPreventDefault(delta));
			if (shouldPrevent) {
				e.stopPropagation();
				e.preventDefault();
			}
		}

		if (typeof window.onwheel !== 'undefined') {
			scrollable.on('wheel' + self.eventClassName, mousewheelHandler);
		} else if (typeof window.onmousewheel !== 'undefined') {
			scrollable.on('mousewheel' + self.eventClassName, mousewheelHandler);
		}

		// fix Firefox scroll problem
		scrollable.on('MozMousePixelScroll' + self.eventClassName, function (e) {
			if (shouldPrevent) {
				e.preventDefault();
			}
		});
	},

	bind_touch: function() {
		var self = this;
		var scrollable = this.options.scrollable.find('.scrollable');
		if(!scrollable.length) return;
		this.bind3_done = true;

		var applyTouchMove = function (differenceX, differenceY) {
			scrollable.scrollTop(self.state.top = scrollable.scrollTop() - differenceY);

			self.update_bar_size_and_position(); // update bar position imediately
			scrollable.parent().parent().trigger('vscroll');
		};

		var startCoords = {}, startTime = 0, speed = {}, breakingProcess = null, inGlobalTouch = false;

		$(window).on('touchstart' + self.eventClassName, function (e) {
			console.log('global touchstart');
			inGlobalTouch = true;
		});
		$(window).on('touchend' + self.eventClassName, function (e) {
			console.log('global touchend');
			inGlobalTouch = false;
		});

		scrollable.on('touchstart' + self.eventClassName, function (e) {
			var touch = e.originalEvent.targetTouches[0];

			startCoords.pageX = touch.pageX;
			startCoords.pageY = touch.pageY;

			startTime = (new Date()).getTime();

			if (breakingProcess !== null) {
				clearInterval(breakingProcess);
			}

			e.stopPropagation();
		});

		scrollable.on('touchmove' + self.eventClassName, function (e) {
			if (!inGlobalTouch && e.originalEvent.targetTouches.length === 1) {
				var touch = e.originalEvent.targetTouches[0];

				var currentCoords = {};
				currentCoords.pageX = touch.pageX;
				currentCoords.pageY = touch.pageY;

				var differenceX = currentCoords.pageX - startCoords.pageX,
				differenceY = currentCoords.pageY - startCoords.pageY;

				applyTouchMove(differenceX, differenceY);
				startCoords = currentCoords;

				var currentTime = (new Date()).getTime();

				var timeGap = currentTime - startTime;
				if (timeGap > 0) {
					speed.x = differenceX / timeGap;
					speed.y = differenceY / timeGap;
					startTime = currentTime;
				}

				e.preventDefault();
			}
		});

		scrollable.on('touchend' + self.eventClassName, function (e) {
			clearInterval(breakingProcess);
			breakingProcess = setInterval(function () {
				if (Math.abs(speed.x) < 0.01 && Math.abs(speed.y) < 0.01) {
					clearInterval(breakingProcess);
					return;
				}

				applyTouchMove(speed.x * 30, speed.y * 30);

				speed.x *= 0.8;
				speed.y *= 0.8;
			}, 10);
		});
	},

	shouldPreventDefault: function (deltaY) {
		var self = this, o = this.options;

		var scrollTop = $(this.options.scrollable).scrollTop();
		if ((scrollTop === 0 && deltaY > 0) || (scrollTop >= o.content_height.value - o.height.value && deltaY < 0)) {
			return !settings.wheelPropagation;
		}

		if (deltaY === 0) {
			return false;
		}
		return true;
	}
}

return Scrollbar

})
