/*
 *  A wintersmith Content plugin for fetching page content from Drupal.
 *
 *  - inherits from the Page class.
 *  - associates with the '.bob' filetype.
 *
 *
 */
(function() {
	var extend = function(child, parent) {
		for (var key in parent) {
			if (hasProp.call(parent, key)) child[key] = parent[key];
		}
		function ctor() { this.constructor = child; }
		ctor.prototype = parent.prototype;
		child.prototype = new ctor();
		child.__super__ = parent.prototype;
		return child;
	};
	var hasProp = {}.hasOwnProperty;

	var async = require('async');
	var fs = require('fs');
	var path = require('path');
	var url = require('url');
	var yaml = require('js-yaml');
	var request = require('sync-request');

  /* Resolve *uri* relative to *content*, resolves using
      *baseUrl* if no matching content is found.
   */
  /*
  var resolveLink = function(content, uri, baseUrl) {

    var nav, part, ref, uriParts;
    uriParts = url.parse(uri);
    if (uriParts.protocol) {
      return uri;
    } else if (uriParts.hash === uri) {
      return uri;
    } else {
      nav = content.parent;
      path = ((ref = uriParts.pathname) != null ? ref.split('/') : void 0) || [];
      while (path.length && (nav != null)) {
        part = path.shift();
        if (part === '') {
          while (nav.parent) {
            nav = nav.parent;
          }
        } else if (part === '..') {
          nav = nav.parent;
        } else {
          nav = nav[part];
        }
      }
      if ((nav != null ? nav.getUrl : void 0) != null) {
        return nav.getUrl() + [uriParts.hash];
      }
      return url.resolve(baseUrl, uri);
    }
  };
  */

	module.exports = function(env, callback) {
		var key, value;

		var RemotePage = (function(superClass) {
			extend(RemotePage, superClass);

			function RemotePage(filepath1, metadata1, markdown1) {
				//console.log('      RemotePage: filepath=', filepath1.relative);
				var y = yaml.safeLoad(fs.readFileSync(filepath1.full, 'utf8'));


				var res = request('GET', y.url + '?_pjax=true', {headers: { 'X-PJAX': '1' }});
				/*
				console.log('-----------------------------------------------------------');
				console.log('-----------------------------------------------------------');
				console.log(res.getBody().toString());
				console.log('-----------------------------------------------------------');
				console.log('-----------------------------------------------------------');
				*/

				this.filepath = filepath1;
				this.metadata = {template: 'page.jade'};
				extend(this.metadata, metadata1);
				extend(this.metadata, y);
				this.markdown = res.getBody().toString()
					// some unresolved issues with mangled tags possibly due to the markdown interpreter.
					.replace(/data-file_info=".*?"/g, '')
					.replace(/%20/, ' ')
					.replace(/en\/forecasts\//g, 'forecasts/')

					// redirects for old urls
					.replace(/forecasts\/charts\/catalogue\?f\[0\]=im_field_chart_type%3A481\&f\[1\]=im_field_chart_type_2%3A607/g, 'forecasts/charts/medium/')
					.replace(/forecasts\/charts\/catalogue\?f%5B0%5D=im_field_chart_type%3A482/g, 'forecasts/charts/mofc_multi/')
					.replace(/forecasts\/charts\/catalogue\?f%5B0%5D=im_field_chart_type%3A483/g, 'forecasts/charts/long-range/');
			}

			RemotePage.prototype.getLocation = function(base) {
				var uri = this.getUrl(base);
				return uri.slice(0, +uri.lastIndexOf('/') + 1 || 9e9);
			};

			RemotePage.prototype.getHtml = function(base) {
				if (base == null) {
					base = env.config.baseUrl;
				}

				/* parse @markdown and return html. also resolves any relative urls to absolute ones */
				//return parseMarkdownSync(this, this.markdown, this.getLocation(base), env.config.markdown || {});
				return this.markdown;
			};

			return RemotePage;

		})(env.plugins.Page);

		RemotePage.fromFile = function(filepath, callback) {
			return async.waterfall([
				function(callback) {
					return fs.readFile(filepath.full, callback);
				},
				function(buffer, callback) {
					return RemotePage.extractMetadata(buffer.toString(), callback);
				},
				(function(_this) {
					return function(result, callback) {
						var markdown = result.markdown;
						var metadata = result.metadata;
						var page = new _this(filepath, metadata, markdown);
						return callback(null, page);
					};
				})(this)
			], callback);
		};

		RemotePage.extractMetadata = function(content, callback) {
			var end, markdown, metadata, parseMetadata, result;
			parseMetadata = function(source, callback) {
				var error, error1, lines, markerPad;
				if (!(source.length > 0)) {
					return callback(null, {});
				}
				try {
					return callback(null, yaml.load(source) || {});
				} catch (error1) {
					error = error1;
					if ((error.problem != null) && (error.problemMark != null)) {
						lines = error.problemMark.buffer.split('\n');
						markerPad = ((function() {
							var i, ref, results;
							results = [];
							for (i = 0, ref = error.problemMark.column; 0 <= ref ? i < ref : i > ref; 0 <= ref ? i++ : i--) {
								results.push(' ');
							}
							return results;
						})()).join('');
						error.message = "YAML: " + error.problem + "\n\n" + lines[error.problemMark.line] + "\n" + markerPad + "^\n";
					} else {
						error.message = "YAML Parsing error " + error.message;
					}
					return callback(error);
				}
			};

			metadata = '';
			markdown = content;
			if (content.slice(0, 3) === '---') {
				result = content.match(/^-{3,}\s([\s\S]*?)-{3,}(\s[\s\S]*|\s?)$/);
				if ((result != null ? result.length : void 0) === 3) {
					metadata = result[1];
					markdown = result[2];
				}
			} else if (content.slice(0, 12) === '```metadata\n') {
				end = content.indexOf('\n```\n');
				if (end !== -1) {
					metadata = content.substring(12, end);
					markdown = content.substring(end + 5);
				}
			}
			return async.parallel({
				metadata: function(callback) {
					return parseMetadata(metadata, callback);
				},
				markdown: function(callback) {
					return callback(null, markdown);
				}
			}, callback);
		};

		//RemotePage.resolveLink = resolveLink;

		// ******************* temporary - keep them separate for debugging
		//env.registerContentPlugin('pages', '**/*.bob', RemotePage);
		env.registerContentPlugin('remote', '**/*.bob', RemotePage);
		return callback();
	};

}).call(this);
