define(['config', 'jquery', 'utils', 'bus', 'jquery-ui/widget'], function(ECMWF, $, Utils, Bus) {

$.widget('ui.ajaxPage', {
	options: {
		links: ''
	},

	_init: function(){
		var self = this, o = self.options;

		self.params = ECMWF.query();

		/*
		 * The dom is very different for different themes.
		 * currently bartik and text-matters themes are supported.
		 * The presence of an 'article' tag is used as a primitive text-matters test.
		 */
		var page_content = '#content article, #content .region-content .content:first'; // the element to be replaced. TODO is possibly wrong for bartik
		var page_container = $(page_content).parent();
		self.page = {
			content: $(page_content),
			h1: page_container.parent().parent().parent().find('h1')
		}

		self.links = $(o.links)
			.orelse(function(){ console.warn('ajaxPage: links is empty'); })
			.attr({'data-pjax': page_content})
			.click(function(event, b){
				console.log('contextual.click url=', $(this).attr('href').split('?')[0] + self._make_query_string());
				var url = $(this).attr('href').split('?')[0] + self._make_query_string();
				if($.pjax){
					$.pjax({url: url, container: page_content});
				}else{
					location = url;
				}
				return false;
		});

		$(window).on('popstate', function(e){
			self.params = ECMWF.query();

			for(var i in self.params){
				self.params[i] = self.params[i].replace('%20', ' ');
			}

			console.log('%cpage.popstate: setting self.params=', 'background-color:#ffa', self.params);
			self._in_popstate = true; // temporary
			Bus().send('page-changed', e);
		});

		/*
		 *  TODO if the page has draft versions, do a full reload
		 *  (this is easier than fully supporting the revisions module).
		 */
		function start(event, response, c){
			console.log('page.start', response, c, c.url);

			$('#content > .messages').remove();

			$(page_content).one('pjax:end', function(e, response){

				self.page.h1.text($.pjax.state.title);

				// when the text matters theme is used, the article does not have a container
				// that does not contain other elements
				// so the content is added inside the article then moved outside afterwards.
				self.page.content.find('article').each(function(){
					$(this).appendTo(page_container);
					self.page.content.remove();
					self.page.content = page_container.find('article');
				});

				self._update_links();

				var tabs = $('#content .tabs-primary, #content .tabs .primary'); // text-matters, bartik

				var content = $($(response.responseText)[1]);

				var node_id = (function get_node_id(content){
					var node_id = parseInt(content.attr('about').split('/')[2], 10);
					if(!node_id){
						node_id = parseInt(content.attr('id'), 10); // bartik ?
						if(!node_id){
							var classes = content.attr('class').split(' ');
							for(var c in classes){
								var s = classes[c].split('-');
								if(s.length > 1){
									node_id = parseInt(s[1], 10);
									if(node_id) break;
								}
							}
						}
					}
					return node_id;
				})(content);

				if(node_id){
					tabs.find('> li').removeClass('is-active');
					tabs.find('a').each(function(){
						// TODO first tab should use the url alias not /node/N
						var link = $(this);
						link.removeClass('active');
						var split = link.attr('href').split('/');
						if(split.length > 2){
							if(split[1] == 'node'){
								split[2] = node_id;
								link.attr('href', split.join('/'));
							}else{
								if(link.attr('href') === self.original_pathname){
									link.attr('href', location.pathname);
								}
							}
						}
					});
					tabs.find('> li:first').addClass('is-active').find('a').addClass('active');
				}else{
					console.warn('page.end: failed to get node_id. responseText=', response.responseText);
					tabs.hide();
				}

				$('#content .workbench-info-block').hide();

				// send google tag manager events
				// -the event type and name of 2nd parameter must match the GTM configuration.
				window.dataLayer.push({
					event: 'pageview',
					virtualUrl: location.href
				});

				$(page_content).one('pjax:start', start);
			});
		}

		self.page.content.on('pjax:beforeSend', function(){
			console.log('beforeSend: orig=', location.pathname);
			self.original_pathname = location.pathname;
		});

		self.page.content.on('pjax:start', start);

		// popstate disabled. see htdocs/sites/all/libraries/jquery-pjax/jquery.pjax.js 799
		/*
		self.element.on('pjax:popstate', page_content, function(e){
			console.log('page: pjax:popstate', e.state);
			self.page.h1.text(e.state.title.replace(' | ECMWF', ''));
			self._update_links();
		});
		*/
	},

	set_params: function(variables){

		var params = Utils.map(variables, function(v, i){ return v.to_string(); });
		console.log('page.set_params: ' + state_id + '. values=', params);

		var changed = false;
		for(var i in params){
			if(this.params[i] !== params[i]){ // must match if self.param is empty
				console.log('page.set_params: changed: ', i, '=', params[i], '(was ' + this.params[i] + ')');
				changed = true;
				break;
			}
		}

		$.extend(this.params, params);

		if(!changed) return;

		if(this._in_popstate){
			console.warn('FIXME in_popstate: changed should be false');
			return;
		}
		this._in_popstate = false;

		var loc = location.href.split('?')[0] + '?' + $.map(this.params, function(v, k){ return k + '=' + v; }).join('&');

		console.log('page.set_params: PUSHSTATE');
		window.history.pushState({
				id: ++state_id,
				url: loc,
				title: document.title + ' ' + state_id
			},
			state_id + ' ' + document.title,
			loc
		);
	},

	_update_links: function(){
		var self = this;
		setTimeout(function(){
			var url = location.pathname.split('?')[0];
			self.links
				.removeClass('active')
				.each(function(){
					var link = $(this);
					if(link.attr('href') === url) link.addClass('active');
				});
		}, 0);
	},

	_make_query_string: function(){
		var blacklist = ['width', 'height'];
		return $.isEmptyObject(this.params)
			? ''
			: ('?' + $.map(
					this.params,
					function(v, k){ v += '';  if(!~$.inArray(k, blacklist)) return k + '=' + v.replace('=', '%3D'); })
				.join('&'))
	}
});

var state_id = 0;

});
