/*
 *  instanceEditor
 *
 *  Provides a form for setting instance properties such as parameter values and size.
 */
(function($){
define(['basic_chart/model/product', 'bus'], function(BasicInstance, Bus){

	// TODO this is the product of refactoring. make sure contents are related only to product selection and parameter selection.
	// A form is created. No dialogue/window creation or management is done.

	$.fn.instanceEditor = function(args){
		var el = $(this), o;

		if(el.data('instanceEditor')){
			o = el.data('instanceEditor');
			o.item = args.item;
			o.variables = args.variables;
			show_axes(o.variables, o.item.values);
			return this;
		}

		o = $.extend({
			item: null,            // pre-selected product. If this is set, the selector will not be shown.
			variables: null,
			legacy_products: null, // model collection object
			form: $(),
			use_legacy_selector: true,
			on_show_form: $.noop,
			done: $.noop,          // called following 'ok' or 'cancel'.
			on_close: $.noop,
			on_resize: $.noop
		}, args || {});

		var item = o.item;

		el.data('instanceEditor', o);

		if(item){
			show_axes(o.variables, o.item.values);
			return this;
		}

		var axes;

		function show_axes(variables, values){
			var item = o.item;
			variables = $.map(variables, function(v){ return v; }); // convert to array.


			function fix(item){
				false && item.variables && $.each(item.variables, function(i, val){
					if(val.type === 'retention'){
						val.type = 'choice';
					}
					if(val.type === 'availability'){
						val.type = 'choice';
					}
					if(val.type === 'selection'){
						val.type = 'choice';
					}
					// see Jira ticket (NWEB-235?)
					/*
					if(val.type === 'choice' && val.values.length > 4){
						val.menu = true;
					}
					*/
				});
				return item;
			}

			var size = (o.widgets === 'interactive')
				? [
					{name: "width", type: "integer", value: 640},
					{name: "height", type: "integer", value: 480}
				  ]
				: [];

			// two different item types, one from get_plot, one from product list (check)
			/*
			item && (function unify(){
				item.variables || (item.product && (item.variables = item.product.variables));
			})();
			*/

			el.find('.variables').removeClass('empty').orelse(function(){
				o.selected = item;
				el.data('instanceEditor', o); // TODO check saving is necessary.
				return el.div({'class': 'variables'});
			}).empty().each(function(){
			}).form({
				items: variables ? $.merge(variables, size) : [],
				values: values,
				ok: function(form_data){
					var chart = o.selected;
					if(!chart){ console.warn('instanceEditor.submit: nothing selected'); return; }
					if(!form_data || $.isEmptyObject(form_data)){
						console.error('instanceEditor.submit: form did not return any data');
						inline_message('Cannot create chart: no data.');
						return;
					}
					console.log('instanceEditor.submit: chart=', chart);
					console.log('instanceEditor.submit: data=', form_data);

					o.form.find('.modal-footer #_ok').prop('disabled', true);   // TODO

					var modifiable = (function get_modifiable(){
						var m = {};
						o.form.find('td.modifiable input').each(function(){
							var title = $(this).parent().parent().find('td.form-item-title').text().replace(':', '');
							var obj = $.grep(variables, function(e){ return e.title === title; });
							obj.length && (m[obj[0].name] = $(this).is(':checked'));
						});
						return m;
					})();

					var instance = (function make_instance(){
						return $.extend(true, {}, chart, {values: form_data, modifiable: modifiable});
					})();

					// TODO instantiate BasicInstance earlier ?
					//if(!chart.save) chart = new BasicInstance($.extend(true, {}, chart, {values: instance}));
					if(!instance.save) instance = new BasicInstance.Instance(instance);

					o.form.attr('disabled', 'disabled');

					var timer = setTimeout(function(){
						console.log('instanceEditor.submit: chart load failed');
						alert('chart load failed (timeout)');
						timer = null;
						o.form.removeAttr('disabled');
					}, 20000);

					instance.save(function(result, error){
						if(error){
							alert("Error creating chart instance. " + error);
							return;
						}

						// the thumbnail widget is designed for this but currently does not work with legacy products
						//
						//var a = $('<div>').thumbnail({name: chart.name, load: true, loaded: function(){
						//});

						//TODO if there is a delay in getting the image, the dialogue box will not close

						instance.load(function(url){
							//note: if the timer is not set, then we have timed out, and should probably not show the image
							timer && (clearTimeout(timer), timer = null);

							var error = instance.error_msg;
							if(instance.error_msg){
								console.log('instanceEditor.submit: chart load failed. error=', error);
								alert('failed to get chart image' + (typeof error === 'string' ? ' (' + error + ')' : ''));
							}else if(!url){
								console.log('instanceEditor.submit: chart load failed. no url');
								alert('failed to get chart image');
							}else{
								console.log('instanceEditor.submit: got image ok. url=', url);

								/*
								var map = {'"': '&quot;', '&': '\u0026'};
								var enc = JSON.stringify(instance).replace(/./g, function(x){ return (x in map) ? map[x] : x; });
								*/

								instance.product && update_page_tags(instance);
								instance.product && update_page_thumbnail(instance);

								var result = {
									alt   : instance.title_() || instance.data.title, // used to use description but this is no longer part of the product definition.
									title : instance.title_() || instance.data.title,
									src   : url,
									'data-chart': instance.data.name
								}
								instance.data.product.viewport && $.extend(result, instance.data.product.viewport);

								var missing;
								if(!(missing = $.map(result, function(i, v){ if(!i) return v})).length){
									//o.on_close($('<div class="www-chart-outer">')
									o.on_close($('<div>')   //CS: Fixing white space between title and chart product
										.append($('<img/>', result))[0]
										.outerHTML);
								}else{
									inline_message('Cannot insert incomplete chart: ' + missing.join(', '));
									return;
								}
							}
							o.done(el.find('select.package-select option:selected').val());
						});
					});
				},
				cancel: function(){
					o.done();
				}
			})
			.each(function(){
				o.on_show_form();

				if(!o.use_legacy_selector){
					o.form.find('td.form-buttons #_ok').remove();
					o.form.find('td.form-buttons #_cancel').remove();
				}

				o.on_resize();

				o.form.find('tr').find('td:eq(2)').each(function(ev){
					$(this).remove();
				});

				o.form.find('.form-row').each(function(){
					$(this).find(':contains(width), :contains(height)').length || $(this).td().addClass('modifiable').checkbox({label: 'User modifiable', checked: true});
				});

				(!variables || !variables.length) && (
					axes && axes.html('<span class="not-available">Parameters not available</span>')
						, console.warn('cannot display axes: no variables property: item=', item)
				);
			});
		}

		return this.each(function(){
			if(o.use_legacy_selector){
				var request_id = 0;
				el.find('.legacy-selector').orelse(function(){
					require(['thumbnail_popup'], function(){
						el.thumbnailPopup();
					});
					o.selector = el.div().legacySelector({
						model: o.legacy_products,
						on_select: function(selection){
							console.log('instance_editor.on_select:', selection);
							if(!selection.package){
								console.error('instance_editor.on_select: missing package');
								return;
							}
							var instance = new BasicInstance.Instance(selection);
							o.selected = selection;
							el.data('instanceEditor', o); // TODO check this is necessary.
							axes && axes.loader().loader('active');
							o.form.find('.variables').addClass('disabled').find(':input').prop('disabled', true);
							el.find('.not-available').remove();
							request_id++;
							var r = request_id;
							instance.set_parameters({}, function() {
								if(r === request_id){
									console.log('instance_editor: got product details. result=', instance.data);
									if(instance.error_msg){
										o.form.find('.variables').empty();
										alert(instance.error_msg);
										return;
									}

									$('.modal-footer').empty(); // temp - just testing.

									// TODO should pass the whole instance directly?
									el.instanceEditor({item: instance.data, variables: instance.variables, form: o.form, widgets: selection.widgets, on_resize: o.on_resize});

									o.form.find('.variables').removeClass('disabled');
								}
								else console.log('instance_editor: ignoring old request: ', r, request_id);
							});

							instance.thumbnail(function(url, error){
								url && Bus().send('product-selection-change', {url: url, instance: instance});
							});

							o.message && o.message.remove();
						}
					})
					o.on_resize();

					require(['ajax_loader'], function(){
						(axes = el.div({'class': 'variables empty'})).loader().loader('active');
					});
				});

			}else{
				list.set_collection(o.legacy_products);
				el.div().grid({
					view_mode: 'list',
					on_click: function(chart){
						o.selected = chart;
					},
					on_model_change : function(items){
						console.log('instance_editor.on_model_change');

						var item = $.extend({}, items[0]);
						el.instanceEditor({item: item, on_resize: o.on_resize});

						//$('.form-buttons').parent().remove();
						// the form widget creates the buttons in the wrong place so they have to be moved.
						o.form.find('.form-buttons #_ok').addClass('btn primary').appendTo('.modal-footer');
						o.form.find('.form-buttons #_cancel').addClass('btn').appendTo('.modal-footer');
					}
				});
			}
		});

		// this functionality is no longer needed (keywords are no longer in the product definition).
		function update_page_tags(instance){
			$('#edit-field-tags-und').each(function(){
				var tags = $(this);
				var chart_keywords = (instance.product.keywords || '').split(' ');
				var k = tags.val() ? $.map(tags.val().split(','), function(i){ return $.trim(i); }) : [];
				for(var i in chart_keywords){
					if(!~$.inArray(chart_keywords[i], k)){
						k.push(chart_keywords[i]);
					}
				}
				tags.val(k.join(', '));
			});
		}

		// FIXME: Thumbnail set here /URL/ ... 
		function update_page_thumbnail(instance){
			var thumbnail = $("#edit-field-thumbnail-url-und-0-value");
			var thumb_url = "/backend/get-thumbnail/product/" + instance.data.product.name;
			thumbnail.val(thumb_url);
		}

		function inline_message(msg){
			console.warn(msg);
			require(['inline_message'], function(){
				o.message = el.div().insertAfter(o.selector).inlineMessage({message: msg, mode: 'error'});
			});
		}
	}
});
})(jQuery);

