/*
 *  Collection object for legacy chart products.
 *
 *  To clarify: this is for Products and not Instances.
 */
define(['config', 'jquery', 'ajax_queue', 'basic_chart/mixins', 'basic_chart/model/product', 'utils'], function(ECMWF, $, Ajaxqueue, mixins, BasicInstance){

	var legacy_products;
	return legacy_products || (legacy_products = new (function LegacyProducts(){

																												// FIXME should return products not instances !!
		this.fetch = function(pkg, on_success){
			if(!on_success && $.isFunction(pkg)){ on_success = pkg; pkg = null; }
			console.log('legacy_products.fetch');

			this.fetch_lite(pkg, function(list){
				on_success($.map(list, function(product){ return new BasicInstance.Instance(product); }));
			});
		}

		/*
		 *   Returns the list more or less as returned by the backend.
		 *   It should be removed if measurements cannot confirm any performance benefits.
		 */
		this.fetch_lite = function(pkg, on_success){
			if(!on_success && $.isFunction(pkg)){ on_success = pkg; pkg = null; }
			var self = this;
			console.log('legacy_products.fetch_lite');

			Ajaxqueue({
				url        : ECMWF.options.ajax.www_product_list,
				data       : JSON.stringify({'package': (pkg || '')}),
				priority   : 'very-high',
				success    : function (result) {
					var products = $.map(result.products, function(product){
						if(self.validate([
								['has name', function(){ return product.name; }],
								['has title', function(){ return product.title; }]
								//['has service', function(){ return product.service; }]
							])){

							(function conform(){
								//product.keywords = (product.keys || '').replace(' ', ', ');
								delete product.keys;
								product.title || (product.title = 'Untitled');
							})();

							return product;
						}
					});

					on_success(self.list = $(products));
				}
			});
		}

		this.packages = function(on_success){
			console.log('legacy_products.packages');
			var self = this;

			if(self._packages){ on_success(self._packages); return; }

			function _fail(){
				timer = null;
				on_success();
			}

			//TODO use ajaxqueue timeout
			var timer = setTimeout(function(){
				_fail();
			}, 6000);

			Ajaxqueue({
				url: '/plots/get-packages/',
				priority: 'very-high',
				success: function(result){
					timer && (clearTimeout(timer), timer = null);
					on_success(self._packages = result.packages);
				},
				error: function(a, b){
					console.warn('packages.error:', a, b);
					_fail();
				}
			});
		}

		/*
		 *  Added for compatibility with grid/products
		 */
		this.facets = function(){
			var self = this;
			if(!this._facets){
				this._facets = {};

				$.each(this.list, function(i, product){
					if(product.data && product.data.tags){
						$.each(product.data.tags, function(i, v){
							if(i === 'title') return; // TODO remove server-side

							self._facets[i] || (self._facets[i] = {});

							if(Array.isArray(v))
								$.each(v, function(j, v){ self._facets[i][v] = true; });
							else
								self._facets[i][v] = true;
						});
					}
				});
				$.each(self._facets, function(i, v){
					self._facets[i] = $.map(v, function(_, i){ return i; });
				});
			}

			return this._facets;
		}

		this.keywords = function(order){
			console.warn('legacy_products.keywords FN NOT USED?');
			var self = this;

			function sort_keywords_by_popularity(keywords){
				function sortfunction(a, b){
					return a.n < b.n;
				}
				var arr = [];
				for(var i in keywords){
					arr.push({name:i, n: keywords[i]});
				}
				arr.sort(sortfunction);
				return arr;
			}

			var tags = {};
			if(true || order === 'by_popularity'){
				self.list.each(function(){
					if(this.keywords){
						var _tags = this.keywords.split(' '); //warning different separator to eccharts
						for(var i in _tags){
							if(_tags[i]){
								if(typeof tags[_tags[i]] === 'undefined') tags[_tags[i]] = 1;
								else tags[_tags[i]]++;
							}
						}
					}
				});
				return sort_keywords_by_popularity(tags);
			}

			//for unsorted tags, return an object not an array.
			//-this path is currently not used.
			self.list.each(function(){
				var _tags = this.keywords.split(', ');
				for(var i in _tags){
					if(_tags[i]){
						if(typeof tags[_tags[i]] === 'undefined') tags[_tags[i]] = 1;
						else tags[_tags[i]]++;
					}
				}
			});
			return tags;
		},

		this._validate_instance = function(result){

			var self = this;
			var instance = result.instance;
			var tests = [
				['has instance', function(){ return instance; }],
				['is instance', function(){ return instance.kind === 'instance'; }]
			];
			for(var t in tests) if(!tests[t][1]()){
				var message = tests[t][0];
				console.warn('validate_instance: "' + message + '" failed.');
				self.error_msg = "test '" + message + "' failed";
				return false;
			}
			return true;
		},

		this.validate_availability = function(availability){
			var self = this;

			if(this.validate([
				['is present', function(){ return availability && availability.availability; }],
				['is array', function(){ return Array.isArray(availability.availability); }],
				['array not empty', function(){ return availability.availability.length; }],
				['item contains 3 elements', function(){ for(var i in availability.availability) if(Array.isArray(availability.availability[i]) && availability.availability[i].length !== 3) return false; return true; }]
			])){
				return true;
			}else{
				self.img && self.img.data('uiWwwChart')._show_message(this.error_msg);
				return false;
			}
		}

		this.save = function(item, done){
			console.warn('LegacyProducts.save: deprecated: use BasicInstance.save instead');

			// item can be either a product, or an existing instance
			var is_new = (!item.kind) || (item.kind !== 'instance');

			var json = $.extend({}, item);

			if(is_new){
				json.package || (json.package = 'medium');
			}

			// keep request size down. jsonp has a hard limit.
			delete json.variables;
			delete json.axis;
			delete json.product;

			Ajaxqueue({
				url: is_new ? ECMWF.options.ajax.add_instance : ECMWF.options.ajax.update_instance,
				data: JSON.stringify(json),
				success: function(result){
					var error = !result
						? 'no result'
						: result.error
							? result.error
							: (!result.instance || !result.instance.name)
								? 'no instance'
								: null;
					console.log('save.complete: instance=', result ? result.instance : null);
					done && done(result.instance, error);
				},
				error: function(a, b){
					console.log('legacy.save.error:', a, b);
					done && done(null, {error: 'error (TODO)'})
				}
			});
		}

		$.extend(this, mixins);
	})());
});
