/*
 *   Wrapper for chart ui elements. includes the chart itself, controls for axes, time navigation etc.
 *
 *   The widget should be instantiated on a dom element that has the 'data-chart' attribute.
 */

define(['config', 'jquery', 'browser', 'bus', './www_chart.css', 'jquery-ui/widget', 'element', 'form', 'utils', 'product_browser/modernizr'], function(ECMWF, $, Browser, Bus) {

	$.widget('ui.wwwChart', {
		options: {
		},

		_init: function(){
		},

		_create: function(){
			var self = this;
			self.first = true;
			self.element.addClass('www-chart');

			Browser.msie8 && $('body').addClass('msie8');

			this.instance = {name: this.element.attr('data-chart')};
			this._load_instance(function(instance, error){
				if(!instance || error){ self._show_message('<b>Chart may be out of date</b></br>Failed to load chart instance: <i>"' + ($.isPlainObject(error) ? error.error : error) + '"</i>.'); return; }

				self._factory()._create_page_navigation();
			});
		},

		_factory: function(){
			var self = this;

			self.container = $('<div>', {'class': 'www-chart-container'}).insertAfter(self.element);

			var width =
				parseInt(self.element.attr('width'), 10)
				|| (function width_from_style(){
					var style = self.element.attr('style');
					if(style){
						var styles = style.split(';');
						for(var i in styles){
							var a = styles[i].split(':');
							if(a[0] === 'width') return parseInt(a[1], 10);
						}
					}
				})()
				|| (console.warn('failed to get width'), null);

			var size = {width: width || parseInt(self.element.width(), 10), height: Math.max(32, parseInt(self.element.height(), 10))};
			width || (size.width && (width = size.width));

			width && self.container.parent().css({width: width});

			if(this.instance.has_widget('interactive')){
				require(['chart/product', 'chart/latlon', 'toolbar', 'tooltip', 'chart/chart.css'], function(){
					require(['www_chart/www_interactive_chart'], function(){

						self.chart = self.element.wrap('<div class="www-chart-image-wrapper">').parent().css(size);
						self.chart.appendTo(self.container);
						self.element.css(size).addClass('prototype');

						self.container.interactiveChart({instance: self.instance, products: self.products});
					});
				});
			}else{
				// 'basic' chart.

				self.container.parent().addClass('basic');
				self.element.appendTo(self.container);

				// TODO a container has already been added. Why is another one needed ?
				self.element.wrap('<div class="www-chart-image-wrapper">');

				var toolbar_wrapper = self.container.find('.www-chart-image-wrapper').div('.toolbar-wrapper');

				var controls = toolbar_wrapper.div('.chart-controls');

				require(['basic_chart', 'basic_chart/plugins/time_navigator', 'basic_chart/parameter_menu', 'basic_chart/tools', 'basic_chart/plugins/fail'], function(_, TimeNavigator, ParameterMenu, Tools, Fail){
					self.container.basicChart({
						instance: [self.instance],
						products: self.products,
						params: ECMWF.query(),
						plugins: [
							{mixin: ParameterMenu, options: {controls: controls}},
							{mixin: Tools, options: {parent: toolbar_wrapper}},
							TimeNavigator,
							Fail
						]
					});

					function set_content() {
						/*
						 * The dom is very different for different themes.
						 * currently bartik and text-matters themes are supported.
						 * The presence of an 'article' tag is used as a primitive text-matters test.
						 */
						self.page = {
							content: $('#content article:visible, #content .region-content .content:first'),
							outer_content: $('#content')
						}
						if(self.page.outer_content.length !== 1) console.warn('n content elements != 1 :', self.page.outer_content.length);
						if(self.page.content.length !== 1) console.warn('n content elements != 1 :', self.page.content.length);
					}
					set_content();

					self.page.content.on('pjax:end', function(e, response){
						console.log('pjax.end: response=', $(response.responseText));
						setTimeout(function() {
							set_content(); // needs to be updated for webkit
							self.page.content.find('img[data-chart]:visible').orelse(function(){ console.warn('chart not found'); }).wwwChart(); // warning: the old content still exists but is unparented
						}, Browser.webkit ? 500 : 0);
					});

					var path = location.pathname;
					Bus().listen('page-changed', function(_, e){
						// this only handles query parameter changes. changes to the url are handled by pjax
						console.warn('wwwChart: page-changed', e);

						(function merge_qparams(){
							self.instance.set_parameters(ECMWF.query());
						})();

						// change location if needed while pjax popstate is temporarily disabled.
						if(location.pathname !== path){
							console.log('path changed');
							location.href = location.href;
						}
						path = location.pathname;
					});

					self.instance.events.on('parameters-changed', function(changed) {
						self._update_page();
					});

					// not entirely sure this is necesary as a parameters-changes seems to be emitted.
					self.container.messageBus().listen('chart-animation-stop', function() {
						self._update_page();
					});

					/*
					// uncomment this to have wider page in order to show 3rd column with epsgrams
					if(this.instance.has_widget('epsgram')){
						$('#header div.section, #featured div.section, #messages div.section, #main, #triptych, #footer-columns, #footer').css({width: 1250});
						$('#content').css({width: 700});
					}
					*/
				});
			}
			return this;
		},

		_update_page: function() {
			$.fn.ajaxPage && $(document).ajaxPage('set_params', this.instance.variables);
		},

		_load_instance: function(done){
			var self = this;
			if(this.instance){
				if(this.instance.variables){ done(this.instance); return; }

				this._load_model(function(_products){
					require(['basic_chart/model/product'], function(BasicInstance){
						BasicInstance.get_info(self.instance, ECMWF.query(), function(instance, error){
							self.instance = instance;
							if(!error){
								self._verify() || (error = 'Product instance verification failed (' + self.verification_message + ').');
							}
							done(self.instance, error);
						});
					});
				});
			}
		},

		_load_model: function(done){
			var self = this;
			require(['www_chart/model/legacy_products'], function (_products){
				done && done(self.products = _products);
			});
		},

		//TODO move into BasicInstance
		_verify: function(){
			var self = this;

			// 'variables' was removed from the list as presumably default values can be used.
			return !$.map(['name', 'title', 'package'], function(v, i){
				if(!self.instance.data.product[v]){
					self.verification_message = "missing '" + v + "'";
					return true;
				}
			}).length;
		},

		_create_page_navigation: function(){
			var self = this;

			setTimeout(function(){
				var catalogue = {
					link: '/en/forecasts/charts/catalogue',
					text: 'Back to charts'
				}

				var from_catalogue = false;
				$.each(['/search/charts/', '/search/site/', '/charts/catalogue'], function(i, route){
					if (document.referrer.match(route)) {
						catalogue.link = document.referrer.replace(/^http:\/\/.*?\//, '/');
						from_catalogue = true;
					}
				});
				if ((!from_catalogue) && localStorage.catalogue)
					catalogue = JSON.parse(localStorage.catalogue);

				var history_nav;
				$('.history-block')
					.each(function(){
						history_nav = $(this).find('> a');
					})
					.orelse(function(){
						var block = $('#block-views-related-charts-block');
						self.history_block = block.clone()
							.empty().removeAttr('id')
							.addClass('history-block')
							.insertBefore(block);
						(history_nav = self.history_block.a({href: catalogue.link}))
							.span({className: 'ui-icon ui-icon-circle-arrow-w'}).parent()
							.span({className: 'link-to-catalogue'});
					});

				history_nav.find('span:last').text(catalogue.text);

				localStorage.catalogue = JSON.stringify(catalogue);
			}, 0);

			return this;
		},

		_show_message: function(message){
			// TODO in many if not all cases, to avoid page stutter and to make less obtrusive,
			//      this message would be better overlayed on the chart.
			console.warn('wwwChart: ', message);
			(this.container || this.element.parent()).prepend('<p class="www-chart-message">' + message + '</p>');
		},

		_destroy: function(){
			this.element.removeClass('www-chart');
		}
	});

$(function(){
	require(['shortcuts', 'jquery-ui/core'], function(){
		$(document).shortcuts({
			shortcuts: [
				// TODO some keys are disabled due to clashes with normal browser usage.
				//{ key: $.ui.keyCode.UP,     signal: 'next-base-time' },
				//{ key: $.ui.keyCode.DOWN,   signal: 'prev-base-time' },
				{ key: $.ui.keyCode.LEFT,   signal: 'prev-step' },
				{ key: $.ui.keyCode.RIGHT,  signal: 'next-step' },
				//{ key: $.ui.keyCode.HOME,   signal: 'first-step' },
				//{ key: $.ui.keyCode.END,    signal: 'last-step' },
				{ key: $.ui.keyCode.COMMA,  signal: 'animation-speed-down' },
				{ key: $.ui.keyCode.PERIOD, signal: 'animation-speed-up' },
				{ key: 70,                  signal: 'fullscreen' }            // 'f' 
			]
		});
	});

	require(['www_chart/ajax_page'], function(){
		$(document).ajaxPage({links: '#sidebar-first .contextual-links-region a, #block-views-related-charts-block .item-list a'});
	});
});

$.getUserPreference = $.getUserPreference || $.noop;
$.setUserPreference = $.setUserPreference || $.noop;

});
