/*
 *  This is a temporary proof-of-concept to evaluate the use of a canvas
 *  element to smooth chart image changes.
 *
 *  Monkey-patching is used to allow the modification to be unobtrusive.
 *
 *  Only panning is supported. Zooming is not supported.
 *
 */
(function ($) {
	var use_trails = window.location.href.indexOf("trail") != -1;

	$.use_trails = use_trails;

	$.chartRegisterPlugin('canvasRenderer', function(chart) {
		var self = this;

		var redraw = chart._redraw;

		chart._redraw = function() {

			var size;
			chart.img.one('load', function(){
				setTimeout(function(){ // wait for product.update (perhaps should use chart-content-update event instead)
					if(size) return; // event is triggered twice

					size = {width: chart.element.width(), height: chart.element.height() || 300};

					// A backbuffer is used for performance but the improvement is not large.
					self.$canvas0 = $('<canvas>', {css: $.extend({overflow: 'hidden', position: 'absolute'}, size)});
					self.canvas0 = self.$canvas0[0];
					self.canvas0.width = size.width;
					self.canvas0.height = size.height;
					self.ctx0 = self.canvas0.getContext('2d');

					self.$canvas = chart.clip.canvas({'class': 'chart-canvas', css: $.extend({overflow: 'hidden', position: 'absolute', backgroundColor: '#fff'}, size)});
					self.canvas = self.$canvas[0];
					self.canvas.width = size.width;
					self.canvas.height = size.height;
					self.ctx = self.canvas.getContext('2d');
					self.ctx.clearRect(0, 0, size.width, size.height);
					self.ctx.globalAlpha = 1.0;

					self.ctx.push_state = function(fn) {
						var tmp = self.ctx.globalCompositeOperation;
						fn();
						self.ctx.globalCompositeOperation = tmp;
					}

					self.ctx0.clearRect(0, 0, size.width, size.height);
					self.ctx0.drawImage(chart.img[0], 0, 0, size.width, size.height);

					blit(0, 0);

					/*
					self.$canvas.mousedown(function(event) {
					});
					self.$canvas.mousemove(function(event) {
					});
					self.$canvas.mouseup(function(event) {
					});
					*/

					var timer;
					function queue_render(x, y) {
						timer || (timer = setTimeout(function(){
							timer = clearTimeout(timer);

							blit(x, y);
						}, 50));
					}

					/*
					var drag = chart._mouseDrag;
					chart._mouseDrag = function(e) {
						drag.apply(this, arguments);

						var start = self.drag.start;
						var end = { x: event.pageX, y: event.pageY };
						self._transient_pan(self.drag.position.top + end.y - start.y, self.drag.position.left + end.x - start.x);

						queue_render();
					}
					*/

					chart._transient_pan = function(y, x) {
						queue_render(x, y);
					}

					chart._redraw = function(done, _change_type) {
						chart.img.load(function(){
							setTimeout(function(){ // wait for product.update
								var product_size = chart.product.size();
								if(product_size.width !== size.width || (product_size.height !== size.height)){
									size = {width: chart.product.size().width, height: chart.product.size().height};

									self.canvas0.width = size.width;
									self.canvas0.height = size.height;
									self.ctx0.globalCompositeOperation = 'copy';

									self.canvas.width = size.width;
									self.canvas.height = size.height;
									self.$canvas.css(size);
									self.ctx.globalCompositeOperation = 'copy';
								}

								on_image_change(chart.img[0]);
							}, 0);
						});

						redraw.apply(chart, arguments);
					}

					var showFrame = chart.showFrame;
					chart.showFrame = function(time, image) {
						on_image_change(image[0]);
						showFrame.apply(chart, arguments);
					}

					function on_image_change(img) {
						self.ctx0.clearRect(0, 0, size.width, size.height);
						self.ctx0.drawImage(img, 0, 0, size.width, size.height);

						blit(0, 0);
					}

					function blit(x, y) {
						if(use_trails){
							self.ctx.push_state(function(){
								self.ctx.fillStyle = 'rgba(1, 1, 1, 0.80)';
								self.ctx.globalCompositeOperation = 'destination-in';
								self.ctx.fillRect(0, 0, size.width, size.height);
							});
						}else{
							self.ctx.clearRect(0, 0, size.width, size.height);
						}
						self.ctx.drawImage(self.canvas0, x, y);
					}
				}, 0);
			});
			redraw.apply(chart, arguments);
		}
	});
} (jQuery));
